@@ -15,11 +15,9 @@ m.Template = m.Class({
 		this._tpCache = new Map();
 	},
 
-	// helper function to return a template value, optionally adjusting for tech.
-	// TODO: there's no support for "_string" values here.
+	// Helper function to return a template value, adjusting for tech.
 	"get": function(string)
 	{
-		let value = this._template;
 		if (this._entityModif && this._entityModif.has(string))
 			return this._entityModif.get(string);
 		else if (this._templateModif)
@@ -31,16 +29,13 @@ m.Template = m.Class({
 
 		if (!this._tpCache.has(string))
 		{
+			let value = this._template;
 			let args = string.split("/");
 			for (let arg of args)
 			{
-				if (value[arg])
-					value = value[arg];
-				else
-				{
-					value = undefined;
+				value = value[arg];
+				if (value == undefined)
 					break;
-				}
 			}
 			this._tpCache.set(string, value);
 		}
@@ -53,6 +48,12 @@ m.Template = m.Class({
 
 	"civ": function() { return this.get("Identity/Civ"); },
 
+	"matchLimit": function() {
+		if (!this.get("TrainingRestrictions"))
+			return undefined;
+		return this.get("TrainingRestrictions/MatchLimit");
+	},
+
 	"classes": function() {
 		let template = this.get("Identity");
 		if (!template)
@@ -63,21 +64,13 @@ m.Template = m.Class({
 	"hasClass": function(name) {
 		if (!this._classes)
 			this._classes = this.classes();
-		let classes = this._classes;
-		return classes && classes.indexOf(name) != -1;
+		return this._classes && this._classes.indexOf(name) != -1;
 	},
 
 	"hasClasses": function(array) {
 		if (!this._classes)
 			this._classes = this.classes();
-		let classes = this._classes;
-		if (!classes)
-			return false;
-
-		for (let cls of array)
-			if (classes.indexOf(cls) == -1)
-				return false;
-		return true;
+		return this._classes && MatchesClassList(this._classes, array);
 	},
 
 	"requiredTech": function() { return this.get("Identity/RequiredTechnology"); },
@@ -107,7 +100,7 @@ m.Template = m.Class({
 
 	"cost": function(productionQueue) {
 		if (!this.get("Cost"))
-			return undefined;
+			return {};
 
 		let ret = {};
 		for (let type in this.get("Cost/Resources"))
@@ -118,7 +111,7 @@ m.Template = m.Class({
 	"costSum": function(productionQueue) {
 		let cost = this.cost(productionQueue);
 		if (!cost)
-			return undefined;
+			return 0;
 		let ret = 0;
 		for (let type in cost)
 			ret += cost[type];
@@ -142,7 +135,7 @@ m.Template = m.Class({
 		{
 			let w = +this.get("Obstruction/Static/@width");
 			let h = +this.get("Obstruction/Static/@depth");
-			return { "max": Math.sqrt(w*w + h*h) / 2, "min": Math.min(h, w) / 2 };
+			return { "max": Math.sqrt(w * w + h * h) / 2, "min": Math.min(h, w) / 2 };
 		}
 
 		if (this.get("Obstruction/Unit"))
@@ -155,10 +148,10 @@ m.Template = m.Class({
 		let left = this.get("Obstruction/Obstructions/Left");
 		if (left && right)
 		{
-			let w = +right["@x"] + right["@width"]/2 - left["@x"] + left["@width"]/2;
-			let h = Math.max(+right["@z"] + right["@depth"]/2, +left["@z"] + left["@depth"]/2) -
-			        Math.min(+right["@z"] - right["@depth"]/2, +left["@z"] - left["@depth"]/2);
-			return { "max": Math.sqrt(w*w + h*h) / 2, "min": Math.min(h, w) / 2 };
+			let w = +right["@x"] + right["@width"] / 2 - left["@x"] + left["@width"] / 2;
+			let h = Math.max(+right["@z"] + right["@depth"] / 2, +left["@z"] + left["@depth"] / 2) -
+			        Math.min(+right["@z"] - right["@depth"] / 2, +left["@z"] - left["@depth"] / 2);
+			return { "max": Math.sqrt(w * w + h * h) / 2, "min": Math.min(h, w) / 2 };
 		}
 
 		return { "max": 0, "min": 0 }; // Units have currently no obstructions
@@ -175,7 +168,7 @@ m.Template = m.Class({
 		{
 			let w = +this.get("Footprint/Square/@width");
 			let h = +this.get("Footprint/Square/@depth");
-			return Math.sqrt(w*w + h*h) / 2;
+			return Math.sqrt(w * w + h * h) / 2;
 		}
 
 		if (this.get("Footprint/Circle"))
@@ -195,31 +188,47 @@ m.Template = m.Class({
 
 	"isRepairable": function() { return this.get("Repairable") !== undefined; },
 
-	"getPopulationBonus": function() { return +this.get("Cost/PopulationBonus"); },
+	"getPopulationBonus": function() {
+		if (!this.get("Population"))
+			return 0;
 
-	"armourStrengths": function() {
-		if (!this.get("Armour"))
+		return +this.get("Population/Bonus");
+	},
+
+	"resistanceStrengths": function() {
+		let resistanceTypes = this.get("Resistance");
+		if (!resistanceTypes || !resistanceTypes.Entity)
 			return undefined;
 
-		return {
-			"Hack": +this.get("Armour/Hack"),
-			"Pierce": +this.get("Armour/Pierce"),
-			"Crush": +this.get("Armour/Crush")
-		};
+		let resistance = {};
+		if (resistanceTypes.Entity.Capture)
+			resistance.Capture = +this.get("Resistance/Entity/Capture");
+
+		if (resistanceTypes.Entity.Damage)
+		{
+			resistance.Damage = {};
+			for (let damageType in resistanceTypes.Entity.Damage)
+				resistance.Damage[damageType] = +this.get("Resistance/Entity/Damage/" + damageType);
+		}
+
+		// ToDo: Resistance to StatusEffects.
+
+		return resistance;
 	},
 
 	"attackTypes": function() {
-		if (!this.get("Attack"))
+		let attack = this.get("Attack");
+		if (!attack)
 			return undefined;
 
 		let ret = [];
-		for (let type in this.get("Attack"))
+		for (let type in attack)
 			ret.push(type);
 		return ret;
 	},
 
 	"attackRange": function(type) {
-		if (!this.get("Attack/" + type +""))
+		if (!this.get("Attack/" + type))
 			return undefined;
 
 		return {
@@ -229,25 +238,26 @@ m.Template = m.Class({
 	},
 
 	"attackStrengths": function(type) {
-		if (!this.get("Attack/" + type +""))
+		let attackDamageTypes = this.get("Attack/" + type + "/Damage");
+		if (!attackDamageTypes)
 			return undefined;
 
-		return {
-			"Hack": +(this.get("Attack/" + type + "/Hack") || 0),
-			"Pierce": +(this.get("Attack/" + type + "/Pierce") || 0),
-			"Crush": +(this.get("Attack/" + type + "/Crush") || 0)
-		};
+		let damage = {};
+		for (let damageType in attackDamageTypes)
+			damage[damageType] = +attackDamageTypes[damageType];
+
+		return damage;
 	},
 
 	"captureStrength": function() {
 		if (!this.get("Attack/Capture"))
 			return undefined;
 
-		return +this.get("Attack/Capture/Value") || 0;
+		return +this.get("Attack/Capture/Capture") || 0;
 	},
 
 	"attackTimes": function(type) {
-		if (!this.get("Attack/" + type +""))
+		if (!this.get("Attack/" + type))
 			return undefined;
 
 		return {
@@ -259,11 +269,12 @@ m.Template = m.Class({
 	// returns the classes this templates counters:
 	// Return type is [ [-neededClasses- , multiplier], … ].
 	"getCounteredClasses": function() {
-		if (!this.get("Attack"))
+		let attack = this.get("Attack");
+		if (!attack)
 			return undefined;
 
 		let Classes = [];
-		for (let type in this.get("Attack"))
+		for (let type in attack)
 		{
 			let bonuses = this.get("Attack/" + type + "/Bonuses");
 			if (!bonuses)
@@ -272,19 +283,20 @@ m.Template = m.Class({
 			{
 				let bonusClasses = this.get("Attack/" + type + "/Bonuses/" + b + "/Classes");
 				if (bonusClasses)
-					Classes.push([bonusClasses.split(" "), +this.get("Attack/" + type +"/Bonuses" + b +"/Multiplier")]);
+					Classes.push([bonusClasses.split(" "), +this.get("Attack/" + type +"/Bonuses/" + b +"/Multiplier")]);
 			}
 		}
 		return Classes;
 	},
 
-	// returns true if the entity counters those classes.
+	// returns true if the entity counters the target entity.
 	// TODO: refine using the multiplier
-	"countersClasses": function(classes) {
-		if (!this.get("Attack"))
+	"counters": function(target) {
+		let attack = this.get("Attack");
+		if (!attack)
 			return false;
 		let mcounter = [];
-		for (let type in this.get("Attack"))
+		for (let type in attack)
 		{
 			let bonuses = this.get("Attack/" + type + "/Bonuses");
 			if (!bonuses)
@@ -296,10 +308,7 @@ m.Template = m.Class({
 					mcounter.concat(bonusClasses.split(" "));
 			}
 		}
-		for (let i in classes)
-			if (mcounter.indexOf(classes[i]) != -1)
-				return true;
-		return false;
+		return target.hasClasses(mcounter);
 	},
 
 	// returns, if it exists, the multiplier from each attack against a given class
@@ -307,9 +316,10 @@ m.Template = m.Class({
 		if (!this.get("Attack/" + type +""))
 			return undefined;
 
-		if (this.get("Attack/" + type + "/Bonuses"))
+		let bonuses = this.get("Attack/" + type + "/Bonuses");
+		if (bonuses)
 		{
-			for (let b in this.get("Attack/" + type + "/Bonuses"))
+			for (let b in bonuses)
 			{
 				let bonusClasses = this.get("Attack/" + type + "/Bonuses/" + b + "/Classes");
 				if (!bonusClasses)
@@ -359,19 +369,16 @@ m.Template = m.Class({
 		let [type, subtype] = this.get("ResourceSupply/Type").split('.');
 		return { "generic": type, "specific": subtype };
 	},
-	// will return either "food", "wood", "stone", "metal" and not treasure.
+
 	"getResourceType": function() {
 		if (!this.get("ResourceSupply"))
 			return undefined;
-		let [type, subtype] = this.get("ResourceSupply/Type").split('.');
-		if (type == "treasure")
-			return subtype;
-		return type;
+		return this.get("ResourceSupply/Type").split('.')[0];
 	},
 
 	"getDiminishingReturns": function() { return +(this.get("ResourceSupply/DiminishingReturns") || 1); },
 
-	"resourceSupplyMax": function() { return +this.get("ResourceSupply/Amount"); },
+	"resourceSupplyMax": function() { return +this.get("ResourceSupply/Max"); },
 
 	"maxGatherers": function() { return +(this.get("ResourceSupply/MaxGatherers") || 0); },
 
@@ -393,11 +400,29 @@ m.Template = m.Class({
 		return types ? types.split(/\s+/) : [];
 	},
 
+	"isResourceDropsite": function(resourceType) {
+		const types = this.resourceDropsiteTypes();
+		return types && (!resourceType || types.indexOf(resourceType) !== -1);
+	},
+
+	"isTreasure": function() { return this.get("Treasure") !== undefined; },
+
+	"treasureResources": function() {
+		if (!this.get("Treasure"))
+			return undefined;
+		let ret = {};
+		for (let r in this.get("Treasure/Resources"))
+			ret[r] = +this.get("Treasure/Resources/" + r);
+		return ret;
+	},
+
 
 	"garrisonableClasses": function() { return this.get("GarrisonHolder/List/_string"); },
 
 	"garrisonMax": function() { return this.get("GarrisonHolder/Max"); },
 
+	"garrisonSize": function() { return this.get("Garrisonable/Size"); },
+
 	"garrisonEjectHealth": function() { return +this.get("GarrisonHolder/EjectHealth"); },
 
 	"getDefaultArrow": function() { return +this.get("BuildingAI/DefaultArrowCount"); },
@@ -416,17 +441,11 @@ m.Template = m.Class({
 
 	"isPackable": function() { return this.get("Pack") != undefined; },
 
-	/**
-	 * Returns whether this is an animal that is too difficult to hunt.
-	 */
 	"isHuntable": function() {
-		if(!this.get("ResourceSupply/KillBeforeGather"))
-			return false;
-
-		// do not hunt retaliating animals (animals without UnitAI are dead animals)
-		let behaviour = this.get("UnitAI/NaturalBehaviour");
-		return !behaviour ||
-		        behaviour != "violent" && behaviour != "aggressive" && behaviour != "defensive";
+		// Do not hunt retaliating animals (dead animals can be used).
+		// Assume entities which can attack, will attack.
+		return this.get("ResourceSupply/KillBeforeGather") &&
+			(!this.get("Health") || !this.get("Attack"));
 	},
 
 	"walkSpeed": function() { return +this.get("UnitMotion/WalkSpeed"); },
@@ -455,9 +474,10 @@ m.Template = m.Class({
 	"buildPlacementType": function() { return this.get("BuildRestrictions/PlacementType"); },
 
 	"buildTerritories": function() {
-		if (!this.get("BuildRestrictions") || !this.get("BuildRestrictions/Territory"))
+		if (!this.get("BuildRestrictions"))
 			return undefined;
-		return this.get("BuildRestrictions/Territory").split(/\s+/);
+		let territory = this.get("BuildRestrictions/Territory");
+		return !territory ? undefined : territory.split(/\s+/);
 	},
 
 	"hasBuildTerritory": function(territory) {
@@ -519,6 +539,8 @@ m.Template = m.Class({
 
 	"isGarrisonHolder": function() { return this.get("GarrisonHolder") !== undefined; },
 
+	"isTurretHolder": function() { return this.get("TurretHolder") !== undefined; },
+
 	/**
 	 * returns true if the tempalte can capture the given target entity
 	 * if no target is given, returns true if the template has the Capture attack
@@ -532,7 +554,7 @@ m.Template = m.Class({
 		if (!target.get("Capturable"))
 			return false;
 		let restrictedClasses = this.get("Attack/Capture/RestrictedClasses/_string");
-		return !restrictedClasses || !MatchesClassList(target.classes(), restrictedClasses);
+		return !restrictedClasses || !target.hasClasses(restrictedClasses);
 	},
 
 	"isCapturable": function() { return this.get("Capturable") !== undefined; },
@@ -540,6 +562,10 @@ m.Template = m.Class({
 	"canGuard": function() { return this.get("UnitAI/CanGuard") === "true"; },
 
 	"canGarrison": function() { return "Garrisonable" in this._template; },
+
+	"canOccupyTurret": function() { return "Turretable" in this._template; },
+
+	"isTreasureCollector": function() { return this.get("TreasureCollector") !== undefined; },
 });
 
 
@@ -587,23 +613,19 @@ m.Entity = m.Class({
 	"position": function() { return this._entity.position; },
 	"angle": function() { return this._entity.angle; },
 
-	"isIdle": function() {
-		if (typeof this._entity.idle === "undefined")
-			return undefined;
-		return this._entity.idle;
-	},
+	"isIdle": function() { return this._entity.idle; },
 
-	"getStance": function() { return this._entity.stance !== undefined ? this._entity.stance : undefined; },
-	"unitAIState": function() { return this._entity.unitAIState !== undefined ? this._entity.unitAIState : undefined; },
-	"unitAIOrderData": function() { return this._entity.unitAIOrderData !== undefined ? this._entity.unitAIOrderData : undefined; },
+	"getStance": function() { return this._entity.stance; },
+	"unitAIState": function() { return this._entity.unitAIState; },
+	"unitAIOrderData": function() { return this._entity.unitAIOrderData; },
 
-	"hitpoints": function() { return this._entity.hitpoints !== undefined ? this._entity.hitpoints : undefined; },
+	"hitpoints": function() { return this._entity.hitpoints; },
 	"isHurt": function() { return this.hitpoints() < this.maxHitpoints(); },
 	"healthLevel": function() { return this.hitpoints() / this.maxHitpoints(); },
 	"needsHeal": function() { return this.isHurt() && this.isHealable(); },
 	"needsRepair": function() { return this.isHurt() && this.isRepairable(); },
-	"decaying": function() { return this._entity.decaying !== undefined ? this._entity.decaying : undefined; },
-	"capturePoints": function() {return this._entity.capturePoints !== undefined ? this._entity.capturePoints : undefined; },
+	"decaying": function() { return this._entity.decaying; },
+	"capturePoints": function() {return this._entity.capturePoints; },
 	"isInvulnerable": function() { return this._entity.invulnerability || false; },
 
 	"isSharedDropsite": function() { return this._entity.sharedDropsite === true; },
@@ -613,8 +635,7 @@ m.Entity = m.Class({
 	 * [ { "id": 0, "template": "...", "count": 1, "progress": 0.5, "metadata": ... }, ... ]
 	 */
 	"trainingQueue": function() {
-		let queue = this._entity.trainingQueue;
-		return queue;
+		return this._entity.trainingQueue;
 	},
 
 	"trainingQueueTime": function() {
@@ -624,12 +645,10 @@ m.Entity = m.Class({
 		let time = 0;
 		for (let item of queue)
 			time += item.timeRemaining;
-		return time/1000;
+		return time / 1000;
 	},
 
 	"foundationProgress": function() {
-		if (this._entity.foundationProgress === undefined)
-			return undefined;
 		return this._entity.foundationProgress;
 	},
 
@@ -660,16 +679,12 @@ m.Entity = m.Class({
 	},
 
 	"resourceSupplyAmount": function() {
-		if (this._entity.resourceSupplyAmount === undefined)
-			return undefined;
 		return this._entity.resourceSupplyAmount;
 	},
 
 	"resourceSupplyNumGatherers": function()
 	{
-		if (this._entity.resourceSupplyNumGatherers !== undefined)
-			return this._entity.resourceSupplyNumGatherers;
-		return undefined;
+		return this._entity.resourceSupplyNumGatherers;
 	},
 
 	"isFull": function()
@@ -681,8 +696,6 @@ m.Entity = m.Class({
 	},
 
 	"resourceCarrying": function() {
-		if (this._entity.resourceCarrying === undefined)
-			return undefined;
 		return this._entity.resourceCarrying;
 	},
 
@@ -692,7 +705,7 @@ m.Entity = m.Class({
 			return undefined;
 
 		if (this.unitAIOrderData().length &&
-			(this.unitAIState().split(".")[1] == "GATHER" || this.unitAIState().split(".")[1] == "RETURNRESOURCE"))
+			this.unitAIState().split(".")[1] == "GATHER")
 		{
 			let res;
 			// this is an abuse of "_ai" but it works.
@@ -706,9 +719,6 @@ m.Entity = m.Class({
 			if (!type)
 				return 0;
 
-			if (type.generic == "treasure")
-				return 1000;
-
 			let tstring = type.generic + "." + type.specific;
 			let rate = +this.get("ResourceGatherer/BaseSpeed");
 			rate *= +this.get("ResourceGatherer/Rates/" +tstring);
@@ -719,18 +729,37 @@ m.Entity = m.Class({
 		return undefined;
 	},
 
+	"garrisonHolderID": function() {
+		return this._entity.garrisonHolderID;
+	},
+
 	"garrisoned": function() { return this._entity.garrisoned; },
-	"canGarrisonInside": function() { return this._entity.garrisoned.length < this.garrisonMax(); },
+
+	"garrisonedSlots": function() {
+		let count = 0;
+
+		if (this._entity.garrisoned)
+			for (let ent of this._entity.garrisoned)
+				count += +this._ai._entities.get(ent).garrisonSize();
+
+		return count;
+	},
+
+	"canGarrisonInside": function()
+	{
+		return this.garrisonedSlots() < this.garrisonMax();
+	},
 
 	/**
 	 * returns true if the entity can attack (including capture) the given class.
 	 */
 	"canAttackClass": function(aClass)
 	{
-		if (!this.get("Attack"))
+		let attack = this.get("Attack");
+		if (!attack)
 			return false;
 
-		for (let type in this.get("Attack"))
+		for (let type in attack)
 		{
 			if (type == "Slaughter")
 				continue;
@@ -741,31 +770,59 @@ m.Entity = m.Class({
 		return false;
 	},
 
-	"move": function(x, z, queued = false) {
-		Engine.PostCommand(PlayerID, { "type": "walk", "entities": [this.id()], "x": x, "z": z, "queued": queued });
+	/**
+	 * Derived from Attack.js' similary named function.
+	 * @return {boolean} - Whether an entity can attack a given target.
+	 */
+	"canAttackTarget": function(target, allowCapture)
+	{
+		let attackTypes = this.get("Attack");
+		if (!attackTypes)
+			return false;
+
+		let canCapture = allowCapture && this.canCapture(target);
+		let health = target.get("Health");
+		if (!health)
+			return canCapture;
+
+		for (let type in attackTypes)
+		{
+			if (type == "Capture" ? !canCapture : target.isInvulnerable())
+				continue;
+
+			let restrictedClasses = this.get("Attack/" + type + "/RestrictedClasses/_string");
+			if (!restrictedClasses || !target.hasClasses(restrictedClasses))
+				return true;
+		}
+
+		return false;
+	},
+
+	"move": function(x, z, queued = false, pushFront = false) {
+		Engine.PostCommand(PlayerID, { "type": "walk", "entities": [this.id()], "x": x, "z": z, "queued": queued, "pushFront": pushFront });
 		return this;
 	},
 
-	"moveToRange": function(x, z, min, max, queued = false) {
-		Engine.PostCommand(PlayerID, { "type": "walk-to-range", "entities": [this.id()], "x": x, "z": z, "min": min, "max": max, "queued": queued });
+	"moveToRange": function(x, z, min, max, queued = false, pushFront = false) {
+		Engine.PostCommand(PlayerID, { "type": "walk-to-range", "entities": [this.id()], "x": x, "z": z, "min": min, "max": max, "queued": queued, "pushFront": pushFront });
 		return this;
 	},
 
-	"attackMove": function(x, z, targetClasses, allowCapture = true, queued = false) {
-		Engine.PostCommand(PlayerID, { "type": "attack-walk", "entities": [this.id()], "x": x, "z": z, "targetClasses": targetClasses, "allowCapture": allowCapture, "queued": queued });
+	"attackMove": function(x, z, targetClasses, allowCapture = true, queued = false, pushFront = false) {
+		Engine.PostCommand(PlayerID, { "type": "attack-walk", "entities": [this.id()], "x": x, "z": z, "targetClasses": targetClasses, "allowCapture": allowCapture, "queued": queued, "pushFront": pushFront });
 		return this;
 	},
 
 	// violent, aggressive, defensive, passive, standground
-	"setStance": function(stance, queued = false) {
+	"setStance": function(stance, queued = false, pushFront = false) {
 		if (this.getStance() === undefined)
 			return undefined;
-		Engine.PostCommand(PlayerID, { "type": "stance", "entities": [this.id()], "name": stance, "queued": queued });
+		Engine.PostCommand(PlayerID, { "type": "stance", "entities": [this.id()], "name": stance, "queued": queued, "pushFront": pushFront });
 		return this;
 	},
 
 	"stopMoving": function() {
-		Engine.PostCommand(PlayerID, { "type": "stop", "entities": [this.id()], "queued": false });
+		Engine.PostCommand(PlayerID, { "type": "stop", "entities": [this.id()], "queued": false, "pushFront": false });
 	},
 
 	"unload": function(id) {
@@ -783,19 +840,36 @@ m.Entity = m.Class({
 		return this;
 	},
 
-	"garrison": function(target, queued = false) {
-		Engine.PostCommand(PlayerID, { "type": "garrison", "entities": [this.id()], "target": target.id(), "queued": queued });
+	"garrison": function(target, queued = false, pushFront = false) {
+		Engine.PostCommand(PlayerID, { "type": "garrison", "entities": [this.id()], "target": target.id(), "queued": queued, "pushFront": pushFront });
 		return this;
 	},
 
-	"attack": function(unitId, allowCapture = true, queued = false) {
-		Engine.PostCommand(PlayerID, { "type": "attack", "entities": [this.id()], "target": unitId, "allowCapture": allowCapture, "queued": queued });
+	"occupy-turret": function(target, queued = false, pushFront = false) {
+		Engine.PostCommand(PlayerID, { "type": "occupy-turret", "entities": [this.id()], "target": target.id(), "queued": queued, "pushFront": pushFront });
+		return this;
+	},
+
+	"attack": function(unitId, allowCapture = true, queued = false, pushFront = false) {
+		Engine.PostCommand(PlayerID, { "type": "attack", "entities": [this.id()], "target": unitId, "allowCapture": allowCapture, "queued": queued, "pushFront": pushFront });
+		return this;
+	},
+
+	"collectTreasure": function(target, queued = false, pushFront = false) {
+		Engine.PostCommand(PlayerID, {
+			"type": "collect-treasure",
+			"entities": [this.id()],
+			"target": target.id(),
+			"queued": queued,
+			"pushFront": pushFront
+		});
 		return this;
 	},
 
 	// moveApart from a point in the opposite direction with a distance dist
 	"moveApart": function(point, dist) {
-		if (this.position() !== undefined) {
+		if (this.position() !== undefined)
+		{
 			let direction = [this.position()[0] - point[0], this.position()[1] - point[1]];
 			let norm = m.VectorDistance(point, this.position());
 			if (norm === 0)
@@ -805,37 +879,38 @@ m.Entity = m.Class({
 				direction[0] /= norm;
 				direction[1] /= norm;
 			}
-			Engine.PostCommand(PlayerID, { "type": "walk", "entities": [this.id()], "x": this.position()[0] + direction[0]*dist, "z": this.position()[1] + direction[1]*dist, "queued": false });
+			Engine.PostCommand(PlayerID, { "type": "walk", "entities": [this.id()], "x": this.position()[0] + direction[0]*dist, "z": this.position()[1] + direction[1]*dist, "queued": false, "pushFront": false });
 		}
 		return this;
 	},
 
 	// Flees from a unit in the opposite direction.
 	"flee": function(unitToFleeFrom) {
-		if (this.position() !== undefined && unitToFleeFrom.position() !== undefined) {
+		if (this.position() !== undefined && unitToFleeFrom.position() !== undefined)
+		{
 			let FleeDirection = [this.position()[0] - unitToFleeFrom.position()[0],
 			                     this.position()[1] - unitToFleeFrom.position()[1]];
 			let dist = m.VectorDistance(unitToFleeFrom.position(), this.position());
-			FleeDirection[0] = 40 * FleeDirection[0]/dist;
-			FleeDirection[1] = 40 * FleeDirection[1]/dist;
+			FleeDirection[0] = 40 * FleeDirection[0] / dist;
+			FleeDirection[1] = 40 * FleeDirection[1] / dist;
 
-			Engine.PostCommand(PlayerID, { "type": "walk", "entities": [this.id()], "x": this.position()[0] + FleeDirection[0], "z": this.position()[1] + FleeDirection[1], "queued": false });
+			Engine.PostCommand(PlayerID, { "type": "walk", "entities": [this.id()], "x": this.position()[0] + FleeDirection[0], "z": this.position()[1] + FleeDirection[1], "queued": false, "pushFront": false });
 		}
 		return this;
 	},
 
-	"gather": function(target, queued = false) {
-		Engine.PostCommand(PlayerID, { "type": "gather", "entities": [this.id()], "target": target.id(), "queued": queued });
+	"gather": function(target, queued = false, pushFront = false) {
+		Engine.PostCommand(PlayerID, { "type": "gather", "entities": [this.id()], "target": target.id(), "queued": queued, "pushFront": pushFront });
 		return this;
 	},
 
-	"repair": function(target, autocontinue = false, queued = false) {
-		Engine.PostCommand(PlayerID, { "type": "repair", "entities": [this.id()], "target": target.id(), "autocontinue": autocontinue, "queued": queued });
+	"repair": function(target, autocontinue = false, queued = false, pushFront = false) {
+		Engine.PostCommand(PlayerID, { "type": "repair", "entities": [this.id()], "target": target.id(), "autocontinue": autocontinue, "queued": queued, "pushFront": pushFront });
 		return this;
 	},
 
-	"returnResources": function(target, queued = false) {
-		Engine.PostCommand(PlayerID, { "type": "returnresource", "entities": [this.id()], "target": target.id(), "queued": queued });
+	"returnResources": function(target, queued = false, pushFront = false) {
+		Engine.PostCommand(PlayerID, { "type": "returnresource", "entities": [this.id()], "target": target.id(), "queued": queued, "pushFront": pushFront });
 		return this;
 	},
 
@@ -850,7 +925,7 @@ m.Entity = m.Class({
 	},
 
 	"tradeRoute": function(target, source) {
-		Engine.PostCommand(PlayerID, { "type": "setup-trade-route", "entities": [this.id()], "target": target.id(), "source": source.id(), "route": undefined, "queued": false });
+		Engine.PostCommand(PlayerID, { "type": "setup-trade-route", "entities": [this.id()], "target": target.id(), "source": source.id(), "route": undefined, "queued": false, "pushFront": false });
 		return this;
 	},
 
@@ -904,6 +979,7 @@ m.Entity = m.Class({
 			"autorepair": false,
 			"autocontinue": false,
 			"queued": false,
+			"pushFront": false,
 			"metadata": metadata	// can be undefined
 		});
 		return this;
@@ -929,8 +1005,8 @@ m.Entity = m.Class({
 		return this;
 	},
 
-	"guard": function(target, queued = false) {
-		Engine.PostCommand(PlayerID, { "type": "guard", "entities": [this.id()], "target": target.id(), "queued": queued });
+	"guard": function(target, queued = false, pushFront = false) {
+		Engine.PostCommand(PlayerID, { "type": "guard", "entities": [this.id()], "target": target.id(), "queued": queued, "pushFront": pushFront });
 		return this;
 	},
 
