@@ -25,9 +25,9 @@ m.GameState.prototype.init = function(SharedScript, state, player)
 	this.ceasefireTimeRemaining = SharedScript.ceasefireTimeRemaining;
 
 	// get the list of possible phases for this civ:
-	// we assume all of them are researchable from the civil centre
+	// we assume all of them are researchable from the civil center
 	this.phases = [];
-	let cctemplate = this.getTemplate(this.applyCiv("structures/{civ}_civil_centre"));
+	let cctemplate = this.getTemplate(this.applyCiv("structures/{civ}/civil_centre"));
 	if (!cctemplate)
 		return;
 	let civ = this.getPlayerCiv();
@@ -465,10 +465,7 @@ m.GameState.prototype.isEntityOwn = function(ent)
 
 m.GameState.prototype.getEntityById = function(id)
 {
-	if (this.entities._entities.has(+id))
-		return this.entities._entities.get(+id);
-
-	return undefined;
+	return this.entities._entities.get(+id);
 };
 
 m.GameState.prototype.getEntities = function(id)
@@ -718,6 +715,7 @@ m.GameState.prototype.findTrainableUnits = function(classes, anticlasses)
 	let ret = [];
 	let limits = this.getEntityLimits();
 	let current = this.getEntityCounts();
+	let matchCounts = this.getEntityMatchCounts();
 	for (let trainable of allTrainable)
 	{
 		if (this.isTemplateDisabled(trainable))
@@ -725,9 +723,10 @@ m.GameState.prototype.findTrainableUnits = function(classes, anticlasses)
 		let template = this.getTemplate(trainable);
 		if (!template || !template.available(this))
 			continue;
-		if (classes.some(c => !template.hasClass(c)))
+		let limit = template.matchLimit();
+		if (matchCounts && limit && matchCounts[trainable] >= limit)
 			continue;
-		if (anticlasses.some(c => template.hasClass(c)))
+		if (!template.hasClasses(classes) || template.hasClasses(anticlasses))
 			continue;
 		let category = template.trainingCategory();
 		if (category && limits[category] && current[category] >= limits[category])
@@ -888,6 +887,11 @@ m.GameState.prototype.getEntityLimits = function()
 	return this.playerData.entityLimits;
 };
 
+m.GameState.prototype.getEntityMatchCounts = function()
+{
+	return this.playerData.matchEntityCounts;
+};
+
 m.GameState.prototype.getEntityCounts = function()
 {
 	return this.playerData.entityCounts;
@@ -918,8 +922,8 @@ m.GameState.prototype.isEntityLimitReached = function(category)
 
 m.GameState.prototype.getTraderTemplatesGains = function()
 {
-	let shipMechantTemplateName = this.applyCiv("units/{civ}_ship_merchant");
-	let supportTraderTemplateName = this.applyCiv("units/{civ}_support_trader");
+	let shipMechantTemplateName = this.applyCiv("units/{civ}/ship_merchant");
+	let supportTraderTemplateName = this.applyCiv("units/{civ}/support_trader");
 	let shipMerchantTemplate = !this.isTemplateDisabled(shipMechantTemplateName) && this.getTemplate(shipMechantTemplateName);
 	let supportTraderTemplate = !this.isTemplateDisabled(supportTraderTemplateName) && this.getTemplate(supportTraderTemplateName);
 	let norm = TradeGainNormalization(this.sharedScript.mapSize);
