@@ -1,9 +1,7 @@
-var PETRA = function(m)
-{
-
-/** Attack Manager */
-
-m.AttackManager = function(Config)
+/**
+ * Attack Manager
+ */
+PETRA.AttackManager = function(Config)
 {
 	this.Config = Config;
 
@@ -22,13 +20,13 @@ m.AttackManager = function(Config)
 };
 
 /** More initialisation for stuff that needs the gameState */
-m.AttackManager.prototype.init = function(gameState)
+PETRA.AttackManager.prototype.init = function(gameState)
 {
 	this.outOfPlan = gameState.getOwnUnits().filter(API3.Filters.byMetadata(PlayerID, "plan", -1));
 	this.outOfPlan.registerUpdates();
 };
 
-m.AttackManager.prototype.setRushes = function(allowed)
+PETRA.AttackManager.prototype.setRushes = function(allowed)
 {
 	if (this.Config.personality.aggressive > this.Config.personalityCut.strong && allowed > 2)
 	{
@@ -47,7 +45,7 @@ m.AttackManager.prototype.setRushes = function(allowed)
 	}
 };
 
-m.AttackManager.prototype.checkEvents = function(gameState, events)
+PETRA.AttackManager.prototype.checkEvents = function(gameState, events)
 {
 	for (let evt of events.PlayerDefeated)
 		this.defeated[evt.playerId] = true;
@@ -102,7 +100,7 @@ m.AttackManager.prototype.checkEvents = function(gameState, events)
 		break;  // take only the first attack request into account
 	}
 	if (targetPlayer !== undefined)
-		m.chatAnswerRequestAttack(gameState, targetPlayer, answer, other);
+		PETRA.chatAnswerRequestAttack(gameState, targetPlayer, answer, other);
 
 	for (let evt of events.EntityRenamed)	// take care of packing units in bombing attacks
 	{
@@ -125,7 +123,7 @@ m.AttackManager.prototype.checkEvents = function(gameState, events)
 /**
  * Check for any structure in range from within our territory, and bomb it
  */
-m.AttackManager.prototype.assignBombers = function(gameState)
+PETRA.AttackManager.prototype.assignBombers = function(gameState)
 {
 	// First some cleaning of current bombing attacks
 	for (let [targetId, unitIds] of this.bombingAttacks)
@@ -153,7 +151,7 @@ m.AttackManager.prototype.assignBombers = function(gameState)
 		}
 	}
 
-	let bombers = gameState.updatingCollection("bombers", API3.Filters.byClassesOr(["BoltShooter", "Catapult"]), gameState.getOwnUnits());
+	const bombers = gameState.updatingCollection("bombers", API3.Filters.byClasses(["BoltShooter", "StoneThrower"]), gameState.getOwnUnits());
 	for (let ent of bombers.values())
 	{
 		if (!ent.position() || !ent.isIdle() || !ent.attackRange("Ranged"))
@@ -179,9 +177,12 @@ m.AttackManager.prototype.assignBombers = function(gameState)
 
 		let range = ent.attackRange("Ranged").max;
 		let entPos = ent.position();
-		let access = m.getLandAccess(gameState, ent);
+		let access = PETRA.getLandAccess(gameState, ent);
 		for (let struct of gameState.getEnemyStructures().values())
 		{
+			if (!ent.canAttackTarget(struct, PETRA.allowCapture(gameState, ent, struct)))
+				continue;
+
 			let structPos = struct.position();
 			let x;
 			let z;
@@ -234,7 +235,7 @@ m.AttackManager.prototype.assignBombers = function(gameState)
  * Some functions are run every turn
  * Others once in a while
  */
-m.AttackManager.prototype.update = function(gameState, queues, events)
+PETRA.AttackManager.prototype.update = function(gameState, queues, events)
 {
 	if (this.Config.debug > 2 && gameState.ai.elapsedTime > this.debugTime + 60)
 	{
@@ -285,7 +286,7 @@ m.AttackManager.prototype.update = function(gameState, queues, events)
 					if (this.Config.debug > 1)
 						API3.warn("Attack Manager: Starting " + attack.getType() + " plan " + attack.getName());
 					if (this.Config.chat)
-						m.chatLaunchAttack(gameState, attack.targetPlayer, attack.getType());
+						PETRA.chatLaunchAttack(gameState, attack.targetPlayer, attack.getType());
 					this.startedAttacks[attackType].push(attack);
 				}
 				else
@@ -324,7 +325,7 @@ m.AttackManager.prototype.update = function(gameState, queues, events)
 		{
 			// we have a barracks and we want to rush, rush.
 			let data = { "targetSize": this.rushSize[this.rushNumber] };
-			let attackPlan = new m.AttackPlan(gameState, this.Config, this.totalNumber, "Rush", data);
+			let attackPlan = new PETRA.AttackPlan(gameState, this.Config, this.totalNumber, "Rush", data);
 			if (!attackPlan.failed)
 			{
 				if (this.Config.debug > 1)
@@ -344,7 +345,7 @@ m.AttackManager.prototype.update = function(gameState, queues, events)
 			!gameState.ai.HQ.baseManagers[1])	// if we have no base ... nothing else to do than attack
 		{
 			let type = this.attackNumber < 2 || this.startedAttacks.HugeAttack.length > 0 ? "Attack" : "HugeAttack";
-			let attackPlan = new m.AttackPlan(gameState, this.Config, this.totalNumber, type);
+			let attackPlan = new PETRA.AttackPlan(gameState, this.Config, this.totalNumber, type);
 			if (attackPlan.failed)
 				this.attackPlansEncounteredWater = true; // hack
 			else
@@ -380,7 +381,7 @@ m.AttackManager.prototype.update = function(gameState, queues, events)
 		this.assignBombers(gameState);
 };
 
-m.AttackManager.prototype.getPlan = function(planName)
+PETRA.AttackManager.prototype.getPlan = function(planName)
 {
 	for (let attackType in this.upcomingAttacks)
 	{
@@ -397,21 +398,21 @@ m.AttackManager.prototype.getPlan = function(planName)
 	return undefined;
 };
 
-m.AttackManager.prototype.pausePlan = function(planName)
+PETRA.AttackManager.prototype.pausePlan = function(planName)
 {
 	let attack = this.getPlan(planName);
 	if (attack)
 		attack.setPaused(true);
 };
 
-m.AttackManager.prototype.unpausePlan = function(planName)
+PETRA.AttackManager.prototype.unpausePlan = function(planName)
 {
 	let attack = this.getPlan(planName);
 	if (attack)
 		attack.setPaused(false);
 };
 
-m.AttackManager.prototype.pauseAllPlans = function()
+PETRA.AttackManager.prototype.pauseAllPlans = function()
 {
 	for (let attackType in this.upcomingAttacks)
 		for (let attack of this.upcomingAttacks[attackType])
@@ -422,7 +423,7 @@ m.AttackManager.prototype.pauseAllPlans = function()
 			attack.setPaused(true);
 };
 
-m.AttackManager.prototype.unpauseAllPlans = function()
+PETRA.AttackManager.prototype.unpauseAllPlans = function()
 {
 	for (let attackType in this.upcomingAttacks)
 		for (let attack of this.upcomingAttacks[attackType])
@@ -433,7 +434,7 @@ m.AttackManager.prototype.unpauseAllPlans = function()
 			attack.setPaused(false);
 };
 
-m.AttackManager.prototype.getAttackInPreparation = function(type)
+PETRA.AttackManager.prototype.getAttackInPreparation = function(type)
 {
 	return this.upcomingAttacks[type].length ? this.upcomingAttacks[type][0] : undefined;
 };
@@ -443,7 +444,7 @@ m.AttackManager.prototype.getAttackInPreparation = function(type)
  * attack.targetPlayer is undefined and in that case, we keep track of the chosen target
  * for future attacks.
  */
-m.AttackManager.prototype.getEnemyPlayer = function(gameState, attack)
+PETRA.AttackManager.prototype.getEnemyPlayer = function(gameState, attack)
 {
 	let enemyPlayer;
 
@@ -474,7 +475,7 @@ m.AttackManager.prototype.getEnemyPlayer = function(gameState, attack)
 				continue;
 			let enemyDefense = 0;
 			for (let ent of gameState.getEnemyStructures(i).values())
-				if (ent.hasClass("Tower") || ent.hasClass("Fortress"))
+				if (ent.hasClasses(["Tower", "WallTower", "Fortress"]))
 					enemyDefense++;
 			if (enemyDefense > 6)
 				veto[i] = true;
@@ -499,14 +500,14 @@ m.AttackManager.prototype.getEnemyPlayer = function(gameState, attack)
 			if (ourcc.owner() != PlayerID)
 				continue;
 			let ourPos = ourcc.position();
-			let access = m.getLandAccess(gameState, ourcc);
+			let access = PETRA.getLandAccess(gameState, ourcc);
 			for (let enemycc of ccEnts.values())
 			{
 				if (veto[enemycc.owner()])
 					continue;
 				if (!gameState.isPlayerEnemy(enemycc.owner()))
 					continue;
-				if (access != m.getLandAccess(gameState, enemycc))
+				if (access != PETRA.getLandAccess(gameState, enemycc))
 					continue;
 				let dist = API3.SquareVectorDistance(ourPos, enemycc.position());
 				if (distmin && dist > distmin)
@@ -557,7 +558,7 @@ m.AttackManager.prototype.getEnemyPlayer = function(gameState, attack)
  * Target the player with the most advanced wonder.
  * TODO currently the first built wonder is kept, should chek on the minimum wonderDuration left instead.
  */
-m.AttackManager.prototype.getWonderEnemyPlayer = function(gameState, attack)
+PETRA.AttackManager.prototype.getWonderEnemyPlayer = function(gameState, attack)
 {
 	let enemyPlayer;
 	let enemyWonder;
@@ -589,7 +590,7 @@ m.AttackManager.prototype.getWonderEnemyPlayer = function(gameState, attack)
 /**
  * Target the player with the most relics (including gaia).
  */
-m.AttackManager.prototype.getRelicEnemyPlayer = function(gameState, attack)
+PETRA.AttackManager.prototype.getRelicEnemyPlayer = function(gameState, attack)
 {
 	let enemyPlayer;
 	let allRelics = gameState.updatingGlobalCollection("allRelics", API3.Filters.byClass("Relic"));
@@ -617,7 +618,7 @@ m.AttackManager.prototype.getRelicEnemyPlayer = function(gameState, attack)
 };
 
 /** f.e. if we have changed diplomacy with another player. */
-m.AttackManager.prototype.cancelAttacksAgainstPlayer = function(gameState, player)
+PETRA.AttackManager.prototype.cancelAttacksAgainstPlayer = function(gameState, player)
 {
 	for (let attackType in this.upcomingAttacks)
 		for (let attack of this.upcomingAttacks[attackType])
@@ -636,10 +637,10 @@ m.AttackManager.prototype.cancelAttacksAgainstPlayer = function(gameState, playe
 		}
 };
 
-m.AttackManager.prototype.raidTargetEntity = function(gameState, ent)
+PETRA.AttackManager.prototype.raidTargetEntity = function(gameState, ent)
 {
 	let data = { "target": ent };
-	let attackPlan = new m.AttackPlan(gameState, this.Config, this.totalNumber, "Raid", data);
+	let attackPlan = new PETRA.AttackPlan(gameState, this.Config, this.totalNumber, "Raid", data);
 	if (attackPlan.failed)
 		return null;
 	if (this.Config.debug > 1)
@@ -654,7 +655,7 @@ m.AttackManager.prototype.raidTargetEntity = function(gameState, ent)
 /**
  * Return the number of units from any of our attacking armies around this position
  */
-m.AttackManager.prototype.numAttackingUnitsAround = function(pos, dist)
+PETRA.AttackManager.prototype.numAttackingUnitsAround = function(pos, dist)
 {
 	let num = 0;
 	for (let attackType in this.startedAttacks)
@@ -674,7 +675,7 @@ m.AttackManager.prototype.numAttackingUnitsAround = function(pos, dist)
  * data.armyID: transform only the defense army ID into a new attack
  * data.uniqueTarget: the attack will stop when the target is destroyed or captured
  */
-m.AttackManager.prototype.switchDefenseToAttack = function(gameState, target, data)
+PETRA.AttackManager.prototype.switchDefenseToAttack = function(gameState, target, data)
 {
 	if (!target || !target.position())
 		return false;
@@ -686,14 +687,14 @@ m.AttackManager.prototype.switchDefenseToAttack = function(gameState, target, da
 	let attackData = data.uniqueTarget ? { "uniqueTargetId": target.id() } : undefined;
 	let pos = target.position();
 	let attackType = "Attack";
-	let attackPlan = new m.AttackPlan(gameState, this.Config, this.totalNumber, attackType, attackData);
+	let attackPlan = new PETRA.AttackPlan(gameState, this.Config, this.totalNumber, attackType, attackData);
 	if (attackPlan.failed)
 		return false;
 	this.totalNumber++;
 	attackPlan.init(gameState);
 	this.startedAttacks[attackType].push(attackPlan);
 
-	let targetAccess = m.getLandAccess(gameState, target);
+	let targetAccess = PETRA.getLandAccess(gameState, target);
 	for (let army of gameState.ai.HQ.defenseManager.armies)
 	{
 		if (data.range)
@@ -713,7 +714,7 @@ m.AttackManager.prototype.switchDefenseToAttack = function(gameState, target, da
 			army.removeOwn(gameState, unitId);
 			let unit = gameState.getEntityById(unitId);
 			let accessOk = unit.getMetadata(PlayerID, "transport") !== undefined ||
-			               unit.position() && m.getLandAccess(gameState, unit) == targetAccess;
+			               unit.position() && PETRA.getLandAccess(gameState, unit) == targetAccess;
 			if (unit && accessOk && attackPlan.isAvailableUnit(gameState, unit))
 			{
 				unit.setMetadata(PlayerID, "plan", attackPlan.name);
@@ -736,7 +737,7 @@ m.AttackManager.prototype.switchDefenseToAttack = function(gameState, target, da
 	return true;
 };
 
-m.AttackManager.prototype.Serialize = function()
+PETRA.AttackManager.prototype.Serialize = function()
 {
 	let properties = {
 		"totalNumber": this.totalNumber,
@@ -769,7 +770,7 @@ m.AttackManager.prototype.Serialize = function()
 	return { "properties": properties, "upcomingAttacks": upcomingAttacks, "startedAttacks": startedAttacks };
 };
 
-m.AttackManager.prototype.Deserialize = function(gameState, data)
+PETRA.AttackManager.prototype.Deserialize = function(gameState, data)
 {
 	for (let key in data.properties)
 		this[key] = data.properties[key];
@@ -780,7 +781,7 @@ m.AttackManager.prototype.Deserialize = function(gameState, data)
 		this.upcomingAttacks[key] = [];
 		for (let dataAttack of data.upcomingAttacks[key])
 		{
-			let attack = new m.AttackPlan(gameState, this.Config, dataAttack.properties.name);
+			let attack = new PETRA.AttackPlan(gameState, this.Config, dataAttack.properties.name);
 			attack.Deserialize(gameState, dataAttack);
 			attack.init(gameState);
 			this.upcomingAttacks[key].push(attack);
@@ -793,13 +794,10 @@ m.AttackManager.prototype.Deserialize = function(gameState, data)
 		this.startedAttacks[key] = [];
 		for (let dataAttack of data.startedAttacks[key])
 		{
-			let attack = new m.AttackPlan(gameState, this.Config, dataAttack.properties.name);
+			let attack = new PETRA.AttackPlan(gameState, this.Config, dataAttack.properties.name);
 			attack.Deserialize(gameState, dataAttack);
 			attack.init(gameState);
 			this.startedAttacks[key].push(attack);
 		}
 	}
 };
-
-return m;
-}(PETRA);
