@@ -1,6 +1,3 @@
-var PETRA = function(m)
-{
-
 /**
  * Base Manager
  * Handles lower level economic stuffs.
@@ -13,7 +10,7 @@ var PETRA = function(m)
  *  -updating whatever needs updating, keeping track of stuffs (rebuilding needs…)
  */
 
-m.BaseManager = function(gameState, Config)
+PETRA.BaseManager = function(gameState, Config)
 {
 	this.Config = Config;
 	this.ID = gameState.ai.uniqueIDs.bases++;
@@ -37,13 +34,13 @@ m.BaseManager = function(gameState, Config)
 	this.timeNextIdleCheck = 0;
 };
 
-m.BaseManager.prototype.init = function(gameState, state)
+PETRA.BaseManager.prototype.init = function(gameState, state)
 {
 	if (state == "unconstructed")
 		this.constructing = true;
 	else if (state != "captured")
 		this.neededDefenders = 0;
-	this.workerObject = new m.Worker(this);
+	this.workerObject = new PETRA.Worker(this);
 	// entitycollections
 	this.units = gameState.getOwnUnits().filter(API3.Filters.byMetadata(PlayerID, "base", this.ID));
 	this.workers = this.units.filter(API3.Filters.byMetadata(PlayerID, "role", "worker"));
@@ -66,7 +63,7 @@ m.BaseManager.prototype.init = function(gameState, state)
 	}
 };
 
-m.BaseManager.prototype.reset = function(gameState, state)
+PETRA.BaseManager.prototype.reset = function(gameState, state)
 {
 	if (state == "unconstructed")
 		this.constructing = true;
@@ -79,17 +76,17 @@ m.BaseManager.prototype.reset = function(gameState, state)
 		this.neededDefenders = 3 + 2 * (this.Config.difficulty - 3);
 };
 
-m.BaseManager.prototype.assignEntity = function(gameState, ent)
+PETRA.BaseManager.prototype.assignEntity = function(gameState, ent)
 {
 	ent.setMetadata(PlayerID, "base", this.ID);
 	this.units.updateEnt(ent);
 	this.workers.updateEnt(ent);
 	this.buildings.updateEnt(ent);
-	if (ent.resourceDropsiteTypes() && !ent.hasClass("Elephant"))
+	if (ent.resourceDropsiteTypes() && !ent.hasClass("Unit"))
 		this.assignResourceToDropsite(gameState, ent);
 };
 
-m.BaseManager.prototype.setAnchor = function(gameState, anchorEntity)
+PETRA.BaseManager.prototype.setAnchor = function(gameState, anchorEntity)
 {
 	if (!anchorEntity.hasClass("CivCentre"))
 		API3.warn("Error: Petra base " + this.ID + " has been assigned " + ent.templateName() + " as anchor.");
@@ -102,12 +99,12 @@ m.BaseManager.prototype.setAnchor = function(gameState, anchorEntity)
 	}
 	anchorEntity.setMetadata(PlayerID, "base", this.ID);
 	this.buildings.updateEnt(anchorEntity);
-	this.accessIndex = m.getLandAccess(gameState, anchorEntity);
+	this.accessIndex = PETRA.getLandAccess(gameState, anchorEntity);
 	return true;
 };
 
-/* we lost our anchor. Let's reaffect our units and buildings */
-m.BaseManager.prototype.anchorLost = function(gameState, ent)
+/* we lost our anchor. Let's reassign our units and buildings */
+PETRA.BaseManager.prototype.anchorLost = function(gameState, ent)
 {
 	this.anchor = undefined;
 	this.anchorId = undefined;
@@ -116,17 +113,17 @@ m.BaseManager.prototype.anchorLost = function(gameState, ent)
 };
 
 /** Set a building of an anchorless base */
-m.BaseManager.prototype.setAnchorlessEntity = function(gameState, ent)
+PETRA.BaseManager.prototype.setAnchorlessEntity = function(gameState, ent)
 {
 	if (!this.buildings.hasEntities())
 	{
-		if (!m.getBuiltEntity(gameState, ent).resourceDropsiteTypes())
+		if (!PETRA.getBuiltEntity(gameState, ent).resourceDropsiteTypes())
 			API3.warn("Error: Petra base " + this.ID + " has been assigned " + ent.templateName() + " as origin.");
-		this.accessIndex = m.getLandAccess(gameState, ent);
+		this.accessIndex = PETRA.getLandAccess(gameState, ent);
 	}
-	else if (this.accessIndex != m.getLandAccess(gameState, ent))
+	else if (this.accessIndex != PETRA.getLandAccess(gameState, ent))
 		API3.warn(" Error: Petra base " + this.ID + " with access " + this.accessIndex +
-		          " has been assigned " + ent.templateName() + " with access" + m.getLandAccess(gameState, ent));
+		          " has been assigned " + ent.templateName() + " with access" + PETRA.getLandAccess(gameState, ent));
 
 	ent.setMetadata(PlayerID, "base", this.ID);
 	this.buildings.updateEnt(ent);
@@ -137,7 +134,7 @@ m.BaseManager.prototype.setAnchorlessEntity = function(gameState, ent)
  * Assign the resources around the dropsites of this basis in three areas according to distance, and sort them in each area.
  * Moving resources (animals) and buildable resources (fields) are treated elsewhere.
  */
-m.BaseManager.prototype.assignResourceToDropsite = function(gameState, dropsite)
+PETRA.BaseManager.prototype.assignResourceToDropsite = function(gameState, dropsite)
 {
 	if (this.dropsites[dropsite.id()])
 	{
@@ -152,7 +149,7 @@ m.BaseManager.prototype.assignResourceToDropsite = function(gameState, dropsite)
 	this.dropsites[dropsiteId] = true;
 
 	if (this.ID == gameState.ai.HQ.baseManagers[0].ID)
-		accessIndex = m.getLandAccess(gameState, dropsite);
+		accessIndex = PETRA.getLandAccess(gameState, dropsite);
 
 	let maxDistResourceSquare = this.maxDistResourceSquare;
 	for (let type of dropsite.resourceDropsiteTypes())
@@ -169,14 +166,11 @@ m.BaseManager.prototype.assignResourceToDropsite = function(gameState, dropsite)
 		{
 			if (!supply.position())
 				return;
-			if (supply.hasClass("Animal"))    // moving resources are treated differently
-				return;
-			if (supply.hasClass("Field"))     // fields are treated separately
-				return;
-			if (supply.resourceSupplyType().generic == "treasure")  // treasures are treated separately
+			// Moving resources and fields are treated differently.
+			if (supply.hasClasses(["Animal", "Field"]))
 				return;
 			// quick accessibility check
-			if (m.getLandAccess(gameState, supply) != accessIndex)
+			if (PETRA.getLandAccess(gameState, supply) != accessIndex)
 				return;
 
 			let dist = API3.SquareVectorDistance(supply.position(), dropsitePos);
@@ -195,7 +189,8 @@ m.BaseManager.prototype.assignResourceToDropsite = function(gameState, dropsite)
 		medium.sort((r1, r2) => r1.dist - r2.dist);
 		faraway.sort((r1, r2) => r1.dist - r2.dist);
 
-/*		let debug = false;
+		/*
+		let debug = false;
 		if (debug)
 		{
 			faraway.forEach(function(res){
@@ -207,7 +202,8 @@ m.BaseManager.prototype.assignResourceToDropsite = function(gameState, dropsite)
 			nearby.forEach(function(res){
 				Engine.PostCommand(PlayerID,{"type": "set-shading-color", "entities": [res.ent.id()], "rgb": [0,0,2]});
 			});
-		} */
+		}
+		*/
 	}
 
 	// Allows all allies to use this dropsite except if base anchor to be sure to keep
@@ -220,7 +216,7 @@ m.BaseManager.prototype.assignResourceToDropsite = function(gameState, dropsite)
 };
 
 // completely remove the dropsite resources from our list.
-m.BaseManager.prototype.removeDropsite = function(gameState, ent)
+PETRA.BaseManager.prototype.removeDropsite = function(gameState, ent)
 {
 	if (!ent.id())
 		return;
@@ -248,12 +244,37 @@ m.BaseManager.prototype.removeDropsite = function(gameState, ent)
 };
 
 /**
- * Returns the position of the best place to build a new dropsite for the specified resource
+ * @return {Object} - The position of the best place to build a new dropsite for the specified resource,
+ *			its quality and its template name.
  */
-m.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource)
+PETRA.BaseManager.prototype.findBestDropsiteAndLocation = function(gameState, resource)
 {
+	let bestResult = {
+		"quality": 0,
+		"pos": [0, 0]
+	};
+	for (const templateName of gameState.ai.HQ.buildManager.findStructuresByFilter(gameState, API3.Filters.isDropsite(resource)))
+	{
+		const dp = this.findBestDropsiteLocation(gameState, resource, templateName);
+		if (dp.quality < bestResult.quality)
+			continue;
+		bestResult = dp;
+		bestResult.templateName = templateName;
+	}
+	return bestResult;
+};
+
+/**
+ * Returns the position of the best place to build a new dropsite for the specified resource and dropsite template.
+ */
+PETRA.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource, templateName)
+{
+	const template = gameState.getTemplate(gameState.applyCiv(templateName));
+
+	// CCs and Docks are handled elsewhere.
+	if (template.hasClasses(["CivCentre", "Dock"]))
+		return { "quality": 0, "pos": [0, 0] };
 
-	let template = gameState.getTemplate(gameState.applyCiv("structures/{civ}_storehouse"));
 	let halfSize = 0;
 	if (template.get("Footprint/Square"))
 		halfSize = Math.max(+template.get("Footprint/Square/@depth"), +template.get("Footprint/Square/@width")) / 2;
@@ -265,10 +286,14 @@ m.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource)
 	// Then checks for a good spot in the territory. If none, and town/city phase, checks outside
 	// The AI will currently not build a CC if it wouldn't connect with an existing CC.
 
-	let obstructions = m.createObstructionMap(gameState, this.accessIndex, template);
+	let obstructions = PETRA.createObstructionMap(gameState, this.accessIndex, template);
 
-	let ccEnts = gameState.getOwnStructures().filter(API3.Filters.byClass("CivCentre")).toEntityArray();
-	let dpEnts = gameState.getOwnStructures().filter(API3.Filters.byClassesOr(["Storehouse", "Dock"])).toEntityArray();
+	const dpEnts = gameState.getOwnStructures().filter(API3.Filters.isDropsite(resource)).toEntityArray();
+
+	// Foundations don't have the dropsite properties yet, so treat them separately.
+	for (const foundation of gameState.getOwnFoundations().toEntityArray())
+		if (PETRA.getBuiltEntity(gameState, foundation).isResourceDropsite(resource))
+			dpEnts.push(foundation);
 
 	let bestIdx;
 	let bestVal = 0;
@@ -278,16 +303,18 @@ m.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource)
 	let width = territoryMap.width;
 	let cellSize = territoryMap.cellSize;
 
+	const droppableResources = template.resourceDropsiteTypes();
+
 	for (let j of this.territoryIndices)
 	{
 		let i = territoryMap.getNonObstructedTile(j, radius, obstructions);
 		if (i < 0)  // no room around
 			continue;
 
-		// we add 3 times the needed resource and once the others (except food)
-		let total = 2*gameState.sharedScript.resourceMaps[resource].map[j];
-		for (let res in gameState.sharedScript.resourceMaps)
-			if (res != "food")
+		// We add 3 times the needed resource and once others that can be dropped here.
+		let total = 2 * gameState.sharedScript.resourceMaps[resource].map[j];
+		for (const res in gameState.sharedScript.resourceMaps)
+			if (droppableResources.indexOf(res) != -1)
 				total += gameState.sharedScript.resourceMaps[res].map[j];
 
 		total *= 0.7;   // Just a normalisation factor as the locateMap is limited to 255
@@ -313,22 +340,6 @@ m.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource)
 		if (total <= bestVal)
 			continue;
 
-		for (let cc of ccEnts)
-		{
-			let ccPos = cc.position();
-			if (!ccPos)
-				continue;
-			let dist = API3.SquareVectorDistance(ccPos, pos);
-			if (dist < 3600)
-			{
-				total = 0;
-				break;
-			}
-			else if (dist < 6400)
-				total *= (Math.sqrt(dist)-60)/20;
-		}
-		if (total <= bestVal)
-			continue;
 		if (gameState.ai.HQ.isDangerousLocation(gameState, pos, halfSize))
 			continue;
 		bestVal = total;
@@ -346,7 +357,7 @@ m.BaseManager.prototype.findBestDropsiteLocation = function(gameState, resource)
 	return { "quality": bestVal, "pos": [x, z] };
 };
 
-m.BaseManager.prototype.getResourceLevel = function(gameState, type, nearbyOnly = false)
+PETRA.BaseManager.prototype.getResourceLevel = function(gameState, type, nearbyOnly = false)
 {
 	let count = 0;
 	let check = {};
@@ -371,13 +382,13 @@ m.BaseManager.prototype.getResourceLevel = function(gameState, type, nearbyOnly
 };
 
 /** check our resource levels and react accordingly */
-m.BaseManager.prototype.checkResourceLevels = function(gameState, queues)
+PETRA.BaseManager.prototype.checkResourceLevels = function(gameState, queues)
 {
 	for (let type of Resources.GetCodes())
 	{
 		if (type == "food")
 		{
-			if (gameState.ai.HQ.canBuild(gameState, "structures/{civ}_field"))	// let's see if we need to add new farms.
+			if (gameState.ai.HQ.canBuild(gameState, "structures/{civ}/field"))	// let's see if we need to add new farms.
 			{
 				let count = this.getResourceLevel(gameState, type, gameState.currentPhase() > 1);  // animals are not accounted
 				let numFarms = gameState.getOwnStructures().filter(API3.Filters.byClass("Field")).length;  // including foundations
@@ -388,7 +399,7 @@ m.BaseManager.prototype.checkResourceLevels = function(gameState, queues)
 				{
 					if (count < 600)
 					{
-						queues.field.addPlan(new m.ConstructionPlan(gameState, "structures/{civ}_field", { "favoredBase": this.ID }));
+						queues.field.addPlan(new PETRA.ConstructionPlan(gameState, "structures/{civ}/field", { "favoredBase": this.ID }));
 						gameState.ai.HQ.needFarm = true;
 					}
 				}
@@ -399,20 +410,20 @@ m.BaseManager.prototype.checkResourceLevels = function(gameState, queues)
 					if (gameState.ai.HQ.saveResources || gameState.ai.HQ.saveSpace || count > 300 || numFarms > 5)
 						goal = Math.max(goal-1, 1);
 					if (numFound + numQueue < goal)
-						queues.field.addPlan(new m.ConstructionPlan(gameState, "structures/{civ}_field", { "favoredBase": this.ID }));
+						queues.field.addPlan(new PETRA.ConstructionPlan(gameState, "structures/{civ}/field", { "favoredBase": this.ID }));
 				}
 				else if (gameState.ai.HQ.needCorral && !gameState.getOwnEntitiesByClass("Corral", true).hasEntities() &&
-				         !queues.corral.hasQueuedUnits() && gameState.ai.HQ.canBuild(gameState, "structures/{civ}_corral"))
-					queues.corral.addPlan(new m.ConstructionPlan(gameState, "structures/{civ}_corral", { "favoredBase": this.ID }));
+				         !queues.corral.hasQueuedUnits() && gameState.ai.HQ.canBuild(gameState, "structures/{civ}/corral"))
+					queues.corral.addPlan(new PETRA.ConstructionPlan(gameState, "structures/{civ}/corral", { "favoredBase": this.ID }));
 				continue;
 			}
 			if (!gameState.getOwnEntitiesByClass("Corral", true).hasEntities() &&
-			    !queues.corral.hasQueuedUnits() && gameState.ai.HQ.canBuild(gameState, "structures/{civ}_corral"))
+			    !queues.corral.hasQueuedUnits() && gameState.ai.HQ.canBuild(gameState, "structures/{civ}/corral"))
 			{
 				let count = this.getResourceLevel(gameState, type, gameState.currentPhase() > 1);  // animals are not accounted
 				if (count < 900)
 				{
-					queues.corral.addPlan(new m.ConstructionPlan(gameState, "structures/{civ}_corral", { "favoredBase": this.ID }));
+					queues.corral.addPlan(new PETRA.ConstructionPlan(gameState, "structures/{civ}/corral", { "favoredBase": this.ID }));
 					gameState.ai.HQ.needCorral = true;
 				}
 			}
@@ -433,7 +444,7 @@ m.BaseManager.prototype.checkResourceLevels = function(gameState, queues)
 		{
 			if (ent.unitAIState() == "INDIVIDUAL.GATHER.GATHERING")
 				++this.gatherers[type].used;
-			else if (ent.unitAIState() == "INDIVIDUAL.RETURNRESOURCE.APPROACHING")
+			else if (ent.unitAIState() == "INDIVIDUAL.GATHER.RETURNINGRESOURCE.APPROACHING")
 				++this.gatherers[type].lost;
 		}
 		// TODO  add also a test on remaining resources.
@@ -443,17 +454,17 @@ m.BaseManager.prototype.checkResourceLevels = function(gameState, queues)
 			let ratio = this.gatherers[type].lost / total;
 			if (ratio > 0.15)
 			{
-				let newDP = this.findBestDropsiteLocation(gameState, type);
-				if (newDP.quality > 50 && gameState.ai.HQ.canBuild(gameState, "structures/{civ}_storehouse"))
-					queues.dropsites.addPlan(new m.ConstructionPlan(gameState, "structures/{civ}_storehouse", { "base": this.ID, "type": type }, newDP.pos));
+				const newDP = this.findBestDropsiteAndLocation(gameState, type);
+				if (newDP.quality > 50 && gameState.ai.HQ.canBuild(gameState, newDP.templateName))
+					queues.dropsites.addPlan(new PETRA.ConstructionPlan(gameState, newDP.templateName, { "base": this.ID, "type": type }, newDP.pos));
 				else if (!gameState.getOwnFoundations().filter(API3.Filters.byClass("CivCentre")).hasEntities() && !queues.civilCentre.hasQueuedUnits())
 				{
 					// No good dropsite, try to build a new base if no base already planned,
 					// and if not possible, be less strict on dropsite quality.
 					if ((!gameState.ai.HQ.canExpand || !gameState.ai.HQ.buildNewBase(gameState, queues, type)) &&
 					    newDP.quality > Math.min(25, 50*0.15/ratio) &&
-					    gameState.ai.HQ.canBuild(gameState, "structures/{civ}_storehouse"))
-						queues.dropsites.addPlan(new m.ConstructionPlan(gameState, "structures/{civ}_storehouse", { "base": this.ID, "type": type }, newDP.pos));
+					    gameState.ai.HQ.canBuild(gameState, newDP.templateName))
+						queues.dropsites.addPlan(new PETRA.ConstructionPlan(gameState, newDP.templateName, { "base": this.ID, "type": type }, newDP.pos));
 				}
 			}
 			this.gatherers[type].nextCheck = gameState.ai.playedTurn + 20;
@@ -467,7 +478,7 @@ m.BaseManager.prototype.checkResourceLevels = function(gameState, queues)
 };
 
 /** Adds the estimated gather rates from this base to the currentRates */
-m.BaseManager.prototype.addGatherRates = function(gameState, currentRates)
+PETRA.BaseManager.prototype.addGatherRates = function(gameState, currentRates)
 {
 	for (let res in currentRates)
 	{
@@ -504,16 +515,14 @@ m.BaseManager.prototype.addGatherRates = function(gameState, currentRates)
 	}
 };
 
-m.BaseManager.prototype.assignRolelessUnits = function(gameState, roleless)
+PETRA.BaseManager.prototype.assignRolelessUnits = function(gameState, roleless)
 {
 	if (!roleless)
 		roleless = this.units.filter(API3.Filters.not(API3.Filters.byHasMetadata(PlayerID, "role"))).values();
 
 	for (let ent of roleless)
 	{
-		if (ent.hasClass("Worker") || ent.hasClass("CitizenSoldier") || ent.hasClass("FishingBoat"))
-			ent.setMetadata(PlayerID, "role", "worker");
-		else if (ent.hasClass("Support") && ent.hasClass("Elephant"))
+		if (ent.hasClasses(["Worker", "CitizenSoldier", "FishingBoat"]))
 			ent.setMetadata(PlayerID, "role", "worker");
 	}
 };
@@ -523,7 +532,7 @@ m.BaseManager.prototype.assignRolelessUnits = function(gameState, roleless)
  * they can be reassigned by reassignIdleWorkers.
  * TODO: actually this probably should be in the HQ.
  */
-m.BaseManager.prototype.setWorkersIdleByPriority = function(gameState)
+PETRA.BaseManager.prototype.setWorkersIdleByPriority = function(gameState)
 {
 	this.timeNextIdleCheck = gameState.ai.elapsedTime + 8;
 	// change resource only towards one which is more needed, and if changing will not change this order
@@ -579,7 +588,7 @@ m.BaseManager.prototype.setWorkersIdleByPriority = function(gameState)
  * and return remaining number of possible switches.
  * Prefer FemaleCitizen for food and CitizenSoldier for other resources.
  */
-m.BaseManager.prototype.switchGatherer = function(gameState, from, to, number)
+PETRA.BaseManager.prototype.switchGatherer = function(gameState, from, to, number)
 {
 	let num = number;
 	let only;
@@ -605,7 +614,7 @@ m.BaseManager.prototype.switchGatherer = function(gameState, from, to, number)
 	return num;
 };
 
-m.BaseManager.prototype.reassignIdleWorkers = function(gameState, idleWorkers)
+PETRA.BaseManager.prototype.reassignIdleWorkers = function(gameState, idleWorkers)
 {
 	// Search for idle workers, and tell them to gather resources based on demand
 	if (!idleWorkers)
@@ -619,12 +628,6 @@ m.BaseManager.prototype.reassignIdleWorkers = function(gameState, idleWorkers)
 		// Check that the worker isn't garrisoned
 		if (!ent.position())
 			continue;
-		// Support elephant can only be builders
-		if (ent.hasClass("Support") && ent.hasClass("Elephant"))
-		{
-			ent.setMetadata(PlayerID, "subrole", "idle");
-			continue;
-		}
 
 		if (ent.hasClass("Worker"))
 		{
@@ -651,19 +654,19 @@ m.BaseManager.prototype.reassignIdleWorkers = function(gameState, idleWorkers)
 				}
 			}
 		}
-		else if (ent.hasClass("Cavalry"))
+		else if (PETRA.isFastMoving(ent) && ent.canGather("food") && ent.canAttackClass("Animal"))
 			ent.setMetadata(PlayerID, "subrole", "hunter");
 		else if (ent.hasClass("FishingBoat"))
 			ent.setMetadata(PlayerID, "subrole", "fisher");
 	}
 };
 
-m.BaseManager.prototype.workersBySubrole = function(gameState, subrole)
+PETRA.BaseManager.prototype.workersBySubrole = function(gameState, subrole)
 {
 	return gameState.updatingCollection("subrole-" + subrole +"-base-" + this.ID, API3.Filters.byMetadata(PlayerID, "subrole", subrole), this.workers);
 };
 
-m.BaseManager.prototype.gatherersByType = function(gameState, type)
+PETRA.BaseManager.prototype.gatherersByType = function(gameState, type)
 {
 	return gameState.updatingCollection("workers-gathering-" + type +"-base-" + this.ID, API3.Filters.byMetadata(PlayerID, "gather-type", type), this.workersBySubrole(gameState, "gatherer"));
 };
@@ -672,7 +675,7 @@ m.BaseManager.prototype.gatherersByType = function(gameState, type)
  * returns an entity collection of workers.
  * They are idled immediatly and their subrole set to idle.
  */
-m.BaseManager.prototype.pickBuilders = function(gameState, workers, number)
+PETRA.BaseManager.prototype.pickBuilders = function(gameState, workers, number)
 {
 	let availableWorkers = this.workers.filter(ent => {
 		if (!ent.position() || !ent.isBuilder())
@@ -715,7 +718,7 @@ m.BaseManager.prototype.pickBuilders = function(gameState, workers, number)
  * try reassigning some other workers who are nearby
  * AI tries to use builders sensibly, not completely stopping its econ.
  */
-m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
+PETRA.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 {
 	let foundations = this.buildings.filter(API3.Filters.and(API3.Filters.isFoundation(), API3.Filters.not(API3.Filters.byClass("Field"))));
 
@@ -782,12 +785,12 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 			continue; // we do not build fields
 
 		if (gameState.ai.HQ.isNearInvadingArmy(target.position()))
-			if (!target.hasClass("CivCentre") && !target.hasClass("StoneWall") &&
+			if (!target.hasClasses(["CivCentre", "Wall"]) &&
 			    (!target.hasClass("Wonder") || !gameState.getVictoryConditions().has("wonder")))
 				continue;
 
 		// if our territory has shrinked since this foundation was positioned, do not build it
-		if (m.isNotWorthBuilding(gameState, target))
+		if (PETRA.isNotWorthBuilding(gameState, target))
 			continue;
 
 		let assigned = gameState.getOwnEntitiesByMetadata("target-foundation", target.id()).length;
@@ -798,13 +801,12 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 		    gameState.getPopulationLimit() < gameState.getPopulationMax())
 			maxTotalBuilders += 2;
 		let targetNB = 2;
-		if (target.hasClass("Fortress") || target.hasClass("Wonder") ||
+		if (target.hasClasses(["Fortress", "Wonder"]) ||
 		    target.getMetadata(PlayerID, "phaseUp") == true)
 			targetNB = 7;
-		else if (target.hasClass("Barracks") || target.hasClass("DefenseTower") ||
-		         target.hasClass("Market"))
+		else if (target.hasClasses(["Barracks", "Range", "Stable", "Tower", "Market"]))
 			targetNB = 4;
-		else if (target.hasClass("House") || target.hasClass("DropsiteWood"))
+		else if (target.hasClasses(["House", "DropsiteWood"]))
 			targetNB = 3;
 
 		if (target.getMetadata(PlayerID, "baseAnchor") == true ||
@@ -849,15 +851,10 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 		let time = target.buildTime();
 		nonBuilderWorkers.sort((workerA, workerB) => {
 			let coeffA = API3.SquareVectorDistance(target.position(), workerA.position());
-			// elephant moves slowly, so when far away they are only useful if build time is long
-			if (workerA.hasClass("Elephant"))
-				coeffA *= 0.5 * (1 + Math.sqrt(coeffA)/5/time);
-			else if (workerA.getMetadata(PlayerID, "gather-type") == "food")
+			if (workerA.getMetadata(PlayerID, "gather-type") == "food")
 				coeffA *= 3;
 			let coeffB = API3.SquareVectorDistance(target.position(), workerB.position());
-			if (workerB.hasClass("Elephant"))
-				coeffB *= 0.5 * (1 + Math.sqrt(coeffB)/5/time);
-			else if (workerB.getMetadata(PlayerID, "gather-type") == "food")
+			if (workerB.getMetadata(PlayerID, "gather-type") == "food")
 				coeffB *= 3;
 			return coeffA - coeffB;
 		});
@@ -881,7 +878,7 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 		if (gameState.ai.HQ.isNearInvadingArmy(target.position()))
 		{
 			if (target.healthLevel() > 0.5 ||
-			    !target.hasClass("CivCentre") && !target.hasClass("StoneWall") &&
+			    !target.hasClasses(["CivCentre", "Wall"]) &&
 			    (!target.hasClass("Wonder") || !gameState.getVictoryConditions().has("wonder")))
 				continue;
 		}
@@ -894,7 +891,7 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 		let assigned = gameState.getOwnEntitiesByMetadata("target-foundation", target.id()).length;
 		let maxTotalBuilders = Math.ceil(workers.length * builderRatio);
 		let targetNB = 1;
-		if (target.hasClass("Fortress") || target.hasClass("Wonder"))
+		if (target.hasClasses(["Fortress", "Wonder"]))
 			targetNB = 3;
 		if (target.getMetadata(PlayerID, "baseAnchor") == true ||
 		    target.hasClass("Wonder") && gameState.getVictoryConditions().has("wonder"))
@@ -948,7 +945,7 @@ m.BaseManager.prototype.assignToFoundations = function(gameState, noRepair)
 };
 
 /** Return false when the base is not active (no workers on it) */
-m.BaseManager.prototype.update = function(gameState, queues, events)
+PETRA.BaseManager.prototype.update = function(gameState, queues, events)
 {
 	if (this.ID == gameState.ai.HQ.baseManagers[0].ID)	// base for unaffected units
 	{
@@ -958,13 +955,13 @@ m.BaseManager.prototype.update = function(gameState, queues, events)
 		{
 			for (let ent of this.units.values())
 			{
-				let bestBase = m.getBestBase(gameState, ent);
+				let bestBase = PETRA.getBestBase(gameState, ent);
 				if (bestBase.ID != this.ID)
 					bestBase.assignEntity(gameState, ent);
 			}
 			for (let ent of this.buildings.values())
 			{
-				let bestBase = m.getBestBase(gameState, ent);
+				let bestBase = PETRA.getBestBase(gameState, ent);
 				if (!bestBase)
 				{
 					if (ent.hasClass("Dock"))
@@ -998,7 +995,7 @@ m.BaseManager.prototype.update = function(gameState, queues, events)
 			// Reassign all remaining entities to its nearest base
 			for (let ent of this.units.values())
 			{
-				let base = m.getBestBase(gameState, ent, false, this.ID);
+				let base = PETRA.getBestBase(gameState, ent, false, this.ID);
 				base.assignEntity(gameState, ent);
 			}
 			return false;
@@ -1009,7 +1006,7 @@ m.BaseManager.prototype.update = function(gameState, queues, events)
 		{
 			if (!ent.position())
 				continue;
-			let base = m.getBestBase(gameState, ent);
+			let base = PETRA.getBestBase(gameState, ent);
 			if (base.anchor)
 				reassignedBase = base;
 			break;
@@ -1083,7 +1080,7 @@ m.BaseManager.prototype.update = function(gameState, queues, events)
 	return true;
 };
 
-m.BaseManager.prototype.Serialize = function()
+PETRA.BaseManager.prototype.Serialize = function()
 {
 	return {
 		"ID": this.ID,
@@ -1098,14 +1095,10 @@ m.BaseManager.prototype.Serialize = function()
 	};
 };
 
-m.BaseManager.prototype.Deserialize = function(gameState, data)
+PETRA.BaseManager.prototype.Deserialize = function(gameState, data)
 {
 	for (let key in data)
 		this[key] = data[key];
 
 	this.anchor = this.anchorId !== undefined ? gameState.getEntityById(this.anchorId) : undefined;
 };
-
-return m;
-
-}(PETRA);
