@@ -1,13 +1,11 @@
-var PETRA = function(m)
+PETRA.DefenseManager = function(Config)
 {
-
-m.DefenseManager = function(Config)
-{
-	this.armies = [];	// array of "army" Objects
+	// Array of "army" Objects.
+	this.armies = [];
 	this.Config = Config;
 	this.targetList = [];
 	this.armyMergeSize = this.Config.Defense.armyMergeSize;
-	// stats on how many enemies are currently attacking our allies
+	// Stats on how many enemies are currently attacking our allies
 	// this.attackingArmies[enemy][ally] = number of enemy armies inside allied territory
 	// this.attackingUnits[enemy][ally] = number of enemy units not in armies inside allied territory
 	// this.attackedAllies[ally] = number of enemies attacking the ally
@@ -16,7 +14,7 @@ m.DefenseManager = function(Config)
 	this.attackedAllies = {};
 };
 
-m.DefenseManager.prototype.update = function(gameState, events)
+PETRA.DefenseManager.prototype.update = function(gameState, events)
 {
 	Engine.ProfileStart("Defense Manager");
 
@@ -24,7 +22,7 @@ m.DefenseManager.prototype.update = function(gameState, events)
 
 	this.checkEvents(gameState, events);
 
-	// Check if our potential targets are still valid
+	// Check if our potential targets are still valid.
 	for (let i = 0; i < this.targetList.length; ++i)
 	{
 		let target = gameState.getEntityById(this.targetList[i]);
@@ -32,8 +30,8 @@ m.DefenseManager.prototype.update = function(gameState, events)
 			this.targetList.splice(i--, 1);
 	}
 
-	// Count the number of enemies attacking our allies in the previous turn
-	// We'll be more cooperative if several enemies are attacking him simultaneously
+	// Count the number of enemies attacking our allies in the previous turn.
+	// We'll be more cooperative if several enemies are attacking him simultaneously.
 	this.attackedAllies = {};
 	let attackingArmies = clone(this.attackingArmies);
 	for (let enemy in this.attackingUnits)
@@ -67,7 +65,7 @@ m.DefenseManager.prototype.update = function(gameState, events)
 	Engine.ProfileStop();
 };
 
-m.DefenseManager.prototype.makeIntoArmy = function(gameState, entityID, type = "default")
+PETRA.DefenseManager.prototype.makeIntoArmy = function(gameState, entityID, type = "default")
 {
 	if (type == "default")
 	{
@@ -78,14 +76,14 @@ m.DefenseManager.prototype.makeIntoArmy = function(gameState, entityID, type = "
 	}
 
 	// Create a new army for it.
-	let army = new m.DefenseArmy(gameState, [entityID], type);
+	let army = new PETRA.DefenseArmy(gameState, [entityID], type);
 
 	this.armies.push(army);
 };
 
-m.DefenseManager.prototype.getArmy = function(partOfArmy)
+PETRA.DefenseManager.prototype.getArmy = function(partOfArmy)
 {
-	// Find the army corresponding to this ID partOfArmy
+	// Find the army corresponding to this ID partOfArmy.
 	for (let army of this.armies)
 		if (army.ID == partOfArmy)
 			return army;
@@ -93,7 +91,7 @@ m.DefenseManager.prototype.getArmy = function(partOfArmy)
 	return undefined;
 };
 
-m.DefenseManager.prototype.isDangerous = function(gameState, entity)
+PETRA.DefenseManager.prototype.isDangerous = function(gameState, entity)
 {
 	if (!entity.position())
 		return false;
@@ -101,8 +99,8 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 	let territoryOwner = this.territoryMap.getOwner(entity.position());
 	if (territoryOwner != 0 && !gameState.isPlayerAlly(territoryOwner))
 		return false;
-	// check if the entity is trying to build a new base near our buildings,
-	// and if yes, add this base in our target list
+	// Check if the entity is trying to build a new base near our buildings,
+	// and if yes, add this base in our target list.
 	if (entity.unitAIState() && entity.unitAIState() == "INDIVIDUAL.REPAIR.REPAIRING")
 	{
 		let targetId = entity.unitAIOrderData()[0].target;
@@ -137,14 +135,15 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 	if (entity.attackTypes() === undefined || entity.hasClass("Support"))
 		return false;
 	let dist2Min = 6000;
-	// TODO the 30 is to take roughly into account the structure size in following checks. Can be improved
+	// TODO the 30 is to take roughly into account the structure size in following checks. Can be improved.
 	if (entity.attackTypes().indexOf("Ranged") != -1)
 		dist2Min = (entity.attackRange("Ranged").max + 30) * (entity.attackRange("Ranged").max + 30);
 
 	for (let targetId of this.targetList)
 	{
 		let target = gameState.getEntityById(targetId);
-		if (!target || !target.position())   // the enemy base is either destroyed or built
+		// The enemy base is either destroyed or built.
+		if (!target || !target.position())
 			continue;
 		if (API3.SquareVectorDistance(target.position(), entity.position()) < dist2Min)
 			return true;
@@ -156,9 +155,7 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 		if (!gameState.isEntityExclusiveAlly(cc) || cc.foundationProgress() == 0)
 			continue;
 		let cooperation = this.GetCooperationLevel(cc.owner());
-		if (cooperation < 0.6 && cc.foundationProgress() !== undefined)
-			continue;
-		if (cooperation < 0.3)
+		if (cooperation < 0.3 || cooperation < 0.6 && !!cc.foundationProgress())
 			continue;
 		if (API3.SquareVectorDistance(cc.position(), entity.position()) < dist2Min)
 			return true;
@@ -175,7 +172,7 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 
 	if (gameState.isPlayerMutualAlly(territoryOwner))
 	{
-		// If ally attacked by more than 2 enemies, help him not only for cc but also for structures
+		// If ally attacked by more than 2 enemies, help him not only for cc but also for structures.
 		if (territoryOwner != PlayerID && this.attackedAllies[territoryOwner] &&
 		                                  this.attackedAllies[territoryOwner] > 1 &&
 		                                  this.GetCooperationLevel(territoryOwner) > 0.7)
@@ -190,7 +187,7 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 			}
 		}
 
-		// Update the number of enemies attacking this ally
+		// Update the number of enemies attacking this ally.
 		let enemy = entity.owner();
 		if (this.attackingUnits[enemy] === undefined)
 			this.attackingUnits[enemy] = {};
@@ -202,7 +199,7 @@ m.DefenseManager.prototype.isDangerous = function(gameState, entity)
 	return false;
 };
 
-m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
+PETRA.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 {
 	const nbPlayers = gameState.sharedScript.playersData.length;
 	let i = gameState.ai.playedTurn % nbPlayers;
@@ -212,7 +209,7 @@ m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 	{
 		if (!this.armies.length)
 		{
-			// check if we can recover capture points from any of our notdecaying structures
+			// Check if we can recover capture points from any of our notdecaying structures.
 			for (let ent of gameState.getOwnStructures().values())
 			{
 				if (ent.decaying())
@@ -235,13 +232,12 @@ m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 	else if (!gameState.isPlayerEnemy(i))
 		return;
 
-	// loop through enemy units
 	for (let ent of gameState.getEnemyUnits(i).values())
 	{
 		if (ent.getMetadata(PlayerID, "PartOfArmy") !== undefined)
 			continue;
 
-		// keep animals attacking us or our allies
+		// Keep animals attacking us or our allies.
 		if (ent.hasClass("Animal"))
 		{
 			if (!ent.unitAIState() || ent.unitAIState().split(".")[1] != "COMBAT")
@@ -255,18 +251,18 @@ m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 		}
 
 		// TODO what to do for ships ?
-		if (ent.hasClass("Ship") || ent.hasClass("Trader"))
+		if (ent.hasClasses(["Ship", "Trader"]))
 			continue;
 
-		// check if unit is dangerous "a priori"
+		// Check if unit is dangerous "a priori".
 		if (this.isDangerous(gameState, ent))
 			this.makeIntoArmy(gameState, ent.id());
 	}
 
 	if (i != 0 || this.armies.length > 1 || gameState.ai.HQ.numActiveBases() == 0)
 		return;
-	// look for possible gaia buildings inside our territory (may happen when enemy resign or after structure decay)
-	// and attack it only if useful (and capturable) or dangereous
+	// Look for possible gaia buildings inside our territory (may happen when enemy resign or after structure decay)
+	// and attack it only if useful (and capturable) or dangereous.
 	for (let ent of gameState.getEnemyStructures(i).values())
 	{
 		if (!ent.position() || ent.getMetadata(PlayerID, "PartOfArmy") !== undefined)
@@ -279,15 +275,16 @@ m.DefenseManager.prototype.checkEnemyUnits = function(gameState)
 	}
 };
 
-m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
+PETRA.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 {
 	for (let i = 0; i < this.armies.length; ++i)
 	{
 		let army = this.armies[i];
-		// this returns a list of IDs: the units that broke away from the army for being too far.
+		// This returns a list of IDs: the units that broke away from the army for being too far.
 		let breakaways = army.update(gameState);
+		// Assume dangerosity.
 		for (let breaker of breakaways)
-			this.makeIntoArmy(gameState, breaker);		// assume dangerosity
+			this.makeIntoArmy(gameState, breaker);
 
 		if (army.getState() == 0)
 		{
@@ -295,10 +292,9 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 				this.switchToAttack(gameState, army);
 			army.clear(gameState);
 			this.armies.splice(i--, 1);
-			continue;
 		}
 	}
-	// Check if we can't merge it with another
+	// Check if we can't merge it with another.
 	for (let i = 0; i < this.armies.length - 1; ++i)
 	{
 		let army = this.armies[i];
@@ -310,7 +306,7 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 			if (otherArmy.getType() != "default" ||
 				API3.SquareVectorDistance(army.foePosition, otherArmy.foePosition) > this.armyMergeSize)
 				continue;
-			// no need to clear here.
+			// No need to clear here.
 			army.merge(gameState, otherArmy);
 			this.armies.splice(j--, 1);
 		}
@@ -318,7 +314,7 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 
 	if (gameState.ai.playedTurn % 5 != 0)
 		return;
-	// Check if any army is no more dangerous (possibly because it has defeated us and destroyed our base)
+	// Check if any army is no more dangerous (possibly because it has defeated us and destroyed our base).
 	this.attackingArmies = {};
 	for (let i = 0; i < this.armies.length; ++i)
 	{
@@ -329,7 +325,7 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 		{
 			if (gameState.isPlayerMutualAlly(owner))
 			{
-				// update the number of enemies attacking this ally
+				// Update the number of enemies attacking this ally.
 				for (let id of army.foeEntities)
 				{
 					let ent = gameState.getEntityById(id);
@@ -346,15 +342,16 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 			}
 			continue;
 		}
-		else if (owner != 0)   // enemy army back in its territory
+		// Enemy army back in its territory.
+		else if (owner != 0)
 		{
 			army.clear(gameState);
 			this.armies.splice(i--, 1);
 			continue;
 		}
 
-		// army in neutral territory
-		// TODO check smaller distance with all our buildings instead of only ccs with big distance
+		// Army in neutral territory.
+		// TODO check smaller distance with all our buildings instead of only ccs with big distance.
 		let stillDangerous = false;
 		let bases = gameState.updatingGlobalCollection("allCCs", API3.Filters.byClass("CivCentre"));
 		for (let base of bases.values())
@@ -373,7 +370,7 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 		}
 		if (stillDangerous)
 			continue;
-		// Need to also check docks because of oversea bases
+		// Need to also check docks because of oversea bases.
 		for (let dock of gameState.getOwnStructures().filter(API3.Filters.byClass("Dock")).values())
 		{
 			if (API3.SquareVectorDistance(dock.position(), army.foePosition) > 10000)
@@ -391,13 +388,13 @@ m.DefenseManager.prototype.checkEnemyArmies = function(gameState)
 	}
 };
 
-m.DefenseManager.prototype.assignDefenders = function(gameState)
+PETRA.DefenseManager.prototype.assignDefenders = function(gameState)
 {
 	if (!this.armies.length)
 		return;
 
 	let armiesNeeding = [];
-	// let's add defenders
+	// Let's add defenders.
 	for (let army of this.armies)
 	{
 		let needsDef = army.needsDefenders(gameState);
@@ -410,7 +407,7 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 			let ent = gameState.getEntityById(entId);
 			if (!ent || !ent.position())
 				continue;
-			armyAccess = m.getLandAccess(gameState, ent);
+			armyAccess = PETRA.getLandAccess(gameState, ent);
 			break;
 		}
 		if (!armyAccess)
@@ -422,7 +419,7 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 	if (!armiesNeeding.length)
 		return;
 
-	// let's get our potential units
+	// Let's get our potential units.
 	let potentialDefenders = [];
 	gameState.getOwnUnits().forEach(function(ent) {
 		if (!ent.position())
@@ -431,9 +428,7 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 			return;
 		if (ent.hasClass("Support") || ent.attackTypes() === undefined)
 			return;
-		if (ent.hasClass("Catapult"))
-			return;
-		if (ent.hasClass("FishingBoat") || ent.hasClass("Trader"))
+		if (ent.hasClasses(["StoneThrower", "Support", "FishingBoat"]))
 			return;
 		if (ent.getMetadata(PlayerID, "transport") !== undefined ||
 		    ent.getMetadata(PlayerID, "transporter") !== undefined)
@@ -451,9 +446,9 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 
 	for (let ipass = 0; ipass < 2; ++ipass)
 	{
-		// First pass only assign defenders with the right access
-		// Second pass assign all defenders
-		// TODO could sort them by distance
+		// First pass only assign defenders with the right access.
+		// Second pass assign all defenders.
+		// TODO could sort them by distance.
 		let backup = 0;
 		for (let i = 0; i < potentialDefenders.length; ++i)
 		{
@@ -462,11 +457,19 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 				continue;
 			let aMin;
 			let distMin;
-			let access = ipass == 0 ? m.getLandAccess(gameState, ent) : undefined;
+			let access = ipass == 0 ? PETRA.getLandAccess(gameState, ent) : undefined;
 			for (let a = 0; a < armiesNeeding.length; ++a)
 			{
 				if (access && armiesNeeding[a].access != access)
 					continue;
+
+				// Do not assign defender if it cannot attack at least part of the attacking army.
+				if (!armiesNeeding[a].army.foeEntities.some(eEnt => {
+					let eEntID = gameState.getEntityById(eEnt);
+					return ent.canAttackTarget(eEntID, PETRA.allowCapture(gameState, ent, eEntID));
+					}))
+					continue;
+
 				let dist = API3.SquareVectorDistance(ent.position(), armiesNeeding[a].army.foePosition);
 				if (aMin !== undefined && dist > distMin)
 					continue;
@@ -475,7 +478,7 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 			}
 
 			// If outside our territory (helping an ally or attacking a cc foundation)
-			// or if in another access, keep some troops in backup
+			// or if in another access, keep some troops in backup.
 			if (backup < 12 && (aMin == undefined || distMin > 40000 &&
 			        this.territoryMap.getOwner(armiesNeeding[aMin].army.foePosition) != PlayerID))
 			{
@@ -486,7 +489,7 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 			else if (aMin === undefined)
 				continue;
 
-			armiesNeeding[aMin].need -= m.getMaxStrength(ent);
+			armiesNeeding[aMin].need -= PETRA.getMaxStrength(ent, this.Config.debug, this.Config.DamageTypeImportance);
 			armiesNeeding[aMin].army.addOwn(gameState, potentialDefenders[i]);
 			armiesNeeding[aMin].army.assignUnit(gameState, potentialDefenders[i]);
 			potentialDefenders[i] = undefined;
@@ -498,14 +501,14 @@ m.DefenseManager.prototype.assignDefenders = function(gameState)
 		}
 	}
 
-	// If shortage of defenders, produce infantry garrisoned in nearest civil centre
+	// If shortage of defenders, produce infantry garrisoned in nearest civil center.
 	let armiesPos = [];
 	for (let a = 0; a < armiesNeeding.length; ++a)
 		armiesPos.push(armiesNeeding[a].army.foePosition);
 	gameState.ai.HQ.trainEmergencyUnits(gameState, armiesPos);
 };
 
-m.DefenseManager.prototype.abortArmy = function(gameState, army)
+PETRA.DefenseManager.prototype.abortArmy = function(gameState, army)
 {
 	army.clear(gameState);
 	for (let i = 0; i < this.armies.length; ++i)
@@ -520,21 +523,23 @@ m.DefenseManager.prototype.abortArmy = function(gameState, army)
 /**
  * If our defense structures are attacked, garrison soldiers inside when possible
  * and if a support unit is attacked and has less than 55% health, garrison it inside the nearest healing structure
- * and if a ranged siege unit (not used for defense) is attacked, garrison it in the nearest fortress
- * If our hero is attacked with regicide victory condition, the victoryManager will handle it
+ * and if a ranged siege unit (not used for defense) is attacked, garrison it in the nearest fortress.
+ * If our hero is attacked with regicide victory condition, the victoryManager will handle it.
  */
-m.DefenseManager.prototype.checkEvents = function(gameState, events)
+PETRA.DefenseManager.prototype.checkEvents = function(gameState, events)
 {
-	// must be called every turn for all armies
+	// Must be called every turn for all armies.
 	for (let army of this.armies)
 		army.checkEvents(gameState, events);
 
-	for (let evt of events.OwnershipChanged)   // capture events
+	// Capture events.
+	for (let evt of events.OwnershipChanged)
 	{
 		if (gameState.isPlayerMutualAlly(evt.from) && evt.to > 0)
 		{
 			let ent = gameState.getEntityById(evt.entity);
-			if (ent && ent.hasClass("CivCentre")) // one of our cc has been captured
+			// One of our cc has been captured.
+			if (ent && ent.hasClass("CivCentre"))
 				gameState.ai.HQ.attackManager.switchDefenseToAttack(gameState, ent, { "range": 150 });
 		}
 	}
@@ -554,7 +559,7 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 		   (!target.hasClass("Structure") || target.attackRange("Ranged")))
 		{
 			// If enemies are in range of one of our defensive structures, garrison it for arrow multiplier
-			// (enemy non-defensive structure are not considered to stay in sync with garrisonManager)
+			// (enemy non-defensive structure are not considered to stay in sync with garrisonManager).
 			if (attacker.position() && attacker.isGarrisonHolder() && attacker.getArrowMultiplier() &&
 			    (target.owner() != 0 || !target.hasClass("Unit") ||
 			     target.unitAIState() && target.unitAIState().split(".")[1] == "COMBAT"))
@@ -564,7 +569,7 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 		if (!gameState.isEntityOwn(target))
 			continue;
 
-		// If attacked by one of our allies (he must trying to recover capture points), do not react
+		// If attacked by one of our allies (he must trying to recover capture points), do not react.
 		if (attacker && gameState.isEntityAlly(attacker))
 			continue;
 
@@ -577,12 +582,13 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 				let pos = attacker.position();
 				let range = attacker.attackRange("Ranged") ? attacker.attackRange("Ranged").max + 15 : 25;
 				if (range * range > API3.SquareVectorDistance(pos, target.position()))
-					target.moveToRange(pos[0], pos[1], range, range);
+					target.moveToRange(pos[0], pos[1], range, range + 5);
 			}
 			continue;
 		}
-
-		if (target.hasClass("Ship"))    // TODO integrate other ships later, need to be sure it is accessible
+		
+		// TODO integrate other ships later, need to be sure it is accessible.
+		if (target.hasClass("Ship"))
 			continue;
 
 		// If a building on a blinking tile is attacked, check if it can be defended.
@@ -604,7 +610,7 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 		}
 
 
-		// If inside a started attack plan, let the plan deal with this unit
+		// If inside a started attack plan, let the plan deal with this unit.
 		let plan = target.getMetadata(PlayerID, "plan");
 		if (plan !== undefined && plan >= 0)
 		{
@@ -613,10 +619,10 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 				continue;
 		}
 
-		// Signal this attacker to our defense manager, except if we are in enemy territory
-		// TODO treat ship attack
+		// Signal this attacker to our defense manager, except if we are in enemy territory.
+		// TODO treat ship attack.
 		if (attacker && attacker.position() && attacker.getMetadata(PlayerID, "PartOfArmy") === undefined &&
-			!attacker.hasClass("Structure") && !attacker.hasClass("Ship"))
+			!attacker.hasClasses(["Structure", "Ship"]))
 		{
 			let territoryOwner = this.territoryMap.getOwner(attacker.position());
 			if (territoryOwner == 0 || gameState.isPlayerAlly(territoryOwner))
@@ -629,13 +635,13 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 			if (army.getType() == "capturing")
 			{
 				let abort = false;
-				// if one of the units trying to capture a structure is attacked,
+				// If one of the units trying to capture a structure is attacked,
 				// abort the army so that the unit can defend itself
 				if (army.ownEntities.indexOf(target.id()) != -1)
 					abort = true;
 				else if (army.foeEntities[0] == target.id() && target.owner() == PlayerID)
 				{
-					// else we may be trying to regain some capture point from one of our structure
+					// else we may be trying to regain some capture point from one of our structure.
 					abort = true;
 					let capture = target.capturePoints();
 					for (let j = 0; j < capture.length; ++j)
@@ -652,7 +658,7 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 			continue;
 		}
 
-		// try to garrison any attacked support unit if low healthlevel
+		// Try to garrison any attacked support unit if low health.
 		if (target.hasClass("Support") && target.healthLevel() < this.Config.garrisonHealthLevel.medium &&
 			!target.getMetadata(PlayerID, "transport") && plan != -2 && plan != -3)
 		{
@@ -660,8 +666,8 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 			continue;
 		}
 
-		// try to garrison any attacked catapult
-		if (target.hasClass("Catapult") &&
+		// Try to garrison any attacked stone thrower.
+		if (target.hasClass("StoneThrower") &&
 			!target.getMetadata(PlayerID, "transport") && plan != -2 && plan != -3)
 		{
 			this.garrisonSiegeUnit(gameState, target);
@@ -676,7 +682,7 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 
 		if (target.hasClass("Unit") && attacker.hasClass("Unit"))
 		{
-			// Consider if we should retaliate or continue our task
+			// Consider whether we should retaliate or continue our task.
 			if (target.hasClass("Support") || target.attackTypes() === undefined)
 				continue;
 			let orderData = target.unitAIOrderData();
@@ -687,14 +693,14 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 				let unitAIState = target.unitAIState();
 				let unitAIStateOrder = unitAIState ? unitAIState.split(".")[1] : "";
 				if (unitAIStateOrder == "COMBAT" && (currentTarget == attacker.id() ||
-					!currentTarget.hasClass("Structure") && !currentTarget.hasClass("Support")))
+					!currentTarget.hasClasses(["Structure", "Support"])))
 					continue;
 				if (unitAIStateOrder == "REPAIR" && currentTarget.hasDefensiveFire())
 					continue;
-				if (unitAIStateOrder == "COMBAT" && !m.isSiegeUnit(currentTarget) &&
+				if (unitAIStateOrder == "COMBAT" && !PETRA.isSiegeUnit(currentTarget) &&
 				    gameState.ai.HQ.capturableTargets.has(orderData[0].target))
 				{
-					// take the nearest unit also attacking this structure to help us
+					// Take the nearest unit also attacking this structure to help us.
 					let capturableTarget = gameState.ai.HQ.capturableTargets.get(orderData[0].target);
 					let minDist;
 					let minEnt;
@@ -705,9 +711,9 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 						if (allAttacked[entId])
 							continue;
 						let ent = gameState.getEntityById(entId);
-						if (!ent || !ent.position())
+						if (!ent || !ent.position() || !ent.canAttackTarget(attacker, PETRA.allowCapture(gameState, ent, attacker)))
 							continue;
-						// Check that the unit is still attacking the structure (since the last played turn)
+						// Check that the unit is still attacking the structure (since the last played turn).
 						let state = ent.unitAIState();
 						if (!state || !state.split(".")[1] || state.split(".")[1] != "COMBAT")
 							continue;
@@ -724,21 +730,23 @@ m.DefenseManager.prototype.checkEvents = function(gameState, events)
 					if (minEnt)
 					{
 						capturableTarget.ents.delete(minEnt.id());
-						minEnt.attack(attacker.id(), m.allowCapture(gameState, minEnt, attacker));
+						minEnt.attack(attacker.id(), PETRA.allowCapture(gameState, minEnt, attacker));
 					}
 				}
 			}
-			target.attack(attacker.id(), m.allowCapture(gameState, target, attacker));
+			let allowCapture = PETRA.allowCapture(gameState, target, attacker);
+			if (target.canAttackTarget(attacker, allowCapture))
+				target.attack(attacker.id(), allowCapture);
 		}
 	}
 };
 
-m.DefenseManager.prototype.garrisonUnitsInside = function(gameState, target, data)
+PETRA.DefenseManager.prototype.garrisonUnitsInside = function(gameState, target, data)
 {
 	if (target.hitpoints() < target.garrisonEjectHealth() * target.maxHitpoints())
 		return false;
 	let minGarrison = data.min || target.garrisonMax();
-	if (gameState.ai.HQ.garrisonManager.numberOfGarrisonedUnits(target) >= minGarrison)
+	if (gameState.ai.HQ.garrisonManager.numberOfGarrisonedSlots(target) >= minGarrison)
 		return false;
 	if (data.attacker)
 	{
@@ -750,23 +758,23 @@ m.DefenseManager.prototype.garrisonUnitsInside = function(gameState, target, dat
 		if (dist >= range*range)
 			return false;
 	}
-	let access = m.getLandAccess(gameState, target);
+	let access = PETRA.getLandAccess(gameState, target);
 	let garrisonManager = gameState.ai.HQ.garrisonManager;
 	let garrisonArrowClasses = target.getGarrisonArrowClasses();
 	let typeGarrison = data.type || "protection";
 	let allowMelee = gameState.ai.HQ.garrisonManager.allowMelee(target);
 	if (allowMelee === undefined)
 	{
-		// Should be kept in sync with garrisonManager to avoid garrisoning-ungarrisoning some units
+		// Should be kept in sync with garrisonManager to avoid garrisoning-ungarrisoning some units.
 		if (data.attacker)
-			allowMelee = data.attacker.hasClass("Structure") ? data.attacker.attackRange("Ranged") : !m.isSiegeUnit(data.attacker);
+			allowMelee = data.attacker.hasClass("Structure") ? data.attacker.attackRange("Ranged") : !PETRA.isSiegeUnit(data.attacker);
 		else
 			allowMelee = true;
 	}
 	let units = gameState.getOwnUnits().filter(ent => {
 		if (!ent.position())
 			return false;
-		if (!MatchesClassList(ent.classes(), garrisonArrowClasses))
+		if (!ent.hasClasses(garrisonArrowClasses))
 			return false;
 		if (typeGarrison != "decay" && !allowMelee && ent.attackTypes().indexOf("Melee") != -1)
 			return false;
@@ -778,11 +786,11 @@ m.DefenseManager.prototype.garrisonUnitsInside = function(gameState, target, dat
 		if (ent.getMetadata(PlayerID, "plan") !== undefined && ent.getMetadata(PlayerID, "plan") >= 0)
 		{
 			let subrole = ent.getMetadata(PlayerID, "subrole");
-			// when structure decaying (usually because we've just captured it in enemy territory), also allow units from an attack plan
+			// When structure decaying (usually because we've just captured it in enemy territory), also allow units from an attack plan.
 			if (typeGarrison != "decay" && subrole && (subrole == "completing" || subrole == "walking" || subrole == "attacking"))
 				return false;
 		}
-		if (m.getLandAccess(gameState, ent) != access)
+		if (PETRA.getLandAccess(gameState, ent) != access)
 			return false;
 		return true;
 	}).filterNearest(target.position());
@@ -790,7 +798,7 @@ m.DefenseManager.prototype.garrisonUnitsInside = function(gameState, target, dat
 	let ret = false;
 	for (let ent of units.values())
 	{
-		if (garrisonManager.numberOfGarrisonedUnits(target) >= minGarrison)
+		if (garrisonManager.numberOfGarrisonedSlots(target) >= minGarrison)
 			break;
 		if (ent.getMetadata(PlayerID, "plan") !== undefined && ent.getMetadata(PlayerID, "plan") >= 0)
 		{
@@ -807,24 +815,24 @@ m.DefenseManager.prototype.garrisonUnitsInside = function(gameState, target, dat
 	return ret;
 };
 
-/** garrison a attacked siege ranged unit inside the nearest fortress */
-m.DefenseManager.prototype.garrisonSiegeUnit = function(gameState, unit)
+/** Garrison a attacked siege ranged unit inside the nearest fortress. */
+PETRA.DefenseManager.prototype.garrisonSiegeUnit = function(gameState, unit)
 {
 	let distmin = Math.min();
 	let nearest;
-	let unitAccess = m.getLandAccess(gameState, unit);
+	let unitAccess = PETRA.getLandAccess(gameState, unit);
 	let garrisonManager = gameState.ai.HQ.garrisonManager;
 	for (let ent of gameState.getAllyStructures().values())
 	{
 		if (!ent.isGarrisonHolder())
 			continue;
-		if (!MatchesClassList(unit.classes(), ent.garrisonableClasses()))
+		if (!unit.hasClasses(ent.garrisonableClasses()))
 			continue;
-		if (garrisonManager.numberOfGarrisonedUnits(ent) >= ent.garrisonMax())
+		if (garrisonManager.numberOfGarrisonedSlots(ent) >= ent.garrisonMax())
 			continue;
 		if (ent.hitpoints() < ent.garrisonEjectHealth() * ent.maxHitpoints())
 			continue;
-		if (m.getLandAccess(gameState, ent) != unitAccess)
+		if (PETRA.getLandAccess(gameState, ent) != unitAccess)
 			continue;
 		let dist = API3.SquareVectorDistance(ent.position(), unit.position());
 		if (dist > distmin)
@@ -838,15 +846,15 @@ m.DefenseManager.prototype.garrisonSiegeUnit = function(gameState, unit)
 };
 
 /**
- * Garrison a hurt unit inside a player-owned or allied structure
- * If emergency is true, the unit will be garrisoned in the closest possible structure
- * Otherwise, it will garrison in the closest healing structure
+ * Garrison a hurt unit inside a player-owned or allied structure.
+ * If emergency is true, the unit will be garrisoned in the closest possible structure.
+ * Otherwise, it will garrison in the closest healing structure.
  */
-m.DefenseManager.prototype.garrisonAttackedUnit = function(gameState, unit, emergency = false)
+PETRA.DefenseManager.prototype.garrisonAttackedUnit = function(gameState, unit, emergency = false)
 {
 	let distmin = Math.min();
 	let nearest;
-	let unitAccess = m.getLandAccess(gameState, unit);
+	let unitAccess = PETRA.getLandAccess(gameState, unit);
 	let garrisonManager = gameState.ai.HQ.garrisonManager;
 	for (let ent of gameState.getAllyStructures().values())
 	{
@@ -854,14 +862,14 @@ m.DefenseManager.prototype.garrisonAttackedUnit = function(gameState, unit, emer
 			continue;
 		if (!emergency && !ent.buffHeal())
 			continue;
-		if (!MatchesClassList(unit.classes(), ent.garrisonableClasses()))
+		if (!unit.hasClasses(ent.garrisonableClasses()))
 			continue;
-		if (garrisonManager.numberOfGarrisonedUnits(ent) >= ent.garrisonMax() &&
+		if (garrisonManager.numberOfGarrisonedSlots(ent) >= ent.garrisonMax() &&
 		    (!emergency || !ent.garrisoned().length))
 			continue;
 		if (ent.hitpoints() < ent.garrisonEjectHealth() * ent.maxHitpoints())
 			continue;
-		if (m.getLandAccess(gameState, ent) != unitAccess)
+		if (PETRA.getLandAccess(gameState, ent) != unitAccess)
 			continue;
 		let dist = API3.SquareVectorDistance(ent.position(), unit.position());
 		if (dist > distmin)
@@ -877,7 +885,7 @@ m.DefenseManager.prototype.garrisonAttackedUnit = function(gameState, unit, emer
 		garrisonManager.garrison(gameState, unit, nearest, "protection");
 		return true;
 	}
-	if (garrisonManager.numberOfGarrisonedUnits(nearest) >= nearest.garrisonMax()) // make room for this ent
+	if (garrisonManager.numberOfGarrisonedSlots(nearest) >= nearest.garrisonMax()) // make room for this ent
 		nearest.unload(nearest.garrisoned()[0]);
 
 	garrisonManager.garrison(gameState, unit, nearest, nearest.buffHeal() ? "protection" : "emergency");
@@ -885,9 +893,9 @@ m.DefenseManager.prototype.garrisonAttackedUnit = function(gameState, unit, emer
 };
 
 /**
- * Be more inclined to help an ally attacked by several enemies
+ * Be more inclined to help an ally attacked by several enemies.
  */
-m.DefenseManager.prototype.GetCooperationLevel = function(ally)
+PETRA.DefenseManager.prototype.GetCooperationLevel = function(ally)
 {
 	let cooperation = this.Config.personality.cooperative;
 	if (this.attackedAllies[ally] && this.attackedAllies[ally] > 1)
@@ -896,9 +904,9 @@ m.DefenseManager.prototype.GetCooperationLevel = function(ally)
 };
 
 /**
- * Switch a defense army into an attack if needed
+ * Switch a defense army into an attack if needed.
  */
-m.DefenseManager.prototype.switchToAttack = function(gameState, army)
+PETRA.DefenseManager.prototype.switchToAttack = function(gameState, army)
 {
 	if (!army)
 		return;
@@ -907,12 +915,12 @@ m.DefenseManager.prototype.switchToAttack = function(gameState, army)
 		let target = gameState.getEntityById(targetId);
 		if (!target || !target.position() || !gameState.isPlayerEnemy(target.owner()))
 			continue;
-		let targetAccess = m.getLandAccess(gameState, target);
+		let targetAccess = PETRA.getLandAccess(gameState, target);
 		let targetPos = target.position();
 		for (let entId of army.ownEntities)
 		{
 			let ent = gameState.getEntityById(entId);
-			if (!ent || !ent.position() || m.getLandAccess(gameState, ent) != targetAccess)
+			if (!ent || !ent.position() || PETRA.getLandAccess(gameState, ent) != targetAccess)
 				continue;
 			if (API3.SquareVectorDistance(targetPos, ent.position()) > 14400)
 				continue;
@@ -922,7 +930,7 @@ m.DefenseManager.prototype.switchToAttack = function(gameState, army)
 	}
 };
 
-m.DefenseManager.prototype.Serialize = function()
+PETRA.DefenseManager.prototype.Serialize = function()
 {
 	let properties = {
 		"targetList": this.targetList,
@@ -939,7 +947,7 @@ m.DefenseManager.prototype.Serialize = function()
 	return { "properties": properties, "armies": armies };
 };
 
-m.DefenseManager.prototype.Deserialize = function(gameState, data)
+PETRA.DefenseManager.prototype.Deserialize = function(gameState, data)
 {
 	for (let key in data.properties)
 		this[key] = data.properties[key];
@@ -947,11 +955,8 @@ m.DefenseManager.prototype.Deserialize = function(gameState, data)
 	this.armies = [];
 	for (let dataArmy of data.armies)
 	{
-		let army = new m.DefenseArmy(gameState, []);
+		let army = new PETRA.DefenseArmy(gameState, []);
 		army.Deserialize(dataArmy);
 		this.armies.push(army);
 	}
 };
-
-return m;
-}(PETRA);
