@@ -1,6 +1,3 @@
-var PETRA = function(m)
-{
-
 /**
  * Manage the garrisonHolders
  * When a unit is ordered to garrison, it must be done through this.garrison() function so that
@@ -9,26 +6,43 @@ var PETRA = function(m)
  * Futhermore garrison units have a metadata garrisonType describing its reason (protection, transport, ...)
  */
 
-m.GarrisonManager = function(Config)
+PETRA.GarrisonManager = function(Config)
 {
 	this.Config = Config;
 	this.holders = new Map();
 	this.decayingStructures = new Map();
 };
 
-m.GarrisonManager.prototype.update = function(gameState, events)
+PETRA.GarrisonManager.prototype.update = function(gameState, events)
 {
 	// First check for possible upgrade of a structure
 	for (let evt of events.EntityRenamed)
 	{
 		for (let id of this.holders.keys())
 		{
-			if (id !== evt.entity)
+			if (id != evt.entity)
 				continue;
 			let data = this.holders.get(id);
-			this.holders.delete(id);
-			this.holders.set(evt.newentity, data);
+			let newHolder = gameState.getEntityById(evt.newentity);
+			if (newHolder && newHolder.isGarrisonHolder())
+			{
+				this.holders.delete(id);
+				this.holders.set(evt.newentity, data);
+			}
+			else
+			{
+				for (let entId of data.list)
+				{
+					let ent = gameState.getEntityById(entId);
+					if (!ent || ent.getMetadata(PlayerID, "garrisonHolder") != id)
+						continue;
+					this.leaveGarrison(ent);
+					ent.stopMoving();
+				}
+				this.holders.delete(id);
+			}
 		}
+
 		for (let id of this.decayingStructures.keys())
 		{
 			if (id !== evt.entity)
@@ -54,11 +68,10 @@ m.GarrisonManager.prototype.update = function(gameState, events)
 			for (let entId of list)
 			{
 				let ent = gameState.getEntityById(entId);
-				if (ent && ent.getMetadata(PlayerID, "garrisonHolder") == id)
-				{
-					this.leaveGarrison(ent);
-					ent.stopMoving();
-				}
+				if (!ent || ent.getMetadata(PlayerID, "garrisonHolder") != id)
+					continue;
+				this.leaveGarrison(ent);
+				ent.stopMoving();
 			}
 			this.holders.delete(id);
 			continue;
@@ -96,7 +109,7 @@ m.GarrisonManager.prototype.update = function(gameState, events)
 						API3.warn("Petra garrison error: unit " + ent.id() + " (" + ent.genericName() +
 							  ") is expected to garrison in " + id + " (" + holder.genericName() +
 							  "), but has no such garrison order " + uneval(ent.unitAIOrderData()));
-						m.dumpEntity(ent);
+						PETRA.dumpEntity(ent);
 					}
 					list.splice(j--, 1);
 				}
@@ -113,19 +126,26 @@ m.GarrisonManager.prototype.update = function(gameState, events)
 			let around = { "defenseStructure": false, "meleeSiege": false, "rangeSiege": false, "unit": false };
 			for (let ent of gameState.getEnemyEntities().values())
 			{
-				if (!ent.position())
+				if (ent.hasClass("Structure"))
+				{
+					if (!ent.attackRange("Ranged"))
+						continue;
+				}
+				else if (ent.hasClass("Unit"))
+				{
+					if (ent.owner() == 0 && (!ent.unitAIState() || ent.unitAIState().split(".")[1] != "COMBAT"))
+						continue;
+				}
+				else
 					continue;
-				if (ent.owner() === 0 && (!ent.unitAIState() || ent.unitAIState().split(".")[1] !== "COMBAT"))
+				if (!ent.position())
 					continue;
 				let dist = API3.SquareVectorDistance(ent.position(), holder.position());
 				if (dist > range*range)
 					continue;
 				if (ent.hasClass("Structure"))
-				{
-					if (ent.attackRange("Ranged"))	// TODO units on wall are not taken into account
-						around.defenseStructure = true;
-				}
-				else if (m.isSiegeUnit(ent))
+					around.defenseStructure = true;
+				else if (PETRA.isSiegeUnit(ent))
 				{
 					if (ent.attackTypes().indexOf("Melee") !== -1)
 						around.meleeSiege = true;
@@ -159,7 +179,7 @@ m.GarrisonManager.prototype.update = function(gameState, events)
 				}
 				list.splice(j--, 1);
 			}
-			if (this.numberOfGarrisonedUnits(holder) === 0)
+			if (this.numberOfGarrisonedSlots(holder) === 0)
 				this.holders.delete(id);
 			else
 				holder.setMetadata(PlayerID, "holderTimeUpdate", gameState.ai.elapsedTime);
@@ -173,13 +193,13 @@ m.GarrisonManager.prototype.update = function(gameState, events)
 		let ent = gameState.getEntityById(id);
 		if (!ent || ent.owner() !== PlayerID)
 			this.decayingStructures.delete(id);
-		else if (this.numberOfGarrisonedUnits(ent) < gmin)
-			gameState.ai.HQ.defenseManager.garrisonUnitsInside(gameState, ent, {"min": gmin, "type": "decay"});
+		else if (this.numberOfGarrisonedSlots(ent) < gmin)
+			gameState.ai.HQ.defenseManager.garrisonUnitsInside(gameState, ent, { "min": gmin, "type": "decay" });
 	}
 };
 
 /** TODO should add the units garrisoned inside garrisoned units */
-m.GarrisonManager.prototype.numberOfGarrisonedUnits = function(holder)
+PETRA.GarrisonManager.prototype.numberOfGarrisonedUnits = function(holder)
 {
 	if (!this.holders.has(holder.id()))
 		return holder.garrisoned().length;
@@ -187,7 +207,16 @@ m.GarrisonManager.prototype.numberOfGarrisonedUnits = function(holder)
 	return holder.garrisoned().length + this.holders.get(holder.id()).list.length;
 };
 
-m.GarrisonManager.prototype.allowMelee = function(holder)
+/** TODO should add the units garrisoned inside garrisoned units */
+PETRA.GarrisonManager.prototype.numberOfGarrisonedSlots = function(holder)
+{
+	if (!this.holders.has(holder.id()))
+		return holder.garrisonedSlots();
+
+	return holder.garrisonedSlots() + this.holders.get(holder.id()).list.length;
+};
+
+PETRA.GarrisonManager.prototype.allowMelee = function(holder)
 {
 	if (!this.holders.has(holder.id()))
 		return undefined;
@@ -196,9 +225,9 @@ m.GarrisonManager.prototype.allowMelee = function(holder)
 };
 
 /** This is just a pre-garrison state, while the entity walk to the garrison holder */
-m.GarrisonManager.prototype.garrison = function(gameState, ent, holder, type)
+PETRA.GarrisonManager.prototype.garrison = function(gameState, ent, holder, type)
 {
-	if (this.numberOfGarrisonedUnits(holder) >= holder.garrisonMax() || !ent.canGarrison())
+	if (this.numberOfGarrisonedSlots(holder) >= holder.garrisonMax() || !ent.canGarrison())
 		return;
 
 	this.registerHolder(gameState, holder);
@@ -227,7 +256,7 @@ m.GarrisonManager.prototype.garrison = function(gameState, ent, holder, type)
  This function is for internal use inside garrisonManager. From outside, you should also update
  the holder and then using cancelGarrison should be the preferred solution
  */
-m.GarrisonManager.prototype.leaveGarrison = function(ent)
+PETRA.GarrisonManager.prototype.leaveGarrison = function(ent)
 {
 	ent.setMetadata(PlayerID, "subrole", undefined);
 	if (ent.getMetadata(PlayerID, "plan") === -2)
@@ -238,7 +267,7 @@ m.GarrisonManager.prototype.leaveGarrison = function(ent)
 };
 
 /** Cancel a pre-garrison state */
-m.GarrisonManager.prototype.cancelGarrison = function(ent)
+PETRA.GarrisonManager.prototype.cancelGarrison = function(ent)
 {
 	ent.stopMoving();
 	this.leaveGarrison(ent);
@@ -251,7 +280,7 @@ m.GarrisonManager.prototype.cancelGarrison = function(ent)
 		list.splice(index, 1);
 };
 
-m.GarrisonManager.prototype.keepGarrisoned = function(ent, holder, around)
+PETRA.GarrisonManager.prototype.keepGarrisoned = function(ent, holder, around)
 {
 	switch (ent.getMetadata(PlayerID, "garrisonType"))
 	{
@@ -262,7 +291,10 @@ m.GarrisonManager.prototype.keepGarrisoned = function(ent, holder, around)
 	case 'protection':	// hurt unit for healing or infantry for defense
 		if (holder.buffHeal() && ent.isHealable() && ent.healthLevel() < this.Config.garrisonHealthLevel.high)
 			return true;
-		if (MatchesClassList(ent.classes(), holder.getGarrisonArrowClasses()))
+		let capture = ent.capturePoints();
+		if (capture && capture[PlayerID] / capture.reduce((a, b) => a + b) < 0.8)
+			return true;
+		if (ent.hasClasses(holder.getGarrisonArrowClasses()))
 		{
 			if (around.unit || around.defenseStructure)
 				return true;
@@ -273,8 +305,8 @@ m.GarrisonManager.prototype.keepGarrisoned = function(ent, holder, around)
 		if (ent.attackTypes() && ent.attackTypes().indexOf("Melee") !== -1)
 			return false;
 		if (around.unit)
-			return ent.hasClass("Support") || m.isSiegeUnit(ent);	// only ranged siege here and below as melee siege already released above
-		if (m.isSiegeUnit(ent))
+			return ent.hasClass("Support") || PETRA.isSiegeUnit(ent);	// only ranged siege here and below as melee siege already released above
+		if (PETRA.isSiegeUnit(ent))
 			return around.meleeSiege;
 		return holder.buffHeal() && ent.needsHeal();
 	case 'decay':
@@ -298,7 +330,7 @@ m.GarrisonManager.prototype.keepGarrisoned = function(ent, holder, around)
 };
 
 /** Add this holder in the list managed by the garrisonManager */
-m.GarrisonManager.prototype.registerHolder = function(gameState, holder)
+PETRA.GarrisonManager.prototype.registerHolder = function(gameState, holder)
 {
 	if (this.holders.has(holder.id()))    // already registered
 		return;
@@ -311,12 +343,12 @@ m.GarrisonManager.prototype.registerHolder = function(gameState, holder)
  * do it only for structures useful for defense, except if we are expanding (justCaptured=true)
  * in which case we also do it for structures useful for unit trainings (TODO only Barracks are done)
  */
-m.GarrisonManager.prototype.addDecayingStructure = function(gameState, entId, justCaptured)
+PETRA.GarrisonManager.prototype.addDecayingStructure = function(gameState, entId, justCaptured)
 {
 	if (this.decayingStructures.has(entId))
 		return true;
 	let ent = gameState.getEntityById(entId);
-	if (!ent || (!(ent.hasClass("Barracks") && justCaptured) && !ent.hasDefensiveFire()))
+	if (!ent || !(ent.hasClass("Barracks") && justCaptured) && !ent.hasDefensiveFire())
 		return false;
 	if (!ent.territoryDecayRate() || !ent.garrisonRegenRate())
 		return false;
@@ -325,23 +357,20 @@ m.GarrisonManager.prototype.addDecayingStructure = function(gameState, entId, ju
 	return true;
 };
 
-m.GarrisonManager.prototype.removeDecayingStructure = function(entId)
+PETRA.GarrisonManager.prototype.removeDecayingStructure = function(entId)
 {
 	if (!this.decayingStructures.has(entId))
 		return;
 	this.decayingStructures.delete(entId);
 };
 
-m.GarrisonManager.prototype.Serialize = function()
+PETRA.GarrisonManager.prototype.Serialize = function()
 {
 	return { "holders": this.holders, "decayingStructures": this.decayingStructures };
 };
 
-m.GarrisonManager.prototype.Deserialize = function(data)
+PETRA.GarrisonManager.prototype.Deserialize = function(data)
 {
 	for (let key in data)
 		this[key] = data[key];
 };
-
-return m;
-}(PETRA);
