@@ -1,14 +1,11 @@
-var PETRA = function(m)
-{
-
 /**
  * Defines a construction plan, ie a building.
  * We'll try to fing a good position if non has been provided
  */
 
-m.ConstructionPlan = function(gameState, type, metadata, position)
+PETRA.ConstructionPlan = function(gameState, type, metadata, position)
 {
-	if (!m.QueuePlan.call(this, gameState, type, metadata))
+	if (!PETRA.QueuePlan.call(this, gameState, type, metadata))
 		return false;
 
 	this.position = position ? position : 0;
@@ -18,9 +15,9 @@ m.ConstructionPlan = function(gameState, type, metadata, position)
 	return true;
 };
 
-m.ConstructionPlan.prototype = Object.create(m.QueuePlan.prototype);
+PETRA.ConstructionPlan.prototype = Object.create(PETRA.QueuePlan.prototype);
 
-m.ConstructionPlan.prototype.canStart = function(gameState)
+PETRA.ConstructionPlan.prototype.canStart = function(gameState)
 {
 	if (gameState.ai.HQ.turnCache.buildingBuilt)   // do not start another building if already one this turn
 		return false;
@@ -34,7 +31,7 @@ m.ConstructionPlan.prototype.canStart = function(gameState)
 	return gameState.ai.HQ.buildManager.hasBuilder(this.type);
 };
 
-m.ConstructionPlan.prototype.start = function(gameState)
+PETRA.ConstructionPlan.prototype.start = function(gameState)
 {
 	Engine.ProfileStart("Building construction start");
 
@@ -57,10 +54,10 @@ m.ConstructionPlan.prototype.start = function(gameState)
 		return;
 	}
 
-	if (this.metadata && this.metadata.expectedGain && (!this.template.hasClass("BarterMarket") ||
-	    gameState.getOwnEntitiesByClass("BarterMarket", true).hasEntities()))
+	if (this.metadata && this.metadata.expectedGain && (!this.template.hasClass("Market") ||
+	    gameState.getOwnEntitiesByClass("Market", true).hasEntities()))
 	{
-		// Check if this market is still worth building (others may have been built making it useless)
+		// Check if this Market is still worth building (others may have been built making it useless).
 		let tradeManager = gameState.ai.HQ.tradeManager;
 		tradeManager.checkRoutes(gameState);
 		if (!tradeManager.isNewMarketWorth(this.metadata.expectedGain))
@@ -109,7 +106,7 @@ m.ConstructionPlan.prototype.start = function(gameState)
 		gameState.ai.HQ.navalManager.createTransportIfNeeded(gameState, this.metadata.proximity, [pos.x, pos.z], this.metadata.access);
 };
 
-m.ConstructionPlan.prototype.findGoodPosition = function(gameState)
+PETRA.ConstructionPlan.prototype.findGoodPosition = function(gameState)
 {
 	let template = this.template;
 
@@ -122,7 +119,7 @@ m.ConstructionPlan.prototype.findGoodPosition = function(gameState)
 		// recompute the best dropsite location in case some conditions have changed
 		let base = HQ.getBaseByID(this.metadata.base);
 		let type = this.metadata.type ? this.metadata.type : "wood";
-		let newpos = base.findBestDropsiteLocation(gameState, type);
+		const newpos = base.findBestDropsiteLocation(gameState, type, template._templateName);
 		if (newpos && newpos.quality > 0)
 		{
 			let pos = newpos.pos;
@@ -146,7 +143,7 @@ m.ConstructionPlan.prototype.findGoodPosition = function(gameState)
 			if (pos)
 				return { "x": pos[0], "z": pos[1], "angle": 3*Math.PI/4, "base": 0 };
 			// No possible location, try to build instead a dock in a not-enemy island
-			let templateName = gameState.applyCiv("structures/{civ}_dock");
+			let templateName = gameState.applyCiv("structures/{civ}/dock");
 			if (gameState.ai.HQ.canBuild(gameState, templateName) && !gameState.isTemplateDisabled(templateName))
 			{
 				template = gameState.getTemplate(templateName);
@@ -155,7 +152,7 @@ m.ConstructionPlan.prototype.findGoodPosition = function(gameState)
 			}
 			return false;
 		}
-		else if (template.hasClass("DefenseTower") || template.hasClass("Fortress") || template.hasClass("ArmyCamp"))
+		else if (template.hasClasses(["Tower", "Fortress", "ArmyCamp"]))
 		{
 			let pos = HQ.findDefensiveLocation(gameState, template);
 			if (pos)
@@ -164,7 +161,7 @@ m.ConstructionPlan.prototype.findGoodPosition = function(gameState)
 			if (!template.hasClass("Fortress") || gameState.getOwnEntitiesByClass("Fortress", true).hasEntities())
 				return false;
 		}
-		else if (template.hasClass("Market"))	// Docks (i.e. NavalMarket) are done before
+		else if (template.hasClass("Market")) // Docks are done before.
 		{
 			let pos = HQ.findMarketLocation(gameState, template);
 			if (pos && pos[2] > 0)
@@ -217,33 +214,33 @@ m.ConstructionPlan.prototype.findGoodPosition = function(gameState)
 				let x = Math.round(pos[0] / cellSize);
 				let z = Math.round(pos[1] / cellSize);
 
-				let struct = m.getBuiltEntity(gameState, ent);
+				let struct = PETRA.getBuiltEntity(gameState, ent);
 				if (struct.resourceDropsiteTypes() && struct.resourceDropsiteTypes().indexOf("food") != -1)
 				{
-					if (template.hasClass("Field") || template.hasClass("Corral"))
-						placement.addInfluence(x, z, 80/cellSize, 50);
+					if (template.hasClasses(["Field", "Corral"]))
+						placement.addInfluence(x, z, 80 / cellSize, 50);
 					else // If this is not a field add a negative influence because we want to leave this area for fields
-						placement.addInfluence(x, z, 80/cellSize, -20);
+						placement.addInfluence(x, z, 80 / cellSize, -20);
 				}
 				else if (template.hasClass("House"))
 				{
 					if (ent.hasClass("House"))
 					{
-						placement.addInfluence(x, z, 60/cellSize, 40);    // houses are close to other houses
+						placement.addInfluence(x, z, 60 / cellSize, 40);    // houses are close to other houses
 						alreadyHasHouses = true;
 					}
-					else if (!ent.hasClass("StoneWall") || ent.hasClass("Gates"))
-						placement.addInfluence(x, z, 60/cellSize, -40);   // and further away from other stuffs
+					else if (ent.hasClasses(["Gate", "!Wall"]))
+						placement.addInfluence(x, z, 60 / cellSize, -40);   // and further away from other stuffs
 				}
-				else if (template.hasClass("Farmstead") && (!ent.hasClass("Field") && !ent.hasClass("Corral") &&
-					(!ent.hasClass("StoneWall") || ent.hasClass("Gates"))))
-					placement.addInfluence(x, z, 100/cellSize, -25);       // move farmsteads away to make room (StoneWall test needed for iber)
+				else if (template.hasClass("Farmstead") && !ent.hasClasses(["Field", "Corral"]) &&
+					ent.hasClasses(["Gate", "!Wall"]))
+					placement.addInfluence(x, z, 100 / cellSize, -25);       // move farmsteads away to make room (Wall test needed for iber)
 				else if (template.hasClass("GarrisonFortress") && ent.hasClass("House"))
-					placement.addInfluence(x, z, 120/cellSize, -50);
+					placement.addInfluence(x, z, 120 / cellSize, -50);
 				else if (template.hasClass("Military"))
-					placement.addInfluence(x, z, 40/cellSize, -40);
+					placement.addInfluence(x, z, 40 / cellSize, -40);
 				else if (template.genericName() == "Rotary Mill" && ent.hasClass("Field"))
-					placement.addInfluence(x, z, 60/cellSize, 40);
+					placement.addInfluence(x, z, 60 / cellSize, 40);
 			});
 		}
 		if (template.hasClass("Farmstead"))
@@ -251,7 +248,7 @@ m.ConstructionPlan.prototype.findGoodPosition = function(gameState)
 			for (let j = 0; j < placement.map.length; ++j)
 			{
 				let value = placement.map[j] - gameState.sharedScript.resourceMaps.wood.map[j]/3;
-				if (HQ.borderMap.map[j] & m.fullBorder_Mask)
+				if (HQ.borderMap.map[j] & PETRA.fullBorder_Mask)
 					value /= 2;	// we need space around farmstead, so disfavor map border
 				placement.set(j, value);
 			}
@@ -259,8 +256,8 @@ m.ConstructionPlan.prototype.findGoodPosition = function(gameState)
 	}
 
 	// Requires to be inside our territory, and inside our base territory if required
-	// and if our first market, put it on border if possible to maximize distance with next market
-	let favorBorder = template.hasClass("BarterMarket");
+	// and if our first market, put it on border if possible to maximize distance with next Market.
+	let favorBorder = template.hasClass("Market");
 	let disfavorBorder = gameState.currentPhase() > 1 && !template.hasDefensiveFire();
 	let favoredBase = this.metadata && (this.metadata.favoredBase ||
 		         (this.metadata.militaryBase ? HQ.findBestBaseForMilitary(gameState) : undefined));
@@ -273,9 +270,9 @@ m.ConstructionPlan.prototype.findGoodPosition = function(gameState)
 				placement.map[j] = 0;
 			else if (placement.map[j] > 0)
 			{
-				if (favorBorder && HQ.borderMap.map[j] & m.border_Mask)
+				if (favorBorder && HQ.borderMap.map[j] & PETRA.border_Mask)
 					placement.set(j, placement.map[j] + 50);
-				else if (disfavorBorder && !(HQ.borderMap.map[j] & m.fullBorder_Mask))
+				else if (disfavorBorder && !(HQ.borderMap.map[j] & PETRA.fullBorder_Mask))
 					placement.set(j, placement.map[j] + 10);
 
 				let x = (j % placement.width + 0.5) * cellSize;
@@ -293,9 +290,9 @@ m.ConstructionPlan.prototype.findGoodPosition = function(gameState)
 				placement.map[j] = 0;
 			else if (placement.map[j] > 0)
 			{
-				if (favorBorder && HQ.borderMap.map[j] & m.border_Mask)
+				if (favorBorder && HQ.borderMap.map[j] & PETRA.border_Mask)
 					placement.set(j, placement.map[j] + 50);
-				else if (disfavorBorder && !(HQ.borderMap.map[j] & m.fullBorder_Mask))
+				else if (disfavorBorder && !(HQ.borderMap.map[j] & PETRA.fullBorder_Mask))
 					placement.set(j, placement.map[j] + 10);
 
 				let x = (j % placement.width + 0.5) * cellSize;
@@ -314,15 +311,14 @@ m.ConstructionPlan.prototype.findGoodPosition = function(gameState)
 	// note: not for houses and dropsites who ought to be closer to either each other or a resource.
 	// also not for fields who can be stacked quite a bit
 
-	let obstructions = m.createObstructionMap(gameState, 0, template);
+	let obstructions = PETRA.createObstructionMap(gameState, 0, template);
 	// obstructions.dumpIm(template.buildPlacementType() + "_obstructions.png");
 
 	let radius = 0;
-	if (template.hasClass("Fortress") || template.hasClass("Workshop") ||
-		this.type == gameState.applyCiv("structures/{civ}_elephant_stables"))
+	if (template.hasClasses(["Fortress", "Arsenal"]) ||
+		this.type == gameState.applyCiv("structures/{civ}/elephant_stable"))
 		radius = Math.floor((template.obstructionRadius().max + 8) / obstructions.cellSize);
-	else if (template.resourceDropsiteTypes() === undefined && !template.hasClass("House") &&
-	         !template.hasClass("Field") && !template.hasClass("BarterMarket"))
+	else if (template.resourceDropsiteTypes() === undefined && !template.hasClasses(["House", "Field", "Market"]))
 		radius = Math.ceil((template.obstructionRadius().max + 4) / obstructions.cellSize);
 	else
 		radius = Math.ceil(template.obstructionRadius().max / obstructions.cellSize);
@@ -363,12 +359,12 @@ m.ConstructionPlan.prototype.findGoodPosition = function(gameState)
  * => we try not to be too far from our territory
  * In all cases, we add a bonus for nearby resources, and when a large extend of water in front ot it.
  */
-m.ConstructionPlan.prototype.findDockPosition = function(gameState)
+PETRA.ConstructionPlan.prototype.findDockPosition = function(gameState)
 {
 	let template = this.template;
 	let territoryMap = gameState.ai.HQ.territoryMap;
 
-	let obstructions = m.createObstructionMap(gameState, 0, template);
+	let obstructions = PETRA.createObstructionMap(gameState, 0, template);
 	// obstructions.dumpIm(template.buildPlacementType() + "_obstructions.png");
 
 	let bestIdx;
@@ -471,7 +467,7 @@ m.ConstructionPlan.prototype.findDockPosition = function(gameState)
 			let dockDist = 0;
 			for (let dock of docks.values())
 			{
-				if (m.getSeaAccess(gameState, dock) != navalPassMap[i])
+				if (PETRA.getSeaAccess(gameState, dock) != navalPassMap[i])
 					continue;
 				let dist = API3.SquareVectorDistance(pos, dock.position());
 				if (dist > dockDist)
@@ -487,7 +483,7 @@ m.ConstructionPlan.prototype.findDockPosition = function(gameState)
 		}
 
 		// Add a penalty if on the map border as ship movement will be difficult
-		if (gameState.ai.HQ.borderMap.map[j] & m.fullBorder_Mask)
+		if (gameState.ai.HQ.borderMap.map[j] & PETRA.fullBorder_Mask)
 			score += 20;
 
 		// Do a pre-selection, supposing we will have the best possible water
@@ -538,7 +534,7 @@ m.ConstructionPlan.prototype.findDockPosition = function(gameState)
 /**
  * Find a good island to build a dock.
  */
-m.ConstructionPlan.prototype.buildOverseaDock = function(gameState, template)
+PETRA.ConstructionPlan.prototype.buildOverseaDock = function(gameState, template)
 {
 	let docks = gameState.getOwnStructures().filter(API3.Filters.byClass("Dock"));
 	if (!docks.hasEntities())
@@ -558,7 +554,7 @@ m.ConstructionPlan.prototype.buildOverseaDock = function(gameState, template)
 		let keep = true;
 		for (let dock of docks.values())
 		{
-			if (m.getLandAccess(gameState, dock) != i)
+			if (PETRA.getLandAccess(gameState, dock) != i)
 				continue;
 			keep = false;
 			break;
@@ -568,7 +564,7 @@ m.ConstructionPlan.prototype.buildOverseaDock = function(gameState, template)
 		let sea;
 		for (let cc of ccEnts.values())
 		{
-			let ccAccess = m.getLandAccess(gameState, cc);
+			let ccAccess = PETRA.getLandAccess(gameState, cc);
 			if (ccAccess != i)
 			{
 				if (cc.owner() == PlayerID && !sea)
@@ -619,7 +615,7 @@ m.ConstructionPlan.prototype.buildOverseaDock = function(gameState, template)
 };
 
 /** Algorithm taken from the function GetDockAngle in simulation/helpers/Commands.js */
-m.ConstructionPlan.prototype.getDockAngle = function(gameState, x, z, size)
+PETRA.ConstructionPlan.prototype.getDockAngle = function(gameState, x, z, size)
 {
 	let pos = gameState.ai.accessibility.gamePosToMapPos([x, z]);
 	let k = pos[0] + pos[1]*gameState.ai.accessibility.width;
@@ -681,7 +677,7 @@ m.ConstructionPlan.prototype.getDockAngle = function(gameState, x, z, size)
  * to determine the special dock requirements
  * returns {"land": land index for this dock, "water": amount of water around this spot}
  */
-m.ConstructionPlan.prototype.checkDockPlacement = function(gameState, x, z, halfDepth, halfWidth, angle)
+PETRA.ConstructionPlan.prototype.checkDockPlacement = function(gameState, x, z, halfDepth, halfWidth, angle)
 {
 	let sz = halfDepth * Math.sin(angle);
 	let cz = halfDepth * Math.cos(angle);
@@ -744,9 +740,9 @@ m.ConstructionPlan.prototype.checkDockPlacement = function(gameState, x, z, half
  * if wantedSea is given, this tile should be inside this sea
  */
 const around = [[ 1.0, 0.0], [ 0.87, 0.50], [ 0.50, 0.87], [ 0.0, 1.0], [-0.50, 0.87], [-0.87, 0.50],
-	        [-1.0, 0.0], [-0.87,-0.50], [-0.50,-0.87], [ 0.0,-1.0], [ 0.50,-0.87], [ 0.87,-0.50]];
+	        [-1.0, 0.0], [-0.87, -0.50], [-0.50, -0.87], [ 0.0, -1.0], [ 0.50, -0.87], [ 0.87, -0.50]];
 
-m.ConstructionPlan.prototype.isDockLocation = function(gameState, j, dimension, wantedLand, wantedSea)
+PETRA.ConstructionPlan.prototype.isDockLocation = function(gameState, j, dimension, wantedLand, wantedSea)
 {
 	let width = gameState.ai.HQ.territoryMap.width;
 	let cellSize = gameState.ai.HQ.territoryMap.cellSize;
@@ -793,7 +789,7 @@ m.ConstructionPlan.prototype.isDockLocation = function(gameState, j, dimension,
  * return a measure of the proximity to our frontier (including our allies)
  * 0=inside, 1=less than 24m, 2= less than 48m, 3= less than 72m, 4=less than 96m, 5=above 96m
  */
-m.ConstructionPlan.prototype.getFrontierProximity = function(gameState, j)
+PETRA.ConstructionPlan.prototype.getFrontierProximity = function(gameState, j)
 {
 	let alliedVictory = gameState.getAlliedVictory();
 	let territoryMap = gameState.ai.HQ.territoryMap;
@@ -817,7 +813,7 @@ m.ConstructionPlan.prototype.getFrontierProximity = function(gameState, j)
 			let jz = iz + Math.round(i*step*a[1]);
 			if (jz < 0 || jz >= width)
 				continue;
-			if (borderMap.map[jx+width*jz] & m.outside_Mask)
+			if (borderMap.map[jx+width*jz] & PETRA.outside_Mask)
 				continue;
 			territoryOwner = territoryMap.getOwnerIndex(jx+width*jz);
 			if (alliedVictory && gameState.isPlayerAlly(territoryOwner) || territoryOwner == PlayerID)
@@ -837,7 +833,7 @@ m.ConstructionPlan.prototype.getFrontierProximity = function(gameState, j)
  * get the sum of the resources (except food) around, inside a given radius
  * resources have a weight (1 if dist=0 and 0 if dist=size) doubled for wood
  */
-m.ConstructionPlan.prototype.getResourcesAround = function(gameState, types, i, radius)
+PETRA.ConstructionPlan.prototype.getResourcesAround = function(gameState, types, i, radius)
 {
 	let resourceMaps = gameState.sharedScript.resourceMaps;
 	let w = resourceMaps.wood.width;
@@ -890,11 +886,12 @@ m.ConstructionPlan.prototype.getResourcesAround = function(gameState, types, i,
 	return nbcell ? total / nbcell : 0;
 };
 
-m.ConstructionPlan.prototype.isGo = function(gameState)
+PETRA.ConstructionPlan.prototype.isGo = function(gameState)
 {
 	if (this.goRequirement && this.goRequirement == "houseNeeded")
 	{
-		if (!gameState.ai.HQ.canBuild(gameState, "structures/{civ}_house"))
+		if (!gameState.ai.HQ.canBuild(gameState, "structures/{civ}/house") &&
+		    !gameState.ai.HQ.canBuild(gameState, "structures/{civ}/apartment"))
 			return false;
 		if (gameState.getPopulationMax() <= gameState.getPopulationLimit())
 			return false;
@@ -917,13 +914,13 @@ m.ConstructionPlan.prototype.isGo = function(gameState)
 	return true;
 };
 
-m.ConstructionPlan.prototype.onStart = function(gameState)
+PETRA.ConstructionPlan.prototype.onStart = function(gameState)
 {
 	if (this.queueToReset)
 		gameState.ai.queueManager.changePriority(this.queueToReset, gameState.ai.Config.priorities[this.queueToReset]);
 };
 
-m.ConstructionPlan.prototype.Serialize = function()
+PETRA.ConstructionPlan.prototype.Serialize = function()
 {
 	return {
 		"category": this.category,
@@ -938,7 +935,7 @@ m.ConstructionPlan.prototype.Serialize = function()
 	};
 };
 
-m.ConstructionPlan.prototype.Deserialize = function(gameState, data)
+PETRA.ConstructionPlan.prototype.Deserialize = function(gameState, data)
 {
 	for (let key in data)
 		this[key] = data[key];
@@ -946,6 +943,3 @@ m.ConstructionPlan.prototype.Deserialize = function(gameState, data)
 	this.cost = new API3.Resources();
 	this.cost.Deserialize(data.cost);
 };
-
-return m;
-}(PETRA);
