@@ -1,11 +1,7 @@
-var PETRA = function(m)
-{
-
 /**
  * Manage the research
  */
-
-m.ResearchManager = function(Config)
+PETRA.ResearchManager = function(Config)
 {
 	this.Config = Config;
 };
@@ -13,30 +9,33 @@ m.ResearchManager = function(Config)
 /**
  * Check if we can go to the next phase
  */
-m.ResearchManager.prototype.checkPhase = function(gameState, queues)
+PETRA.ResearchManager.prototype.checkPhase = function(gameState, queues)
 {
 	if (queues.majorTech.hasQueuedUnits())
 		return;
+	// Don't try to phase up if already trying to gather resources for a civil-centre or wonder
+	if (queues.civilCentre.hasQueuedUnits() || queues.wonder.hasQueuedUnits())
+		return;
 
-	let townPhase = gameState.townPhase();
-	let cityPhase = gameState.cityPhase();
+	let currentPhaseIndex = gameState.currentPhase();
+	let nextPhaseName = gameState.getPhaseName(currentPhaseIndex+1);
+	if (!nextPhaseName)
+		return;
 
-	if (gameState.canResearch(townPhase,true) && gameState.getPopulation() >= this.Config.Economy.popForTown - 10 &&
-		gameState.hasResearchers(townPhase, true))
+	let petraRequirements =
+		currentPhaseIndex == 1 && gameState.ai.HQ.getAccountedPopulation(gameState) >= this.Config.Economy.popPhase2 ||
+		currentPhaseIndex == 2 && gameState.ai.HQ.getAccountedWorkers(gameState) > this.Config.Economy.workPhase3 ||
+		currentPhaseIndex >= 3 && gameState.ai.HQ.getAccountedWorkers(gameState) > this.Config.Economy.workPhase4;
+	if (petraRequirements && gameState.hasResearchers(nextPhaseName, true))
 	{
-		let plan = new m.ResearchPlan(gameState, townPhase, true);
-		queues.majorTech.addPlan(plan);
-	}
-	else if (gameState.canResearch(cityPhase,true) && gameState.ai.elapsedTime > this.Config.Economy.cityPhase &&
-			gameState.getOwnEntitiesByRole("worker", true).length > this.Config.Economy.workForCity &&
-			gameState.hasResearchers(cityPhase, true) && !queues.civilCentre.hasQueuedUnits())
-	{
-		let plan = new m.ResearchPlan(gameState, cityPhase, true);
-		queues.majorTech.addPlan(plan);
+		gameState.ai.HQ.phasing = currentPhaseIndex + 1;
+		// Reset the queue priority in case it was changed during a previous phase update
+		gameState.ai.queueManager.changePriority("majorTech", gameState.ai.Config.priorities.majorTech);
+		queues.majorTech.addPlan(new PETRA.ResearchPlan(gameState, nextPhaseName, true));
 	}
 };
 
-m.ResearchManager.prototype.researchPopulationBonus = function(gameState, queues)
+PETRA.ResearchManager.prototype.researchPopulationBonus = function(gameState, queues)
 {
 	if (queues.minorTech.hasQueuedUnits())
 		return;
@@ -47,14 +46,14 @@ m.ResearchManager.prototype.researchPopulationBonus = function(gameState, queues
 		if (!tech[1]._template.modifications)
 			continue;
 		// TODO may-be loop on all modifs and check if the effect if positive ?
-		if (tech[1]._template.modifications[0].value !== "Cost/PopulationBonus")
+		if (tech[1]._template.modifications[0].value !== "Population/Bonus")
 			continue;
-		queues.minorTech.addPlan(new m.ResearchPlan(gameState, tech[0]));
+		queues.minorTech.addPlan(new PETRA.ResearchPlan(gameState, tech[0]));
 		break;
 	}
 };
 
-m.ResearchManager.prototype.researchTradeBonus = function(gameState, queues)
+PETRA.ResearchManager.prototype.researchTradeBonus = function(gameState, queues)
 {
 	if (queues.minorTech.hasQueuedUnits())
 		return;
@@ -70,19 +69,24 @@ m.ResearchManager.prototype.researchTradeBonus = function(gameState, queues)
 		if (tech[1]._template.modifications[0].value !== "UnitMotion/WalkSpeed" &&
                     tech[1]._template.modifications[0].value !== "Trader/GainMultiplier")
 			continue;
-		queues.minorTech.addPlan(new m.ResearchPlan(gameState, tech[0]));
+		queues.minorTech.addPlan(new PETRA.ResearchPlan(gameState, tech[0]));
 		break;
 	}
 };
 
 /** Techs to be searched for as soon as they are available */
-m.ResearchManager.prototype.researchWantedTechs = function(gameState, techs)
+PETRA.ResearchManager.prototype.researchWantedTechs = function(gameState, techs)
 {
 	let phase1 = gameState.currentPhase() === 1;
 	let available = phase1 ? gameState.ai.queueManager.getAvailableResources(gameState) : null;
 	let numWorkers = phase1 ? gameState.getOwnEntitiesByRole("worker", true).length : 0;
 	for (let tech of techs)
 	{
+		if (tech[0].indexOf("unlock_champion") == 0)
+			return { "name": tech[0], "increasePriority": true };
+		if (tech[0] == "traditional_army_sele" || tech[0] == "reformed_army_sele")
+			return { "name": pickRandom(["traditional_army_sele", "reformed_army_sele"]), "increasePriority": true };
+
 		if (!tech[1]._template.modifications)
 			continue;
 		let template = tech[1]._template;
@@ -115,7 +119,7 @@ m.ResearchManager.prototype.researchWantedTechs = function(gameState, techs)
 };
 
 /** Techs to be searched for as soon as they are available, but only after phase 2 */
-m.ResearchManager.prototype.researchPreferredTechs = function(gameState, techs)
+PETRA.ResearchManager.prototype.researchPreferredTechs = function(gameState, techs)
 {
 	let phase2 = gameState.currentPhase() === 2;
 	let available = phase2 ? gameState.ai.queueManager.getAvailableResources(gameState) : null;
@@ -125,7 +129,7 @@ m.ResearchManager.prototype.researchPreferredTechs = function(gameState, techs)
 		if (!tech[1]._template.modifications)
 			continue;
 		let template = tech[1]._template;
-	    	if (phase2)
+		if (phase2)
 		{
 			let cost = template.cost;
 			let costMax = 0;
@@ -151,7 +155,7 @@ m.ResearchManager.prototype.researchPreferredTechs = function(gameState, techs)
 	return null;
 };
 
-m.ResearchManager.prototype.update = function(gameState, queues)
+PETRA.ResearchManager.prototype.update = function(gameState, queues)
 {
 	if (queues.minorTech.hasQueuedUnits() || queues.majorTech.hasQueuedUnits())
 		return;
@@ -164,12 +168,12 @@ m.ResearchManager.prototype.update = function(gameState, queues)
 		if (techName.increasePriority)
 		{
 			gameState.ai.queueManager.changePriority("minorTech", 2*this.Config.priorities.minorTech);
-			let plan = new m.ResearchPlan(gameState, techName.name);
+			let plan = new PETRA.ResearchPlan(gameState, techName.name);
 			plan.queueToReset = "minorTech";
 			queues.minorTech.addPlan(plan);
 		}
 		else
-			queues.minorTech.addPlan(new m.ResearchPlan(gameState, techName.name));
+			queues.minorTech.addPlan(new PETRA.ResearchPlan(gameState, techName.name));
 		return;
 	}
 
@@ -182,12 +186,12 @@ m.ResearchManager.prototype.update = function(gameState, queues)
 		if (techName.increasePriority)
 		{
 			gameState.ai.queueManager.changePriority("minorTech", 2*this.Config.priorities.minorTech);
-			let plan = new m.ResearchPlan(gameState, techName.name);
+			let plan = new PETRA.ResearchPlan(gameState, techName.name);
 			plan.queueToReset = "minorTech";
 			queues.minorTech.addPlan(plan);
 		}
 		else
-			queues.minorTech.addPlan(new m.ResearchPlan(gameState, techName.name));
+			queues.minorTech.addPlan(new PETRA.ResearchPlan(gameState, techName.name));
 		return;
 	}
 
@@ -200,7 +204,7 @@ m.ResearchManager.prototype.update = function(gameState, queues)
 	{
 		let template = techs[i][1]._template;
 		if (template.affects && template.affects.length === 1 &&
-			(template.affects[0] === "Healer" || template.affects[0] === "Outpost" || template.affects[0] === "StoneWall"))
+			(template.affects[0] === "Healer" || template.affects[0] === "Outpost" || template.affects[0] === "Wall"))
 		{
 			techs.splice(i--, 1);
 			continue;
@@ -217,10 +221,10 @@ m.ResearchManager.prototype.update = function(gameState, queues)
 		return;
 
 	// randomly pick one. No worries about pairs in that case.
-	queues.minorTech.addPlan(new m.ResearchPlan(gameState, pickRandom(techs)[0]));
+	queues.minorTech.addPlan(new PETRA.ResearchPlan(gameState, pickRandom(techs)[0]));
 };
 
-m.ResearchManager.prototype.CostSum = function(cost)
+PETRA.ResearchManager.prototype.CostSum = function(cost)
 {
 	let costSum = 0;
 	for (let res in cost)
@@ -228,14 +232,11 @@ m.ResearchManager.prototype.CostSum = function(cost)
 	return costSum;
 };
 
-m.ResearchManager.prototype.Serialize = function()
+PETRA.ResearchManager.prototype.Serialize = function()
 {
 	return {};
 };
 
-m.ResearchManager.prototype.Deserialize = function(data)
+PETRA.ResearchManager.prototype.Deserialize = function(data)
 {
 };
-
-return m;
-}(PETRA);
