@@ -1,10 +1,9 @@
-var PETRA = function(m)
-{
 /**
- * determines the strategy to adopt when starting a new game, depending on the initial conditions
+ * Determines the strategy to adopt when starting a new game,
+ * depending on the initial conditions
  */
 
-m.HQ.prototype.gameAnalysis = function(gameState)
+PETRA.HQ.prototype.gameAnalysis = function(gameState)
 {
 	// Analysis of the terrain and the different access regions
 	if (!this.regionAnalysis(gameState))
@@ -20,7 +19,7 @@ m.HQ.prototype.gameAnalysis = function(gameState)
 	this.structureAnalysis(gameState);
 
 	// Let's get our initial situation here.
-	let nobase = new m.BaseManager(gameState, this.Config);
+	let nobase = new PETRA.BaseManager(gameState, this.Config);
 	nobase.init(gameState);
 	nobase.accessIndex = 0;
 	this.baseManagers.push(nobase);   // baseManagers[0] will deal with unit/structure without base
@@ -42,7 +41,7 @@ m.HQ.prototype.gameAnalysis = function(gameState)
 	this.canBuildUnits = true;
 	if (!gameState.getOwnStructures().filter(API3.Filters.byClass("CivCentre")).hasEntities())
 	{
-		let template = gameState.applyCiv("structures/{civ}_civil_centre");
+		let template = gameState.applyCiv("structures/{civ}/civil_centre");
 		if (!gameState.isTemplateAvailable(template) || !gameState.getTemplate(template).available(gameState))
 		{
 			if (this.Config.debug > 1)
@@ -62,12 +61,12 @@ m.HQ.prototype.gameAnalysis = function(gameState)
 /**
  * Assign the starting entities to the different bases
  */
-m.HQ.prototype.assignStartingEntities = function(gameState)
+PETRA.HQ.prototype.assignStartingEntities = function(gameState)
 {
 	for (let ent of gameState.getOwnEntities().values())
 	{
 		// do not affect merchant ship immediately to trade as they may-be useful for transport
-		if (ent.hasClass("Trader") && !ent.hasClass("Ship"))
+		if (ent.hasClasses(["Trader+!Ship"]))
 			this.tradeManager.assignTrader(ent);
 
 		let pos = ent.position();
@@ -113,7 +112,7 @@ m.HQ.prototype.assignStartingEntities = function(gameState)
 			if (ent.hasClass("Structure") && !ent.decaying() && ent.resourceDropsiteTypes())
 				bestbase = this.createBase(gameState, ent, "anchorless");
 			else
-				bestbase = m.getBestBase(gameState, ent) || this.baseManagers[0];
+				bestbase = PETRA.getBestBase(gameState, ent) || this.baseManagers[0];
 			bestbase.assignEntity(gameState, ent);
 		}
 		// now assign entities garrisoned inside this entity
@@ -137,7 +136,7 @@ m.HQ.prototype.assignStartingEntities = function(gameState)
  * determine the main land Index (or water index if none)
  * as well as the list of allowed (land andf water) regions
  */
-m.HQ.prototype.regionAnalysis = function(gameState)
+PETRA.HQ.prototype.regionAnalysis = function(gameState)
 {
 	let accessibility = gameState.ai.accessibility;
 	let landIndex;
@@ -231,7 +230,7 @@ m.HQ.prototype.regionAnalysis = function(gameState)
  * load units and buildings from the config files
  * TODO: change that to something dynamic
  */
-m.HQ.prototype.structureAnalysis = function(gameState)
+PETRA.HQ.prototype.structureAnalysis = function(gameState)
 {
 	let civref = gameState.playerData.civ;
 	let civ = civref in this.Config.buildings ? civref : 'default';
@@ -245,11 +244,11 @@ m.HQ.prototype.structureAnalysis = function(gameState)
  * build our first base
  * if not enough resource, try first to do a dock
  */
-m.HQ.prototype.buildFirstBase = function(gameState)
+PETRA.HQ.prototype.buildFirstBase = function(gameState)
 {
 	if (gameState.ai.queues.civilCentre.hasQueuedUnits())
 		return;
-	let templateName = gameState.applyCiv("structures/{civ}_civil_centre");
+	let templateName = gameState.applyCiv("structures/{civ}/civil_centre");
 	if (gameState.isTemplateDisabled(templateName))
 		return;
 	let template = gameState.getTemplate(templateName);
@@ -267,19 +266,19 @@ m.HQ.prototype.buildFirstBase = function(gameState)
 				continue;
 			// If we can get a treasure around, just do it
 			if (ent.isIdle())
-				m.gatherTreasure(gameState, ent);
+				PETRA.gatherTreasure(gameState, ent);
 			// Then count the resources from the treasures being collected
-			let supplyId = ent.getMetadata(PlayerID, "supply");
-			if (!supplyId)
+			let treasureId = ent.getMetadata(PlayerID, "treasure");
+			if (!treasureId)
 				continue;
-			let supply = gameState.getEntityById(supplyId);
-			if (!supply || supply.resourceSupplyType().generic != "treasure")
+			let treasure = gameState.getEntityById(treasureId);
+			if (!treasure)
 				continue;
-			let type = supply.resourceSupplyType().specific;
-			if (!(type in totalExpected))
-				continue;
-			totalExpected[type] += supply.resourceSupplyMax();
-			// If we can collect enough resources from these treasures, wait for them
+			let types = treasure.treasureResources();
+			for (let type in types)
+				if (type in totalExpected)
+					totalExpected[type] += types[type];
+			// If we can collect enough resources from these treasures, wait for them.
 			if (totalExpected.canAfford(new API3.Resources(template.cost())))
 				return;
 		}
@@ -289,7 +288,7 @@ m.HQ.prototype.buildFirstBase = function(gameState)
 		{
 			if (gameState.ai.queues.dock.hasQueuedUnits())
 				return;
-			templateName = gameState.applyCiv("structures/{civ}_dock");
+			templateName = gameState.applyCiv("structures/{civ}/dock");
 			if (gameState.isTemplateDisabled(templateName))
 				return;
 			template = gameState.getTemplate(templateName);
@@ -305,20 +304,20 @@ m.HQ.prototype.buildFirstBase = function(gameState)
 	let startingPoint = [];
 	for (let ent of gameState.getOwnUnits().values())
 	{
-		if (!ent.hasClass("Worker") && !(ent.hasClass("Support") && ent.hasClass("Elephant")))
+		if (!ent.hasClass("Worker"))
 			continue;
-		if (ent.hasClass("Cavalry"))
+		if (PETRA.isFastMoving(ent))
 			continue;
 		let pos = ent.position();
 		if (!pos)
 		{
-			let holder = m.getHolder(gameState, ent);
+			let holder = PETRA.getHolder(gameState, ent);
 			if (!holder || !holder.position())
 				continue;
 			pos = holder.position();
 		}
 		let gamepos = gameState.ai.accessibility.gamePosToMapPos(pos);
-		let index = gamepos[0] + gamepos[1]*gameState.ai.accessibility.width;
+		let index = gamepos[0] + gamepos[1] * gameState.ai.accessibility.width;
 		let land = gameState.ai.accessibility.landPassMap[index];
 		let sea = gameState.ai.accessibility.navalPassMap[index];
 		let found = false;
@@ -346,10 +345,10 @@ m.HQ.prototype.buildFirstBase = function(gameState)
 	if (goal == "dock")
 	{
 		let sea = startingPoint[imax].sea > 1 ? startingPoint[imax].sea : undefined;
-		gameState.ai.queues.dock.addPlan(new m.ConstructionPlan(gameState, "structures/{civ}_dock", { "sea": sea, "proximity": startingPoint[imax].pos }));
+		gameState.ai.queues.dock.addPlan(new PETRA.ConstructionPlan(gameState, "structures/{civ}/dock", { "sea": sea, "proximity": startingPoint[imax].pos }));
 	}
 	else
-		gameState.ai.queues.civilCentre.addPlan(new m.ConstructionPlan(gameState, "structures/{civ}_civil_centre", { "base": -1, "resource": "wood", "proximity": startingPoint[imax].pos }));
+		gameState.ai.queues.civilCentre.addPlan(new PETRA.ConstructionPlan(gameState, "structures/{civ}/civil_centre", { "base": -1, "resource": "wood", "proximity": startingPoint[imax].pos }));
 };
 
 /**
@@ -357,7 +356,7 @@ m.HQ.prototype.buildFirstBase = function(gameState)
  *   - if one of our allies has a cc, affect a small fraction of our army for his defense, the rest will attack
  *   - otherwise all units will attack
  */
-m.HQ.prototype.dispatchUnits = function(gameState)
+PETRA.HQ.prototype.dispatchUnits = function(gameState)
 {
 	let allycc = gameState.getExclusiveAllyEntities().filter(API3.Filters.byClass("CivCentre")).toEntityArray();
 	if (allycc.length)
@@ -369,40 +368,40 @@ m.HQ.prototype.dispatchUnits = function(gameState)
 		let num1 = Math.floor(num / 2);
 		let num2 = num1;
 		// first pass to affect ranged infantry
-		units.filter(API3.Filters.byClassesAnd(["Infantry", "Ranged"])).forEach(ent => {
+		units.filter(API3.Filters.byClasses(["Infantry+Ranged"])).forEach(ent => {
 			if (!num || !num1)
 				return;
 			if (ent.getMetadata(PlayerID, "allied"))
 				return;
-			let access = m.getLandAccess(gameState, ent);
+			let access = PETRA.getLandAccess(gameState, ent);
 			for (let cc of allycc)
 			{
-				if (!cc.position() || m.getLandAccess(gameState, cc) != access)
+				if (!cc.position() || PETRA.getLandAccess(gameState, cc) != access)
 					continue;
 				--num;
 				--num1;
 				ent.setMetadata(PlayerID, "allied", true);
 				let range = 1.5 * cc.footprintRadius();
-				ent.moveToRange(cc.position()[0], cc.position()[1], range, range);
+				ent.moveToRange(cc.position()[0], cc.position()[1], range, range + 5);
 				break;
 			}
 		});
 		// second pass to affect melee infantry
-		units.filter(API3.Filters.byClassesAnd(["Infantry", "Melee"])).forEach(ent => {
+		units.filter(API3.Filters.byClasses(["Infantry+Melee"])).forEach(ent => {
 			if (!num || !num2)
 				return;
 			if (ent.getMetadata(PlayerID, "allied"))
 				return;
-			let access = m.getLandAccess(gameState, ent);
+			let access = PETRA.getLandAccess(gameState, ent);
 			for (let cc of allycc)
 			{
-				if (!cc.position() || m.getLandAccess(gameState, cc) != access)
+				if (!cc.position() || PETRA.getLandAccess(gameState, cc) != access)
 					continue;
 				--num;
 				--num2;
 				ent.setMetadata(PlayerID, "allied", true);
 				let range = 1.5 * cc.footprintRadius();
-				ent.moveToRange(cc.position()[0], cc.position()[1], range, range);
+				ent.moveToRange(cc.position()[0], cc.position()[1], range, range + 5);
 				break;
 			}
 		});
@@ -412,16 +411,16 @@ m.HQ.prototype.dispatchUnits = function(gameState)
 				return;
 			if (ent.getMetadata(PlayerID, "allied"))
 				return;
-			let access = m.getLandAccess(gameState, ent);
+			let access = PETRA.getLandAccess(gameState, ent);
 			for (let cc of allycc)
 			{
-				if (!cc.position() || m.getLandAccess(gameState, cc) != access)
+				if (!cc.position() || PETRA.getLandAccess(gameState, cc) != access)
 					continue;
 				if (!ent.hasClass("Support"))
 					--num;
 				ent.setMetadata(PlayerID, "allied", true);
 				let range = 1.5 * cc.footprintRadius();
-				ent.moveToRange(cc.position()[0], cc.position()[1], range, range);
+				ent.moveToRange(cc.position()[0], cc.position()[1], range, range + 5);
 				break;
 			}
 		});
@@ -433,7 +432,7 @@ m.HQ.prototype.dispatchUnits = function(gameState)
  *   - if on a small island, favor fishing
  *   - count the available wood resource, and allow rushes only if enough (we should otherwise favor expansion)
  */
-m.HQ.prototype.configFirstBase = function(gameState)
+PETRA.HQ.prototype.configFirstBase = function(gameState)
 {
 	if (this.baseManagers.length < 2)
 		return;
@@ -476,7 +475,7 @@ m.HQ.prototype.configFirstBase = function(gameState)
 	else
 		this.maxFields = false;
 
-	// - count the available wood resource, and react accordingly
+	// - count the available food resource, and react accordingly
 	let startingFood = gameState.getResources().food;
 	let check = {};
 	for (let proxim of ["nearby", "medium", "faraway"])
@@ -546,11 +545,10 @@ m.HQ.prototype.configFirstBase = function(gameState)
 	}
 
 	// immediatly build a wood dropsite if possible.
-	let template = gameState.applyCiv("structures/{civ}_storehouse");
-	if (!gameState.getOwnEntitiesByClass("Storehouse", true).hasEntities() && this.canBuild(gameState, template))
+	if (!gameState.getOwnEntitiesByClass("DropsiteWood", true).hasEntities())
 	{
-		let newDP = this.baseManagers[1].findBestDropsiteLocation(gameState, "wood");
-		if (newDP.quality > 40)
+		const newDP = this.baseManagers[1].findBestDropsiteAndLocation(gameState, "wood");
+		if (newDP.quality > 40 && this.canBuild(gameState, newDP.templateName))
 		{
 			// if we start with enough workers, put our available resources in this first dropsite
 			// same thing if our pop exceed the allowed one, as we will need several houses
@@ -558,21 +556,17 @@ m.HQ.prototype.configFirstBase = function(gameState)
 			if (numWorkers > 12 && newDP.quality > 60 ||
 				gameState.getPopulation() > gameState.getPopulationLimit() + 20)
 			{
-				let cost = new API3.Resources(gameState.getTemplate(template).cost());
+				const cost = new API3.Resources(gameState.getTemplate(newDP.templateName).cost());
 				gameState.ai.queueManager.setAccounts(gameState, cost, "dropsites");
 			}
-			gameState.ai.queues.dropsites.addPlan(new m.ConstructionPlan(gameState, template, { "base": this.baseManagers[1].ID }, newDP.pos));
+			gameState.ai.queues.dropsites.addPlan(new PETRA.ConstructionPlan(gameState, newDP.templateName, { "base": this.baseManagers[1].ID }, newDP.pos));
 		}
 	}
 	// and build immediately a corral if needed
 	if (this.needCorral)
 	{
-		template = gameState.applyCiv("structures/{civ}_corral");
+		const template = gameState.applyCiv("structures/{civ}/corral");
 		if (!gameState.getOwnEntitiesByClass("Corral", true).hasEntities() && this.canBuild(gameState, template))
-			gameState.ai.queues.corral.addPlan(new m.ConstructionPlan(gameState, template, { "base": this.baseManagers[1].ID }));
+			gameState.ai.queues.corral.addPlan(new PETRA.ConstructionPlan(gameState, template, { "base": this.baseManagers[1].ID }));
 	}
 };
-
-return m;
-
-}(PETRA);
