@@ -1,18 +1,14 @@
-var PETRA = function(m)
-{
-
 /**
  * This class makes a worker do as instructed by the economy manager
  */
-
-m.Worker = function(base)
+PETRA.Worker = function(base)
 {
 	this.ent = undefined;
 	this.base = base;
 	this.baseID = base.ID;
 };
 
-m.Worker.prototype.update = function(gameState, ent)
+PETRA.Worker.prototype.update = function(gameState, ent)
 {
 	if (!ent.position() || ent.getMetadata(PlayerID, "plan") == -2 || ent.getMetadata(PlayerID, "plan") == -3)
 		return;
@@ -40,9 +36,9 @@ m.Worker.prototype.update = function(gameState, ent)
 				let gatherType = ent.getMetadata(PlayerID, "gather-type") || "food";
 				for (let structure of gameState.getOwnStructures().values())
 				{
-					if (m.getLandAccess(gameState, structure) != plan.endIndex)
+					if (PETRA.getLandAccess(gameState, structure) != plan.endIndex)
 						continue;
-					let resourceDropsiteTypes = m.getBuiltEntity(gameState, structure).resourceDropsiteTypes();
+					let resourceDropsiteTypes = PETRA.getBuiltEntity(gameState, structure).resourceDropsiteTypes();
 					if (!resourceDropsiteTypes || resourceDropsiteTypes.indexOf(gatherType) == -1)
 						continue;
 					hasDropsite = true;
@@ -52,7 +48,7 @@ m.Worker.prototype.update = function(gameState, ent)
 				{
 					for (let unit of gameState.getOwnUnits().filter(API3.Filters.byClass("Support")).values())
 					{
-						if (!unit.position() || m.getLandAccess(gameState, unit) != plan.endIndex)
+						if (!unit.position() || PETRA.getLandAccess(gameState, unit) != plan.endIndex)
 							continue;
 						let resourceDropsiteTypes = unit.resourceDropsiteTypes();
 						if (!resourceDropsiteTypes || resourceDropsiteTypes.indexOf(gatherType) == -1)
@@ -69,7 +65,7 @@ m.Worker.prototype.update = function(gameState, ent)
 			return;
 	}
 
-	this.entAccess = m.getLandAccess(gameState, ent);
+	this.entAccess = PETRA.getLandAccess(gameState, ent);
 	// base 0 for unassigned entities has no accessIndex, so take the one from the entity
 	if (this.baseID == gameState.ai.HQ.baseManagers[0].ID)
 		this.baseAccess = this.entAccess;
@@ -116,8 +112,8 @@ m.Worker.prototype.update = function(gameState, ent)
 					}
 					else if (!gameState.isPlayerAlly(territoryOwner))
 					{
-						let distanceSquare = ent.hasClass("Cavalry") ? 90000 : 30000;
-						let targetAccess = m.getLandAccess(gameState, target);
+						let distanceSquare = PETRA.isFastMoving(ent) ? 90000 : 30000;
+						let targetAccess = PETRA.getLandAccess(gameState, target);
 						let foodDropsites = gameState.playerData.hasSharedDropsites ?
 						                    gameState.getAnyDropsites("food") : gameState.getOwnDropsites("food");
 						let hasFoodDropsiteWithinDistance = false;
@@ -129,7 +125,7 @@ m.Worker.prototype.update = function(gameState, ent)
 							// owner != PlayerID can only happen when hasSharedDropsites == true, so no need to test it again
 							if (owner != PlayerID && (!dropsite.isSharedDropsite() || !gameState.isPlayerMutualAlly(owner)))
 								continue;
-							if (targetAccess != m.getLandAccess(gameState, dropsite))
+							if (targetAccess != PETRA.getLandAccess(gameState, dropsite))
 								continue;
 							if (API3.SquareVectorDistance(target.position(), dropsite.position()) < distanceSquare)
 							{
@@ -169,7 +165,7 @@ m.Worker.prototype.update = function(gameState, ent)
 			if (orderData && orderData.target)
 			{
 				let target = gameState.getEntityById(orderData.target);
-				if (target && (!target.position() || m.getLandAccess(gameState, target) != this.entAccess))
+				if (target && (!target.position() || PETRA.getLandAccess(gameState, target) != this.entAccess))
 				{
 					if (this.retryWorking(gameState, subrole))
 						return;
@@ -187,7 +183,7 @@ m.Worker.prototype.update = function(gameState, ent)
 				// and UnitAI sent it fight back with allowCapture=true
 				let target = gameState.getEntityById(orderData.target);
 				if (target && target.owner() > 0 && !gameState.isPlayerAlly(target.owner()))
-					ent.attack(orderData.target, m.allowCapture(gameState, ent, target));
+					ent.attack(orderData.target, PETRA.allowCapture(gameState, ent, target));
 			}
 		}
 		return;
@@ -209,7 +205,7 @@ m.Worker.prototype.update = function(gameState, ent)
 			{
 				this.startGathering(gameState);
 			}
-			else if (!m.returnResources(gameState, ent))     // try to deposit resources
+			else if (!PETRA.returnResources(gameState, ent))     // try to deposit resources
 			{
 				// no dropsite, abandon old resources and start gathering new ones
 				this.startGathering(gameState);
@@ -223,8 +219,7 @@ m.Worker.prototype.update = function(gameState, ent)
 			{
 				let supplyId = ent.unitAIOrderData()[0].target;
 				let supply = gameState.getEntityById(supplyId);
-				if (supply && !supply.hasClass("Field") && !supply.hasClass("Animal") &&
-					supply.resourceSupplyType().generic != "treasure" &&
+				if (supply && !supply.hasClasses(["Field", "Animal"]) &&
 					supplyId != ent.getMetadata(PlayerID, "supply"))
 				{
 					let nbGatherers = supply.resourceSupplyNumGatherers() + gameState.ai.HQ.GetTCGatherer(supplyId);
@@ -258,37 +253,37 @@ m.Worker.prototype.update = function(gameState, ent)
 					}
 				}
 			}
-		}
-		else if (unitAIState == "INDIVIDUAL.RETURNRESOURCE.APPROACHING")
-		{
-			if (gameState.ai.playedTurn % 10 == 0)
+			if (unitAIState == "INDIVIDUAL.GATHER.RETURNINGRESOURCE.APPROACHING")
 			{
-				// Check from time to time that UnitAI does not send us to an inaccessible dropsite
-				let dropsite = gameState.getEntityById(ent.unitAIOrderData()[0].target);
-				if (dropsite && dropsite.position() && this.entAccess != m.getLandAccess(gameState, dropsite))
-					m.returnResources(gameState, this.ent);
-			}
-
-			// If gathering a sparse resource, we may have been sent to a faraway resource if the one nearby was full.
-			// Let's check if it is still the case. If so, we reset its metadata supplyId so that the unit will be
-			// reordered to gather after having returned the resources (when comparing its supplyId with the UnitAI one).
-			let gatherType = ent.getMetadata(PlayerID, "gather-type");
-			let influenceGroup = Resources.GetResource(gatherType).aiAnalysisInfluenceGroup;
-			if (influenceGroup && influenceGroup == "sparse")
-			{
-				let supplyId = ent.getMetadata(PlayerID, "supply");
-				if (supplyId)
+				if (gameState.ai.playedTurn % 10 == 0)
 				{
-					let nearby = this.base.dropsiteSupplies[gatherType].nearby;
-					if (!nearby.some(sup => sup.id == supplyId))
+					// Check from time to time that UnitAI does not send us to an inaccessible dropsite
+					let dropsite = gameState.getEntityById(ent.unitAIOrderData()[0].target);
+					if (dropsite && dropsite.position() && this.entAccess != PETRA.getLandAccess(gameState, dropsite))
+						PETRA.returnResources(gameState, this.ent);
+				}
+
+				// If gathering a sparse resource, we may have been sent to a faraway resource if the one nearby was full.
+				// Let's check if it is still the case. If so, we reset its metadata supplyId so that the unit will be
+				// reordered to gather after having returned the resources (when comparing its supplyId with the UnitAI one).
+				let gatherType = ent.getMetadata(PlayerID, "gather-type");
+				let influenceGroup = Resources.GetResource(gatherType).aiAnalysisInfluenceGroup;
+				if (influenceGroup && influenceGroup == "sparse")
+				{
+					let supplyId = ent.getMetadata(PlayerID, "supply");
+					if (supplyId)
 					{
-						if (nearby.length)
-							ent.setMetadata(PlayerID, "supply", undefined);
-						else
+						let nearby = this.base.dropsiteSupplies[gatherType].nearby;
+						if (!nearby.some(sup => sup.id == supplyId))
 						{
-							let medium = this.base.dropsiteSupplies[gatherType].medium;
-							if (!medium.some(sup => sup.id == supplyId) && medium.length)
+							if (nearby.length)
 								ent.setMetadata(PlayerID, "supply", undefined);
+							else
+							{
+								let medium = this.base.dropsiteSupplies[gatherType].medium;
+								if (!medium.some(sup => sup.id == supplyId) && medium.length)
+									ent.setMetadata(PlayerID, "supply", undefined);
+							}
 						}
 					}
 				}
@@ -335,10 +330,7 @@ m.Worker.prototype.update = function(gameState, ent)
 		{
 			ent.setMetadata(PlayerID, "subrole", "idle");
 			ent.setMetadata(PlayerID, "target-foundation", undefined);
-			// If worker elephant, move away to avoid being trapped in between constructions
-			if (ent.hasClass("Elephant"))
-				this.moveToGatherer(gameState, ent, true);
-			else if (this.baseID != gameState.ai.HQ.baseManagers[0].ID)
+			if (this.baseID != gameState.ai.HQ.baseManagers[0].ID)
 			{
 				// reassign it to something useful
 				this.base.reassignIdleWorkers(gameState, [ent]);
@@ -348,8 +340,8 @@ m.Worker.prototype.update = function(gameState, ent)
 		}
 		else
 		{
-			let goalAccess = m.getLandAccess(gameState, target);
-			let queued = m.returnResources(gameState, ent);
+			let goalAccess = PETRA.getLandAccess(gameState, target);
+			let queued = PETRA.returnResources(gameState, ent);
 			if (this.entAccess == goalAccess)
 				ent.repair(target, target.hasClass("House"), queued);  // autocontinue=true for houses
 			else
@@ -387,18 +379,18 @@ m.Worker.prototype.update = function(gameState, ent)
 		}
 		else	// Perform some sanity checks
 		{
-			if (unitAIStateOrder == "GATHER" || unitAIStateOrder == "RETURNRESOURCE")
+			if (unitAIStateOrder == "GATHER")
 			{
 				// we may have drifted towards ennemy territory during the hunt, if yes go home
 				let territoryOwner = gameState.ai.HQ.territoryMap.getOwner(ent.position());
 				if (territoryOwner != 0 && !gameState.isPlayerAlly(territoryOwner))  // player is its own ally
 					this.startHunting(gameState);
-				else if (unitAIState == "INDIVIDUAL.RETURNRESOURCE.APPROACHING")
+				else if (unitAIState == "INDIVIDUAL.GATHER.RETURNINGRESOURCE.APPROACHING")
 				{
 					// Check that UnitAI does not send us to an inaccessible dropsite
 					let dropsite = gameState.getEntityById(ent.unitAIOrderData()[0].target);
-					if (dropsite && dropsite.position() && this.entAccess != m.getLandAccess(gameState, dropsite))
-						m.returnResources(gameState, ent);
+					if (dropsite && dropsite.position() && this.entAccess != PETRA.getLandAccess(gameState, dropsite))
+						PETRA.returnResources(gameState, ent);
 				}
 			}
 		}
@@ -416,7 +408,7 @@ m.Worker.prototype.update = function(gameState, ent)
 	}
 };
 
-m.Worker.prototype.retryWorking = function(gameState, subrole)
+PETRA.Worker.prototype.retryWorking = function(gameState, subrole)
 {
 	switch (subrole)
 	{
@@ -433,21 +425,21 @@ m.Worker.prototype.retryWorking = function(gameState, subrole)
 	}
 };
 
-m.Worker.prototype.startBuilding = function(gameState)
+PETRA.Worker.prototype.startBuilding = function(gameState)
 {
 	let target = gameState.getEntityById(this.ent.getMetadata(PlayerID, "target-foundation"));
 	if (!target || target.foundationProgress() === undefined && target.needsRepair() == false)
 		return false;
-	if (m.getLandAccess(gameState, target) != this.entAccess)
+	if (PETRA.getLandAccess(gameState, target) != this.entAccess)
 		return false;
 	this.ent.repair(target, target.hasClass("House"));  // autocontinue=true for houses
 	return true;
 };
 
-m.Worker.prototype.startGathering = function(gameState)
+PETRA.Worker.prototype.startGathering = function(gameState)
 {
 	// First look for possible treasure if any
-	if (m.gatherTreasure(gameState, this.ent))
+	if (PETRA.gatherTreasure(gameState, this.ent))
 		return true;
 
 	let resource = this.ent.getMetadata(PlayerID, "gather-type");
@@ -467,7 +459,7 @@ m.Worker.prototype.startGathering = function(gameState)
 				supplies.splice(i--, 1);
 				continue;
 			}
-			if (m.IsSupplyFull(gameState, supplies[i].ent))
+			if (PETRA.IsSupplyFull(gameState, supplies[i].ent))
 				continue;
 			let inaccessibleTime = supplies[i].ent.getMetadata(PlayerID, "inaccessibleTime");
 			if (inaccessibleTime && gameState.ai.elapsedTime < inaccessibleTime)
@@ -586,7 +578,7 @@ m.Worker.prototype.startGathering = function(gameState)
 	// Try to help building one if any accessible foundation available
 	let foundations = gameState.getOwnFoundations().toEntityArray();
 	let shouldBuild = this.ent.isBuilder() && foundations.some(function(foundation) {
-		if (!foundation || m.getLandAccess(gameState, foundation) != this.entAccess)
+		if (!foundation || PETRA.getLandAccess(gameState, foundation) != this.entAccess)
 			return false;
 		let structure = gameState.getBuiltTemplate(foundation.templateName());
 		if (structure.resourceDropsiteTypes() && structure.resourceDropsiteTypes().indexOf(resource) != -1)
@@ -653,12 +645,12 @@ m.Worker.prototype.startGathering = function(gameState)
 	// Okay so we haven't found any appropriate dropsite anywhere.
 	// Try to help building one if any non-accessible foundation available
 	shouldBuild = this.ent.isBuilder() && foundations.some(function(foundation) {
-		if (!foundation || m.getLandAccess(gameState, foundation) == this.entAccess)
+		if (!foundation || PETRA.getLandAccess(gameState, foundation) == this.entAccess)
 			return false;
 		let structure = gameState.getBuiltTemplate(foundation.templateName());
 		if (structure.resourceDropsiteTypes() && structure.resourceDropsiteTypes().indexOf(resource) != -1)
 		{
-			let foundationAccess = m.getLandAccess(gameState, foundation);
+			let foundationAccess = PETRA.getLandAccess(gameState, foundation);
 			if (navalManager.requireTransport(gameState, this.ent, this.entAccess, foundationAccess, foundation.position()))
 			{
 				if (foundation.getMetadata(PlayerID, "base") != this.baseID)
@@ -680,8 +672,8 @@ m.Worker.prototype.startGathering = function(gameState)
 	{
 		if (gameState.ai.HQ.turnCache.allowDistantFood === undefined)
 			gameState.ai.HQ.turnCache.allowDistantFood =
-				!gameState.ai.HQ.canBuild(gameState, "structures/{civ}_field") &&
-				!gameState.ai.HQ.canBuild(gameState, "structures/{civ}_corral");
+				!gameState.ai.HQ.canBuild(gameState, "structures/{civ}/field") &&
+				!gameState.ai.HQ.canBuild(gameState, "structures/{civ}/corral");
 		allowDistant = gameState.ai.HQ.turnCache.allowDistantFood;
 	}
 	if (allowDistant)
@@ -735,10 +727,10 @@ m.Worker.prototype.startGathering = function(gameState)
  * if position is given, we only check if we could hunt from this position but do nothing
  * otherwise the position of the entity is taken, and if something is found, we directly start the hunt
  */
-m.Worker.prototype.startHunting = function(gameState, position)
+PETRA.Worker.prototype.startHunting = function(gameState, position)
 {
 	// First look for possible treasure if any
-	if (!position && m.gatherTreasure(gameState, this.ent))
+	if (!position && PETRA.gatherTreasure(gameState, this.ent))
 		return true;
 
 	let resources = gameState.getHuntableSupplies();
@@ -748,7 +740,7 @@ m.Worker.prototype.startHunting = function(gameState, position)
 	let nearestSupplyDist = Math.min();
 	let nearestSupply;
 
-	let isCavalry = this.ent.hasClass("Cavalry");
+	let isFastMoving = PETRA.isFastMoving(this.ent);
 	let isRanged = this.ent.hasClass("Ranged");
 	let entPosition = position ? position : this.ent.position();
 	let foodDropsites = gameState.playerData.hasSharedDropsites ?
@@ -764,7 +756,7 @@ m.Worker.prototype.startHunting = function(gameState, position)
 			// owner != PlayerID can only happen when hasSharedDropsites == true, so no need to test it again
 			if (owner != PlayerID && (!dropsite.isSharedDropsite() || !gameState.isPlayerMutualAlly(owner)))
 				continue;
-			if (supplyAccess != m.getLandAccess(gameState, dropsite))
+			if (supplyAccess != PETRA.getLandAccess(gameState, dropsite))
 				continue;
 			if (API3.SquareVectorDistance(supplyPosition, dropsite.position()) < distSquare)
 				return true;
@@ -786,43 +778,43 @@ m.Worker.prototype.startHunting = function(gameState, position)
 		if (!gatherRates[supplyType])
 			continue;
 
-		if (m.IsSupplyFull(gameState, supply))
+		if (PETRA.IsSupplyFull(gameState, supply))
 			continue;
-		// check if available resource is worth one additionnal gatherer (except for farms)
+		// Check if available resource is worth one additionnal gatherer (except for farms).
 		let nbGatherers = supply.resourceSupplyNumGatherers() + gameState.ai.HQ.GetTCGatherer(supply.id());
 		if (nbGatherers > 0 && supply.resourceSupplyAmount()/(1+nbGatherers) < 30)
 			continue;
 
 		let canFlee = !supply.hasClass("Domestic") && supply.templateName().indexOf("resource|") == -1;
-		// Only cavalry and range units should hunt fleeing animals
-		if (canFlee && !isCavalry && !isRanged)
+		// Only FastMoving and Ranged units should hunt fleeing animals.
+		if (canFlee && !isFastMoving && !isRanged)
 			continue;
 
-		let supplyAccess = m.getLandAccess(gameState, supply);
+		let supplyAccess = PETRA.getLandAccess(gameState, supply);
 		if (supplyAccess != this.entAccess)
 			continue;
 
-		// measure the distance to the resource
+		// measure the distance to the resource.
 		let dist = API3.SquareVectorDistance(entPosition, supply.position());
 		if (dist > nearestSupplyDist)
 			continue;
 
-		// Only cavalry should hunt faraway
-		if (!isCavalry && dist > 25000)
+		// Only FastMoving should hunt faraway.
+		if (!isFastMoving && dist > 25000)
 			continue;
 
-		// Avoid ennemy territory
+		// Avoid enemy territory.
 		let territoryOwner = gameState.ai.HQ.territoryMap.getOwner(supply.position());
-		if (territoryOwner != 0 && !gameState.isPlayerAlly(territoryOwner))  // player is its own ally
+		if (territoryOwner != 0 && !gameState.isPlayerAlly(territoryOwner))  // Player is its own ally.
 			continue;
-		// And if in ally territory, don't hunt this ally's cattle
+		// And if in ally territory, don't hunt this ally's cattle.
 		if (territoryOwner != 0 && territoryOwner != PlayerID && supply.owner() == territoryOwner)
 			continue;
 
-		// Only cavalry should hunt far from dropsite (specially for non domestic animals which flee)
-		if (!isCavalry && canFlee && territoryOwner == 0)
+		// Only FastMoving should hunt far from dropsite (specially for non-Domestic animals which flee).
+		if (!isFastMoving && canFlee && territoryOwner == 0)
 			continue;
-		let distanceSquare = isCavalry ? 35000 : (canFlee ? 7000 : 12000);
+		let distanceSquare = isFastMoving ? 35000 : (canFlee ? 7000 : 12000);
 		if (!hasFoodDropsiteWithinDistance(supply.position(), supplyAccess, distanceSquare))
 			continue;
 
@@ -843,7 +835,7 @@ m.Worker.prototype.startHunting = function(gameState, position)
 	return false;
 };
 
-m.Worker.prototype.startFishing = function(gameState)
+PETRA.Worker.prototype.startFishing = function(gameState)
 {
 	if (!this.ent.position())
 		return false;
@@ -859,7 +851,7 @@ m.Worker.prototype.startFishing = function(gameState)
 	let nearestSupplyDist = Math.min();
 	let nearestSupply;
 
-	let fisherSea = m.getSeaAccess(gameState, this.ent);
+	let fisherSea = PETRA.getSeaAccess(gameState, this.ent);
 	let fishDropsites = (gameState.playerData.hasSharedDropsites ? gameState.getAnyDropsites("food") : gameState.getOwnDropsites("food")).
 	                    filter(API3.Filters.byClass("Dock")).toEntityArray();
 
@@ -874,7 +866,7 @@ m.Worker.prototype.startFishing = function(gameState)
 			// owner != PlayerID can only happen when hasSharedDropsites == true, so no need to test it again
 			if (owner != PlayerID && (!dropsite.isSharedDropsite() || !gameState.isPlayerMutualAlly(owner)))
 				continue;
-			if (fisherSea != m.getSeaAccess(gameState, dropsite))
+			if (fisherSea != PETRA.getSeaAccess(gameState, dropsite))
 				continue;
 			distMin = Math.min(distMin, API3.SquareVectorDistance(pos, dropsite.position()));
 		}
@@ -898,7 +890,7 @@ m.Worker.prototype.startFishing = function(gameState)
 		if (!gatherRates[supplyType])
 			return;
 
-		if (m.IsSupplyFull(gameState, supply))
+		if (PETRA.IsSupplyFull(gameState, supply))
 			return;
 		// check if available resource is worth one additionnal gatherer (except for farms)
 		let nbGatherers = supply.resourceSupplyNumGatherers() + gameState.ai.HQ.GetTCGatherer(supply.id());
@@ -938,7 +930,7 @@ m.Worker.prototype.startFishing = function(gameState)
 	return false;
 };
 
-m.Worker.prototype.gatherNearestField = function(gameState, baseID)
+PETRA.Worker.prototype.gatherNearestField = function(gameState, baseID)
 {
 	let ownFields = gameState.getOwnEntitiesByClass("Field", true).filter(API3.Filters.isBuilt()).filter(API3.Filters.byMetadata(PlayerID, "base", baseID));
 	let bestFarm;
@@ -946,7 +938,7 @@ m.Worker.prototype.gatherNearestField = function(gameState, baseID)
 	let gatherRates = this.ent.resourceGatherRates();
 	for (let field of ownFields.values())
 	{
-		if (m.IsSupplyFull(gameState, field))
+		if (PETRA.IsSupplyFull(gameState, field))
 			continue;
 		let supplyType = field.get("ResourceSupply/Type");
 		if (!gatherRates[supplyType])
@@ -978,7 +970,7 @@ m.Worker.prototype.gatherNearestField = function(gameState, baseID)
  * WARNING with the present options of AI orders, the unit will not gather after building the farm.
  * This is done by calling the gatherNearestField function when construction is completed.
  */
-m.Worker.prototype.buildAnyField = function(gameState, baseID)
+PETRA.Worker.prototype.buildAnyField = function(gameState, baseID)
 {
 	if (!this.ent.isBuilder())
 		return false;
@@ -1007,7 +999,7 @@ m.Worker.prototype.buildAnyField = function(gameState, baseID)
  * For the time being, we move towards the nearest gatherer (providing him a dropsite).
  * BaseManager does also use that function to deal with its mobile dropsites.
  */
-m.Worker.prototype.moveToGatherer = function(gameState, ent, forced)
+PETRA.Worker.prototype.moveToGatherer = function(gameState, ent, forced)
 {
 	let pos = ent.position();
 	if (!pos || ent.getMetadata(PlayerID, "target-foundation") !== undefined)
@@ -1017,7 +1009,7 @@ m.Worker.prototype.moveToGatherer = function(gameState, ent, forced)
 	let gatherers = this.base.workersBySubrole(gameState, "gatherer");
 	let dist = Math.min();
 	let destination;
-	let access = m.getLandAccess(gameState, ent);
+	let access = PETRA.getLandAccess(gameState, ent);
 	let types = ent.resourceDropsiteTypes();
 	for (let gatherer of gatherers.values())
 	{
@@ -1025,7 +1017,7 @@ m.Worker.prototype.moveToGatherer = function(gameState, ent, forced)
 		if (!gathererType || types.indexOf(gathererType) == -1)
 			continue;
 		if (!gatherer.position() || gatherer.getMetadata(PlayerID, "transport") !== undefined ||
-		    m.getLandAccess(gameState, gatherer) != access || gatherer.isIdle())
+		    PETRA.getLandAccess(gameState, gatherer) != access || gatherer.isIdle())
 			continue;
 		let distance = API3.SquareVectorDistance(pos, gatherer.position());
 		if (distance > dist)
@@ -1043,7 +1035,7 @@ m.Worker.prototype.moveToGatherer = function(gameState, ent, forced)
  * inside obstruction of other entities). The resource will be flagged as inaccessible during 10 mn (in case
  * it will be cleared later).
  */
-m.Worker.prototype.isInaccessibleSupply = function(gameState)
+PETRA.Worker.prototype.isInaccessibleSupply = function(gameState)
 {
 	if (!this.ent.unitAIOrderData()[0] || !this.ent.unitAIOrderData()[0].target)
 		return false;
@@ -1109,6 +1101,3 @@ m.Worker.prototype.isInaccessibleSupply = function(gameState)
 	}
 	return false;
 };
-
-return m;
-}(PETRA);
