@@ -1,7 +1,7 @@
 /**
  * Used to highlight hotkeys in tooltip descriptions.
  */
-var g_HotkeyTags = {"color": "255 251 131" };
+var g_HotkeyTags = { "color": "255 251 131" };
 
 /**
  * Concatenate integer color values to a string (for use in GUI objects)
@@ -34,7 +34,7 @@ function guiToRgbColor(string)
 		"r": +color[0],
 		"g": +color[1],
 		"b": +color[2],
-		"alpha": color.length == 4 ? +color[3] : undefined
+		"a": color.length == 4 ? +color[3] : undefined
 	};
 }
 
@@ -92,7 +92,11 @@ function rgbToHsl(r, g, b)
 	let h, s, l = (max + min) / 2;
 
 	if (max == min)
-		h = s = 0; // achromatic
+	{
+		// achromatic
+		h = 0;
+		s = 0;
+	}
 	else
 	{
 		let d = max - min;
@@ -108,7 +112,11 @@ function rgbToHsl(r, g, b)
 		case b:
 			h = (r - g) / d + 4;
 			break;
+		default:
+			error("rgbToHsl could not determine maximum!");
+			break;
 		}
+
 		h /= 6;
 	}
 
@@ -144,8 +152,14 @@ function hslToRgb(h, s, l)
 	[h, s, l] = [h, s, l].map(clampColorValue);
 	let r, g, b;
 	if (s == 0)
-		r = g = b = l; // achromatic
-	else {
+	{
+		// achromatic
+		b = l;
+		r = l;
+		g = l;
+	}
+	else
+	{
 		let q = l < 0.5 ? l * (1 + s) : l + s - l * s;
 		let p = 2 * l - q;
 		r = hue2rgb(p, q, h + 1/3);
@@ -158,12 +172,15 @@ function hslToRgb(h, s, l)
 
 function colorizeHotkey(text, hotkey)
 {
-	let key = Engine.ConfigDB_GetValue("user", "hotkey." + hotkey);
+	// TODO: Be more efficient in retrieving the mapping(s) for a specific hotkey
+	let key = Engine.GetHotkeyMap()[hotkey];
 
-	if (!key || key.toLowerCase() == "unused")
+	if (!key)
 		key = sprintf(translate("Unassigned hotkey: %(hotkeyName)s"), {
 			"hotkeyName": hotkey
 		});
+	else
+		key = formatHotkeyCombinations(key);
 
 	return sprintf(text, {
 		"hotkey": setStringTags("\\[" + key + "]", g_HotkeyTags)
@@ -176,7 +193,7 @@ function colorizeHotkey(text, hotkey)
  */
 function colorizeAutocompleteHotkey(string)
 {
-	return sprintf(string || translate("Press %(hotkey)s to autocomplete playernames."), {
+	return sprintf(string || translate("Press %(hotkey)s to autocomplete player names."), {
 		"hotkey":
 			setStringTags("\\[" + translateWithContext("hotkey", "Tab") + "]", g_HotkeyTags)
 	});
