@@ -106,6 +106,13 @@ AIProxy.prototype.OnHealthChanged = function(msg)
 	this.changes.hitpoints = msg.to;
 };
 
+AIProxy.prototype.OnGarrisonedStateChanged = function(msg)
+{
+	if (!this.NotifyChange())
+		return;
+	this.changes.garrisonHolderID = msg.holderID;
+};
+
 AIProxy.prototype.OnCapturePointsChanged = function(msg)
 {
 	if (!this.NotifyChange())
@@ -260,9 +267,9 @@ AIProxy.prototype.GetFullRepresentation = function()
 		ret.hitpoints = cmpHealth.GetHitpoints();
 	}
 
-	let cmpDamageReceiver = Engine.QueryInterface(this.entity, IID_DamageReceiver);
-	if (cmpDamageReceiver)
-		ret.invulnerability = cmpDamageReceiver.IsInvulnerable();
+	let cmpResistance = Engine.QueryInterface(this.entity, IID_Resistance);
+	if (cmpResistance)
+		ret.invulnerability = cmpResistance.IsInvulnerable();
 
 	let cmpOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
 	if (cmpOwnership)
@@ -327,6 +334,13 @@ AIProxy.prototype.GetFullRepresentation = function()
 		ret.garrisoned = cmpGarrisonHolder.GetEntities();
 	}
 
+	let cmpGarrisonable = Engine.QueryInterface(this.entity, IID_Garrisonable);
+	if (cmpGarrisonable)
+	{
+		// Updated by OnGarrisonedStateChanged
+		ret.garrisonHolderID = cmpGarrisonable.HolderID();
+	}
+
 	let cmpTerritoryDecay = Engine.QueryInterface(this.entity, IID_TerritoryDecay);
 	if (cmpTerritoryDecay)
 		ret.decaying = cmpTerritoryDecay.IsDecaying();
