@@ -29,16 +29,16 @@ Barter.prototype.RESTORE_TIMER_INTERVAL = 5000;
 Barter.prototype.Init = function()
 {
 	this.priceDifferences = {};
-	for (let resource of Resources.GetCodes())
+	for (let resource of Resources.GetBarterableCodes())
 		this.priceDifferences[resource] = 0;
 	this.restoreTimer = undefined;
 };
 
-Barter.prototype.GetPrices = function(playerID)
+Barter.prototype.GetPrices = function(cmpPlayer)
 {
-	var prices = { "buy": {}, "sell": {} };
-	let multiplier = QueryPlayerIDInterface(playerID).GetBarterMultiplier();
-	for (let resource of Resources.GetCodes())
+	let prices = { "buy": {}, "sell": {} };
+	let multiplier = cmpPlayer.GetBarterMultiplier();
+	for (let resource of Resources.GetBarterableCodes())
 	{
 		let truePrice = Resources.GetResource(resource).truePrice;
 		prices.buy[resource] = truePrice * (100 + this.CONSTANT_DIFFERENCE + this.priceDifferences[resource]) * multiplier.buy[resource] / 100;
@@ -47,20 +47,6 @@ Barter.prototype.GetPrices = function(playerID)
 	return prices;
 };
 
-Barter.prototype.PlayerHasMarket = function(playerID)
-{
-	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
-	var entities = cmpRangeManager.GetEntitiesByPlayer(playerID);
-	for (var entity of entities)
-	{
-		var cmpFoundation = Engine.QueryInterface(entity, IID_Foundation);
-		var cmpIdentity = Engine.QueryInterface(entity, IID_Identity);
-		if (!cmpFoundation && cmpIdentity && cmpIdentity.HasClass("BarterMarket"))
-			return true;
-	}
-	return false;
-};
-
 Barter.prototype.ExchangeResources = function(playerID, resourceToSell, resourceToBuy, amount)
 {
 	if (amount <= 0)
@@ -69,7 +55,7 @@ Barter.prototype.ExchangeResources = function(playerID, resourceToSell, resource
 		return;
 	}
 
-	let availResources = Resources.GetCodes();
+	let availResources = Resources.GetBarterableCodes();
 	if (availResources.indexOf(resourceToSell) == -1)
 	{
 		warn("ExchangeResources: incorrect resource to sell: " + uneval(resourceToSell));
@@ -82,35 +68,34 @@ Barter.prototype.ExchangeResources = function(playerID, resourceToSell, resource
 		return;
 	}
 
-	// This can occur when the player issues the order just before the market is destroyed or captured
-	if (!this.PlayerHasMarket(playerID))
-		return;
-
 	if (amount != 100 && amount != 500)
 		return;
 
-	var cmpPlayer = QueryPlayerIDInterface(playerID);
-	var prices = this.GetPrices(playerID);
-	var amountsToSubtract = {};
+	let cmpPlayer = QueryPlayerIDInterface(playerID);
+	if (!cmpPlayer || !cmpPlayer.CanBarter())
+		return;
+
+	let prices = this.GetPrices(cmpPlayer);
+	let amountsToSubtract = {};
 	amountsToSubtract[resourceToSell] = amount;
 	if (cmpPlayer.TrySubtractResources(amountsToSubtract))
 	{
-		var amountToAdd = Math.round(prices["sell"][resourceToSell] / prices["buy"][resourceToBuy] * amount);
+		let amountToAdd = Math.round(prices.sell[resourceToSell] / prices.buy[resourceToBuy] * amount);
 		cmpPlayer.AddResource(resourceToBuy, amountToAdd);
 
-		// Display chat message to observers
-		var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
+		// Display chat message to observers.
+		let cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 		if (cmpGUIInterface)
 			cmpGUIInterface.PushNotification({
 				"type": "barter",
 				"players": [playerID],
-				"amountsSold": amount,
-				"amountsBought": amountToAdd,
-				"resourceSold": resourceToSell,
-				"resourceBought": resourceToBuy
+				"amountGiven": amount,
+				"amountGained": amountToAdd,
+				"resourceGiven": resourceToSell,
+				"resourceGained": resourceToBuy
 			});
 
-		var cmpStatisticsTracker = QueryPlayerIDInterface(playerID, IID_StatisticsTracker);
+		let cmpStatisticsTracker = QueryPlayerIDInterface(playerID, IID_StatisticsTracker);
 		if (cmpStatisticsTracker)
 		{
 			cmpStatisticsTracker.IncreaseResourcesSoldCounter(resourceToSell, amount);
@@ -127,19 +112,16 @@ Barter.prototype.ExchangeResources = function(playerID, resourceToSell, resource
 	}
 
 	if (this.restoreTimer === undefined)
-	{
-		var cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
-		this.restoreTimer = cmpTimer.SetInterval(this.entity, IID_Barter, "ProgressTimeout", this.RESTORE_TIMER_INTERVAL, this.RESTORE_TIMER_INTERVAL, {});
-	}
+		this.restoreTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer).SetInterval(this.entity, IID_Barter, "ProgressTimeout", this.RESTORE_TIMER_INTERVAL, this.RESTORE_TIMER_INTERVAL, {});
 };
 
 Barter.prototype.ProgressTimeout = function(data)
 {
-	var needRestore = false;
-	for (let resource of Resources.GetCodes())
+	let needRestore = false;
+	for (let resource of Resources.GetBarterableCodes())
 	{
 		// Calculate value to restore, it should be limited to [-DIFFERENCE_RESTORE; DIFFERENCE_RESTORE] interval
-		var differenceRestore = Math.min(this.DIFFERENCE_RESTORE, Math.max(-this.DIFFERENCE_RESTORE, this.priceDifferences[resource]));
+		let differenceRestore = Math.min(this.DIFFERENCE_RESTORE, Math.max(-this.DIFFERENCE_RESTORE, this.priceDifferences[resource]));
 		differenceRestore = -differenceRestore;
 		this.priceDifferences[resource] += differenceRestore;
 		// If price difference still exists then set flag to run timer again
@@ -149,7 +131,7 @@ Barter.prototype.ProgressTimeout = function(data)
 
 	if (!needRestore)
 	{
-		var cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
+		let cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
 		cmpTimer.CancelTimer(this.restoreTimer);
 		this.restoreTimer = undefined;
 	}
