@@ -96,9 +96,9 @@ CeasefireManager.prototype.ShowCeasefireCountdownMessage = function()
 {
 	let cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 	this.ceasefireCountdownMessage = cmpGuiInterface.AddTimeNotification({
-			"message": markForTranslation("You can attack in %(time)s"),
-			"translateMessage": true
-		}, this.countdownMessageDuration);
+		"message": markForTranslation("You can attack in %(time)s"),
+		"translateMessage": true
+	}, this.countdownMessageDuration);
 };
 
 CeasefireManager.prototype.StopCeasefire = function()
@@ -124,6 +124,11 @@ CeasefireManager.prototype.StopCeasefire = function()
 	this.diplomacyBeforeCeasefire = [];
 
 	Engine.PostMessage(SYSTEM_ENTITY, MT_CeasefireEnded);
+
+	cmpGuiInterface.PushNotification({
+		"type": "ceasefire-ended",
+		"players": [-1] // processed globally
+	});
 };
 
 Engine.RegisterSystemComponentType(IID_CeasefireManager, "CeasefireManager", CeasefireManager);
