@@ -4,12 +4,12 @@ EntityLimits.prototype.Schema =
 	"<a:help>Specifies per category limits on number of entities (buildings or units) that can be created for each player</a:help>" +
 	"<a:example>" +
 		"<Limits>" +
-			"<DefenseTower>25</DefenseTower>" +
-			"<Fortress>10</Fortress>" +
-			"<Wonder>1</Wonder>" +
-			"<Hero>1</Hero>" +
 			"<Apadana>1</Apadana>" +
+			"<Fortress>10</Fortress>" +
+			"<Hero>1</Hero>" +
 			"<Monument>5</Monument>" +
+			"<Tower>25</Tower>" +
+			"<Wonder>1</Wonder>" +
 		"</Limits>" +
 		"<LimitChangers>" +
 			"<Monument>" +
@@ -72,6 +72,7 @@ EntityLimits.prototype.Init = function()
 	this.removers = {};
 	this.classCount = {};	// counts entities with the given class, used in the limit removal
 	this.removedLimit = {};
+	this.matchTemplateCount = {};
 	for (var category in this.template.Limits)
 	{
 		this.limit[category] = +this.template.Limits[category];
@@ -103,6 +104,14 @@ EntityLimits.prototype.ChangeCount = function(category, value)
 		this.count[category] += value;
 };
 
+EntityLimits.prototype.ChangeMatchCount = function(template, value)
+{
+	if (!this.matchTemplateCount[template])
+		this.matchTemplateCount[template] = 0;
+
+	this.matchTemplateCount[template] += value;
+};
+
 EntityLimits.prototype.GetLimits = function()
 {
 	return this.limit;
@@ -113,6 +122,11 @@ EntityLimits.prototype.GetCounts = function()
 	return this.count;
 };
 
+EntityLimits.prototype.GetMatchCounts = function()
+{
+	return this.matchTemplateCount;
+};
+
 EntityLimits.prototype.GetLimitChangers = function()
 {
 	return this.changers;
@@ -121,8 +135,8 @@ EntityLimits.prototype.GetLimitChangers = function()
 EntityLimits.prototype.UpdateLimitsFromTech = function(tech)
 {
 	for (var category in this.removers)
-		if ("RequiredTechs" in this.removers[category] && this.removers[category]["RequiredTechs"].indexOf(tech) !== -1)
-			this.removers[category]["RequiredTechs"].splice(this.removers[category]["RequiredTechs"].indexOf(tech), 1);
+		if ("RequiredTechs" in this.removers[category] && this.removers[category].RequiredTechs.indexOf(tech) !== -1)
+			this.removers[category].RequiredTechs.splice(this.removers[category].RequiredTechs.indexOf(tech), 1);
 
 	this.UpdateLimitRemoval();
 };
@@ -133,9 +147,9 @@ EntityLimits.prototype.UpdateLimitRemoval = function()
 	{
 		var nolimit = true;
 		if ("RequiredTechs" in this.removers[category])
-			nolimit = !this.removers[category]["RequiredTechs"].length;
+			nolimit = !this.removers[category].RequiredTechs.length;
 		if (nolimit && "RequiredClasses" in this.removers[category])
-			for (var cls of this.removers[category]["RequiredClasses"])
+			for (var cls of this.removers[category].RequiredClasses)
 				nolimit = nolimit && this.classCount[cls] > 0;
 
 		if (nolimit && this.limit[category] !== undefined)
@@ -145,40 +159,48 @@ EntityLimits.prototype.UpdateLimitRemoval = function()
 	}
 };
 
-EntityLimits.prototype.AllowedToCreate = function(limitType, category, count)
+EntityLimits.prototype.AllowedToCreate = function(limitType, category, count, templateName, matchLimit)
 {
-	// Allow unspecified categories and those with no limit
-	if (this.count[category] === undefined || this.limit[category] === undefined)
-		return true;
-
-	if (this.count[category] + count > this.limit[category])
+	if (this.count[category] !== undefined && this.limit[category] !== undefined &&
+		this.count[category] + count > this.limit[category])
 	{
-		var cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
-		var notification = {
-			"players": [cmpPlayer.GetPlayerID()],
-			"translateMessage": true,
-			"translateParameters": ["category"],
-			"parameters": {"category": category, "limit": this.limit[category]},
-		};
-
-		if (limitType == BUILD)
-			notification.message = markForTranslation("%(category)s build limit of %(limit)s reached");
-		else if (limitType == TRAINING)
-			notification.message = markForTranslation("%(category)s training limit of %(limit)s reached");
-		else
-		{
-			warn("EntityLimits.js: Unknown LimitType " + limitType);
-			notification.message = markForTranslation("%(category)s limit of %(limit)s reached");
-		}
-		var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-		cmpGUIInterface.PushNotification(notification);
+		this.NotifyLimit(limitType, category, this.limit[category]);
+		return false;
+	}
 
+	if (this.matchTemplateCount[templateName] !== undefined && matchLimit !== undefined &&
+		this.matchTemplateCount[templateName] + count > matchLimit)
+	{
+		this.NotifyLimit(limitType, category, matchLimit);
 		return false;
 	}
 
 	return true;
 };
 
+EntityLimits.prototype.NotifyLimit = function(limitType, category, limit)
+{
+	let cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
+	let notification = {
+		"players": [cmpPlayer.GetPlayerID()],
+		"translateMessage": true,
+		"translateParameters": ["category"],
+		"parameters": { "category": category, "limit": limit },
+	};
+
+	if (limitType == BUILD)
+		notification.message = markForTranslation("%(category)s build limit of %(limit)s reached");
+	else if (limitType == TRAINING)
+		notification.message = markForTranslation("%(category)s training limit of %(limit)s reached");
+	else
+	{
+		warn("EntityLimits.js: Unknown LimitType " + limitType);
+		notification.message = markForTranslation("%(category)s limit of %(limit)s reached");
+	}
+	let cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
+	cmpGUIInterface.PushNotification(notification);
+};
+
 EntityLimits.prototype.AllowedToBuild = function(category)
 {
 	// We pass count 0 as the creation of the building has already taken place and
@@ -186,9 +208,9 @@ EntityLimits.prototype.AllowedToBuild = function(category)
 	return this.AllowedToCreate(BUILD, category, 0);
 };
 
-EntityLimits.prototype.AllowedToTrain = function(category, count)
+EntityLimits.prototype.AllowedToTrain = function(category, count, templateName, matchLimit)
 {
-	return this.AllowedToCreate(TRAINING, category, count);
+	return this.AllowedToCreate(TRAINING, category, count, templateName, matchLimit);
 };
 
 /**
@@ -266,7 +288,7 @@ EntityLimits.prototype.OnGlobalOwnershipChanged = function(msg)
 
 	for (var category in this.removers)
 		if ("RequiredClasses" in this.removers[category])
-			for (var cls of this.removers[category]["RequiredClasses"])
+			for (var cls of this.removers[category].RequiredClasses)
 				if (classes.indexOf(cls) !== -1)
 					this.classCount[cls] += modifier;
 
