@@ -15,6 +15,7 @@ Gate.prototype.Schema =
 Gate.prototype.Init = function()
 {
 	this.allies = [];
+	this.ignoreList = [];
 	this.opened = false;
 	this.locked = false;
 };
@@ -32,10 +33,11 @@ Gate.prototype.OnOwnershipChanged = function(msg)
 
 Gate.prototype.OnDiplomacyChanged = function(msg)
 {
-	var cmpOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
+	let cmpOwnership = Engine.QueryInterface(this.entity, IID_Ownership);
 	if (cmpOwnership && cmpOwnership.GetOwner() == msg.player)
 	{
 		this.allies = [];
+		this.ignoreList = [];
 		this.SetupRangeQuery(msg.player);
 	}
 };
@@ -76,7 +78,7 @@ Gate.prototype.SetupRangeQuery = function(owner)
 	if (range > 0)
 	{
 		// Only find entities with IID_UnitAI interface
-		this.unitsQuery = cmpRangeManager.CreateActiveQuery(this.entity, 0, range, players, IID_UnitAI, cmpRangeManager.GetEntityFlagMask("normal"));
+		this.unitsQuery = cmpRangeManager.CreateActiveQuery(this.entity, 0, range, players, IID_UnitAI, cmpRangeManager.GetEntityFlagMask("normal"), true);
 		cmpRangeManager.EnableActiveQuery(this.unitsQuery);
 	}
 };
@@ -91,11 +93,36 @@ Gate.prototype.OnRangeUpdate = function(msg)
 
 	if (msg.added.length > 0)
 		for (let entity of msg.added)
+		{
+			// Ignore entities that cannot move as those won't be able to go through the gate.
+			let unitAI = Engine.QueryInterface(entity, IID_UnitAI);
+			if (!unitAI || !unitAI.AbleToMove())
+				this.ignoreList.push(entity);
 			this.allies.push(entity);
+		}
 
 	if (msg.removed.length > 0)
 		for (let entity of msg.removed)
+		{
+			let index = this.ignoreList.indexOf(entity);
+			if (index !== -1)
+				this.ignoreList.splice(index, 1);
 			this.allies.splice(this.allies.indexOf(entity), 1);
+		}
+
+	this.OperateGate();
+};
+
+Gate.prototype.OnGlobalUnitAbleToMoveChanged = function(msg)
+{
+	if (this.allies.indexOf(msg.entity) === -1)
+		return;
+
+	let index = this.ignoreList.indexOf(msg.entity);
+	if (msg.ableToMove && index !== -1)
+		this.ignoreList.splice(index, 1);
+	else if (!msg.ableToMove && index === -1)
+		this.ignoreList.push(msg.entity);
 
 	this.OperateGate();
 };
@@ -108,6 +135,11 @@ Gate.prototype.GetPassRange = function()
 	return +this.template.PassRange;
 };
 
+Gate.prototype.ShouldOpen = function()
+{
+	return this.allies.some(ent => this.ignoreList.indexOf(ent) === -1);
+};
+
 /**
  * Attempt to open or close the gate.
  * An ally must be in range to open the gate, but an unlocked gate will only close
@@ -122,10 +154,9 @@ Gate.prototype.OperateGate = function()
 		cmpTimer.CancelTimer(this.timer);
 		this.timer = undefined;
 	}
-
-	if (this.opened && (this.allies.length == 0 || this.locked))
+	if (this.opened && (this.locked || !this.ShouldOpen()))
 		this.CloseGate();
-	else if (!this.opened && this.allies.length)
+	else if (!this.opened && this.ShouldOpen())
 		this.OpenGate();
 };
 
@@ -143,7 +174,7 @@ Gate.prototype.LockGate = function()
 
 	// Delete animal corpses to prevent units trying to gather the unreachable entity
 	let cmpObstruction = Engine.QueryInterface(this.entity, IID_Obstruction);
-	if (cmpObstruction && cmpObstruction.GetBlockMovementFlag())
+	if (cmpObstruction && cmpObstruction.GetBlockMovementFlag(true))
 		for (let ent of cmpObstruction.GetEntitiesDeletedUponConstruction())
 			Engine.DestroyEntity(ent);
 
@@ -215,19 +246,24 @@ Gate.prototype.OpenGate = function()
  */
 Gate.prototype.CloseGate = function()
 {
-	var cmpObstruction = Engine.QueryInterface(this.entity, IID_Obstruction);
+	let cmpObstruction = Engine.QueryInterface(this.entity, IID_Obstruction);
 	if (!cmpObstruction)
 		return;
 
 	// The gate can't be closed if there are entities colliding with it.
-	var collisions = cmpObstruction.GetEntitiesBlockingConstruction();
+	// NB: because walls are overlapping, they requires special care to not break
+	// in particular, walls do not block construction, so walls from skirmish maps
+	// do not appear in this check even if they have different control groups from the gate.
+	// This no longer works if gates are made to check for entities blocking movement.
+	// Fixing that would let us change this code, but it sounds decidedly non-trivial.
+	let collisions = cmpObstruction.GetEntitiesBlockingConstruction();
 	if (collisions.length)
 	{
 		if (!this.timer)
 		{
 			// Set an "instant" timer which will run on the next simulation turn.
-			var cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
-			this.timer = cmpTimer.SetTimeout(this.entity, IID_Gate, "OperateGate", 0, {});
+			let cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
+			this.timer = cmpTimer.SetTimeout(this.entity, IID_Gate, "OperateGate", 0);
 		}
 		return;
 	}
@@ -241,7 +277,7 @@ Gate.prototype.CloseGate = function()
 	this.opened = false;
 
 	PlaySound("gate_closing", this.entity);
-	var cmpVisual = Engine.QueryInterface(this.entity, IID_Visual);
+	let cmpVisual = Engine.QueryInterface(this.entity, IID_Visual);
 	if (cmpVisual)
 		cmpVisual.SelectAnimation("gate_closing", true, 1.0);
 };
