@@ -49,6 +49,16 @@ Pack.prototype.IsPacking = function()
 	return this.packing;
 };
 
+Pack.prototype.CanPack = function()
+{
+	return !this.packing && !this.packed;
+};
+
+Pack.prototype.CanUnpack = function()
+{
+	return !this.packing && this.packed;
+};
+
 Pack.prototype.Pack = function()
 {
 	if (this.IsPacked() || this.IsPacking())
@@ -57,11 +67,11 @@ Pack.prototype.Pack = function()
 	this.packing = true;
 
 	let cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
-	this.timer = cmpTimer.SetInterval(this.entity, IID_Pack, "PackProgress", 0, PACKING_INTERVAL, { "packing": true });
+	this.timer = cmpTimer.SetInterval(this.entity, IID_Pack, "PackProgress", 0, PACKING_INTERVAL, null);
 
 	let cmpVisual = Engine.QueryInterface(this.entity, IID_Visual);
 	if (cmpVisual)
-		cmpVisual.SelectAnimation("packing", true, 1.0, "packing");
+		cmpVisual.SelectAnimation("packing", true, 1.0);
 };
 
 Pack.prototype.Unpack = function()
@@ -72,7 +82,7 @@ Pack.prototype.Unpack = function()
 	this.packing = true;
 
 	let cmpTimer = Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer);
-	this.timer = cmpTimer.SetInterval(this.entity, IID_Pack, "PackProgress", 0, PACKING_INTERVAL, { "packing": false });
+	this.timer = cmpTimer.SetInterval(this.entity, IID_Pack, "PackProgress", 0, PACKING_INTERVAL, null);
 
 	let cmpVisual = Engine.QueryInterface(this.entity, IID_Visual);
 	if (cmpVisual)
@@ -96,9 +106,7 @@ Pack.prototype.CancelPack = function()
 
 Pack.prototype.GetPackTime = function()
 {
-	let cmpPlayer = QueryOwnerInterface(this.entity, IID_Player);
-
-	return ApplyValueModificationsToEntity("Pack/Time", +this.template.Time, this.entity) * cmpPlayer.GetTimeMultiplier();
+	return ApplyValueModificationsToEntity("Pack/Time", +this.template.Time, this.entity);
 };
 
 Pack.prototype.GetElapsedTime = function()
