@@ -36,6 +36,12 @@ Player.prototype.Serialize = function()
 	return state;
 };
 
+Player.prototype.Deserialize = function(state)
+{
+	for (let prop in state)
+		this[prop] = state[prop];
+};
+
 /**
  * Which units will be shown with special icons at the top.
  */
@@ -44,56 +50,56 @@ var panelEntityClasses = "Hero Relic";
 Player.prototype.Init = function()
 {
 	this.playerID = undefined;
-	this.name = undefined;	// define defaults elsewhere (supporting other languages)
+	this.name = undefined;	// Define defaults elsewhere (supporting other languages).
 	this.civ = undefined;
 	this.color = undefined;
 	this.diplomacyColor = undefined;
 	this.displayDiplomacyColor = false;
-	this.popUsed = 0; // population of units owned or trained by this player
-	this.popBonuses = 0; // sum of population bonuses of player's entities
-	this.maxPop = 300; // maximum population
-	this.trainingBlocked = false; // indicates whether any training queue is currently blocked
+	this.popUsed = 0; // Population of units owned or trained by this player.
+	this.popBonuses = 0; // Sum of population bonuses of player's entities.
+	this.maxPop = 300; // Maximum population.
+	this.trainingBlocked = false; // Indicates whether any training queue is currently blocked.
 	this.resourceCount = {};
-	this.tradingGoods = []; // goods for next trade-route and its proba in % (the sum of probas must be 100)
-	this.team = -1;	// team number of the player, players on the same team will always have ally diplomatic status - also this is useful for team emblems, scoring, etc.
+	this.resourceGatherers = {};
+	this.tradingGoods = []; // Goods for next trade-route and its probabilities * 100.
+	this.team = -1;	// Team number of the player, players on the same team will always have ally diplomatic status. Also this is useful for team emblems, scoring, etc.
 	this.teamsLocked = false;
-	this.state = "active"; // game state - one of "active", "defeated", "won"
-	this.diplomacy = [];	// array of diplomatic stances for this player with respect to other players (including gaia and self)
+	this.state = "active"; // Game state. One of "active", "defeated", "won".
+	this.diplomacy = [];	// Array of diplomatic stances for this player with respect to other players (including gaia and self).
 	this.sharedDropsites = false;
 	this.formations = [];
 	this.startCam = undefined;
 	this.controlAllUnits = false;
 	this.isAI = false;
-	this.timeMultiplier = 1;
-	this.gatherRateMultiplier = 1;
-	this.tradeRateMultiplier = 1;
 	this.cheatsEnabled = false;
-	this.cheatTimeMultiplier = 1;
 	this.panelEntities = [];
 	this.resourceNames = {};
 	this.disabledTemplates = {};
 	this.disabledTechnologies = {};
-	this.startingTechnologies = [];
 	this.spyCostMultiplier = +this.template.SpyCostMultiplier;
+	this.barterEntities = [];
 	this.barterMultiplier = {
 		"buy": clone(this.template.BarterMultiplier.Buy),
 		"sell": clone(this.template.BarterMultiplier.Sell)
 	};
 
-	// Initial resources and trading goods probability in steps of 5
+	// Initial resources.
 	let resCodes = Resources.GetCodes();
-	let quotient = Math.floor(20 / resCodes.length);
-	let remainder = 20 % resCodes.length;
-	for (let i in resCodes)
+	for (let res of resCodes)
 	{
-		let res = resCodes[i];
 		this.resourceCount[res] = 300;
 		this.resourceNames[res] = Resources.GetResource(res).name;
+		this.resourceGatherers[res] = 0;
+	}
+	// Trading goods probability in steps of 5.
+	let resTradeCodes = Resources.GetTradableCodes();
+	let quotient = Math.floor(20 / resTradeCodes.length);
+	let remainder = 20 % resTradeCodes.length;
+	for (let i in resTradeCodes)
 		this.tradingGoods.push({
-			"goods": res,
+			"goods": resTradeCodes[i],
 			"proba": 5 * (quotient + (+i < remainder ? 1 : 0))
 		});
-	}
 };
 
 Player.prototype.SetPlayerID = function(id)
@@ -118,14 +124,12 @@ Player.prototype.GetName = function()
 
 Player.prototype.SetCiv = function(civcode)
 {
-	var oldCiv = this.civ;
+	let oldCiv = this.civ;
 	this.civ = civcode;
-	// Normally, the civ is only set once
-	// But in Atlas, the map designers can change civs at any time
-	var playerID = this.GetPlayerID();
-	if (oldCiv && playerID && oldCiv != civcode)
+	// Normally, the civ is only set once. But in Atlas, map designers can change civs at any time.
+	if (oldCiv && this.playerID && oldCiv != civcode)
 		Engine.BroadcastMessage(MT_CivChanged, {
-			"player": playerID,
+			"player": this.playerID,
 			"from": oldCiv,
 			"to": civcode
 		});
@@ -138,11 +142,11 @@ Player.prototype.GetCiv = function()
 
 Player.prototype.SetColor = function(r, g, b)
 {
-	var colorInitialized = !!this.color;
+	let colorInitialized = !!this.color;
 
 	this.color = { "r": r / 255, "g": g / 255, "b": b / 255, "a": 1 };
 
-	// Used in Atlas
+	// Used in Atlas.
 	if (colorInitialized)
 		Engine.BroadcastMessage(MT_PlayerColorChanged, {
 			"player": this.playerID
@@ -172,8 +176,8 @@ Player.prototype.GetDisplayedColor = function()
 // Try reserving num population slots. Returns 0 on success or number of missing slots otherwise.
 Player.prototype.TryReservePopulationSlots = function(num)
 {
-	if (num != 0 && num > (this.GetPopulationLimit() - this.GetPopulationCount()))
-		return num - (this.GetPopulationLimit() - this.GetPopulationCount());
+	if (num != 0 && num > (this.GetPopulationLimit() - this.popUsed))
+		return num - (this.GetPopulationLimit() - this.popUsed);
 
 	this.popUsed += num;
 	return 0;
@@ -219,51 +223,21 @@ Player.prototype.GetMaxPopulation = function()
 	return Math.round(ApplyValueModificationsToEntity("Player/MaxPopulation", this.maxPop, this.entity));
 };
 
+Player.prototype.CanBarter = function()
+{
+	return this.barterEntities.length > 0;
+};
+
 Player.prototype.GetBarterMultiplier = function()
 {
 	return this.barterMultiplier;
 };
 
-Player.prototype.GetGatherRateMultiplier = function()
-{
-	return this.gatherRateMultiplier / this.cheatTimeMultiplier;
-};
-
-Player.prototype.GetTimeMultiplier = function()
-{
-	return this.timeMultiplier * this.cheatTimeMultiplier;
-};
-
-Player.prototype.GetTradeRateMultiplier = function()
-{
-	return this.tradeRateMultiplier;
-};
-
 Player.prototype.GetSpyCostMultiplier = function()
 {
 	return this.spyCostMultiplier;
 };
 
-/**
- * Setters currently used by the AI to set the difficulty level
- */
-Player.prototype.SetGatherRateMultiplier = function(value)
-{
-	this.gatherRateMultiplier = value;
-	Engine.BroadcastMessage(MT_MultiplierChanged, { "player": this.playerID, "type": "gather" });
-};
-
-Player.prototype.SetTimeMultiplier = function(value)
-{
-	this.timeMultiplier = value;
-	Engine.BroadcastMessage(MT_MultiplierChanged, { "player": this.playerID, "type": "time" });
-};
-
-Player.prototype.SetTradeRateMultiplier = function(value)
-{
-	this.tradeRateMultiplier = value;
-};
-
 Player.prototype.GetPanelEntities = function()
 {
 	return this.panelEntities;
@@ -295,10 +269,31 @@ Player.prototype.GetResourceCounts = function()
 	return this.resourceCount;
 };
 
+Player.prototype.GetResourceGatherers = function()
+{
+	return this.resourceGatherers;
+};
+
 /**
- * Add resource of specified type to player
- * @param type Generic type of resource (string)
- * @param amount Amount of resource, which should be added (integer)
+ * @param {string} type - The generic type of resource to add the gatherer for.
+ */
+Player.prototype.AddResourceGatherer = function(type)
+{
+	++this.resourceGatherers[type];
+};
+
+/**
+ * @param {string} type - The generic type of resource to remove the gatherer from.
+ */
+Player.prototype.RemoveResourceGatherer = function(type)
+{
+	--this.resourceGatherers[type];
+};
+
+/**
+ * Add resource of specified type to player.
+ * @param {string} type - Generic type of resource.
+ * @param {number} amount - Amount of resource, which should be added.
  */
 Player.prototype.AddResource = function(type, amount)
 {
@@ -306,19 +301,19 @@ Player.prototype.AddResource = function(type, amount)
 };
 
 /**
- * Add resources to player
+ * Add resources to player.
  */
 Player.prototype.AddResources = function(amounts)
 {
-	for (var type in amounts)
+	for (let type in amounts)
 		this.resourceCount[type] += +amounts[type];
 };
 
 Player.prototype.GetNeededResources = function(amounts)
 {
-	// Check if we can afford it all
-	var amountsNeeded = {};
-	for (var type in amounts)
+	// Check if we can afford it all.
+	let amountsNeeded = {};
+	for (let type in amounts)
 		if (this.resourceCount[type] != undefined && amounts[type] > this.resourceCount[type])
 			amountsNeeded[type] = amounts[type] - Math.floor(this.resourceCount[type]);
 
@@ -329,23 +324,23 @@ Player.prototype.GetNeededResources = function(amounts)
 
 Player.prototype.SubtractResourcesOrNotify = function(amounts)
 {
-	var amountsNeeded = this.GetNeededResources(amounts);
+	let amountsNeeded = this.GetNeededResources(amounts);
 
-	// If we don't have enough resources, send a notification to the player
+	// If we don't have enough resources, send a notification to the player.
 	if (amountsNeeded)
 	{
-		var parameters = {};
-		var i = 0;
-		for (var type in amountsNeeded)
+		let parameters = {};
+		let i = 0;
+		for (let type in amountsNeeded)
 		{
 			++i;
-			parameters["resourceType"+i] = this.resourceNames[type];
-			parameters["resourceAmount"+i] = amountsNeeded[type];
+			parameters["resourceType" + i] = this.resourceNames[type];
+			parameters["resourceAmount" + i] = amountsNeeded[type];
 		}
 
-		var msg = "";
-		// when marking strings for translations, you need to include the actual string,
-		// not some way to derive the string
+		let msg = "";
+		// When marking strings for translations, you need to include the actual string,
+		// not some way to derive the string.
 		if (i < 1)
 			warn("Amounts needed but no amounts given?");
 		else if (i == 1)
@@ -359,7 +354,7 @@ Player.prototype.SubtractResourcesOrNotify = function(amounts)
 		else
 			warn("Localisation: Strings are not localised for more than 4 resources");
 
-		// Send as time-notification
+		// Send as time-notification.
 		let cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 		cmpGUIInterface.PushNotification({
 			"players": [this.playerID],
@@ -370,13 +365,13 @@ Player.prototype.SubtractResourcesOrNotify = function(amounts)
 				"resourceType1": "withinSentence",
 				"resourceType2": "withinSentence",
 				"resourceType3": "withinSentence",
-				"resourceType4": "withinSentence",
-			},
+				"resourceType4": "withinSentence"
+			}
 		});
 		return false;
 	}
 
-	for (var type in amounts)
+	for (let type in amounts)
 		this.resourceCount[type] -= amounts[type];
 
 	return true;
@@ -387,9 +382,9 @@ Player.prototype.TrySubtractResources = function(amounts)
 	if (!this.SubtractResourcesOrNotify(amounts))
 		return false;
 
-	var cmpStatisticsTracker = QueryPlayerIDInterface(this.playerID, IID_StatisticsTracker);
+	let cmpStatisticsTracker = QueryPlayerIDInterface(this.playerID, IID_StatisticsTracker);
 	if (cmpStatisticsTracker)
-		for (var type in amounts)
+		for (let type in amounts)
 			cmpStatisticsTracker.IncreaseResourceUsedCounter(type, amounts[type]);
 
 	return true;
@@ -397,10 +392,10 @@ Player.prototype.TrySubtractResources = function(amounts)
 
 Player.prototype.GetNextTradingGoods = function()
 {
-	var value = randFloat(0, 100);
-	var last = this.tradingGoods.length - 1;
-	var sumProba = 0;
-	for (var i = 0; i < last; ++i)
+	let value = randFloat(0, 100);
+	let last = this.tradingGoods.length - 1;
+	let sumProba = 0;
+	for (let i = 0; i < last; ++i)
 	{
 		sumProba += this.tradingGoods[i].proba;
 		if (value < sumProba)
@@ -411,7 +406,7 @@ Player.prototype.GetNextTradingGoods = function()
 
 Player.prototype.GetTradingGoods = function()
 {
-	var tradingGoods = {};
+	let tradingGoods = {};
 	for (let resource of this.tradingGoods)
 		tradingGoods[resource.goods] = resource.proba;
 
@@ -420,11 +415,11 @@ Player.prototype.GetTradingGoods = function()
 
 Player.prototype.SetTradingGoods = function(tradingGoods)
 {
-	let resCodes = Resources.GetCodes();
+	let resTradeCodes = Resources.GetTradableCodes();
 	let sumProba = 0;
 	for (let resource in tradingGoods)
 	{
-		if (resCodes.indexOf(resource) == -1 || tradingGoods[resource] < 0)
+		if (resTradeCodes.indexOf(resource) == -1 || tradingGoods[resource] < 0)
 		{
 			error("Invalid trading goods: " + uneval(tradingGoods));
 			return;
@@ -434,7 +429,7 @@ Player.prototype.SetTradingGoods = function(tradingGoods)
 
 	if (sumProba != 100)
 	{
-		error("Invalid trading goods: " + uneval(tradingGoods));
+		error("Invalid trading goods probability: " + uneval(sumProba));
 		return;
 	}
 
@@ -468,6 +463,12 @@ Player.prototype.SetState = function(newState, message)
 		return;
 	}
 
+	if (!this.playerID)
+	{
+		warn("Gaia can't change state.");
+		return;
+	}
+
 	this.state = newState;
 
 	let won = newState == "won";
@@ -476,7 +477,7 @@ Player.prototype.SetState = function(newState, message)
 		cmpRangeManager.SetLosRevealAll(this.playerID, true);
 	else
 	{
-		// Reassign all player's entities to Gaia
+		// Reassign all player's entities to Gaia.
 		let entities = cmpRangeManager.GetEntitiesByPlayer(this.playerID);
 
 		// The ownership change is done in two steps so that entities don't hit idle
@@ -523,7 +524,7 @@ Player.prototype.SetTeam = function(team)
 
 	this.team = team;
 
-	// Set all team members as allies
+	// Set all team members as allies.
 	if (this.team != -1)
 	{
 		let numPlayers = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager).GetNumPlayers();
@@ -585,7 +586,7 @@ Player.prototype.SetDiplomacyIndex = function(idx, value)
 		"otherPlayer": cmpPlayer.GetPlayerID()
 	});
 
-	// Mutual worsening of relations
+	// Mutual worsening of relations.
 	if (cmpPlayer.diplomacy[this.playerID] > value)
 		cmpPlayer.SetDiplomacyIndex(this.playerID, value);
 };
@@ -668,8 +669,8 @@ Player.prototype.IsAI = function()
 
 Player.prototype.GetPlayersByDiplomacy = function(func)
 {
-	var players = [];
-	for (var i = 0; i < this.diplomacy.length; ++i)
+	let players = [];
+	for (let i = 0; i < this.diplomacy.length; ++i)
 		if (this[func](i))
 			players.push(i);
 	return players;
@@ -681,7 +682,7 @@ Player.prototype.SetAlly = function(id)
 };
 
 /**
- * Check if given player is our ally
+ * Check if given player is our ally.
  */
 Player.prototype.IsAlly = function(id)
 {
@@ -706,7 +707,7 @@ Player.prototype.IsExclusiveAlly = function(id)
  */
 Player.prototype.IsMutualAlly = function(id)
 {
-	var cmpPlayer = QueryPlayerIDInterface(id);
+	let cmpPlayer = QueryPlayerIDInterface(id);
 	return this.IsAlly(id) && cmpPlayer && cmpPlayer.IsAlly(this.playerID);
 };
 
@@ -759,12 +760,7 @@ Player.prototype.IsNeutral = function(id)
  */
 Player.prototype.OnGlobalInitGame = function(msg)
 {
-	let cmpTechnologyManager = Engine.QueryInterface(this.entity, IID_TechnologyManager);
-	if (cmpTechnologyManager)
-		for (let tech of this.startingTechnologies)
-			cmpTechnologyManager.ResearchTechnology(tech);
-
-	// Replace the "{civ}" code with this civ ID
+	// Replace the "{civ}" code with this civ ID.
 	let disabledTemplates = this.disabledTemplates;
 	this.disabledTemplates = {};
 	for (let template in disabledTemplates)
@@ -774,35 +770,42 @@ Player.prototype.OnGlobalInitGame = function(msg)
 
 /**
  * Keep track of population effects of all entities that
- * become owned or unowned by this player
+ * become owned or unowned by this player.
  */
 Player.prototype.OnGlobalOwnershipChanged = function(msg)
 {
 	if (msg.from != this.playerID && msg.to != this.playerID)
 		return;
 
-	var cmpIdentity = Engine.QueryInterface(msg.entity, IID_Identity);
-	var cmpCost = Engine.QueryInterface(msg.entity, IID_Cost);
+	let cmpCost = Engine.QueryInterface(msg.entity, IID_Cost);
 
 	if (msg.from == this.playerID)
 	{
 		if (cmpCost)
 			this.popUsed -= cmpCost.GetPopCost();
 
-		if (cmpIdentity && MatchesClassList(cmpIdentity.GetClassesList(), panelEntityClasses))
-		{
-			let index = this.panelEntities.indexOf(msg.entity);
-			if (index >= 0)
-				this.panelEntities.splice(index, 1);
-		}
+		let panelIndex = this.panelEntities.indexOf(msg.entity);
+		if (panelIndex >= 0)
+			this.panelEntities.splice(panelIndex, 1);
+
+		let barterIndex = this.barterEntities.indexOf(msg.entity);
+		if (barterIndex >= 0)
+			this.barterEntities.splice(barterIndex, 1);
 	}
 	if (msg.to == this.playerID)
 	{
 		if (cmpCost)
 			this.popUsed += cmpCost.GetPopCost();
 
-		if (cmpIdentity && MatchesClassList(cmpIdentity.GetClassesList(), panelEntityClasses))
+		let cmpIdentity = Engine.QueryInterface(msg.entity, IID_Identity);
+		if (!cmpIdentity)
+			return;
+
+		if (MatchesClassList(cmpIdentity.GetClassesList(), panelEntityClasses))
 			this.panelEntities.push(msg.entity);
+
+		if (cmpIdentity.HasClass("Barter") && !Engine.QueryInterface(msg.entity, IID_Foundation))
+			this.barterEntities.push(msg.entity);
 	}
 };
 
@@ -845,32 +848,22 @@ Player.prototype.GetCheatsEnabled = function()
 	return this.cheatsEnabled;
 };
 
-Player.prototype.SetCheatTimeMultiplier = function(time)
-{
-	this.cheatTimeMultiplier = time;
-	Engine.BroadcastMessage(MT_MultiplierChanged, { "player": this.playerID, "type": "cheat" });
-};
-
-Player.prototype.GetCheatTimeMultiplier = function()
-{
-	return this.cheatTimeMultiplier;
-};
-
 Player.prototype.TributeResource = function(player, amounts)
 {
-	var cmpPlayer = QueryPlayerIDInterface(player);
+	let cmpPlayer = QueryPlayerIDInterface(player);
 	if (!cmpPlayer)
 		return;
 
 	if (this.state != "active" || cmpPlayer.state != "active")
 		return;
 
+	let resTribCodes = Resources.GetTributableCodes();
 	for (let resCode in amounts)
-		if (Resources.GetCodes().indexOf(resCode) == -1 ||
+		if (resTribCodes.indexOf(resCode) == -1 ||
 		    !Number.isInteger(amounts[resCode]) ||
 		    amounts[resCode] < 0)
 		{
-			warn("Invalid tribute amounts: " + uneval(amounts));
+			warn("Invalid tribute amounts: " + uneval(resCode) + ": " + uneval(amounts));
 			return;
 		}
 
@@ -878,15 +871,15 @@ Player.prototype.TributeResource = function(player, amounts)
 		return;
 	cmpPlayer.AddResources(amounts);
 
-	var total = Object.keys(amounts).reduce((sum, type) => sum + amounts[type], 0);
-	var cmpOurStatisticsTracker = QueryPlayerIDInterface(this.playerID, IID_StatisticsTracker);
+	let total = Object.keys(amounts).reduce((sum, type) => sum + amounts[type], 0);
+	let cmpOurStatisticsTracker = QueryPlayerIDInterface(this.playerID, IID_StatisticsTracker);
 	if (cmpOurStatisticsTracker)
 		cmpOurStatisticsTracker.IncreaseTributesSentCounter(total);
-	var cmpTheirStatisticsTracker = QueryPlayerIDInterface(player, IID_StatisticsTracker);
+	let cmpTheirStatisticsTracker = QueryPlayerIDInterface(player, IID_StatisticsTracker);
 	if (cmpTheirStatisticsTracker)
 		cmpTheirStatisticsTracker.IncreaseTributesReceivedCounter(total);
 
-	var cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
+	let cmpGUIInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
 	if (cmpGUIInterface)
 		cmpGUIInterface.PushNotification({
 			"type": "tribute",
@@ -905,24 +898,13 @@ Player.prototype.TributeResource = function(player, amounts)
 Player.prototype.AddDisabledTemplate = function(template)
 {
 	this.disabledTemplates[template] = true;
-	Engine.BroadcastMessage(MT_DisabledTemplatesChanged, {});
-	var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-	cmpGuiInterface.PushNotification({
-		"type": "resetselectionpannel",
-		"players": [this.GetPlayerID()]
-	});
+	Engine.BroadcastMessage(MT_DisabledTemplatesChanged, { "player": this.playerID });
 };
 
 Player.prototype.RemoveDisabledTemplate = function(template)
 {
 	this.disabledTemplates[template] = false;
-	Engine.BroadcastMessage(MT_DisabledTemplatesChanged, {});
-
-	var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-	cmpGuiInterface.PushNotification({
-		"type": "resetselectionpannel",
-		"players": [this.GetPlayerID()]
-	});
+	Engine.BroadcastMessage(MT_DisabledTemplatesChanged, { "player": this.playerID });
 };
 
 Player.prototype.SetDisabledTemplates = function(templates)
@@ -930,13 +912,7 @@ Player.prototype.SetDisabledTemplates = function(templates)
 	this.disabledTemplates = {};
 	for (let template of templates)
 		this.disabledTemplates[template] = true;
-	Engine.BroadcastMessage(MT_DisabledTemplatesChanged, {});
-
-	var cmpGuiInterface = Engine.QueryInterface(SYSTEM_ENTITY, IID_GuiInterface);
-	cmpGuiInterface.PushNotification({
-		"type": "resetselectionpannel",
-		"players": [this.GetPlayerID()]
-	});
+	Engine.BroadcastMessage(MT_DisabledTemplatesChanged, { "player": this.playerID });
 };
 
 Player.prototype.GetDisabledTemplates = function()
@@ -947,13 +923,13 @@ Player.prototype.GetDisabledTemplates = function()
 Player.prototype.AddDisabledTechnology = function(tech)
 {
 	this.disabledTechnologies[tech] = true;
-	Engine.BroadcastMessage(MT_DisabledTechnologiesChanged, {});
+	Engine.BroadcastMessage(MT_DisabledTechnologiesChanged, { "player": this.playerID });
 };
 
 Player.prototype.RemoveDisabledTechnology = function(tech)
 {
 	this.disabledTechnologies[tech] = false;
-	Engine.BroadcastMessage(MT_DisabledTechnologiesChanged, {});
+	Engine.BroadcastMessage(MT_DisabledTechnologiesChanged, { "player": this.playerID });
 };
 
 Player.prototype.SetDisabledTechnologies = function(techs)
@@ -961,7 +937,7 @@ Player.prototype.SetDisabledTechnologies = function(techs)
 	this.disabledTechnologies = {};
 	for (let tech of techs)
 		this.disabledTechnologies[tech] = true;
-	Engine.BroadcastMessage(MT_DisabledTechnologiesChanged, {});
+	Engine.BroadcastMessage(MT_DisabledTechnologiesChanged, { "player": this.playerID });
 };
 
 Player.prototype.GetDisabledTechnologies = function()
@@ -969,15 +945,15 @@ Player.prototype.GetDisabledTechnologies = function()
 	return this.disabledTechnologies;
 };
 
-Player.prototype.AddStartingTechnology = function(tech)
+Player.prototype.OnGlobalPlayerDefeated = function(msg)
 {
-	if (this.startingTechnologies.indexOf(tech) == -1)
-		this.startingTechnologies.push(tech);
-};
+	let cmpSound = Engine.QueryInterface(this.entity, IID_Sound);
+	if (!cmpSound)
+		return;
 
-Player.prototype.SetStartingTechnologies = function(techs)
-{
-	this.startingTechnologies = techs;
+	let soundGroup = cmpSound.GetSoundGroup(this.playerID === msg.playerId ? "defeated" : this.IsAlly(msg.playerId) ? "defeated_ally" : this.state === "won" ? "won" : "defeated_enemy");
+	if (soundGroup)
+		Engine.QueryInterface(SYSTEM_ENTITY, IID_SoundManager).PlaySoundGroupForPlayer(soundGroup, this.playerID);
 };
 
 Engine.RegisterComponentType(IID_Player, "Player", Player);
