@@ -15,14 +15,9 @@ SkirmishReplacer.prototype.Init = function()
 
 SkirmishReplacer.prototype.Serialize = null; // We have no dynamic state to save
 
-//this function gets the replacement entities from the {civ}.json file
 function getReplacementEntities(civ)
 {
-	var rawCivData = Engine.ReadCivJSONFile(civ+".json");
-	if (rawCivData && rawCivData.SkirmishReplacements)
-		return rawCivData.SkirmishReplacements;
-	warn("SkirmishReplacer.js: no replacements found in '"+civ+".json'");
-	return {};
+	return Engine.ReadJSONFile("simulation/data/civs/" + civ + ".json").SkirmishReplacements;
 }
 
 SkirmishReplacer.prototype.OnOwnershipChanged = function(msg)
@@ -34,12 +29,18 @@ SkirmishReplacer.prototype.OnOwnershipChanged = function(msg)
 SkirmishReplacer.prototype.ReplaceEntities = function()
 {
 	var cmpPlayer = QueryOwnerInterface(this.entity);
+	if (!cmpPlayer)
+		return;
+
 	var civ = cmpPlayer.GetCiv();
 	var replacementEntities = getReplacementEntities(civ);
 
 	var cmpTemplateManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
 	var templateName = cmpTemplateManager.GetCurrentTemplateName(this.entity);
 
+	let specialFilters = templateName.substr(0, templateName.lastIndexOf("|") + 1);
+	templateName = removeFiltersFromTemplateName(templateName);
+
 	if (templateName in replacementEntities)
 		templateName = replacementEntities[templateName];
 	else if (this.template && "general" in this.template)
@@ -53,7 +54,7 @@ SkirmishReplacer.prototype.ReplaceEntities = function()
 		return;
 	}
 
-	templateName = templateName.replace(/\{civ\}/g, civ);
+	templateName = specialFilters + templateName.replace(/\{civ\}/g, civ);
 
 	var cmpCurPosition = Engine.QueryInterface(this.entity, IID_Position);
 	var replacement = Engine.AddEntity(templateName);
@@ -71,7 +72,9 @@ SkirmishReplacer.prototype.ReplaceEntities = function()
 	var cmpReplacementOwnership = Engine.QueryInterface(replacement, IID_Ownership);
 	cmpReplacementOwnership.SetOwner(cmpCurOwnership.GetOwner());
 
-	Engine.BroadcastMessage(MT_EntityRenamed, { "entity": this.entity, "newentity": replacement });
+	let msg = { "entity": this.entity, "newentity": replacement };
+	Engine.PostMessage(this.entity, MT_EntityRenamed, msg);
+	Engine.PostMessage(this.entity, MT_SkirmishReplacerReplaced, msg);
 	Engine.DestroyEntity(this.entity);
 };
 
