@@ -3,21 +3,6 @@ function TechnologyManager() {}
 TechnologyManager.prototype.Schema =
 	"<empty/>";
 
-TechnologyManager.prototype.Serialize = function()
-{
-	// The modifications cache will be affected by property reads from the GUI and other places so we shouldn't
-	// serialize it.
-
-	var ret = {};
-	for (var i in this)
-	{
-		if (this.hasOwnProperty(i))
-			ret[i] = this[i];
-	}
-	ret.modificationCache = {};
-	return ret;
-};
-
 TechnologyManager.prototype.Init = function()
 {
 	// Holds names of technologies that have been researched.
@@ -29,16 +14,6 @@ TechnologyManager.prototype.Init = function()
 	// Holds technologies which are being researched currently (non-queued).
 	this.researchStarted = new Set();
 
-	// This stores the modifications to unit stats from researched technologies
-	// Example data: {"ResourceGatherer/Rates/food.grain": [
-	//                     {"multiply": 1.15, "affects": ["FemaleCitizen", "Infantry Sword"]},
-	//                     {"add": 2}
-	//                 ]}
-	this.modifications = {};
-	this.modificationCache = {}; // Caches the values after technologies have been applied
-	                             // e.g. { "Attack/Melee/Hack" : {5: {"origValue": 8, "newValue": 10}, 7: {"origValue": 9, "newValue": 12}, ...}, ...}
-	                             // where 5 and 7 are entity id's
-
 	this.classCounts = {}; // stores the number of entities of each Class
 	this.typeCountsByClass = {}; // stores the number of entities of each type for each class i.e.
 	                             // {"someClass": {"unit/spearman": 2, "unit/cav": 5} "someOtherClass":...}
@@ -64,8 +39,8 @@ TechnologyManager.prototype.UpdateAutoResearch = function()
 	for (let key of this.unresearchedAutoResearchTechs)
 	{
 		let tech = TechnologyTemplates.Get(key);
-		if ((tech.autoResearch && this.CanResearch(key))
-			|| (tech.top && (this.IsTechnologyResearched(tech.top) || this.IsTechnologyResearched(tech.bottom))))
+		if ((tech.autoResearch && this.CanResearch(key)) ||
+			(tech.top && (this.IsTechnologyResearched(tech.top) || this.IsTechnologyResearched(tech.bottom))))
 		{
 			this.unresearchedAutoResearchTechs.delete(key);
 			this.ResearchTechnology(key);
@@ -75,7 +50,7 @@ TechnologyManager.prototype.UpdateAutoResearch = function()
 };
 
 // Checks an entity template to see if its technology requirements have been met
-TechnologyManager.prototype.CanProduce = function (templateName)
+TechnologyManager.prototype.CanProduce = function(templateName)
 {
 	var cmpTempManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_TemplateManager);
 	var template = cmpTempManager.GetTemplate(templateName);
@@ -130,7 +105,7 @@ TechnologyManager.prototype.CanResearch = function(tech)
 
 /**
  * Private function for checking a set of requirements is met
- * @param {object} reqs - Technology requirements as derived from the technology template by globalscripts
+ * @param {Object} reqs - Technology requirements as derived from the technology template by globalscripts
  * @param {boolean} civonly - True if only the civ requirement is to be checked
  *
  * @return true if the requirements pass, false otherwise
@@ -204,31 +179,6 @@ TechnologyManager.prototype.OnGlobalOwnershipChanged = function(msg)
 				this.typeCountsByClass[cls][template] += 1;
 			}
 		}
-
-		// Newly created entity, check if any researched techs might apply
-		// (only do this for new entities because even if an entity is converted or captured,
-		//	we want it to maintain whatever technologies previously applied)
-		if (msg.from == INVALID_PLAYER)
-		{
-			var modifiedComponents = {};
-			for (var name in this.modifications)
-			{
-				// We only need to find one one tech per component for a match
-				var modifications = this.modifications[name];
-				var component = name.split("/")[0];
-				for (let modif of modifications)
- 					if (DoesModificationApply(modif, classes))
-					{
-						if (!modifiedComponents[component])
-							modifiedComponents[component] = [];
-						modifiedComponents[component].push(name);
-					}
-			}
-
-			// Send mesage(s) to the entity so it knows about researched techs
-			for (var component in modifiedComponents)
-				Engine.PostMessage(msg.entity, MT_ValueModification, { "entities": [msg.entity], "component": component, "valueNames": modifiedComponents[component] });
-		}
 	}
 	if (msg.from == playerID)
 	{
@@ -254,47 +204,40 @@ TechnologyManager.prototype.OnGlobalOwnershipChanged = function(msg)
 				}
 			}
 		}
-
-		this.clearModificationCache(msg.entity);
 	}
 };
 
-// Marks a technology as researched.  Note that this does not verify that the requirements are met.
+/**
+ * Marks a technology as researched.
+ * Note that this does not verify that the requirements are met.
+ *
+ * @param {string} tech - The technology to mark as researched.
+ */
 TechnologyManager.prototype.ResearchTechnology = function(tech)
 {
 	this.StoppedResearch(tech, false);
 
-	var modifiedComponents = {};
+	let modifiedComponents = {};
 	this.researchedTechs.add(tech);
-	// store the modifications in an easy to access structure
+
+	// Store the modifications in an easy to access structure.
 	let template = TechnologyTemplates.Get(tech);
 	if (template.modifications)
 	{
-		let derivedModifiers = DeriveModificationsFromTech(template);
-		for (let modifierPath in derivedModifiers)
-		{
-			if (!this.modifications[modifierPath])
-				this.modifications[modifierPath] = [];
-			this.modifications[modifierPath] = this.modifications[modifierPath].concat(derivedModifiers[modifierPath]);
-
-			let component = modifierPath.split("/")[0];
-			if (!modifiedComponents[component])
-				modifiedComponents[component] = [];
-			modifiedComponents[component].push(modifierPath);
-			this.modificationCache[modifierPath] = {};
-		}
+		let cmpModifiersManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_ModifiersManager);
+		cmpModifiersManager.AddModifiers("tech/" + tech, DeriveModificationsFromTech(template), this.entity);
 	}
 
 	if (template.replaces && template.replaces.length > 0)
 	{
-		for (var i of template.replaces)
+		for (let i of template.replaces)
 		{
 			if (!i || this.IsTechnologyResearched(i))
 				continue;
 
 			this.researchedTechs.add(i);
 
-			// Change the EntityLimit if any
+			// Change the EntityLimit if any.
 			let cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
 			if (cmpPlayer && cmpPlayer.GetPlayerID() !== undefined)
 			{
@@ -308,27 +251,18 @@ TechnologyManager.prototype.ResearchTechnology = function(tech)
 
 	this.UpdateAutoResearch();
 
-	var cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
+	let cmpPlayer = Engine.QueryInterface(this.entity, IID_Player);
 	if (!cmpPlayer || cmpPlayer.GetPlayerID() === undefined)
 		return;
-	var playerID = cmpPlayer.GetPlayerID();
-	var cmpRangeManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager);
-	var ents = cmpRangeManager.GetEntitiesByPlayer(playerID);
-	ents.push(this.entity);
+	let playerID = cmpPlayer.GetPlayerID();
 
-	// Change the EntityLimit if any
-	var cmpPlayerEntityLimits = QueryPlayerIDInterface(playerID, IID_EntityLimits);
+	// Change the EntityLimit if any.
+	let cmpPlayerEntityLimits = QueryPlayerIDInterface(playerID, IID_EntityLimits);
 	if (cmpPlayerEntityLimits)
 		cmpPlayerEntityLimits.UpdateLimitsFromTech(tech);
 
-	// always send research finished message
-	Engine.PostMessage(this.entity, MT_ResearchFinished, {"player": playerID, "tech": tech});
-
-	for (var component in modifiedComponents)
-	{
-		Engine.PostMessage(SYSTEM_ENTITY, MT_TemplateModification, { "player": playerID, "component": component, "valueNames": modifiedComponents[component]});
-		Engine.BroadcastMessage(MT_ValueModification, { "entities": ents, "component": component, "valueNames": modifiedComponents[component]});
-	}
+	// Always send research finished message.
+	Engine.PostMessage(this.entity, MT_ResearchFinished, { "player": playerID, "tech": tech });
 
 	if (tech.startsWith("phase") && !template.autoResearch)
 	{
@@ -342,45 +276,6 @@ TechnologyManager.prototype.ResearchTechnology = function(tech)
 	}
 };
 
-// Clears the cached data for an entity from the modifications cache
-TechnologyManager.prototype.clearModificationCache = function(ent)
-{
-	for (var valueName in this.modificationCache)
-		delete this.modificationCache[valueName][ent];
-};
-
-// Caching layer in front of ApplyModificationsWorker
-//	Note: be careful with the type of curValue, if it should be a numerical
-//		value and is derived from template data, you must convert the string
-//		from the template to a number using the + operator, before calling
-//		this function!
-TechnologyManager.prototype.ApplyModifications = function(valueName, curValue, ent)
-{
-	if (!this.modificationCache[valueName])
-		this.modificationCache[valueName] = {};
-
-	if (!this.modificationCache[valueName][ent] || this.modificationCache[valueName][ent].origValue != curValue)
-	{
-		let cmpIdentity = Engine.QueryInterface(ent, IID_Identity);
-		if (!cmpIdentity)
-			return curValue;
-		this.modificationCache[valueName][ent] = {
-			"origValue": curValue,
-			"newValue": GetTechModifiedProperty(this.modifications, cmpIdentity.GetClassesList(), valueName, curValue)
-		};
-	}
-
-	return this.modificationCache[valueName][ent].newValue;
-};
-
-// Alternative version of ApplyModifications, applies to templates instead of entities
-TechnologyManager.prototype.ApplyModificationsTemplate = function(valueName, curValue, template)
-{
-	if (!template || !template.Identity)
-		return curValue;
-	return GetTechModifiedProperty(this.modifications, GetIdentityClasses(template.Identity), valueName, curValue);
-};
-
 /**
  * Marks a technology as being queued for research at the given entityID.
  */
@@ -449,7 +344,7 @@ TechnologyManager.prototype.GetStartedTechs = function()
  */
 TechnologyManager.prototype.GetResearcher = function(tech)
 {
-	return this.researchQueued.get(tech)
+	return this.researchQueued.get(tech);
 };
 
 /**
