@@ -10,7 +10,7 @@ Engine.LoadComponentScript("Barter.js");
 const truePrice = 110;
 
 Resources = {
-	"GetCodes": () => ["wood", "stone", "metal"],
+	"GetBarterableCodes": () => ["wood", "stone", "metal"],
 	"GetResource": (resource) => ({ "truePrice": truePrice })
 };
 
@@ -36,6 +36,7 @@ let multiplier = {
 		"metal": 1.0
 	}
 };
+let cmpBarter = ConstructComponent(SYSTEM_ENTITY, "Barter");
 
 AddMock(SYSTEM_ENTITY, IID_Timer, {
 	"CancelTimer": id => { timerActivated = false; },
@@ -47,13 +48,11 @@ AddMock(SYSTEM_ENTITY, IID_Timer, {
 	}
 });
 
-let cmpBarter = ConstructComponent(SYSTEM_ENTITY, "Barter");
-
 // Init
 TS_ASSERT_EQUALS(cmpBarter.restoreTimer, undefined);
 TS_ASSERT_UNEVAL_EQUALS(cmpBarter.priceDifferences, { "wood": 0, "stone": 0, "metal": 0 });
 
-AddMock(playerEnt, IID_Player, {
+let cmpPlayer = AddMock(playerEnt, IID_Player, {
 	"TrySubtractResources": amounts => {
 		sold = amounts[Object.keys(amounts)[0]];
 		return true;
@@ -62,54 +61,36 @@ AddMock(playerEnt, IID_Player, {
 		bought = amount;
 		return true;
 	},
+	"CanBarter": () => true,
 	"GetBarterMultiplier": () => multiplier
 });
 
-AddMock(SYSTEM_ENTITY, IID_RangeManager, {
-	"GetEntitiesByPlayer": (id) => id == playerID ? [62, 60, 61, 63] : []
-});
-
-AddMock(60, IID_Identity, {
-	"HasClass": (cl) => true
-});
-
-AddMock(60, IID_Foundation, {});
-
 // GetPrices
 cmpBarter.priceDifferences = { "wood": 8, "stone": 0, "metal": 0 };
-TS_ASSERT_UNEVAL_EQUALS(cmpBarter.GetPrices(playerID).buy, {
+TS_ASSERT_UNEVAL_EQUALS(cmpBarter.GetPrices(cmpPlayer).buy, {
 	"wood": truePrice * (100 + 8 + cmpBarter.CONSTANT_DIFFERENCE) / 100,
 	"stone": truePrice * (100 + cmpBarter.CONSTANT_DIFFERENCE) / 100,
 	"metal": truePrice * (100 + cmpBarter.CONSTANT_DIFFERENCE) / 100
 });
-TS_ASSERT_UNEVAL_EQUALS(cmpBarter.GetPrices(playerID).sell, {
+TS_ASSERT_UNEVAL_EQUALS(cmpBarter.GetPrices(cmpPlayer).sell, {
 	"wood": truePrice * (100 + 8 - cmpBarter.CONSTANT_DIFFERENCE) / 100,
 	"stone": truePrice * (100 - cmpBarter.CONSTANT_DIFFERENCE) / 100,
 	"metal": truePrice * (100 - cmpBarter.CONSTANT_DIFFERENCE) / 100
 });
 
 multiplier.buy.stone = 2.0;
-TS_ASSERT_UNEVAL_EQUALS(cmpBarter.GetPrices(playerID).buy, {
+TS_ASSERT_UNEVAL_EQUALS(cmpBarter.GetPrices(cmpPlayer).buy, {
 	"wood": truePrice * (100 + 8 + cmpBarter.CONSTANT_DIFFERENCE) / 100,
 	"stone": truePrice * (100 + cmpBarter.CONSTANT_DIFFERENCE) * 2.0 / 100,
 	"metal": truePrice * (100 + cmpBarter.CONSTANT_DIFFERENCE) / 100
 });
-TS_ASSERT_UNEVAL_EQUALS(cmpBarter.GetPrices(playerID).sell, {
+TS_ASSERT_UNEVAL_EQUALS(cmpBarter.GetPrices(cmpPlayer).sell, {
 	"wood": truePrice * (100 + 8 - cmpBarter.CONSTANT_DIFFERENCE) / 100,
 	"stone": truePrice * (100 - cmpBarter.CONSTANT_DIFFERENCE) / 100,
 	"metal": truePrice * (100 - cmpBarter.CONSTANT_DIFFERENCE) / 100
 });
 multiplier.buy.stone = 1.0;
 
-// PlayerHasMarket
-TS_ASSERT(!cmpBarter.PlayerHasMarket(playerID));
-
-AddMock(61, IID_Identity, {
-	"HasClass": (cl) => true
-});
-
-TS_ASSERT(cmpBarter.PlayerHasMarket(playerID));
-
 // ExchangeResources
 // Price differences magnitude are caped by 99 - CONSTANT_DIFFERENCE.
 // If we have a bigger DIFFERENCE_PER_DEAL than 99 - CONSTANT_DIFFERENCE at each deal we reach the cap.
@@ -158,6 +139,7 @@ timerActivated = false;
 AddMock(playerEnt, IID_Player, {
 	"TrySubtractResources": () => false,
 	"AddResource": () => {},
+	"CanBarter": () => true,
 	"GetBarterMultiplier": () => multiplier
 });
 cmpBarter.ExchangeResources(playerID, "wood", "stone", 100);
