@@ -13,8 +13,8 @@ Resources = {
 		return "<interleave>" + schema + "</interleave>";
 	}
 };
-Engine.LoadComponentScript("interfaces/AuraManager.js"); // Provides `IID_AuraManager`, tested for in helpers/ValueModification.js.
-Engine.LoadComponentScript("interfaces/TechnologyManager.js"); // Provides `IID_TechnologyManager`, used below.
+Engine.LoadComponentScript("interfaces/ProductionQueue.js");
+Engine.LoadComponentScript("interfaces/ModifiersManager.js"); // Provides `IID_ModifiersManager`, used below.
 Engine.LoadComponentScript("interfaces/Timer.js"); // Provides `IID_Timer`, used below.
 
 // What we're testing:
@@ -40,7 +40,7 @@ let template = {
 	"Upgrade": {
 		"Tower": {
 			"Cost": { "stone": "100", "wood": "50" },
-			"Entity": "structures/{civ}_defense_tower",
+			"Entity": "structures/{civ}/defense_tower",
 			"Time": "100"
 		}
 	}
@@ -89,21 +89,17 @@ AddMock(SYSTEM_ENTITY, IID_Timer, {
 	"CancelTimer": () => {} // Called in components/Upgrade.js::CancelUpgrade().
 });
 
-// Init Player:
-AddMock(10, IID_Player, {
-	"AddResources": () => {}, // Called in components/Upgrade.js::CancelUpgrade().
-	"GetPlayerID": () => playerID, // Called in helpers/Player.js::QueryOwnerInterface() (and several times below).
-	"GetTimeMultiplier": () => 1.0, // Called in components/Upgrade.js::GetUpgradeTime().
-	"TrySubtractResources": () => true // Called in components/Upgrade.js::Upgrade().
-});
-AddMock(10, IID_TechnologyManager, {
-	"ApplyModificationsTemplate": (valueName, curValue, template) => {
+AddMock(SYSTEM_ENTITY, IID_ModifiersManager, {
+	"ApplyTemplateModifiers": (valueName, curValue, template, player) => {
 		// Called in helpers/ValueModification.js::ApplyValueModificationsToTemplate()
 		// as part of Tests T2 and T5 below.
 		let mods = isResearched ? templateTechModifications.with : templateTechModifications.without;
-		return GetTechModifiedProperty(mods, GetIdentityClasses(template.Identity), valueName, curValue);
+
+		if (mods[valueName])
+			return GetTechModifiedProperty(mods[valueName], GetIdentityClasses(template.Identity), curValue);
+		return curValue;
 	},
-	"ApplyModifications": (valueName, curValue, ent) => {
+	"ApplyModifiers": (valueName, curValue, ent) => {
 		// Called in helpers/ValueModification.js::ApplyValueModificationsToEntity()
 		// as part of Tests T3, T6 and T7 below.
 		let mods = isResearched ? entityTechModifications.with : entityTechModifications.without;
@@ -111,6 +107,13 @@ AddMock(10, IID_TechnologyManager, {
 	}
 });
 
+// Init Player:
+AddMock(10, IID_Player, {
+	"AddResources": () => {}, // Called in components/Upgrade.js::CancelUpgrade().
+	"GetPlayerID": () => playerID, // Called in helpers/Player.js::QueryOwnerInterface() (and several times below).
+	"TrySubtractResources": () => true // Called in components/Upgrade.js::Upgrade().
+});
+
 // Create an entity with an Upgrade component:
 AddMock(20, IID_Ownership, {
 	"GetOwner": () => playerID // Called in helpers/Player.js::QueryOwnerInterface().
@@ -118,6 +121,9 @@ AddMock(20, IID_Ownership, {
 AddMock(20, IID_Identity, {
 	"GetCiv": () => civCode // Called in components/Upgrade.js::init().
 });
+AddMock(20, IID_ProductionQueue, {
+	"HasQueuedProduction": () => false
+});
 let cmpUpgrade = ConstructComponent(20, "Upgrade", template.Upgrade);
 cmpUpgrade.owner = playerID;
 
@@ -126,11 +132,11 @@ cmpUpgrade.owner = playerID;
  * To start with, no techs are researched...
  */
 // T1: Check the cost of the upgrade without a player value being passed (as it would be in the structree).
-let parsed_template = GetTemplateDataHelper(template, null, {}, Resources, DamageTypes);
+let parsed_template = GetTemplateDataHelper(template, null, {});
 TS_ASSERT_UNEVAL_EQUALS(parsed_template.upgrades[0].cost, { "stone": 100, "wood": 50, "time": 100 });
 
 // T2: Check the value, with a player ID (as it would be in-session).
-parsed_template = GetTemplateDataHelper(template, playerID, {}, Resources, DamageTypes);
+parsed_template = GetTemplateDataHelper(template, playerID, {});
 TS_ASSERT_UNEVAL_EQUALS(parsed_template.upgrades[0].cost, { "stone": 100, "wood": 50, "time": 100 });
 
 // T3: Check that the value is correct within the Update Component.
@@ -140,15 +146,15 @@ TS_ASSERT_UNEVAL_EQUALS(cmpUpgrade.GetUpgrades()[0].cost, { "stone": 100, "wood"
  * Tell the Upgrade component to start the Upgrade,
  * then mark the technology that alters the upgrade cost as researched.
  */
-cmpUpgrade.Upgrade("structures/"+civCode+"_defense_tower");
+cmpUpgrade.Upgrade("structures/" + civCode + "/defense_tower");
 isResearched = true;
 
 // T4: Check that the player-less value hasn't increased...
-parsed_template = GetTemplateDataHelper(template, null, {}, Resources, DamageTypes);
+parsed_template = GetTemplateDataHelper(template, null, {});
 TS_ASSERT_UNEVAL_EQUALS(parsed_template.upgrades[0].cost, { "stone": 100, "wood": 50, "time": 100 });
 
 // T5: ...but the player-backed value has.
-parsed_template = GetTemplateDataHelper(template, playerID, {}, Resources, DamageTypes);
+parsed_template = GetTemplateDataHelper(template, playerID, {});
 TS_ASSERT_UNEVAL_EQUALS(parsed_template.upgrades[0].cost, { "stone": 160, "wood": 25, "time": 90 });
 
 // T6: The upgrade component should still be using the old resource cost (but new time cost) for the upgrade in progress...
