@@ -24,11 +24,16 @@ function Cheat(input)
 		cmpRangeManager.SetLosRevealAll(-1, true);
 		return;
 	case "maxpopulation":
-		cmpPlayer.SetPopulationBonuses(500);
+		cmpPlayer.SetPopulationBonuses((cmpPlayerManager.GetMaxWorldPopulation() || cmpPlayer.GetMaxPopulation()) + 500);
 		return;
 	case "changemaxpopulation":
-		cmpPlayer.SetMaxPopulation(500);
+	{
+		let cmpModifiersManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_ModifiersManager);
+		cmpModifiersManager.AddModifiers("cheat/maxpopulation", {
+			"Player/MaxPopulation": [{ "affects": ["Player"], "add": 500 }],
+		}, playerEnt);
 		return;
+	}
 	case "convertunit":
 		for (let ent of input.selected)
 		{
@@ -65,15 +70,33 @@ function Cheat(input)
 			return;
 		}
 
+		let owner = input.player;
+		let cmpOwnership = Engine.QueryInterface(input.selected[0], IID_Ownership);
+		if (cmpOwnership)
+			owner = cmpOwnership.GetOwner();
 		for (let i = 0; i < Math.min(input.parameter, cmpPlayer.GetMaxPopulation() - cmpPlayer.GetPopulationCount()); ++i)
-			cmpProductionQueue.SpawnUnits(input.templates[i % input.templates.length], 1, null);
+			cmpProductionQueue.SpawnUnits({
+				"count": 1,
+				"player": owner,
+				"metadata": null,
+				"unitTemplate": input.templates[i % input.templates.length]
+			});
 		return;
 	case "fastactions":
-		cmpPlayer.SetCheatTimeMultiplier((cmpPlayer.GetCheatTimeMultiplier() == 1) ? 0.01 : 1);
-		return;
-	case "changespeed":
-		cmpPlayer.SetCheatTimeMultiplier(input.parameter);
+	{
+		let cmpModifiersManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_ModifiersManager);
+		if (cmpModifiersManager.HasAnyModifier("cheat/fastactions", playerEnt))
+			cmpModifiersManager.RemoveAllModifiers("cheat/fastactions", playerEnt);
+		else
+			cmpModifiersManager.AddModifiers("cheat/fastactions", {
+				"Cost/BuildTime": [{ "affects": [["Structure"], ["Unit"]], "multiply": 0.01 }],
+				"ResourceGatherer/BaseSpeed": [{ "affects": [["Structure"], ["Unit"]], "multiply": 1000 }],
+				"Pack/Time": [{ "affects": [["Structure"], ["Unit"]], "multiply": 0.01 }],
+				"Upgrade/Time": [{ "affects": [["Structure"], ["Unit"]], "multiply": 0.01 }],
+				"ProductionQueue/TechCostMultiplier/time": [{ "affects": [["Structure"], ["Unit"]], "multiply": 0.01 }]
+			}, playerEnt);
 		return;
+	}
 	case "changephase":
 		var cmpTechnologyManager = Engine.QueryInterface(playerEnt, IID_TechnologyManager);
 		if (!cmpTechnologyManager)
