@@ -95,31 +95,36 @@ function LoadPlayerSettings(settings, newPlayers)
 			continue;
 		}
 
-		// Note: this is not yet implemented but I leave it commented to highlight it's easy
-		// If anyone ever adds handicap.
-		//if (getSetting(playerData, playerDefaults, i, "GatherRateMultiplier") !== undefined)
-		//	cmpPlayer.SetGatherRateMultiplier(getSetting(playerData, playerDefaults, i, "GatherRateMultiplier"));
+		// PopulationLimit
+		{
+			let maxPopulation =
+				settings.PlayerData[i].PopulationLimit !== undefined ?
+					settings.PlayerData[i].PopulationLimit :
+				settings.PopulationCap !== undefined ?
+					settings.PopulationCap :
+				playerDefaults[i].PopulationLimit !== undefined ?
+					playerDefaults[i].PopulationLimit :
+					undefined;
 
-		if (getSetting(playerData, playerDefaults, i, "PopulationLimit") !== undefined)
-			cmpPlayer.SetMaxPopulation(getSetting(playerData, playerDefaults, i, "PopulationLimit"));
+			if (maxPopulation !== undefined)
+				cmpPlayer.SetMaxPopulation(maxPopulation);
+		}
 
-		if (getSetting(playerData, playerDefaults, i, "Resources") !== undefined)
-			cmpPlayer.SetResourceCounts(getSetting(playerData, playerDefaults, i, "Resources"));
-
-		if (getSetting(playerData, playerDefaults, i, "StartingTechnologies") !== undefined)
-			cmpPlayer.SetStartingTechnologies(getSetting(playerData, playerDefaults, i, "StartingTechnologies"));
-
-		if (getSetting(playerData, playerDefaults, i, "DisabledTechnologies") !== undefined)
-			cmpPlayer.SetDisabledTechnologies(getSetting(playerData, playerDefaults, i, "DisabledTechnologies"));
-
-		let disabledTemplates = [];
-		if (settings.DisabledTemplates !== undefined)
-			disabledTemplates = settings.DisabledTemplates;
-		if (getSetting(playerData, playerDefaults, i, "DisabledTemplates") !== undefined)
-			disabledTemplates = disabledTemplates.concat(getSetting(playerData, playerDefaults, i, "DisabledTemplates"));
-		if (disabledTemplates.length)
-			cmpPlayer.SetDisabledTemplates(disabledTemplates);
+		// StartingResources
+		if (settings.PlayerData[i].Resources !== undefined)
+			cmpPlayer.SetResourceCounts(settings.PlayerData[i].Resources);
+		else if (settings.StartingResources)
+		{
+			let resourceCounts = cmpPlayer.GetResourceCounts();
+			let newResourceCounts = {};
+			for (let resouces in resourceCounts)
+				newResourceCounts[resouces] = settings.StartingResources;
+			cmpPlayer.SetResourceCounts(newResourceCounts);
+		}
+		else if (playerDefaults[i].Resources !== undefined)
+			cmpPlayer.SetResourceCounts(playerDefaults[i].Resources);
 
+		// DisableSpies
 		if (settings.DisableSpies)
 		{
 			cmpPlayer.AddDisabledTechnology("unlock_spies");
@@ -184,6 +189,31 @@ function GetPlayerTemplateName(civ)
 	return path;
 }
 
+/**
+ * @param id An entity's ID
+ * @returns The entity ID of the owner player (not his player ID) or ent if ent is a player entity.
+ */
+function QueryOwnerEntityID(ent)
+{
+	let cmpPlayer = Engine.QueryInterface(ent, IID_Player);
+	if (cmpPlayer)
+		return ent;
+
+	let cmpOwnership = Engine.QueryInterface(ent, IID_Ownership);
+	if (!cmpOwnership)
+		return null;
+
+	let owner = cmpOwnership.GetOwner();
+	if (owner == INVALID_PLAYER)
+		return null;
+
+	let cmpPlayerManager = Engine.QueryInterface(SYSTEM_ENTITY, IID_PlayerManager);
+	if (!cmpPlayerManager)
+		return null;
+
+	return cmpPlayerManager.GetPlayerByID(owner);
+}
+
 /**
  * Similar to Engine.QueryInterface but applies to the player entity
  * that owns the given entity.
@@ -224,13 +254,13 @@ function QueryPlayerIDInterface(id, iid = IID_Player)
  */
 function QueryMiragedInterface(ent, iid)
 {
-	var cmp = Engine.QueryInterface(ent, IID_Mirage);
-	if (cmp && !cmp.Mirages(iid))
+	let cmpMirage = Engine.QueryInterface(ent, IID_Mirage);
+	if (cmpMirage && !cmpMirage.Mirages(iid))
 		return null;
-	else if (!cmp)
-		cmp = Engine.QueryInterface(ent, iid);
+	else if (!cmpMirage)
+		return Engine.QueryInterface(ent, iid);
 
-	return cmp;
+	return cmpMirage.Get(iid);
 }
 
 /**
@@ -303,7 +333,7 @@ function IsOwnedByMutualAllyOfPlayer(player, target)
 	return IsOwnedByHelper(player, target, "IsMutualAlly");
 }
 
-function IsOwnedByNeutralOfPlayer(player,target)
+function IsOwnedByNeutralOfPlayer(player, target)
 {
 	return IsOwnedByHelper(player, target, "IsNeutral");
 }
@@ -326,6 +356,7 @@ function IsOwnedByHelper(player, target, check)
 }
 
 Engine.RegisterGlobal("LoadPlayerSettings", LoadPlayerSettings);
+Engine.RegisterGlobal("QueryOwnerEntityID", QueryOwnerEntityID);
 Engine.RegisterGlobal("QueryOwnerInterface", QueryOwnerInterface);
 Engine.RegisterGlobal("QueryPlayerIDInterface", QueryPlayerIDInterface);
 Engine.RegisterGlobal("QueryMiragedInterface", QueryMiragedInterface);
