@@ -22,20 +22,20 @@ const tCoralsUpper = "medit_sea_coral_plants";
 const tCoralsLower = "medit_sea_coral_deep";
 const tSeaDepths = "medit_sea_depths";
 
-const oBerryBush = "gaia/flora_bush_berry";
+const oBerryBush = "gaia/fruit/berry_01";
 const oDeer = "gaia/fauna_deer";
-const oFish = "gaia/fauna_fish";
+const oFish = "gaia/fish/generic";
 const oSheep = "gaia/fauna_sheep";
 const oGoat = "gaia/fauna_goat";
-const oStoneLarge = "gaia/geology_stonemine_medit_quarry";
-const oStoneSmall = "gaia/geology_stone_mediterranean";
-const oMetalLarge = "gaia/geology_metal_mediterranean_slabs";
-const oDatePalm = "gaia/flora_tree_cretan_date_palm_short";
-const oSDatePalm = "gaia/flora_tree_cretan_date_palm_tall";
-const oCarob = "gaia/flora_tree_carob";
-const oFanPalm = "gaia/flora_tree_medit_fan_palm";
-const oPoplar = "gaia/flora_tree_poplar_lombardy";
-const oCypress = "gaia/flora_tree_cypress";
+const oStoneLarge = "gaia/rock/mediterranean_large";
+const oStoneSmall = "gaia/rock/mediterranean_small";
+const oMetalLarge = "gaia/ore/mediterranean_large";
+const oDatePalm = "gaia/tree/cretan_date_palm_short";
+const oSDatePalm = "gaia/tree/cretan_date_palm_tall";
+const oCarob = "gaia/tree/carob";
+const oFanPalm = "gaia/tree/medit_fan_palm";
+const oPoplar = "gaia/tree/poplar_lombardy";
+const oCypress = "gaia/tree/cypress";
 
 const aBush1 = "actor|props/flora/bush_medit_sm.xml";
 const aBush2 = "actor|props/flora/bush_medit_me.xml";
