@@ -20,19 +20,19 @@ var tShore = "alpine_shore_rocks";
 var tWater = "polar_ice_b";
 var tHill = "polar_ice_cracked";
 
-var oBush = "gaia/flora_bush_badlands";
-var oBush2 = "gaia/flora_bush_temperate";
-var oBerryBush = "gaia/flora_bush_berry";
+var oBush = "gaia/tree/bush_badlands";
+var oBush2 = "gaia/tree/bush_temperate";
+var oBerryBush = "gaia/fruit/berry_01";
 var oRabbit = "gaia/fauna_rabbit";
 var oMuskox = "gaia/fauna_muskox";
 var oDeer = "gaia/fauna_deer";
 var oWolf = "gaia/fauna_wolf";
 var oWhaleFin = "gaia/fauna_whale_fin";
 var oWhaleHumpback = "gaia/fauna_whale_humpback";
-var oFish = "gaia/fauna_fish";
-var oStoneLarge = "gaia/geology_stonemine_alpine_quarry";
-var oStoneSmall = "gaia/geology_stone_alpine_a";
-var oMetalLarge = "gaia/geology_metal_alpine_slabs";
+var oFish = "gaia/fish/generic";
+var oStoneLarge = "gaia/rock/alpine_large";
+var oStoneSmall = "gaia/rock/alpine_small";
+var oMetalLarge = "gaia/ore/alpine_large";
 var oWoodTreasure = "gaia/treasure/wood";
 
 var aRockLarge = "actor|geology/stone_granite_med.xml";
@@ -312,8 +312,7 @@ placePlayersNomad(clPlayer, avoidClasses(clForest, 1, clWater, 4, clMetal, 4, cl
 setSkySet("sunset 1");
 setSunRotation(randomAngle());
 setSunColor(0.8, 0.7, 0.6);
-setTerrainAmbientColor(0.7, 0.6, 0.7);
-setUnitsAmbientColor(0.6, 0.5, 0.6);
+setAmbientColor(0.6, 0.5, 0.6);
 setSunElevation(Math.PI * randFloat(1/12, 1/7));
 setWaterColor(0, 0.047, 0.286);
 setWaterTint(0.462, 0.756, 0.866);
