@@ -1,33 +1,36 @@
 Engine.LoadLibrary("rmgen");
 Engine.LoadLibrary("rmgen-common");
+Engine.LoadLibrary("rmbiome");
 
-const tPrimary = ["temp_forestfloor_pine", "temp_forestfloor_pine", "alpine_cliff_c", "alpine_grass_rocky"];
-const tGrass = ["new_alpine_grass_b", "new_alpine_grass_c", "new_alpine_grass_d"];
-const tPineForestFloor = "temp_forestfloor_pine";
-const tForestFloor = [tPineForestFloor, tPineForestFloor, "alpine_dirt_grass_50"];
-const tCliff = ["alpine_cliff_c", "alpine_cliff_c", "alpine_grass_rocky"];
+setBiome("generic/temperate");
+
+const tPrimary = ["steppe_grass_03", "steppe_grass_03", "alpine_cliff_c", "temperate_grass_mud_01"];
+const tGrass = ["steppe_grass_04", "steppe_grass_04", "aegean_grass_dirt_03"];
+const tPineForestFloor = "steppe_grass_03";
+const tForestFloor = [tPineForestFloor, tPineForestFloor, "temperate_grass_mud_01"];
+const tCliff = ["alpine_cliff_c", "alpine_cliff_c", "temperate_cliff_01"];
 const tCity = ["new_alpine_citytile", "new_alpine_grass_dirt_a"];
-const tGrassPatch = ["alpine_grass_a", "alpine_grass_b"];
+const tGrassPatch = ["alpine_grass_d"];
 
 const oBoar = "gaia/fauna_boar";
 const oDeer = "gaia/fauna_deer";
-const oBear = "gaia/fauna_bear";
+const oBear = "gaia/fauna_bear_brown";
 const oPig = "gaia/fauna_pig";
-const oBerryBush = "gaia/flora_bush_berry";
-const oMetalSmall = "gaia/geology_metal_alpine";
-const oMetalLarge = "gaia/geology_metal_temperate_slabs";
-const oStoneSmall = "gaia/geology_stone_alpine_a";
-const oStoneLarge = "gaia/geology_stonemine_temperate_quarry";
+const oBerryBush = "gaia/fruit/berry_01";
+const oMetalSmall = "gaia/ore/alpine_small";
+const oMetalLarge = "gaia/ore/temperate_large";
+const oStoneSmall = "gaia/rock/alpine_small";
+const oStoneLarge = "gaia/rock/temperate_large";
 
-const oOak = "gaia/flora_tree_oak";
-const oOakLarge = "gaia/flora_tree_oak_large";
-const oPine = "gaia/flora_tree_pine";
-const oAleppoPine = "gaia/flora_tree_aleppo_pine";
+const oOak = "gaia/tree/oak";
+const oOakLarge = "gaia/tree/oak_large";
+const oPine = "gaia/tree/pine";
+const oAleppoPine = "gaia/tree/fir";
 
 const aTreeA = "actor|flora/trees/oak.xml";
 const aTreeB = "actor|flora/trees/oak_large.xml";
 const aTreeC = "actor|flora/trees/pine.xml";
-const aTreeD = "actor|flora/trees/aleppo_pine.xml";
+const aTreeD = "actor|flora/trees/fir_tree.xml";
 
 const aTrees = [aTreeA, aTreeB, aTreeC, aTreeD];
 
@@ -69,10 +72,11 @@ var clMetal = g_Map.createTileClass();
 var clFood = g_Map.createTileClass();
 var clBaseResource = g_Map.createTileClass();
 var clHillDeco = g_Map.createTileClass();
+var clExplorable = g_Map.createTileClass();
 
 g_Map.log("Creating the central dip");
 createArea(
-	new ClumpPlacer(diskArea(fractionToTiles(0.42)), 0.94, 0.05, 0.1, mapCenter),
+	new ClumpPlacer(diskArea(fractionToTiles(0.44)), 0.94, 0.05, 0.1, mapCenter),
 	[
 		new LayeredPainter([tCliff, tGrass], [3]),
 		new SmoothElevationPainter(ELEVATION_SET, heightLand, 3)
@@ -276,6 +280,8 @@ var explorableArea = createArea(
 		avoidClasses(clPlayer, 1)
 	]);
 
+new TileClassPainter(clExplorable).paint(explorableArea);
+
 Engine.SetProgress(55);
 
 // Add some general noise - after placing height dependant trees
@@ -296,23 +302,23 @@ for (var ix = 0; ix < mapSize; ix++)
 Engine.SetProgress(60);
 
 g_Map.log("Creating forests");
-var [forestTrees, stragglerTrees] = getTreeCounts(400, 6000, 0.8);
+var [forestTrees, stragglerTrees] = getTreeCounts(1300, 8000, 0.8);
 var [forestTreesJoin, forestTrees] = getTreeCounts(forestTrees, forestTrees, 0.25);
 
-var num = forestTrees / (scaleByMapSize(6, 16) * numPlayers);
+var num = forestTrees / scaleByMapSize(20, 70);
 createAreasInAreas(
 	new ClumpPlacer(forestTrees / num, 0.1, 0.1, Infinity),
 	[
 		new TerrainPainter(pForest),
 		new TileClassPainter(clForest)
 	],
-	avoidClasses(clPlayer, 5, clBaseResource, 4, clForest, 6, clHill, 4),
+	avoidClasses(clPlayer, 5, clBaseResource, 4, clForest, 5, clHill, 4),
 	num,
 	100,
 	[explorableArea]
 );
 
-var num = forestTreesJoin / (scaleByMapSize(4,6) * numPlayers);
+var num = forestTreesJoin / (scaleByMapSize(4, 6) * numPlayers);
 createAreasInAreas(
 	new ClumpPlacer(forestTreesJoin / num, 0.1, 0.1, Infinity),
 	[
@@ -346,40 +352,27 @@ for (let size of [scaleByMapSize(20, 120)])
 
 Engine.SetProgress(75);
 
-g_Map.log("Creating stone mines");
-createObjectGroupsByAreasDeprecated(
-	new SimpleGroup(
-		[
-			new SimpleObject(oStoneSmall, 1, 2, 0, 4, 0, 2 * Math.PI, 1),
-			new SimpleObject(oStoneLarge, 0, 1, 0, 4, 0, 2 * Math.PI, 4)
-		],
-		true,
-		clRock),
-	0,
-	[avoidClasses(clHill, 4, clForest, 2, clPlayer, 20, clRock, 10)],
-	scaleByMapSize(6,20), 100,
-	[explorableArea]);
-
-g_Map.log("Creating small stone mines");
-var group = new SimpleGroup([new SimpleObject(oStoneSmall, 2, 5, 1, 3)], true, clRock);
-createObjectGroupsByAreasDeprecated(group, 0,
-	[avoidClasses(clHill, 4, clForest, 2, clPlayer, 20, clRock, 10)],
-	scaleByMapSize(6,20), 100,
-	[explorableArea]
+g_Map.log("Creating metal mines");
+createBalancedMetalMines(
+	oMetalSmall,
+	oMetalLarge,
+	clMetal,
+	[stayClasses(clExplorable, 1), avoidClasses(clForest, 0, clPlayer, scaleByMapSize(15, 25), clHill, 1)]
 );
 
-g_Map.log("Creating metal mines");
-group = new SimpleGroup([new SimpleObject(oMetalSmall, 1,2, 0,4), new SimpleObject(oMetalLarge, 0,1, 0,4)], true, clMetal);
-createObjectGroupsByAreasDeprecated(group, 0,
-	[avoidClasses(clHill, 4, clForest, 2, clPlayer, 20, clMetal, 10, clRock, 5)],
-	scaleByMapSize(6,20), 100,
-	[explorableArea]
+
+g_Map.log("Creating stone mines");
+createBalancedStoneMines(
+	oStoneSmall,
+	oStoneLarge,
+	clRock,
+	[stayClasses(clExplorable, 1), avoidClasses(clForest, 0, clPlayer, scaleByMapSize(15, 25), clHill, 1, clMetal, 10)]
 );
 
 Engine.SetProgress(80);
 
 g_Map.log("Creating wildlife");
-group = new SimpleGroup(
+let group = new SimpleGroup(
 	[new SimpleObject(oDeer, 5,7, 0,4)],
 	true, clFood
 );
@@ -466,7 +459,4 @@ createObjectGroupsByAreasDeprecated(group, 0,
 
 placePlayersNomad(clPlayer, avoidClasses(clForest, 1, clMetal, 4, clRock, 4, clHill, 4, clFood, 2));
 
-setTerrainAmbientColor(0.44,0.51,0.56);
-setUnitsAmbientColor(0.44,0.51,0.56);
-
 g_Map.ExportMap();
