@@ -6,7 +6,7 @@ Engine.LoadLibrary("heightmap");
 var tGrove = "temp_grass_plants";
 var tPath = "road_rome_a";
 
-var oGroveEntities = ["structures/gaul_outpost", "gaia/flora_tree_oak_new"];
+var oGroveEntities = ["structures/gaul/outpost", "gaia/tree/oak_new"];
 
 var g_Map = new RandomMap(0, "whiteness");
 
@@ -39,12 +39,12 @@ function placeMine(point, centerEntity)
 // Food, fences with domestic animals
 g_WallStyles.other = {
 	"overlap": 0,
-	"fence": readyWallElement("other/fence_long", "gaia"),
-	"fence_short": readyWallElement("other/fence_short", "gaia"),
-	"bench":     { "angle": Math.PI / 2, "length": 1.5,  "indent": 0,    "bend": 0, "templateName": "other/bench" },
-	"sheep":     { "angle": 0,           "length": 0,    "indent": 0.75, "bend": 0, "templateName": "gaia/fauna_sheep" },
-	"foodBin":   { "angle": Math.PI / 2, "length": 1.5,  "indent": 0,    "bend": 0, "templateName": "gaia/treasure/food_bin" },
-	"farmstead": { "angle": Math.PI,     "length": 0,    "indent": -3,   "bend": 0, "templateName": "structures/brit_farmstead" }
+	"fence": readyWallElement("structures/fence_long", "gaia"),
+	"fence_short": readyWallElement("structures/fence_short", "gaia"),
+	"bench": { "angle": Math.PI / 2, "length": 1.5, "indent": 0, "bend": 0, "templateName": "structures/bench" },
+	"sheep": { "angle": 0, "length": 0, "indent": 0.75, "bend": 0, "templateName": "gaia/fauna_sheep" },
+	"foodBin": { "angle": Math.PI / 2, "length": 1.5, "indent": 0, "bend": 0, "templateName": "gaia/treasure/food_bin" },
+	"farmstead": { "angle": Math.PI, "length": 0, "indent": -3, "bend": 0, "templateName": "structures/brit/farmstead" }
 };
 
 let fences = [
@@ -83,8 +83,8 @@ let num = fences.length;
 for (let i = 0; i < num; ++i)
 	fences.push(new Fortress("fence", clone(fences[i].wall).reverse()));
 
-// Groves, only Wood
-let groveEntities = ["gaia/flora_bush_temperate", "gaia/flora_tree_euro_beech"];
+// Groves, only wood
+let groveEntities = ["gaia/tree/bush_temperate", "gaia/tree/euro_beech"];
 let groveActors = [
 	"actor|geology/highland1_moss.xml", "actor|geology/highland2_moss.xml",
 	"actor|props/flora/bush.xml", "actor|props/flora/bush_dry_a.xml", "actor|props/flora/bush_highlands.xml",
@@ -119,7 +119,7 @@ function placeGrove(point)
 function placeCamp(point,
 	centerEntity = "actor|props/special/eyecandy/campfire.xml",
 	otherEntities = ["gaia/treasure/metal", "gaia/treasure/standing_stone",
-		"units/brit_infantry_slinger_b", "units/brit_infantry_javelinist_b", "units/gaul_infantry_slinger_b", "units/gaul_infantry_javelinist_b", "units/gaul_champion_fanatic",
+		"units/brit/infantry_slinger_b", "units/brit/infantry_javelineer_b", "units/gaul/infantry_slinger_b", "units/gaul/infantry_javelineer_b", "units/gaul/champion_fanatic",
 		"actor|props/special/common/waypoint_flag.xml", "actor|props/special/eyecandy/barrel_a.xml", "actor|props/special/eyecandy/basket_celt_a.xml", "actor|props/special/eyecandy/crate_a.xml", "actor|props/special/eyecandy/dummy_a.xml", "actor|props/special/eyecandy/handcart_1.xml", "actor|props/special/eyecandy/handcart_1_broken.xml", "actor|props/special/eyecandy/sack_1.xml", "actor|props/special/eyecandy/sack_1_rough.xml"
 	]
 )
@@ -135,14 +135,14 @@ function placeCamp(point,
 	}
 }
 
-function placeStartLocationResources(point, foodEntities = ["gaia/flora_bush_berry", "gaia/fauna_chicken", "gaia/fauna_chicken"])
+function placeStartLocationResources(point, foodEntities = ["gaia/fruit/berry_01", "gaia/fauna_chicken", "gaia/fauna_chicken"])
 {
 	let currentAngle = randomAngle();
 	// Stone and chicken
 	let dAngle = 4/9 * Math.PI;
 	let angle = currentAngle + randFloat(1, 3) * dAngle / 4;
 	let stonePosition = Vector2D.add(point, new Vector2D(12, 0).rotate(-angle));
-	placeMine(stonePosition, "gaia/geology_stonemine_temperate_quarry");
+	placeMine(stonePosition, "gaia/rock/temperate_large");
 
 	currentAngle += dAngle;
 
@@ -170,7 +170,7 @@ function placeStartLocationResources(point, foodEntities = ["gaia/flora_bush_ber
 	dAngle = 2 * Math.PI * 2 / 9;
 	angle = currentAngle + dAngle * randFloat(1, 3) / 4;
 	let metalPosition = Vector2D.add(point, new Vector2D(13, 0).rotate(-angle));
-	placeMine(metalPosition, "gaia/geology_metal_temperate_slabs");
+	placeMine(metalPosition, "gaia/ore/temperate_large");
 	currentAngle += dAngle;
 
 	// Berries
@@ -190,7 +190,7 @@ function placeStartLocationResources(point, foodEntities = ["gaia/flora_bush_ber
  */
 setBiome("generic/alpine");
 g_Environment.Fog.FogColor = { "r": 0.8, "g": 0.8, "b": 0.8, "a": 0.01 };
-g_Environment.Water.WaterBody.Colour = { "r" : 0.3, "g" : 0.05, "b" : 0.1, "a" : 0.1 };
+g_Environment.Water.WaterBody.Colour = { "r": 0.3, "g": 0.05, "b": 0.1, "a": 0.1 };
 g_Environment.Water.WaterBody.Murkiness = 0.4;
 
 /**
@@ -238,8 +238,8 @@ g_Map.log("Determining height-dependent biome");
 let myBiome = [];
 myBiome.push({ // 0 Deep water
 	"texture": ["shoreline_stoney_a"],
-	"actor": [["gaia/fauna_fish", "actor|geology/stone_granite_boulder.xml"], 0.02],
-	"textureHS": ["alpine_mountainside"], "actorHS": [["gaia/fauna_fish"], 0.1]
+	"actor": [["gaia/fish/generic", "actor|geology/stone_granite_boulder.xml"], 0.02],
+	"textureHS": ["alpine_mountainside"], "actorHS": [["gaia/fish/generic"], 0.1]
 });
 myBiome.push({ // 1 Medium Water
 	"texture": ["shoreline_stoney_a", "alpine_shore_rocks"],
@@ -253,7 +253,7 @@ myBiome.push({ // 2 Shallow water
 });
 myBiome.push({ // 3 Shore
 	"texture": ["alpine_shore_rocks_grass_50", "alpine_grass_rocky"],
-	"actor": [["gaia/flora_tree_pine", "gaia/flora_bush_badlands", "actor|geology/highland1_moss.xml", "actor|props/flora/grass_soft_tuft_a.xml", "actor|props/flora/bush.xml"], 0.3],
+	"actor": [["gaia/tree/pine", "gaia/tree/bush_badlands", "actor|geology/highland1_moss.xml", "actor|props/flora/grass_soft_tuft_a.xml", "actor|props/flora/bush.xml"], 0.3],
 	"textureHS": ["alpine_mountainside"], "actorHS": [["actor|props/flora/grass_soft_tuft_a.xml"], 0.1]
 });
 myBiome.push({ // 4 Low ground
@@ -273,17 +273,17 @@ myBiome.push({ // 6 High ground
 });
 myBiome.push({ // 7 Lower forest border
 	"texture": ["new_alpine_grass_mossy", "alpine_grass_rocky"],
-	"actor": [["gaia/flora_tree_pine", "gaia/flora_tree_oak", "actor|props/flora/grass_tufts_a.xml", "gaia/flora_bush_berry", "actor|geology/highland2_moss.xml", "gaia/fauna_goat", "actor|props/flora/bush_tempe_underbrush.xml"], 0.3],
+	"actor": [["gaia/tree/pine", "gaia/tree/oak", "actor|props/flora/grass_tufts_a.xml", "gaia/fruit/berry_01", "actor|geology/highland2_moss.xml", "gaia/fauna_goat", "actor|props/flora/bush_tempe_underbrush.xml"], 0.3],
 	"textureHS": ["alpine_cliff_c"], "actorHS": [["actor|props/flora/grass_tufts_a.xml", "actor|geology/highland2_moss.xml"], 0.1]
 });
 myBiome.push({ // 8 Forest
 	"texture": ["alpine_forrestfloor"],
-	"actor": [["gaia/flora_tree_pine", "gaia/flora_tree_pine", "gaia/flora_tree_pine", "gaia/flora_tree_pine", "actor|geology/highland2_moss.xml", "actor|props/flora/bush_highlands.xml"], 0.5],
+	"actor": [["gaia/tree/pine", "gaia/tree/pine", "gaia/tree/pine", "gaia/tree/pine", "actor|geology/highland2_moss.xml", "actor|props/flora/bush_highlands.xml"], 0.5],
 	"textureHS": ["alpine_cliff_c"], "actorHS": [["actor|geology/highland2_moss.xml", "actor|geology/stone_granite_med.xml"], 0.1]
 });
 myBiome.push({ // 9 Upper forest border
 	"texture": ["alpine_forrestfloor_snow", "new_alpine_grass_dirt_a"],
-	"actor": [["gaia/flora_tree_pine", "actor|geology/snow1.xml"], 0.3],
+	"actor": [["gaia/tree/pine", "actor|geology/snow1.xml"], 0.3],
 	"textureHS": ["alpine_cliff_b"], "actorHS": [["actor|geology/stone_granite_med.xml", "actor|geology/snow1.xml"], 0.1]
 });
 myBiome.push({ // 10 Hilltop
@@ -416,9 +416,9 @@ for (let i = 0; i < resourceSpots.length; ++i)
 	let pos = new Vector2D(resourceSpots[i].x, resourceSpots[i].y);
 	let choice = i % (isNomad() ? 4 : 5);
 	if (choice == 0)
-		placeMine(pos, "gaia/geology_stonemine_temperate_formation");
+		placeMine(pos, "gaia/rock/temperate_large_02");
 	if (choice == 1)
-		placeMine(pos, "gaia/geology_metal_temperate_slabs");
+		placeMine(pos, "gaia/ore/temperate_large");
 	if (choice == 2)
 		placeCustomFortress(pos, pickRandom(fences), "other", 0, randomAngle());
 	if (choice == 3)
