@@ -30,19 +30,19 @@ const tGrass = ["savanna_shrubs_a_wetseason", "alpine_grass_b_wild", "medit_shru
 const tForestFloorLand = "savanna_forestfloor_b_red";
 const tForestFloorIsland = pickRandom(tGrass);
 
-const oAcacia = "gaia/flora_tree_acacia";
+const oAcacia = "gaia/tree/acacia";
 const oPalms = [
-	"gaia/flora_tree_cretan_date_palm_tall",
-	"gaia/flora_tree_cretan_date_palm_short",
-	"gaia/flora_tree_palm_tropic",
-	"gaia/flora_tree_date_palm",
-	"gaia/flora_tree_senegal_date_palm",
-	"gaia/flora_tree_medit_fan_palm"
+	"gaia/tree/cretan_date_palm_tall",
+	"gaia/tree/cretan_date_palm_short",
+	"gaia/tree/palm_tropic",
+	"gaia/tree/date_palm",
+	"gaia/tree/senegal_date_palm",
+	"gaia/tree/medit_fan_palm"
 ];
-const oStoneLarge = "gaia/geology_stonemine_savanna_quarry";
-const oStoneSmall = "gaia/geology_stone_desert_small";
-const oMetalLarge = "gaia/geology_metal_savanna_slabs";
-const oMetalSmall = "gaia/geology_metal_desert_small";
+const oStoneLarge = "gaia/rock/savanna_large";
+const oStoneSmall = "gaia/rock/desert_small";
+const oMetalLarge = "gaia/ore/savanna_large";
+const oMetalSmall = "gaia/ore/desert_small";
 
 const oTreasure = [
 	"gaia/treasure/food_barrel",
@@ -51,9 +51,9 @@ const oTreasure = [
 	"gaia/treasure/metal",
 	"gaia/treasure/stone"
 ];
-const oBerryBush = "gaia/flora_bush_berry_desert";
+const oBerryBush = "gaia/fruit/berry_05";
 const oGazelle = "gaia/fauna_gazelle";
-const oRhino = "gaia/fauna_rhino";
+const oRhino = "gaia/fauna_rhinoceros_white";
 const oWarthog = "gaia/fauna_boar";
 const oGiraffe = "gaia/fauna_giraffe";
 const oGiraffeInfant = "gaia/fauna_giraffe_infant";
@@ -61,19 +61,19 @@ const oElephant = "gaia/fauna_elephant_african_bush";
 const oElephantInfant = "gaia/fauna_elephant_african_infant";
 const oLion = "gaia/fauna_lion";
 const oLioness = "gaia/fauna_lioness";
-const oCrocodile = "gaia/fauna_crocodile";
-const oFish = "gaia/fauna_fish_tilapia";
-const oHawk = "gaia/fauna_hawk";
+const oCrocodile = "gaia/fauna_crocodile_nile";
+const oFish = "gaia/fish/tilapia";
+const oHawk = "birds/buzzard";
 
 // The main temple on elephantine was very similar looking (Greco-Roman-Egyptian):
-const oWonder = "structures/ptol_wonder";
-const oTemples = ["structures/kush_temple_amun", "structures/kush_temple"];
-const oPyramid = "structures/kush_pyramid_large";
-const oTowers = new Array(2).fill("uncapturable|structures/kush_sentry_tower").concat(["uncapturable|structures/kush_defense_tower"]);
+const oWonder = "structures/ptol/wonder";
+const oTemples = ["structures/kush/temple_amun", "structures/kush/temple"];
+const oPyramid = "structures/kush/pyramid_large";
+const oTowers = new Array(2).fill("uncapturable|structures/kush/sentry_tower").concat(["uncapturable|structures/kush/defense_tower"]);
 
-const oHeroes = Engine.FindTemplates("units/", true).filter(templateName => templateName.startsWith("units/kush_hero_"));
-const oUnits = Engine.FindTemplates("units/", false).filter(templateName =>
-	templateName.startsWith("units/kush_") &&
+const oHeroes = Engine.FindTemplates("units/kush/", true).filter(templateName => templateName.startsWith("units/kush/hero_"));
+const oUnits = Engine.FindTemplates("units/kush/", false).filter(templateName =>
+	templateName.startsWith("units/kush/") &&
 	oHeroes.every(heroTemplateName => heroTemplateName != templateName) &&
 	Engine.GetTemplate(templateName).Identity.VisibleClasses._string.split(" ").some(type => ["Soldier", "Healer", "Female"].indexOf(type) != -1));
 
@@ -554,13 +554,12 @@ g_Environment = {
 	},
 	"SunElevation": 0.908117,
 	"SunRotation": -0.558369,
-	"TerrainAmbientColor": {
+	"AmbientColor": {
 		"r": 0.54902,
 		"g": 0.419608,
 		"b": 0.352941,
 		"a": 0
 	},
-	"UnitsAmbientColor": { "r": 0.721569, "g": 0.529412, "b": 0.4, "a": 0 },
 	"Fog": {
 		"FogFactor": 0.00195313,
 		"FogThickness": 0,
