@@ -13,16 +13,16 @@ const tGrassPatch = "temp_grass_plants_aut";
 const tShore = "temp_plants_bog_aut";
 const tWater = "temp_mud_a";
 
-const oBeech = "gaia/flora_tree_euro_beech_aut";
-const oOak = "gaia/flora_tree_oak_aut";
-const oPine = "gaia/flora_tree_pine";
+const oBeech = "gaia/tree/euro_beech_aut";
+const oOak = "gaia/tree/oak_aut";
+const oPine = "gaia/tree/pine";
 const oDeer = "gaia/fauna_deer";
-const oFish = "gaia/fauna_fish";
+const oFish = "gaia/fish/generic";
 const oSheep = "gaia/fauna_rabbit";
-const oBerryBush = "gaia/flora_bush_berry";
-const oStoneLarge = "gaia/geology_stonemine_temperate_quarry";
-const oStoneSmall = "gaia/geology_stone_temperate";
-const oMetalLarge = "gaia/geology_metal_temperate_slabs";
+const oBerryBush = "gaia/fruit/berry_01";
+const oStoneLarge = "gaia/rock/temperate_large";
+const oStoneSmall = "gaia/rock/temperate_small";
+const oMetalLarge = "gaia/ore/temperate_large";
 const oFoodTreasure = "gaia/treasure/food_bin";
 const oWoodTreasure = "gaia/treasure/wood";
 const oStoneTreasure = "gaia/treasure/stone";
@@ -109,11 +109,11 @@ for (let i = 0; i < numPlayers; ++i)
 
 	// Fortress
 	// To take into account houses that offer a lower population bonus
-	if (Engine.GetTemplate("structures/" + civ + "_house").Cost.PopulationBonus <= 5)
+	if (Engine.GetTemplate("structures/" + civ + "/house").Population.Bonus <= 5)
 	{
 		var wall = ["gate", "tower", "long",
-			"cornerIn", "long", "barracks", "tower", "long", "tower", "house", "long",
-			"cornerIn", "long", "house", "tower", "gate", "tower", "house", "long",
+			"cornerIn", "long", "barracks", "tower", "long", "tower", "long",
+			"cornerIn", "long", "stable", "tower", "gate", "tower", "house", "long",
 			"cornerIn", "long", "house", "tower", "long", "tower", "house", "long",
 			"cornerIn", "long", "house", "tower"];
 	}
@@ -121,7 +121,7 @@ for (let i = 0; i < numPlayers; ++i)
 	{
 		var wall = ["gate", "tower", "long",
 			"cornerIn", "long", "barracks", "tower", "long", "tower", "long",
-			"cornerIn", "long", "house", "tower", "gate", "tower", "long",
+			"cornerIn", "long", "stable", "tower", "gate", "tower", "long",
 			"cornerIn", "long", "house", "tower", "long", "tower", "long",
 			"cornerIn", "long", "house", "tower"];
 	}
