@@ -261,7 +261,7 @@ Engine.SetProgress(65);
 
 var planetm = 1;
 
-if (currentBiome() == "generic/tropic")
+if (currentBiome() == "generic/india")
 	planetm = 8;
 
 createDecoration(
