@@ -13,16 +13,16 @@ const tGrassPatch = "medit_grass_shrubs";
 const tShore = "sand_grass_25";
 const tWater = "medit_sand_wet";
 
-const oPoplar = "gaia/flora_tree_poplar";
-const oApple = "gaia/flora_tree_apple";
-const oCarob = "gaia/flora_tree_carob";
-const oBerryBush = "gaia/flora_bush_berry";
+const oPoplar = "gaia/tree/poplar";
+const oApple = "gaia/fruit/apple";
+const oCarob = "gaia/tree/carob";
+const oBerryBush = "gaia/fruit/berry_01";
 const oDeer = "gaia/fauna_deer";
-const oFish = "gaia/fauna_fish";
+const oFish = "gaia/fish/generic";
 const oSheep = "gaia/fauna_sheep";
-const oStoneLarge = "gaia/geology_stonemine_medit_quarry";
-const oStoneSmall = "gaia/geology_stone_mediterranean";
-const oMetalLarge = "gaia/geology_metal_mediterranean_slabs";
+const oStoneLarge = "gaia/rock/mediterranean_large";
+const oStoneSmall = "gaia/rock/mediterranean_small";
+const oMetalLarge = "gaia/ore/mediterranean_large";
 
 const aGrass = "actor|props/flora/grass_soft_large_tall.xml";
 const aGrassShort = "actor|props/flora/grass_soft_large.xml";
