@@ -22,26 +22,26 @@ const tGrassTransition2 = "steppe_grass_dirt_66";
 const tPath = "road2";
 const tPathWild = "road_med";
 
-const oAcacia = "gaia/flora_tree_acacia";
-const oPalmPath = "gaia/flora_tree_cretan_date_palm_tall";
+const oAcacia = "gaia/tree/acacia";
+const oPalmPath = "gaia/tree/cretan_date_palm_tall";
 const oPalms = [
-	"gaia/flora_tree_cretan_date_palm_tall",
-	"gaia/flora_tree_cretan_date_palm_short",
-	"gaia/flora_tree_palm_tropic",
-	"gaia/flora_tree_date_palm",
-	"gaia/flora_tree_senegal_date_palm",
-	"gaia/flora_tree_medit_fan_palm"
+	"gaia/tree/cretan_date_palm_tall",
+	"gaia/tree/cretan_date_palm_short",
+	"gaia/tree/palm_tropic",
+	"gaia/tree/date_palm",
+	"gaia/tree/senegal_date_palm",
+	"gaia/tree/medit_fan_palm"
 ];
-const oBerryBushGrapes = "gaia/flora_bush_grapes";
-const oBerryBushDesert = "gaia/flora_bush_berry_desert";
-const oStoneLargeDesert = "gaia/geology_stonemine_desert_quarry";
-const oStoneSmallDesert = "gaia/geology_stone_desert_small";
-const oMetalLargeDesert = "gaia/geology_metal_desert_slabs";
-const oMetalSmallDesert = "gaia/geology_metal_desert_small";
-const oStoneLargeFertileLand = "gaia/geology_stonemine_desert_quarry";
-const oStoneSmallFertileLand = "gaia/geology_stone_greek";
-const oMetalLargeFertileLand = "gaia/geology_metal_desert_slabs";
-const oMetalSmallFertileLand = "gaia/geology_metal_temperate";
+const oBerryBushGrapes = "gaia/fruit/grapes";
+const oBerryBushDesert = "gaia/fruit/berry_05";
+const oStoneLargeDesert = "gaia/rock/desert_large";
+const oStoneSmallDesert = "gaia/rock/desert_small";
+const oMetalLargeDesert = "gaia/ore/desert_large";
+const oMetalSmallDesert = "gaia/ore/desert_small";
+const oStoneLargeFertileLand = "gaia/rock/desert_large";
+const oStoneSmallFertileLand = "gaia/rock/greece_small";
+const oMetalLargeFertileLand = "gaia/ore/desert_large";
+const oMetalSmallFertileLand = "gaia/ore/temperate_small";
 const oFoodTreasureBin = "gaia/treasure/food_bin";
 const oFoodTreasureCrate = "gaia/treasure/food_crate";
 const oFoodTreasureJars = "gaia/treasure/food_jars";
@@ -53,46 +53,46 @@ const oTreasuresCity = [oFoodTreasureBin, oFoodTreasureCrate, oFoodTreasureJars]
 const oGiraffe = "gaia/fauna_giraffe";
 const oGiraffeInfant = "gaia/fauna_giraffe_infant";
 const oGazelle = "gaia/fauna_gazelle";
-const oRhino = "gaia/fauna_rhino";
+const oRhino = "gaia/fauna_rhinoceros_white";
 const oWarthog = "gaia/fauna_boar";
 const oElephant = "gaia/fauna_elephant_african_bush";
 const oElephantInfant = "gaia/fauna_elephant_african_infant";
 const oLion = "gaia/fauna_lion";
 const oLioness = "gaia/fauna_lioness";
-const oCrocodile = "gaia/fauna_crocodile";
-const oFish = "gaia/fauna_fish_tilapia";
-const oHawk = "gaia/fauna_hawk";
-const oTempleApedemak = "structures/kush_temple";
-const oTempleAmun = "structures/kush_temple_amun";
-const oPyramidLarge = "structures/kush_pyramid_large";
-const oPyramidSmall = "structures/kush_pyramid_small";
-const oWonderPtol = "structures/ptol_wonder";
-const oFortress = "structures/kush_fortress";
-const oTower = g_MapSettings.Size >= 256 && getDifficulty() >= 3 ? "structures/kush_defense_tower" : "structures/kush_sentry_tower";
-const oHouse = "structures/kush_house";
-const oMarket = "structures/kush_market";
-const oBlacksmith = "structures/kush_blacksmith";
-const oBlemmyeCamp = "structures/kush_blemmye_camp";
-const oNubaVillage = "structures/kush_nuba_village";
-const oCivicCenter = "structures/kush_civil_centre";
-const oBarracks = "structures/kush_barracks";
-const oStable = "structures/kush_stable";
-const oElephantStables = "structures/kush_elephant_stables";
-const oWallMedium = "structures/kush_wall_medium";
-const oWallGate = "structures/kush_wall_gate";
-const oWallTower = "structures/kush_wall_tower";
-const oPalisadeMedium = "other/palisades_rocks_medium";
-const oPalisadeGate = "other/palisades_rocks_gate";
-const oPalisadeTower = "other/palisades_rocks_tower";
-const oKushCitizenArcher = "units/kush_infantry_archer_b";
-const oKushHealer = "units/kush_support_healer_b";
-const oKushChampionArcher = "units/kush_champion_infantry";
+const oCrocodile = "gaia/fauna_crocodile_nile";
+const oFish = "gaia/fish/tilapia";
+const oHawk = "birds/buzzard";
+const oTempleApedemak = "structures/kush/temple";
+const oTempleAmun = "structures/kush/temple_amun";
+const oPyramidLarge = "structures/kush/pyramid_large";
+const oPyramidSmall = "structures/kush/pyramid_small";
+const oWonderPtol = "structures/ptol/wonder";
+const oFortress = "structures/kush/fortress";
+const oTower = g_MapSettings.Size >= 256 && getDifficulty() >= 3 ? "structures/kush/defense_tower" : "structures/kush/sentry_tower";
+const oHouse = "structures/kush/house";
+const oMarket = "structures/kush/market";
+const oForge = "structures/kush/forge";
+const oBlemmyeCamp = "structures/kush/camp_blemmye";
+const oNobaCamp = "structures/kush/camp_noba";
+const oCivicCenter = "structures/kush/civil_centre";
+const oBarracks = "structures/kush/barracks";
+const oStable = "structures/kush/stable";
+const oElephantStable = "structures/kush/elephant_stable";
+const oWallMedium = "structures/kush/wall_medium";
+const oWallGate = "structures/kush/wall_gate";
+const oWallTower = "structures/kush/wall_tower";
+const oPalisadeMedium = "structures/palisades_medium";
+const oPalisadeGate = "structures/palisades_gate";
+const oPalisadeTower = "structures/palisades_tower";
+const oKushCitizenArcher = "units/kush/infantry_archer_b";
+const oKushHealer = "units/kush/support_healer_b";
+const oKushChampionArcher = "units/kush/champion_infantry_archer";
 const oKushChampions = [
 	oKushChampionArcher,
-	"units/kush_champion_infantry_amun",
-	"units/kush_champion_infantry_apedemak"
+	"units/kush/champion_infantry_amun",
+	"units/kush/champion_infantry_apedemak"
 ];
-const oPtolSiege = ["units/ptol_mechanical_siege_lithobolos_unpacked", "units/ptol_mechanical_siege_polybolos_unpacked"];
+const oPtolSiege = ["units/ptol/siege_lithobolos_unpacked", "units/ptol/siege_polybolos_unpacked"];
 const oTriggerPointCityPath = "trigger/trigger_point_A";
 const oTriggerPointAttackerPatrol = "trigger/trigger_point_B";
 
@@ -181,13 +181,13 @@ const clTemple = g_Map.createTileClass();
 const clRitualPlace = g_Map.createTileClass();
 const clPyramid = g_Map.createTileClass();
 const clHouse = g_Map.createTileClass();
-const clBlacksmith = g_Map.createTileClass();
+const clForge = g_Map.createTileClass();
 const clStable = g_Map.createTileClass();
-const clElephantStables = g_Map.createTileClass();
+const clElephantStable = g_Map.createTileClass();
 const clCivicCenter = g_Map.createTileClass();
 const clBarracks = g_Map.createTileClass();
 const clBlemmyeCamp = g_Map.createTileClass();
-const clNubaVillage = g_Map.createTileClass();
+const clNobaCamp = g_Map.createTileClass();
 const clMarket = g_Map.createTileClass();
 const clDecorative = g_Map.createTileClass();
 
@@ -270,10 +270,10 @@ const layoutKushCity = [
 		"painters": new TileClassPainter(clCivicCenter)
 	},
 	{
-		"templateName": oElephantStables,
+		"templateName": oElephantStable,
 		"difficulty": "Easy",
-		"constraints": avoidClasses(clElephantStables, 10),
-		"painters": new TileClassPainter(clElephantStables)
+		"constraints": avoidClasses(clElephantStable, 10),
+		"painters": new TileClassPainter(clElephantStable)
 	},
 	{
 		"templateName": oStable,
@@ -300,16 +300,16 @@ const layoutKushCity = [
 		"painters": new TileClassPainter(clMarket)
 	},
 	{
-		"templateName": "uncapturable|" + oBlacksmith,
+		"templateName": "uncapturable|" + oForge,
 		"difficulty": "Very Easy",
-		"constraints": avoidClasses(clBlacksmith, 30),
-		"painters": new TileClassPainter(clBlacksmith)
+		"constraints": avoidClasses(clForge, 30),
+		"painters": new TileClassPainter(clForge)
 	},
 	{
-		"templateName": oNubaVillage,
+		"templateName": oNobaCamp,
 		"difficulty": "Easy",
-		"constraints": avoidClasses(clNubaVillage, 30),
-		"painters": new TileClassPainter(clNubaVillage)
+		"constraints": avoidClasses(clNobaCamp, 30),
+		"painters": new TileClassPainter(clNobaCamp)
 	},
 	{
 		"templateName": oBlemmyeCamp,
@@ -665,7 +665,7 @@ Engine.SetProgress(60);
 // The city is a circle segment of this maximum size
 g_Map.log("Computing city grid");
 var gridCenter = new Vector2D(0, fractionToTiles(0.3)).rotate(-riverAngle).add(mapCenter).round();
-var gridMaxAngle = scaleByMapSize(Math.PI / 3, Math.PI);
+var gridMaxAngle = Math.min(scaleByMapSize(1/3, 1), 2/3) * Math.PI;
 var gridStartAngle = -Math.PI / 2 -gridMaxAngle / 2 + riverAngle;
 var gridRadius = y => hillRadius + 18 * y;
 
@@ -976,7 +976,7 @@ var areaCityPaths = new Area(areasCityPaths.reduce((points, area) => points.conc
 var areaRoads = [];
 for (let roadStart of roadStartLocations)
 {
-	if (areaRoads.length >= scaleByMapSize(2, 4))
+	if (areaRoads.length >= scaleByMapSize(2, 5))
 		break;
 
 	let closestPoint = areaCityPaths.getClosestPointTo(roadStart);
@@ -987,7 +987,7 @@ for (let roadStart of roadStartLocations)
 			new PathPlacer(
 				Vector2D.add(closestPoint, new Vector2D(0, 3/4 * mapSize).rotate(closestPoint.angleTo(roadStart))),
 				roadStart,
-				4,
+				scaleByMapSize(5, 3),
 				0.1,
 				5,
 				0.5,
@@ -1061,13 +1061,13 @@ var areaCityBushes =
 				clTower, 1,
 				clFortress, 1,
 				clHouse, 1,
-				clBlacksmith, 1,
-				clElephantStables, 1,
+				clForge, 1,
+				clElephantStable, 1,
 				clStable, 1,
 				clCivicCenter, 1,
 				clBarracks, 1,
 				clBlemmyeCamp, 1,
-				clNubaVillage, 1,
+				clNobaCamp, 1,
 				clMarket, 1)
 		]);
 
@@ -1078,7 +1078,7 @@ var areaCityPalms =
 		undefined,
 		[
 			new StayAreasConstraint([areaCityBushes]),
-			avoidClasses(clElephantStables, 3)
+			avoidClasses(clElephantStable, 3)
 		]);
 
 g_Map.log("Placing city palms");
@@ -1487,7 +1487,7 @@ setWaterWaviness(8);
 setWaterMurkiness(0.87);
 setWaterType("lake");
 
-setTerrainAmbientColor(0.58, 0.443, 0.353);
+setAmbientColor(0.58, 0.443, 0.353);
 
 setSunColor(0.733, 0.746, 0.574);
 setSunRotation(Math.PI / 2 * randFloat(-1, 1));
