@@ -30,6 +30,7 @@ const oSecondaryHuntableAnimal = g_Gaia.secondaryHuntableAnimal;
 const oStoneLarge = g_Gaia.stoneLarge;
 const oStoneSmall = g_Gaia.stoneSmall;
 const oMetalLarge = g_Gaia.metalLarge;
+const oMetalSmall = g_Gaia.metalSmall;
 
 const aGrass = g_Decoratives.grass;
 const aGrassShort = g_Decoratives.grassShort;
@@ -66,7 +67,7 @@ g_Map.log("Preventing water in player territory");
 for (let i = 0; i < numPlayers; ++i)
 	addCivicCenterAreaToClass(playerPosition[i], clPlayer);
 
-g_Map.log("Creating the lake...")
+g_Map.log("Creating the lake...");
 createArea(
 	new ChainPlacer(
 		2,
@@ -134,12 +135,12 @@ if (randBool())
 else
 	createMountains(tCliff, avoidClasses(clPlayer, 20, clHill, 15, clWater, 2), clHill, scaleByMapSize(1, 4) * numPlayers);
 
-var [forestTrees, stragglerTrees] = getTreeCounts(...rBiomeTreeCount(1));
-createForests(
- [tMainTerrain, tForestFloor1, tForestFloor2, pForest1, pForest2],
- avoidClasses(clPlayer, 20, clForest, 17, clHill, 0, clWater, 2),
- clForest,
- forestTrees);
+const [forestTrees, stragglerTrees] = getTreeCounts(...rBiomeTreeCount(1));
+createDefaultForests(
+	[tMainTerrain, tForestFloor1, tForestFloor2, pForest1, pForest2],
+	avoidClasses(clPlayer, 20, clForest, 17, clHill, 0, clWater, 2),
+	clForest,
+	forestTrees);
 
 Engine.SetProgress(50);
 
@@ -161,29 +162,27 @@ createPatches(
  clDirt);
 Engine.SetProgress(55);
 
-g_Map.log("Creating stone mines");
-createMines(
- [
-  [new SimpleObject(oStoneSmall, 0, 2, 0, 4, 0, 2 * Math.PI, 1), new SimpleObject(oStoneLarge, 1, 1, 0, 4, 0, 2 * Math.PI, 4)],
-  [new SimpleObject(oStoneSmall, 2,5, 1,3)]
- ],
- avoidClasses(clWater, 3, clForest, 1, clPlayer, 20, clRock, 10, clHill, 1),
- clRock);
-
 g_Map.log("Creating metal mines");
-createMines(
- [
-  [new SimpleObject(oMetalLarge, 1,1, 0,4)]
- ],
- avoidClasses(clWater, 3, clForest, 1, clPlayer, 20, clMetal, 10, clRock, 5, clHill, 1),
- clMetal
+createBalancedMetalMines(
+	oMetalSmall,
+	oMetalLarge,
+	clMetal,
+	avoidClasses(clWater, 3, clForest, 1, clPlayer, scaleByMapSize(20, 35), clHill, 1)
+);
+
+g_Map.log("Creating stone mines");
+createBalancedStoneMines(
+	oStoneSmall,
+	oStoneLarge,
+	clRock,
+	avoidClasses(clWater, 3, clForest, 1, clPlayer, scaleByMapSize(20, 35), clHill, 1, clMetal, 10)
 );
 
 Engine.SetProgress(65);
 
 var planetm = 1;
 
-if (currentBiome() == "generic/tropic")
+if (currentBiome() == "generic/india")
 	planetm = 8;
 
 createDecoration(
