@@ -20,20 +20,20 @@ const tGrassCliff = "medit_cliff_italia_grass";
 const tCliff = ["medit_cliff_italia", "medit_cliff_italia", "medit_cliff_italia_grass"];
 const tForestFloor = "medit_grass_wild";
 
-const oBeech = "gaia/flora_tree_euro_beech";
-const oBerryBush = "gaia/flora_bush_berry";
-const oCarob = "gaia/flora_tree_carob";
-const oCypress1 = "gaia/flora_tree_cypress";
-const oCypress2 = "gaia/flora_tree_cypress";
-const oLombardyPoplar = "gaia/flora_tree_poplar_lombardy";
-const oPalm = "gaia/flora_tree_medit_fan_palm";
-const oPine = "gaia/flora_tree_aleppo_pine";
+const oBeech = "gaia/tree/euro_beech";
+const oBerryBush = "gaia/fruit/berry_01";
+const oCarob = "gaia/tree/carob";
+const oCypress1 = "gaia/tree/cypress";
+const oCypress2 = "gaia/tree/cypress";
+const oLombardyPoplar = "gaia/tree/poplar_lombardy";
+const oPalm = "gaia/tree/medit_fan_palm";
+const oPine = "gaia/tree/aleppo_pine";
 const oDeer = "gaia/fauna_deer";
-const oFish = "gaia/fauna_fish";
+const oFish = "gaia/fish/generic";
 const oSheep = "gaia/fauna_sheep";
-const oStoneLarge = "gaia/geology_stonemine_medit_quarry";
-const oStoneSmall = "gaia/geology_stone_mediterranean";
-const oMetalLarge = "gaia/geology_metal_mediterranean_slabs";
+const oStoneLarge = "gaia/rock/mediterranean_large";
+const oStoneSmall = "gaia/rock/mediterranean_small";
+const oMetalLarge = "gaia/ore/mediterranean_large";
 
 const aBushMedDry = "actor|props/flora/bush_medit_me_dry.xml";
 const aBushMed = "actor|props/flora/bush_medit_me.xml";
