@@ -14,13 +14,13 @@ const tShoreBlend = "alpine_shore_rocks_icy";
 const tShore = "alpine_shore_rocks";
 const tWater = "alpine_shore_rocks";
 
-const oPine = "gaia/flora_tree_pine_w";
-const oStoneLarge = "gaia/geology_stonemine_alpine_quarry";
-const oStoneSmall = "gaia/geology_stone_alpine_a";
-const oMetalLarge = "gaia/geology_metal_alpine_slabs";
-const oFish = "gaia/fauna_fish";
+const oPine = "gaia/tree/pine_w";
+const oStoneLarge = "gaia/rock/alpine_large";
+const oStoneSmall = "gaia/rock/alpine_small";
+const oMetalLarge = "gaia/ore/alpine_large";
+const oFish = "gaia/fish/generic";
 const oWalrus = "gaia/fauna_walrus";
-const oArcticWolf = "gaia/fauna_arctic_wolf";
+const oArcticWolf = "gaia/fauna_wolf_arctic";
 
 const aIceberg = "actor|props/special/eyecandy/iceberg.xml";
 
