@@ -12,15 +12,15 @@ var tShore = "polar_ice_snow";
 var tWater = "polar_ice_c";
 
 var oArcticFox = "gaia/fauna_fox_arctic";
-var oArcticWolf = "trigger/fauna_arctic_wolf_attack";
+var oArcticWolf = "gaia/fauna_wolf_arctic_violent";
 var oMuskox = "gaia/fauna_muskox";
 var oWalrus = "gaia/fauna_walrus";
 var oWhaleFin = "gaia/fauna_whale_fin";
 var oWhaleHumpback = "gaia/fauna_whale_humpback";
-var oFish = "gaia/fauna_fish";
-var oStoneLarge = "gaia/geology_stonemine_medit_quarry";
-var oStoneSmall = "gaia/geology_stone_alpine_a";
-var oMetalLarge = "gaia/geology_metal_desert_badlands_slabs";
+var oFish = "gaia/fish/generic";
+var oStoneLarge = "gaia/rock/mediterranean_large";
+var oStoneSmall = "gaia/rock/alpine_small";
+var oMetalLarge = "gaia/ore/badlands_large";
 var oWoodTreasure = "gaia/treasure/wood";
 var oMarket = "skirmish/structures/default_market";
 
@@ -134,7 +134,7 @@ Engine.SetProgress(65);
 g_Map.log("Creating dirt patches");
 createLayeredPatches(
 	[scaleByMapSize(3, 6), scaleByMapSize(5, 10), scaleByMapSize(8, 21)],
-	[[tDirt,tHalfSnow], [tHalfSnow,tSnowLimited]],
+	[[tDirt, tHalfSnow], [tHalfSnow, tSnowLimited]],
 	[2],
 	avoidClasses(clWater, 3, clDirt, 5, clPlayer, 12),
 	scaleByMapSize(15, 45),
@@ -150,10 +150,10 @@ createPatches(
 Engine.SetProgress(70);
 
 g_Map.log("Creating stone mines");
-	createMines(
+createMines(
 	[
 		[new SimpleObject(oStoneSmall, 0, 2, 0, 4, 0, 2 * Math.PI, 1), new SimpleObject(oStoneLarge, 1, 1, 0, 4, 0, 2 * Math.PI, 4)],
-		[new SimpleObject(oStoneSmall, 2,5, 1,3)]
+		[new SimpleObject(oStoneSmall, 2, 5, 1, 3)]
 	],
 	avoidClasses(clWater, 3, clPlayer, 20, clRock, 18, clHill, 2),
 	clRock);
@@ -161,7 +161,7 @@ g_Map.log("Creating stone mines");
 g_Map.log("Creating metal mines");
 createMines(
 	[
-		[new SimpleObject(oMetalLarge, 1,1, 0,4)]
+		[new SimpleObject(oMetalLarge, 1, 1, 0, 4)]
 	],
 	avoidClasses(clWater, 3, clPlayer, 20, clMetal, 18, clRock, 5, clHill, 2),
 	clMetal);
@@ -241,12 +241,11 @@ createObjectGroupsDeprecated(
 	100);
 Engine.SetProgress(95);
 
-if (randBool(1/3))
+if (g_MapSettings.Daytime !== undefined ? g_MapSettings.Daytime == "dawn" : randBool(1/3))
 {
 	setSkySet("sunset 1");
 	setSunColor(0.8, 0.7, 0.6);
-	setTerrainAmbientColor(0.7, 0.6, 0.7);
-	setUnitsAmbientColor(0.6, 0.5, 0.6);
+	setAmbientColor(0.7, 0.6, 0.7);
 	setSunElevation(Math.PI * randFloat(1/24, 1/7));
 }
 else
