@@ -14,21 +14,21 @@ const tShoreBlend = "tropic_beach_dry_plants";
 const tShore = "tropic_beach_dry";
 const tWater = "tropic_beach_wet";
 
-const oTree = "gaia/flora_tree_toona";
-const oPalm1 = "gaia/flora_tree_palm_tropic";
-const oPalm2 = "gaia/flora_tree_palm_tropical";
-const oStoneLarge = "gaia/geology_stonemine_tropic_quarry";
-const oStoneSmall = "gaia/geology_stone_tropic_a";
-const oMetalLarge = "gaia/geology_metal_tropic_slabs";
-const oFish = "gaia/fauna_fish";
+const oTree = "gaia/tree/toona";
+const oPalm1 = "gaia/tree/palm_tropic";
+const oPalm2 = "gaia/tree/palm_tropical";
+const oStoneLarge = "gaia/rock/tropical_large";
+const oStoneSmall = "gaia/rock/tropical_small";
+const oMetalLarge = "gaia/ore/tropical_large";
+const oFish = "gaia/fish/generic";
 const oDeer = "gaia/fauna_deer";
 const oTiger = "gaia/fauna_tiger";
 const oBoar = "gaia/fauna_boar";
 const oPeacock = "gaia/fauna_peacock";
-const oBush = "gaia/flora_bush_berry";
-const oSpearman = "units/maur_infantry_spearman_b";
-const oArcher = "units/maur_infantry_archer_b";
-const oArcherElephant = "units/maur_elephant_archer_b";
+const oBush = "gaia/fruit/berry_01";
+const oSpearman = "units/maur/infantry_spearman_b";
+const oArcher = "units/maur/infantry_archer_b";
+const oArcherElephant = "units/maur/elephant_archer_b";
 
 const aRockLarge = "actor|geology/stone_granite_large.xml";
 const aRockMedium = "actor|geology/stone_granite_med.xml";
