@@ -461,14 +461,14 @@ function placePlayersNomad(playerClass, constraints)
 			ents => new SimpleObject(ents.Template, ents.Count || 1, ents.Count || 1, 1, 3));
 
 		// Add treasure if too few resources for a civic center
-		let ccCost = Engine.GetTemplate("structures/" + getCivCode(playerIDs[i]) + "_civil_centre").Cost.Resources;
+		let ccCost = Engine.GetTemplate("structures/" + getCivCode(playerIDs[i]) + "/civil_centre").Cost.Resources;
 		for (let resourceType in ccCost)
 		{
 			let treasureTemplate = g_NomadTreasureTemplates[resourceType];
 
 			let count = Math.max(0, Math.ceil(
 				(ccCost[resourceType] - (g_MapSettings.StartingResources || 0)) /
-				Engine.GetTemplate(treasureTemplate).ResourceSupply.Amount));
+				Engine.GetTemplate(treasureTemplate).Treasure.Resources[resourceType]));
 
 			objects.push(new SimpleObject(treasureTemplate, count, count, 3, 5));
 		}
@@ -689,7 +689,9 @@ function playerPlacementRandom(playerIDs, constraints = undefined)
 
 	for (let i = 0; i < getNumPlayers(); ++i)
 	{
-		let position = pickRandom(area.getPoints());
+		const position = pickRandom(area.getPoints());
+		if (!position)
+			return undefined;
 
 		// Minimum distance between initial bases must be a quarter of the map diameter
 		if (locations.some(loc => loc.distanceToSquared(position) < playerMinDistSquared) ||
