@@ -24,10 +24,11 @@ function loadWallsetsFromCivData()
 		for (let path of civInfo.WallSets)
 		{
 			// File naming conventions:
-			// - other/wallset_{style}
-			// - structures/{civ}_wallset_{style}
-			let style = basename(path).split("_");
-			style = style[0] == "wallset" ? style[1] : style[0] + "_" + style[2];
+			// - structures/wallset_{style}
+			// - structures/{civ}/wallset_{style}
+			let style = basename(path).split("_")[1];
+			if (path.split("/").indexOf(civ) != -1)
+				style = civ + "/" + style;
 
 			if (!wallsets[style])
 				wallsets[style] = loadWallset(Engine.GetTemplate(path), civ);
@@ -64,7 +65,7 @@ function loadWallset(wallsetPath, civ)
  * @param {string} type - Descriptive string, example: "tiny". Not really needed (WallTool.wallTypes["type string"] is used). Mainly for custom wall elements.
  * @param {array} [wall] - Array of wall element strings. May be defined at a later point.
  *                         Example: ["medium", "cornerIn", "gate", "cornerIn", "medium", "cornerIn", "gate", "cornerIn"]
- * @param {object} [centerToFirstElement] - Vector from the visual center of the fortress to the first wall element.
+ * @param {Object} [centerToFirstElement] - Vector from the visual center of the fortress to the first wall element.
  * @param {number} [centerToFirstElement.x]
  * @param {number} [centerToFirstElement.y]
  */
@@ -132,11 +133,11 @@ function createDefaultFortressTypes()
  *   `gap_{x}` returns a non-blocking gap of length `x` meters.
  *   `turn_{x}` returns a zero-length turn of angle `x` radians.
  *
- * Any other arbitrary string passed will be attempted to be used as: `structures/{civ}_{arbitrary_string}`.
+ * Any other arbitrary string passed will be attempted to be used as: `structures/{civ}/{arbitrary_string}`.
  *
  * @param {string} element - What sort of element to fetch.
  * @param {string} [style] - The style from which this element should come from.
- * @returns {object} The wall element requested. Or a tower element.
+ * @returns {Object} The wall element requested. Or a tower element.
  */
 function getWallElement(element, style)
 {
@@ -148,7 +149,7 @@ function getWallElement(element, style)
 	// Defaults to a wall tower piece
 	const quarterBend = Math.PI / 2;
 	let wallset = g_WallStyles[style];
-	let civ = style.split("_")[0];
+	let civ = style.split("/")[0];
 	let ret = wallset.tower ? clone(wallset.tower) : { "angle": 0, "bend": 0, "length": 0, "indent": 0 };
 
 	switch (element)
@@ -193,7 +194,7 @@ function getWallElement(element, style)
 		break;
 
 	case "entryTower":
-		ret.templateName = g_CivData[civ] ? "structures/" + civ + "_defense_tower" : "other/palisades_rocks_watchtower";
+		ret.templateName = g_CivData[civ] ? "structures/" + civ + "/defense_tower" : "structures/palisades_watchtower";
 		ret.indent = ret.length * -3;
 		ret.length = wallset.gate.length;
 		break;
@@ -236,7 +237,7 @@ function getWallElement(element, style)
 			if (!g_CivData[civ])
 				civ = Object.keys(g_CivData)[0];
 
-			let templateName = "structures/" + civ + "_" + element;
+			let templateName = "structures/" + civ + "/" + element;
 			if (Engine.TemplateExists(templateName))
 			{
 				ret.indent = ret.length * (element == "outpost" || element.endsWith("_tower") ? -3 : 3.5);
@@ -262,7 +263,7 @@ function getWallElement(element, style)
 function readyWallElement(path, civCode)
 {
 	path = path.replace(/\{civ\}/g, civCode);
-	let template = GetTemplateDataHelper(Engine.GetTemplate(path), null, null, {}, g_DamageTypes, {});
+	let template = GetTemplateDataHelper(Engine.GetTemplate(path), null, null);
 	let length = template.wallPiece ? template.wallPiece.length : template.obstruction.shape.width;
 
 	return deepfreeze({
@@ -345,7 +346,7 @@ function getWallAlignment(position, wall = [], style = "athen_stone", orientatio
  * Used to get centerToFirstElement of fortresses by default
  *
  * @param {number} alignment
- * @returns {object} Vector from the center of the set of aligned wallpieces to the first wall element.
+ * @returns {Object} Vector from the center of the set of aligned wallpieces to the first wall element.
  */
 function getCenterToFirstElement(alignment)
 {
@@ -385,7 +386,7 @@ function validateStyle(style, playerId = 0)
 		if (playerId == 0)
 			return Object.keys(g_WallStyles)[0];
 
-		style = getCivCode(playerId) + "_stone";
+		style = getCivCode(playerId) + "/stone";
 		return !g_WallStyles[style] ? Object.keys(g_WallStyles)[0] : style;
 	}
 	return style;
@@ -429,7 +430,7 @@ function placeWall(position, wall = [], style, playerId = 0, orientation = 0, co
  * "entry" or "gate") to get the orientation correct.
  *
  * @param {Vector2D} centerPosition
- * @param {object} [fortress] - If not provided, defaults to the predefined "medium" fortress type.
+ * @param {Object} [fortress] - If not provided, defaults to the predefined "medium" fortress type.
  * @param {string} [style] - Wall style string.
  * @param {number} [playerId] - Identifier of the player for whom the wall will be placed.
  * @param {number} [orientation] - Angle the first wall element (should be a gate or entrance) is placed. Default is 0
@@ -612,7 +613,11 @@ function placeCircularWall(center, radius, wallPart, style, playerId = 0, orient
 
 			// Placement
 			if (wallEle.templateName && g_Map.inMapBounds(place) && constraint.allows(place.clone().floor()))
-				entities.push(g_Map.placeEntityPassable(wallEle.templateName, playerId, place, placeAngle + wallEle.angle));
+			{
+				let entity = g_Map.placeEntityPassable(wallEle.templateName, playerId, place, placeAngle + wallEle.angle);
+				if (entity)
+					entities.push(entity);
+			}
 
 			// Prepare for the next wall element
 			actualAngle += addAngle;
@@ -623,7 +628,7 @@ function placeCircularWall(center, radius, wallPart, style, playerId = 0, orient
 	{
 		let wallEle = getWallElement(wallPart[0], style);
 		let addAngle = scaleFactor * wallEle.length / radius;
-		let target = Vector2D.add(center, new Vector2D(radius, 0).rotate(-actualAngle - addAngle))
+		let target = Vector2D.add(center, new Vector2D(radius, 0).rotate(-actualAngle - addAngle));
 		let place = Vector2D.average([position, target]);
 		let placeAngle = actualAngle + addAngle / 2;
 		if (g_Map.inMapBounds(place) && constraint.allows(place.clone().floor()))
@@ -665,8 +670,11 @@ function placePolygonalWall(centerPosition, radius, wallPart, cornerWallElement
 	{
 		let angleToCorner = getAngle(corners[i].x, corners[i].y, centerPosition.x, centerPosition.y);
 		if (g_Map.inMapBounds(corners[i]) && constraint.allows(corners[i].clone().floor()))
-			entities.push(
-				g_Map.placeEntityPassable(getWallElement(cornerWallElement, style).templateName, playerId, corners[i], angleToCorner));
+		{
+			let entity = g_Map.placeEntityPassable(getWallElement(cornerWallElement, style).templateName, playerId, corners[i], angleToCorner);
+			if (entity)
+				entities.push(entity);
+		}
 
 		if (!skipFirstWall || i != 0)
 		{
