@@ -4,11 +4,10 @@
 
 var g_Environment = {
 	"SkySet": "default", // textures for the skybox, subdirectory name of art/textures/skies
-	"SunColor": { "r": 0.749020, "g": 0.749020, "b": 0.749020, "a": 0 }, // all rgb from 0 to 1
-	"SunElevation": 0.785398, // 0 to 2pi
-	"SunRotation": 5.49779, // 0 to 2pi
-	"TerrainAmbientColor": { "r": 0.501961, "g": 0.501961, "b": 0.501961, "a": 0 },
-	"UnitsAmbientColor": { "r": 0.501961, "g": 0.501961, "b": 0.501961, "a": 0 },
+	"SunColor": { "r": 1.03162, "g": 0.99521, "b": 0.865752, "a": 0 }, // all rgb from 0 to 1
+	"SunElevation": 1.85, // 0 to 2pi
+	"SunRotation": 0.0, // 0 to 2pi
+	"AmbientColor": { "r": 0.364706, "g": 0.376471, "b": 0.419608, "a": 0 },
 	"Water": {
 		"WaterBody": {
 			"Type": "ocean", // Subdirectory name of art/textures/animated/water
@@ -67,14 +66,9 @@ function setSunRotation(r)
 	g_Environment.SunRotation = r;
 }
 
-function setTerrainAmbientColor(r, g, b)
+function setAmbientColor(r, g, b)
 {
-	g_Environment.TerrainAmbientColor = { "r" : r, "g" : g, "b" : b, "a" : 0 };
-}
-
-function setUnitsAmbientColor(r, g, b)
-{
-	g_Environment.UnitsAmbientColor = { "r" : r, "g" : g, "b" : b, "a" : 0 };
+	g_Environment.AmbientColor = { "r" : r, "g" : g, "b" : b, "a" : 0 };
 }
 
 ////////////////////////////////////////////////////////////////////////////
