@@ -21,25 +21,12 @@ const TERRAIN_SEPARATOR = "|";
 const SEA_LEVEL = 20.0;
 const HEIGHT_UNITS_PER_METRE = 92;
 
-/**
- * Number of impassable, unexplorable tiles at the map border.
- */
-const MAP_BORDER_WIDTH = 3;
-
-const g_DamageTypes = new DamageTypes();
-
 /**
  * Constants needed for heightmap_manipulation.js
  */
 const MAX_HEIGHT_RANGE = 0xFFFF / HEIGHT_UNITS_PER_METRE; // Engine limit, Roughly 700 meters
 const MIN_HEIGHT = - SEA_LEVEL;
 
-/**
- * Length of one tile of the terrain grid in metres.
- * Useful to transform footprint sizes of templates to the coordinate system used by getMapSize.
- */
-const TERRAIN_TILE_SIZE = Engine.GetTerrainTileSize();
-
 const MAX_HEIGHT = MAX_HEIGHT_RANGE - SEA_LEVEL;
 
 /**
@@ -153,6 +140,12 @@ function createAreas(centeredPlacer, painter, constraints, amount, retryFactor =
  */
 function createAreasInAreas(centeredPlacer, painter, constraints, amount, retryFactor, areas)
 {
+	areas = areas.filter(area => area.getPoints().length);
+	if (!areas.length) {
+		log("createAreasInAreas: 'areas' was either empty or only contained empty areas thus returning an empty array.\n" + new Error().stack)
+		return [];
+	}
+	
 	let placeFunc = function() {
 		centeredPlacer.setCenterPosition(pickRandom(pickRandom(areas).getPoints()));
 		return createArea(centeredPlacer, painter, constraints);
@@ -181,6 +174,12 @@ function createObjectGroups(group, player, constraints, amount, retryFactor = 10
  */
 function createObjectGroupsByAreas(group, player, constraints, amount, retryFactor, areas, behaveDeprecated = false)
 {
+	areas = areas.filter(area => area.getPoints().length);
+	if (!areas.length) {
+		log("createObjectGroupsByAreas: 'areas' was either empty or only contained empty areas.\n" + new Error().stack)
+		return [];
+	}
+	
 	let placeFunc = function() {
 		group.setCenterPosition(pickRandom(pickRandom(areas).getPoints()));
 		return createObjectGroup(group, player, constraints);
