@@ -141,64 +141,13 @@ let wildLakeBiome = [
 	}
 ];
 
-var mercenaryCampGuards = {
-	"generic/temperate": [
-		{ "Template" : "structures/merc_camp_egyptian" },
-		{ "Template" : "units/mace_infantry_javelinist_b", "Count" : 4 },
-		{ "Template" : "units/mace_cavalry_spearman_e", "Count" : 3 },
-		{ "Template" : "units/mace_infantry_archer_a", "Count" : 4 },
-		{ "Template" : "units/mace_champion_infantry_a", "Count" : 3 }
-	],
-	"generic/snowy": [
-		{ "Template" : "structures/ptol_mercenary_camp" },
-		{ "Template" : "units/brit_infantry_javelinist_b", "Count" : 4 },
-		{ "Template" : "units/brit_cavalry_swordsman_e", "Count" : 3 },
-		{ "Template" : "units/brit_infantry_slinger_a", "Count" : 4 },
-		{ "Template" : "units/brit_champion_infantry", "Count" : 3 }
-	],
-	"generic/desert": [
-		{ "Template" : "structures/ptol_mercenary_camp" },
-		{ "Template" : "units/pers_infantry_javelinist_b", "Count" : 4 },
-		{ "Template" : "units/pers_cavalry_swordsman_e", "Count" : 3 },
-		{ "Template" : "units/pers_infantry_archer_a", "Count" : 4 },
-		{ "Template" : "units/pers_champion_infantry", "Count" : 3 }
-	],
-	"generic/alpine": [
-		{ "Template" : "structures/ptol_mercenary_camp" },
-		{ "Template" : "units/rome_infantry_swordsman_b", "Count" : 4 },
-		{ "Template" : "units/rome_cavalry_spearman_e", "Count" : 3 },
-		{ "Template" : "units/rome_infantry_javelinist_a", "Count" : 4 },
-		{ "Template" : "units/rome_champion_infantry", "Count" : 3 }
-	],
-	"generic/mediterranean": [
-		{ "Template" : "structures/merc_camp_egyptian" },
-		{ "Template" : "units/iber_infantry_javelinist_b", "Count" : 4 },
-		{ "Template" : "units/iber_cavalry_spearman_e", "Count" : 3 },
-		{ "Template" : "units/iber_infantry_slinger_a", "Count" : 4 },
-		{ "Template" : "units/iber_champion_infantry", "Count" : 3 }
-	],
-	"generic/savanna": [
-		{ "Template" : "structures/merc_camp_egyptian" },
-		{ "Template" : "units/sele_infantry_javelinist_b", "Count" : 4 },
-		{ "Template" : "units/sele_cavalry_spearman_merc_e", "Count" : 3 },
-		{ "Template" : "units/sele_infantry_spearman_a", "Count" : 4 },
-		{ "Template" : "units/sele_champion_infantry_swordsman", "Count" : 3 }
-	],
-	"generic/tropic": [
-		{ "Template" : "structures/merc_camp_egyptian" },
-		{ "Template" : "units/ptol_infantry_javelinist_b", "Count" : 4 },
-		{ "Template" : "units/ptol_cavalry_archer_e", "Count" : 3 },
-		{ "Template" : "units/ptol_infantry_slinger_a", "Count" : 4 },
-		{ "Template" : "units/ptol_champion_infantry_pikeman", "Count" : 3 }
-	],
-	"generic/autumn": [
-		{ "Template" : "structures/ptol_mercenary_camp" },
-		{ "Template" : "units/gaul_infantry_javelinist_b", "Count" : 4 },
-		{ "Template" : "units/gaul_cavalry_swordsman_e", "Count" : 3 },
-		{ "Template" : "units/gaul_infantry_slinger_a", "Count" : 4 },
-		{ "Template" : "units/gaul_champion_infantry", "Count" : 3 }
-	]
-};
+const wildLakeEntities = Engine.ReadJSONFile("maps/random/wild_lake_biomes.json");
+const farmEntities = wildLakeEntities[currentBiome()].farmEntities;
+const mercenaryCampEntities = wildLakeEntities[currentBiome()].mercenaryCampEntities;
+const guards = mercenaryCampEntities
+	.map(ent => ent.Template)
+	.filter(ent => ent.indexOf("units/") != -1);
+const campEntities = wildLakeEntities.campEntities.concat(guards);
 
 /**
  * Resource spots and other points of interest
@@ -224,7 +173,7 @@ function placeMine(position, centerEntity,
 			randomAngle());
 }
 
-// Groves, only Wood
+// Groves, only wood
 let groveActors = [g_Decoratives.grass, g_Decoratives.rockMedium, g_Decoratives.bushMedium];
 let clGrove = g_Map.createTileClass();
 let clGaiaCamp = g_Map.createTileClass();
@@ -241,7 +190,7 @@ function placeGrove(point,
 )
 {
 	let position = new Vector2D(point.x, point.y);
-	g_Map.placeEntityPassable(pickRandom(["structures/gaul_outpost", "gaia/flora_tree_oak_new"]), 0, position, randomAngle());
+	g_Map.placeEntityPassable(pickRandom(["structures/gaul/outpost", g_Gaia.tree1]), 0, position, randomAngle());
 
 	let quantity = randIntInclusive(20, 30);
 	let dAngle = 2 * Math.PI / quantity;
@@ -266,25 +215,16 @@ function placeGrove(point,
 	}
 }
 
-var farmEntities = {
-	"generic/temperate": { "building": "structures/mace_farmstead", "animal": "gaia/fauna_pig" },
-	"generic/snowy": { "building": "structures/brit_farmstead", "animal": "gaia/fauna_sheep" },
-	"generic/desert": { "building": "structures/pers_farmstead", "animal": "gaia/fauna_camel" },
-	"generic/alpine": { "building": "structures/rome_farmstead", "animal": "gaia/fauna_sheep" },
-	"generic/mediterranean": { "building": "structures/iber_farmstead", "animal": "gaia/fauna_pig" },
-	"generic/savanna": { "building": "structures/sele_farmstead", "animal": "gaia/fauna_horse" },
-	"generic/tropic": { "building": "structures/ptol_farmstead", "animal": "gaia/fauna_camel" },
-	"generic/autumn": { "building": "structures/gaul_farmstead", "animal": "gaia/fauna_horse" }
-};
+
 
 g_WallStyles.other = {
 	"overlap": 0,
-	"fence": readyWallElement("other/fence_long", "gaia"),
-	"fence_short": readyWallElement("other/fence_short", "gaia"),
-	"bench":     { "angle": Math.PI / 2, "length": 1.5,  "indent": 0,    "bend": 0, "templateName": "other/bench" },
-	"foodBin":   { "angle": Math.PI / 2, "length": 1.5,  "indent": 0,    "bend": 0, "templateName": "gaia/treasure/food_bin" },
-	"animal":    { "angle": 0,           "length": 0,    "indent": 0.75, "bend": 0, "templateName": farmEntities[currentBiome()].animal },
-	"farmstead": { "angle": Math.PI,     "length": 0,    "indent": -3,   "bend": 0, "templateName": farmEntities[currentBiome()].building }
+	"fence": readyWallElement("structures/fence_long", "gaia"),
+	"fence_short": readyWallElement("structures/fence_short", "gaia"),
+	"bench": { "angle": Math.PI / 2, "length": 1.5, "indent": 0, "bend": 0, "templateName": "structures/bench" },
+	"foodBin": { "angle": Math.PI / 2, "length": 1.5, "indent": 0, "bend": 0, "templateName": "gaia/treasure/food_bin" },
+	"animal": { "angle": 0, "length": 0, "indent": 0.75, "bend": 0, "templateName": farmEntities.animal },
+	"farmstead": { "angle": Math.PI, "length": 0, "indent": -3, "bend": 0, "templateName": farmEntities.building }
 };
 
 let fences = [
@@ -325,11 +265,7 @@ for (let i = 0; i < num; ++i)
 
 // Camps with fire and gold treasure
 function placeCamp(position,
-	centerEntity = "actor|props/special/eyecandy/campfire.xml",
-	otherEntities = ["gaia/treasure/metal", "gaia/treasure/standing_stone",
-		"units/brit_infantry_slinger_b", "units/brit_infantry_javelinist_b", "units/gaul_infantry_slinger_b", "units/gaul_infantry_javelinist_b", "units/gaul_champion_fanatic",
-		"actor|props/special/common/waypoint_flag.xml", "actor|props/special/eyecandy/barrel_a.xml", "actor|props/special/eyecandy/basket_celt_a.xml", "actor|props/special/eyecandy/crate_a.xml", "actor|props/special/eyecandy/dummy_a.xml", "actor|props/special/eyecandy/handcart_1.xml", "actor|props/special/eyecandy/handcart_1_broken.xml", "actor|props/special/eyecandy/sack_1.xml", "actor|props/special/eyecandy/sack_1_rough.xml"
-	]
+	centerEntity = "actor|props/special/eyecandy/campfire.xml"
 )
 {
 	g_Map.placeEntityPassable(centerEntity, 0, position, randomAngle());
@@ -340,7 +276,7 @@ function placeCamp(position,
 	{
 		let angle = dAngle * randFloat(i, i + 1);
 		let dist = randFloat(1, 3);
-		g_Map.placeEntityPassable(pickRandom(otherEntities), 0, Vector2D.add(position, new Vector2D(dist, 0).rotate(-angle)), randomAngle());
+		g_Map.placeEntityPassable(pickRandom(campEntities), 0, Vector2D.add(position, new Vector2D(dist, 0).rotate(-angle)), randomAngle());
 	}
 
 	addCivicCenterAreaToClass(position, clGaiaCamp);
@@ -418,12 +354,13 @@ function placeStartLocationResources(
 /**
  * Base terrain shape generation and settings
  */
- // Height range by map size
-let heightScale = (g_Map.size + 256) / 768 / 4;
+
+// Height range by map size
+let heightScale = (g_Map.size + 512) / 1024 / 5;
 let heightRange = { "min": MIN_HEIGHT * heightScale, "max": MAX_HEIGHT * heightScale };
 
 // Water coverage
-let averageWaterCoverage = 1/5; // NOTE: Since terrain generation is quite unpredictable actual water coverage might vary much with the same value
+let averageWaterCoverage = 1 / 5; // NOTE: Since terrain generation is quite unpredictable actual water coverage might vary much with the same value
 let heightSeaGround = -MIN_HEIGHT + heightRange.min + averageWaterCoverage * (heightRange.max - heightRange.min); // Water height in environment and the engine
 let heightSeaGroundAdjusted = heightSeaGround + MIN_HEIGHT; // Water height as terrain height
 setWaterHeight(heightSeaGround);
@@ -474,7 +411,7 @@ for (let i = 0; i < 5; ++i)
 g_Map.log("Smoothing map");
 createArea(
 	new MapBoundsPlacer(),
-	new SmoothingPainter(1, 0.8, 1));
+	new SmoothingPainter(1, 0.5, Math.ceil(g_Map.size/128) + 1));
 
 g_Map.log("Rescaling map");
 rescaleHeightmap(heightRange.min, heightRange.max);
@@ -494,8 +431,8 @@ let heighLimits = [
 	heightSeaGroundAdjusted + 7/8 * (heightRange.max - heightSeaGroundAdjusted), // 6 Lower forest border
 	heightRange.max // 7 Forest
 ];
-let playerHeightRange = { "min" : heighLimits[3], "max" : heighLimits[4] };
-let resourceSpotHeightRange = { "min" : (heighLimits[2] + heighLimits[3]) / 2, "max" : (heighLimits[4] + heighLimits[5]) / 2 };
+let playerHeightRange = { "min": heighLimits[3], "max": heighLimits[4] };
+let resourceSpotHeightRange = { "min": (heighLimits[2] + heighLimits[3]) / 2, "max": (heighLimits[4] + heighLimits[5]) / 2 };
 let playerHeight = (playerHeightRange.min + playerHeightRange.max) / 2; // Average player height
 
 g_Map.log("Chosing starting locations");
@@ -623,7 +560,7 @@ for (let i = 0; i < resourceSpots.length; ++i)
 	{
 		if (mercenaryCamps)
 		{
-			placeStartingEntities(resourceSpots[i], 0, mercenaryCampGuards[currentBiome()]);
+			placeStartingEntities(resourceSpots[i], 0, mercenaryCampEntities);
 			radius = 15;
 			--mercenaryCamps;
 		}
