@@ -16,6 +16,7 @@ var g_DurationFilterIntervals = [
  * Allow to filter by population capacity.
  */
 const g_PopulationCapacities = prepareForDropdown(g_Settings && g_Settings.PopulationCapacities);
+const g_WorldPopulationCapacities = prepareForDropdown(g_Settings && g_Settings.WorldPopulationCapacities);
 
 /**
  * Reloads the selectable values in the filters. The filters depend on g_Settings and g_Replays
@@ -23,7 +24,8 @@ const g_PopulationCapacities = prepareForDropdown(g_Settings && g_Settings.Popul
  */
 function initFilters(filters)
 {
-	Engine.GetGUIObjectByName("compatibilityFilter").checked = !filters || filters.compatibility === undefined || filters.compatibility;
+	if (filters && filters.compatibility !== undefined)
+		Engine.GetGUIObjectByName("compatibilityFilter").checked = filters.compatibility;
 
 	if (filters && filters.playernames)
 		Engine.GetGUIObjectByName("playersFilter").caption = filters.playernames;
@@ -139,8 +141,13 @@ function initDurationFilter(filters)
 function initSingleplayerFilter(filters)
 {
 	let singleplayerFilter = Engine.GetGUIObjectByName("singleplayerFilter");
-	singleplayerFilter.list = [translate("Single- and multiplayer"), translate("Single Player"), translate("Multiplayer")];
-	singleplayerFilter.list_data = ["", "Singleplayer", "Multiplayer"];
+	singleplayerFilter.list = [
+		translateWithContext("replay filter", "Any"),
+		translateWithContext("replay filter", "Single-player"),
+		translateWithContext("replay filter", "Multiplayer"),
+		translateWithContext("replay filter", "Campaigns")
+	];
+	singleplayerFilter.list_data = ["", "Single-player", "Multiplayer", "Campaigns"];
 
 	if (filters && filters.singleplayer)
 		singleplayerFilter.selected = singleplayerFilter.list_data.indexOf(filters.singleplayer);
@@ -152,7 +159,7 @@ function initSingleplayerFilter(filters)
 function initVictoryConditionFilter(filters)
 {
 	let victoryConditionFilter = Engine.GetGUIObjectByName("victoryConditionFilter");
-	victoryConditionFilter.list = [translate("Any Victory Condition")].concat(g_VictoryConditions.map(victoryCondition => translateVictoryCondition(victoryCondition.Name)));
+	victoryConditionFilter.list = [translate("Any victory condition")].concat(g_VictoryConditions.map(victoryCondition => translateVictoryCondition(victoryCondition.Name)));
 	victoryConditionFilter.list_data = [""].concat(g_VictoryConditions.map(victoryCondition => victoryCondition.Name));
 
 	if (filters && filters.victoryCondition)
@@ -234,11 +241,12 @@ function filterReplay(replay)
 	if (compatibilityFilter.checked && !isReplayCompatible(replay))
 		return false;
 
-	// Filter by singleplayer / multiplayer
+	// Filter by single-player or multiplayer.
 	let singleplayerFilter = Engine.GetGUIObjectByName("singleplayerFilter");
 	let selectedSingleplayerFilter = singleplayerFilter.list_data[singleplayerFilter.selected] || "";
-	if (selectedSingleplayerFilter == "Singleplayer" && replay.isMultiplayer ||
-	    selectedSingleplayerFilter == "Multiplayer" && !replay.isMultiplayer)
+	if (selectedSingleplayerFilter == "Campaigns" && !replay.isCampaign ||
+	    selectedSingleplayerFilter == "Single-player" && (replay.isMultiplayer || replay.isCampaign) ||
+	    selectedSingleplayerFilter == "Multiplayer" && (!replay.isMultiplayer || replay.isCampaign))
 		return false;
 
 	// Filter by victory condition
@@ -272,7 +280,7 @@ function filterReplay(replay)
 
 	// Filter by map name
 	let mapNameFilter = Engine.GetGUIObjectByName("mapNameFilter");
-	if (mapNameFilter.selected > 0 && getReplayMapName(replay) != mapNameFilter.list_data[mapNameFilter.selected])
+	if (mapNameFilter.selected > 0 && replay.attribs.settings.Name != mapNameFilter.list_data[mapNameFilter.selected])
 		return false;
 
 	// Filter by map size
