@@ -61,7 +61,7 @@ TriggerHelper.GetTime = function()
 	return Engine.QueryInterface(SYSTEM_ENTITY, IID_Timer).GetTime();
 };
 
- /**
+/**
   * Returns the elpased ingame time in minutes since the gamestart. Useful for balancing.
   */
 TriggerHelper.GetMinutes = function()
@@ -91,7 +91,7 @@ TriggerHelper.SetUnitFormation = function(playerID, entities, formation)
 	ProcessCommand(playerID, {
 		"type": "formation",
 		"entities": entities,
-		"name": formation
+		"formation": formation
 	});
 };
 
@@ -184,14 +184,9 @@ TriggerHelper.SpawnGarrisonedUnits = function(entity, template, count, owner)
 		if (cmpOwnership)
 			cmpOwnership.SetOwner(owner);
 
-		if (cmpGarrisonHolder.Garrison(ent))
-		{
-			let cmpUnitAI = Engine.QueryInterface(ent, IID_UnitAI);
-			if (cmpUnitAI)
-				cmpUnitAI.Autogarrison(entity);
-
+		let cmpGarrisonable = Engine.QueryInterface(ent, IID_Garrisonable);
+		if (cmpGarrisonable && cmpGarrisonable.Garrison(entity))
 			entities.push(ent);
-		}
 		else
 			error("failed to garrison entity " + ent + " (" + template + ") inside " + entity);
 	}
@@ -199,6 +194,47 @@ TriggerHelper.SpawnGarrisonedUnits = function(entity, template, count, owner)
 	return entities;
 };
 
+/**
+ * Can be used to spawn turreted units on top of a building/ship.
+ *
+ * @param entity Entity id of the turret holder on which units will be turreted
+ * @param template Name of the template
+ * @param count Number of units to spawn
+ * @param owner Player id of the owner of the new units. By default, the owner
+ * of the turretholder entity.
+ */
+TriggerHelper.SpawnTurretedUnits = function(entity, template, count, owner)
+{
+	let entities = [];
+
+	let cmpTurretHolder = Engine.QueryInterface(entity, IID_TurretHolder);
+	if (!cmpTurretHolder)
+	{
+		error("tried to create turreted entities inside a non-turretholder");
+		return entities;
+	}
+
+	if (owner == null)
+		owner = TriggerHelper.GetOwner(entity);
+
+	for (let i = 0; i < count; ++i)
+	{
+		let ent = Engine.AddEntity(template);
+
+		let cmpOwnership = Engine.QueryInterface(ent, IID_Ownership);
+		if (cmpOwnership)
+			cmpOwnership.SetOwner(owner);
+
+		let cmpTurretable = Engine.QueryInterface(ent, IID_Turretable);
+		if (cmpTurretable && cmpTurretable.OccupyTurret(entity))
+			entities.push(ent);
+		else
+			error("failed to turret entity " + ent + " (" + template + ") inside " + entity);
+	}
+
+	return entities;
+};
+
 /**
  * Spawn units from all trigger points with this reference
  * If player is defined, only spaw units from the trigger points
@@ -357,11 +393,11 @@ TriggerHelper.HasDealtWithTech = function(playerID, techName)
 /**
  * Returns all names of templates that match the given identity classes, constrainted to an optional civ.
  *
- * @param {String} classes - See MatchesClassList for the accepted formats, for example "Class1 Class2+!Class3".
- * @param [String] civ - Optionally only retrieve templates of the given civ. Can be left undefined.
- * @param [String] packedState - When retrieving siege engines filter for the "packed" or "unpacked" state
- * @param [String] rank - If given, only return templates that have no or the given rank. For example "Elite".
- * @param [Boolean] excludeBarracksVariants - Optionally exclude templates whose name ends with "_barracks"
+ * @param {string} classes - See MatchesClassList for the accepted formats, for example "Class1 Class2+!Class3".
+ * @param [string] civ - Optionally only retrieve templates of the given civ. Can be left undefined.
+ * @param [string] packedState - When retrieving siege engines filter for the "packed" or "unpacked" state
+ * @param [string] rank - If given, only return templates that have no or the given rank. For example "Elite".
+ * @param [boolean] excludeBarracksVariants - Optionally exclude templates whose name ends with "_barracks"
  */
 TriggerHelper.GetTemplateNamesByClasses = function(classes, civ, packedState, rank, excludeBarracksVariants)
 {
@@ -397,10 +433,10 @@ TriggerHelper.GetTemplateNamesByClasses = function(classes, civ, packedState, ra
 /**
  * Composes a random set of the given templates of the given total size.
  *
- * @param {String[]} templateNames - for example ["brit_infantry_javelinist_b", "brit_cavalry_swordsman_e"]
- * @param {Number} totalCount - total amount of templates, in this  example 12
+ * @param {string[]} templateNames - for example ["brit_infantry_javelineer_b", "brit_cavalry_swordsman_e"]
+ * @param {number} totalCount - total amount of templates, in this  example 12
  * @returns an object where the keys are template names and values are amounts,
- *          for example { "brit_infantry_javelinist_b": 4, "brit_cavalry_swordsman_e": 8 }
+ *          for example { "brit_infantry_javelineer_b": 4, "brit_cavalry_swordsman_e": 8 }
  */
 TriggerHelper.RandomTemplateComposition = function(templateNames, totalCount)
 {
@@ -436,7 +472,7 @@ TriggerHelper.RandomTemplateComposition = function(templateNames, totalCount)
  *        { "templates": ["template3"], "frequency": 1 },
  *        { "templates": ["hero1", "hero2"], "unique_entities": [380, 495], "count": 1 }
  *     ]
- * @param {Number} totalCount - total amount of templates, for example 5.
+ * @param {number} totalCount - total amount of templates, for example 5.
  *
  * @returns an object where the keys are template names and values are amounts,
  *    for example { "template1": 1, "template2": 3, "template3": 2, "hero1": 1 }
@@ -489,7 +525,7 @@ TriggerHelper.BalancedTemplateComposition = function(templateBalancing, totalCou
 				Math.min(remainder, Math.round(templateBalancingFrequencies[i].frequency / templateBalancingFrequencySum * totalCount)));
 
 	if (remainder != 0)
-		warn("Could not chose as many templates as intended, remaining " + remainder + ", chosen: " + uneval(results));
+		warn("Could not choose as many templates as intended, remaining " + remainder + ", chosen: " + uneval(results));
 
 	return results;
 };
@@ -513,7 +549,7 @@ TriggerHelper.SpawnAndGarrisonAtClasses = function(playerID, classes, templates,
 		if (!cmpGarrisonHolder)
 			continue;
 
-		// TODO: account for already garrisoned entities
+		// TODO: account for already garrisoned entities and garrison size.
 		results[entGarrisonHolder] = this.RandomTemplateComposition(templates, Math.floor(cmpGarrisonHolder.GetCapacity() * capacityPercent));
 
 		for (let template in results[entGarrisonHolder])
@@ -523,4 +559,32 @@ TriggerHelper.SpawnAndGarrisonAtClasses = function(playerID, classes, templates,
 	return results;
 };
 
+/**
+ * This will spawn random compositions of entities of the given templates in all turretholders of the given targetClass of the given player.
+ * The turretholder will be filled to capacityPercent.
+ * Returns an object where keys are entityIDs of the affected turretholders and the properties are template compositions, see RandomTemplateComposition.
+ */
+TriggerHelper.SpawnAndTurretAtClasses = function(playerID, classes, templates, capacityPercent)
+{
+	let results = {};
+
+	for (let entTurretHolder of Engine.QueryInterface(SYSTEM_ENTITY, IID_RangeManager).GetEntitiesByPlayer(playerID))
+	{
+		let cmpTurretHolder = Engine.QueryInterface(entTurretHolder, IID_TurretHolder);
+		if (!cmpTurretHolder)
+			continue;
+
+		let cmpIdentity = Engine.QueryInterface(entTurretHolder, IID_Identity);
+		if (!cmpIdentity || !MatchesClassList(cmpIdentity.GetClassesList(), classes))
+			continue;
+
+		results[entTurretHolder] = this.RandomTemplateComposition(templates, Math.max(Math.floor(cmpTurretHolder.GetTurretPoints().length * capacityPercent - cmpTurretHolder.GetEntities().length), 0));
+
+		for (let template in results[entTurretHolder])
+			TriggerHelper.SpawnTurretedUnits(entTurretHolder, template, results[entTurretHolder][template], playerID);
+	}
+
+	return results;
+};
+
 Engine.RegisterGlobal("TriggerHelper", TriggerHelper);
