@@ -10,14 +10,6 @@ function layoutSelectionMultiple()
 	Engine.GetGUIObjectByName("detailsAreaSingle").hidden = true;
 }
 
-function getResourceTypeDisplayName(resourceType)
-{
-	return resourceNameFirstWord(
-		resourceType.generic == "treasure" ?
-			resourceType.specific :
-			resourceType.generic);
-}
-
 // Updates the health bar of garrisoned units
 function updateGarrisonHealthBar(entState, selection)
 {
@@ -59,20 +51,27 @@ function updateGarrisonHealthBar(entState, selection)
 // Fills out information that most entities have
 function displaySingle(entState)
 {
-	// Get general unit and player data
 	let template = GetTemplateData(entState.template);
-	let specificName = template.name.specific;
-	let genericName = template.name.generic;
-	// If packed, add that to the generic name (reduces template clutter)
-	if (genericName && template.pack && template.pack.state == "packed")
-		genericName = sprintf(translate("%(genericName)s — Packed"), { "genericName": genericName });
+
+	let primaryName = g_SpecificNamesPrimary ? template.name.specific : template.name.generic;
+	let secondaryName;
+	if (g_ShowSecondaryNames)
+		secondaryName = g_SpecificNamesPrimary ? template.name.generic : template.name.specific;
+
+	// If packed, add that to the generic name (reduces template clutter).
+	if (template.pack && template.pack.state == "packed")
+	{
+		if (secondaryName && g_ShowSecondaryNames)
+			secondaryName = sprintf(translate("%(secondaryName)s — Packed"), { "secondaryName": secondaryName });
+		else
+			secondaryName = sprintf(translate("Packed"));
+	}
 	let playerState = g_Players[entState.player];
 
 	let civName = g_CivData[playerState.civ].Name;
 	let civEmblem = g_CivData[playerState.civ].Emblem;
 
 	let playerName = playerState.name;
-	let playerColor = rgbToGuiColor(g_DisplayedPlayerColors[entState.player], 128);
 
 	// Indicate disconnected players by prefixing their name
 	if (g_Players[entState.player].offline)
@@ -93,8 +92,35 @@ function displaySingle(entState)
 		Engine.GetGUIObjectByName("rankIcon").tooltip = "";
 	}
 
+	if (entState.statusEffects)
+	{
+		let statusEffectsSection = Engine.GetGUIObjectByName("statusEffectsIcons");
+		statusEffectsSection.hidden = false;
+		let statusIcons = statusEffectsSection.children;
+		let i = 0;
+		for (let effectCode in entState.statusEffects)
+		{
+			let effect = entState.statusEffects[effectCode];
+			statusIcons[i].hidden = false;
+			statusIcons[i].sprite = "stretched:session/icons/status_effects/" + g_StatusEffectsMetadata.getIcon(effect.baseCode) + ".png";
+			statusIcons[i].tooltip = getStatusEffectsTooltip(effect.baseCode, effect, false);
+			let size = statusIcons[i].size;
+			size.top = i * 18;
+			size.bottom = i * 18 + 16;
+			statusIcons[i].size = size;
+
+			if (++i >= statusIcons.length)
+				break;
+		}
+		for (; i < statusIcons.length; ++i)
+			statusIcons[i].hidden = true;
+	}
+	else
+		Engine.GetGUIObjectByName("statusEffectsIcons").hidden = true;
+
 	let showHealth = entState.hitpoints;
 	let showResource = entState.resourceSupply;
+	let showCapture = entState.capturePoints;
 
 	let healthSection = Engine.GetGUIObjectByName("healthSection");
 	let captureSection = Engine.GetGUIObjectByName("captureSection");
@@ -120,11 +146,13 @@ function displaySingle(entState)
 		captureSection.size = showResource ? sectionPosMiddle.size : sectionPosBottom.size;
 		resourceSection.size = showResource ? sectionPosBottom.size : sectionPosMiddle.size;
 	}
-	else
+	else if (showResource)
 	{
 		captureSection.size = sectionPosBottom.size;
 		resourceSection.size = sectionPosTop.size;
 	}
+	else if (showCapture)
+		captureSection.size = sectionPosTop.size;
 
 	// CapturePoints
 	captureSection.hidden = !entState.capturePoints;
@@ -138,7 +166,7 @@ function displaySingle(entState)
 			let size = 100 * Math.max(0, Math.min(1, entState.capturePoints[playerID] / entState.maxCapturePoints));
 			sizeObj.rright = startSize + size;
 			unitCaptureBar.size = sizeObj;
-			unitCaptureBar.sprite = "color:" + rgbToGuiColor(g_DisplayedPlayerColors[playerID], 128);
+			unitCaptureBar.sprite = "color:" + g_DiplomacyColors.getPlayerColor(playerID, 128);
 			unitCaptureBar.hidden = false;
 			return startSize + size;
 		};
@@ -200,7 +228,7 @@ function displaySingle(entState)
 		unitResourceBar.size = resourceSize;
 
 		Engine.GetGUIObjectByName("resourceLabel").caption = sprintf(translate("%(resource)s:"), {
-			"resource": getResourceTypeDisplayName(entState.resourceSupply.type)
+			"resource": resourceNameFirstWord(entState.resourceSupply.type.generic)
 		});
 		Engine.GetGUIObjectByName("resourceStats").caption = resources;
 
@@ -239,8 +267,10 @@ function displaySingle(entState)
 	{
 		resourceCarryingIcon.sprite = "stretched:session/icons/repair.png";
 		resourceCarryingIcon.tooltip = getBuildTimeTooltip(entState);
-		resourceCarryingText.caption = entState.foundation.numBuilders ?
-			Engine.FormatMillisecondsIntoDateStringGMT(entState.foundation.buildTime.timeRemaining * 1000, translateWithContext("countdown format", "m:ss")) : "";
+		resourceCarryingText.caption = entState.foundation.numBuilders ? sprintf(translate("(%(number)s)\n%(time)s"), {
+			"number": entState.foundation.numBuilders,
+			"time": Engine.FormatMillisecondsIntoDateStringGMT(entState.foundation.buildTime.timeRemaining * 1000, translateWithContext("countdown format", "m:ss"))
+		}) : "";
 	}
 	else if (entState.resourceSupply && (!entState.resourceSupply.killBeforeGather || !entState.hitpoints))
 	{
@@ -255,8 +285,10 @@ function displaySingle(entState)
 	{
 		resourceCarryingIcon.sprite = "stretched:session/icons/repair.png";
 		resourceCarryingIcon.tooltip = getRepairTimeTooltip(entState);
-		resourceCarryingText.caption = entState.repairable.numBuilders ?
-			Engine.FormatMillisecondsIntoDateStringGMT(entState.repairable.buildTime.timeRemaining * 1000, translateWithContext("countdown format", "m:ss")) : "";
+		resourceCarryingText.caption = entState.repairable.numBuilders ? sprintf(translate("(%(number)s)\n%(time)s"), {
+			"number": entState.repairable.numBuilders,
+			"time": Engine.FormatMillisecondsIntoDateStringGMT(entState.repairable.buildTime.timeRemaining * 1000, translateWithContext("countdown format", "m:ss"))
+		}) : "";
 	}
 	else
 	{
@@ -264,47 +296,58 @@ function displaySingle(entState)
 		resourceCarryingText.hidden = true;
 	}
 
-	Engine.GetGUIObjectByName("specific").caption = specificName;
 	Engine.GetGUIObjectByName("player").caption = playerName;
-	Engine.GetGUIObjectByName("playerColorBackground").sprite = "color:" + playerColor;
-	Engine.GetGUIObjectByName("generic").caption = genericName == specificName ? "" :
-		sprintf(translate("(%(genericName)s)"), {
-			"genericName": genericName
+
+	Engine.GetGUIObjectByName("playerColorBackground").sprite =
+		"color:" + g_DiplomacyColors.getPlayerColor(entState.player, 128);
+
+	Engine.GetGUIObjectByName("primary").caption = primaryName;
+	Engine.GetGUIObjectByName("secondary").caption = !secondaryName || primaryName == secondaryName ? "" :
+		sprintf(translate("(%(secondaryName)s)"), {
+			"secondaryName": secondaryName
 		});
 
 	let isGaia = playerState.civ == "gaia";
-	Engine.GetGUIObjectByName("playerCivIcon").sprite = isGaia ? "" : "stretched:grayscale:" + civEmblem;
+	Engine.GetGUIObjectByName("playerCivIcon").sprite = isGaia ? "" : "cropped:1.0, 0.15625 center:grayscale:" + civEmblem;
 	Engine.GetGUIObjectByName("player").tooltip = isGaia ? "" : civName;
 
 	// TODO: we should require all entities to have icons
 	Engine.GetGUIObjectByName("icon").sprite = template.icon ? ("stretched:session/portraits/" + template.icon) : "BackgroundBlack";
 	if (template.icon)
 		Engine.GetGUIObjectByName("iconBorder").onPressRight = () => {
-			showTemplateDetails(entState.template);
+			showTemplateDetails(entState.template, playerState.civ);
 		};
 
-	Engine.GetGUIObjectByName("attackAndArmorStats").tooltip = [
+	let detailedTooltip = [
 		getAttackTooltip,
-		getSplashDamageTooltip,
 		getHealerTooltip,
-		getArmorTooltip,
+		getResistanceTooltip,
 		getGatherTooltip,
 		getSpeedTooltip,
 		getGarrisonTooltip,
+		getTurretsTooltip,
+		getPopulationBonusTooltip,
 		getProjectilesTooltip,
 		getResourceTrickleTooltip,
+		getUpkeepTooltip,
 		getLootTooltip
 	].map(func => func(entState)).filter(tip => tip).join("\n");
+	if (detailedTooltip)
+	{
+		Engine.GetGUIObjectByName("attackAndResistanceStats").hidden = false;
+		Engine.GetGUIObjectByName("attackAndResistanceStats").tooltip = detailedTooltip;
+	}
+	else
+		Engine.GetGUIObjectByName("attackAndResistanceStats").hidden = true;
 
 	let iconTooltips = [];
 
-	if (genericName)
-		iconTooltips.push("[font=\"sans-bold-16\"]" + genericName + "[/font]");
-
+	iconTooltips.push(setStringTags(primaryName, g_TooltipTextFormats.namePrimaryBig));
 	iconTooltips = iconTooltips.concat([
 		getVisibleEntityClassesFormatted,
 		getAurasTooltip,
 		getEntityTooltip,
+		getTreasureTooltip,
 		showTemplateViewerOnRightClickTooltip
 	].map(func => func(template)));
 
@@ -324,6 +367,7 @@ function displayMultiple(entStates)
 	let playerID = 0;
 	let totalCarrying = {};
 	let totalLoot = {};
+	let garrisonSize = 0;
 
 	for (let entState of entStates)
 	{
@@ -353,6 +397,12 @@ function displayMultiple(entStates)
 			totalCarrying[type] = (totalCarrying[type] || 0) + carrying[type];
 			totalLoot[type] = (totalLoot[type] || 0) + carrying[type];
 		}
+
+		if (entState.garrisonable)
+			garrisonSize += entState.garrisonable.size;
+
+		if (entState.garrisonHolder)
+			garrisonSize += entState.garrisonHolder.occupiedSlots;
 	}
 
 	Engine.GetGUIObjectByName("healthMultiple").hidden = averageHealth <= 0;
@@ -381,7 +431,7 @@ function displayMultiple(entStates)
 			let size = 100 * Math.max(0, Math.min(1, capturePoints[pID] / maxCapturePoints));
 			sizeObj.rbottom = startSize + size;
 			unitCaptureBar.size = sizeObj;
-			unitCaptureBar.sprite = "color:" + rgbToGuiColor(g_DisplayedPlayerColors[pID], 128);
+			unitCaptureBar.sprite = "color:" + g_DiplomacyColors.getPlayerColor(pID, 128);
 			unitCaptureBar.hidden = false;
 			return startSize + size;
 		};
@@ -406,6 +456,12 @@ function displayMultiple(entStates)
 	numberOfUnits.caption = entStates.length;
 	numberOfUnits.tooltip = "";
 
+	if (garrisonSize)
+		numberOfUnits.tooltip = sprintf(translate("%(label)s: %(details)s\n"), {
+			"label": headerFont(translate("Garrison Size")),
+			"details": bodyFont(garrisonSize)
+		});
+
 	if (Object.keys(totalCarrying).length)
 		numberOfUnits.tooltip = sprintf(translate("%(label)s %(details)s\n"), {
 			"label": headerFont(translate("Carrying:")),
