@@ -28,10 +28,12 @@
 
 // Cache some formation info
 // Available formations per player
-let g_AvailableFormations = new Map();
-let g_FormationsInfo = new Map();
+var g_AvailableFormations = new Map();
+var g_FormationsInfo = new Map();
 
-let g_SelectionPanels = {};
+var g_SelectionPanels = {};
+
+var g_SelectionPanelBarterButtonManager;
 
 g_SelectionPanels.Alert = {
 	"getMaxNumberOfItems": function()
@@ -77,28 +79,24 @@ g_SelectionPanels.Alert = {
 g_SelectionPanels.Barter = {
 	"getMaxNumberOfItems": function()
 	{
-		return 4;
+		return 5;
 	},
-	"rowLength": 4,
+	"rowLength": 5,
 	"conflictsWith": ["Garrison"],
 	"getItems": function(unitEntStates)
 	{
 		// If more than `rowLength` resources, don't display icons.
-		if (unitEntStates.every(state => !state.isBarterMarket) || g_ResourceData.GetCodes().length > this.rowLength)
+		if (unitEntStates.every(state => !state.isBarterMarket) || g_ResourceData.GetBarterableCodes().length > this.rowLength)
 			return [];
-		return g_ResourceData.GetCodes();
+		return g_ResourceData.GetBarterableCodes();
 	},
 	"setupButton": function(data)
 	{
-		barterOpenCommon(data.item, data.i, "unitBarter");
-		barterUpdateCommon(data.item, data.i, "unitBarter", data.player);
-
-		let button = {};
-		for (let action of g_BarterActions)
-			button[action] = Engine.GetGUIObjectByName("unitBarter" + action + "Button[" + data.i + "]");
-
-		setPanelObjectPosition(button.Sell, data.i, data.rowLength);
-		setPanelObjectPosition(button.Buy, data.i + data.rowLength, data.rowLength);
+		if (g_SelectionPanelBarterButtonManager)
+		{
+			g_SelectionPanelBarterButtonManager.setViewedPlayer(data.player);
+			g_SelectionPanelBarterButtonManager.update();
+		}
 		return true;
 	}
 };
@@ -114,7 +112,7 @@ g_SelectionPanels.Command = {
 
 		for (let command in g_EntityCommands)
 		{
-			let info = g_EntityCommands[command].getInfo(unitEntStates);
+			let info = getCommandInfo(command, unitEntStates);
 			if (info)
 			{
 				info.name = command;
@@ -136,10 +134,7 @@ g_SelectionPanels.Command = {
 
 		data.countDisplay.caption = data.item.count || "";
 
-		data.button.enabled =
-			g_IsObserver && data.item.name == "focus-rally" ||
-			controlsPlayer(data.player) && (data.item.name != "delete" ||
-				data.unitEntStates.some(state => !isUndeletable(state)));
+		data.button.enabled = data.item.enabled == true;
 
 		data.icon.sprite = "stretched:session/icons/" + data.item.icon;
 
@@ -155,71 +150,19 @@ g_SelectionPanels.Command = {
 	}
 };
 
-g_SelectionPanels.AllyCommand = {
-	"getMaxNumberOfItems": function()
-	{
-		return 2;
-	},
-	"conflictsWith": ["Command"],
-	"getItems": function(unitEntStates)
-	{
-		let commands = [];
-		for (let command in g_AllyEntityCommands)
-			for (let state of unitEntStates)
-			{
-				let info = g_AllyEntityCommands[command].getInfo(state);
-				if (info)
-				{
-					info.name = command;
-					commands.push(info);
-					break;
-				}
-			}
-		return commands;
-	},
-	"setupButton": function(data)
-	{
-		data.button.tooltip = data.item.tooltip;
-
-		data.button.onPress = function() {
-			if (data.item.callback)
-				data.item.callback(data.item);
-			else
-				performAllyCommand(data.unitEntStates[0].id, data.item.name);
-		};
-
-		data.countDisplay.caption = data.item.count || "";
-
-		data.button.enabled = !!data.item.count;
-
-		let grayscale = data.button.enabled ? "" : "grayscale:";
-		data.icon.sprite = "stretched:" + grayscale + "session/icons/" + data.item.icon;
-
-		let size = data.button.size;
-		// relative to the center ( = 50%)
-		size.rleft = 50;
-		size.rright = 50;
-		// offset from the center calculation, count on square buttons, so size.bottom is the width too
-		size.left = (data.i - data.numberOfItems / 2) * (size.bottom + 1);
-		size.right = size.left + size.bottom;
-		data.button.size = size;
-
-		return true;
-	}
-};
-
 g_SelectionPanels.Construction = {
 	"getMaxNumberOfItems": function()
 	{
-		return 24 - getNumberOfRightPanelButtons();
+		return 40 - getNumberOfRightPanelButtons();
 	},
+	"rowLength": 10,
 	"getItems": function()
 	{
 		return getAllBuildableEntitiesFromSelection();
 	},
 	"setupButton": function(data)
 	{
-		let template = GetTemplateData(data.item);
+		let template = GetTemplateData(data.item, data.player);
 		if (!template)
 			return false;
 
@@ -236,22 +179,30 @@ g_SelectionPanels.Construction = {
 			});
 
 		data.button.onPress = function() { startBuildingPlacement(data.item, data.playerState); };
-		data.button.onPressRight = function() { showTemplateDetails(data.item); };
+		let showTemplateFunc = () => { showTemplateDetails(data.item, data.playerState.civ); };
+		data.button.onPressRight = showTemplateFunc;
+		data.button.onPressRightDisabled = showTemplateFunc;
 
 		let tooltips = [
 			getEntityNamesFormatted,
 			getVisibleEntityClassesFormatted,
 			getAurasTooltip,
-			getEntityTooltip,
-			getEntityCostTooltip,
-			getGarrisonTooltip,
-			getPopulationBonusTooltip,
-			showTemplateViewerOnRightClickTooltip
+			getEntityTooltip
 		].map(func => func(template));
+		tooltips.push(
+			getEntityCostTooltip(template, data.player),
+			getResourceDropsiteTooltip(template),
+			getGarrisonTooltip(template),
+			getTurretsTooltip(template),
+			getPopulationBonusTooltip(template),
+			showTemplateViewerOnRightClickTooltip(template)
+		);
+
 
 		let limits = getEntityLimitAndCount(data.playerState, data.item);
 		tooltips.push(
 			formatLimitString(limits.entLimit, limits.entCount, limits.entLimitChangers),
+			formatMatchLimitString(limits.matchLimit, limits.matchCount, limits.type),
 			getRequiredTechnologyTooltip(technologyEnabled, template.requiredTechnology, GetSimState().players[data.player].civ),
 			getNeededResourcesTooltip(neededResources));
 
@@ -282,32 +233,28 @@ g_SelectionPanels.Construction = {
 g_SelectionPanels.Formation = {
 	"getMaxNumberOfItems": function()
 	{
-		return 16;
+		return 15;
 	},
-	"rowLength": 4,
+	"rowLength": 5,
 	"conflictsWith": ["Garrison"],
 	"getItems": function(unitEntStates)
 	{
 		if (unitEntStates.some(state => !hasClass(state, "Unit")))
 			return [];
 
+		if (unitEntStates.every(state => !state.identity || !state.identity.hasSomeFormation))
+			return [];
+
 		if (!g_AvailableFormations.has(unitEntStates[0].player))
 			g_AvailableFormations.set(unitEntStates[0].player, Engine.GuiInterfaceCall("GetAvailableFormations", unitEntStates[0].player));
 
-		let availableFormations = g_AvailableFormations.get(unitEntStates[0].player);
-
-		// Hide the panel if all formations are disabled
-		if (availableFormations.some(formation => canMoveSelectionIntoFormation(formation)))
-			return availableFormations;
-
-		return [];
+		return g_AvailableFormations.get(unitEntStates[0].player).filter(formation => unitEntStates.some(state => !!state.identity && state.identity.formations.indexOf(formation) != -1));
 	},
 	"setupButton": function(data)
 	{
 		if (!g_FormationsInfo.has(data.item))
 			g_FormationsInfo.set(data.item, Engine.GuiInterfaceCall("GetFormationInfoFromTemplate", { "templateName": data.item }));
 
-		let formationInfo = g_FormationsInfo.get(data.item);
 		let formationOk = canMoveSelectionIntoFormation(data.item);
 		let unitIds = data.unitEntStates.map(state => state.id);
 		let formationSelected = Engine.GuiInterfaceCall("IsFormationSelected", {
@@ -319,7 +266,21 @@ g_SelectionPanels.Formation = {
 			performFormation(unitIds, data.item);
 		};
 
+		data.button.onMouseRightPress = () => g_AutoFormation.setDefault(data.item);
+
+		let formationInfo = g_FormationsInfo.get(data.item);
 		let tooltip = translate(formationInfo.name);
+
+		let isDefaultFormation = g_AutoFormation.isDefault(data.item);
+		if (data.item === NULL_FORMATION)
+			tooltip += "\n" + (isDefaultFormation ?
+				translate("Default formation is disabled.") :
+				translate("Right-click to disable the default formation feature."));
+		else
+			tooltip += "\n" + (isDefaultFormation ?
+				translate("This is the default formation, used for movement orders.") :
+				translate("Right-click to set this as the default formation."));
+
 		if (!formationOk && formationInfo.tooltip)
 			tooltip += "\n" + coloredText(translate(formationInfo.tooltip), "red");
 		data.button.tooltip = tooltip;
@@ -327,6 +288,7 @@ g_SelectionPanels.Formation = {
 		data.button.enabled = formationOk && controlsPlayer(data.player);
 		let grayscale = formationOk ? "" : "grayscale:";
 		data.guiSelection.hidden = !formationSelected;
+		data.countDisplay.hidden = !isDefaultFormation;
 		data.icon.sprite = "stretched:" + grayscale + "session/icons/" + formationInfo.icon;
 
 		setPanelObjectPosition(data.button, data.i, data.rowLength);
@@ -368,13 +330,11 @@ g_SelectionPanels.Garrison = {
 
 		data.countDisplay.caption = data.item.ents.length || "";
 
-		let canUngarrison =
-			g_ViewedPlayer == data.player ||
-			g_ViewedPlayer == entState.player;
+		let canUngarrison = controlsPlayer(data.player) || controlsPlayer(entState.player);
 
-		data.button.enabled = canUngarrison && controlsPlayer(g_ViewedPlayer);
+		data.button.enabled = canUngarrison;
 
-		data.button.tooltip = (canUngarrison || g_IsObserver ?
+		data.button.tooltip = (canUngarrison ?
 			sprintf(translate("Unload %(name)s"), { "name": getEntityNames(template) }) + "\n" +
 			translate("Single-click to unload 1. Shift-click to unload all of this type.") :
 			getEntityNames(template)) + "\n" +
@@ -382,11 +342,11 @@ g_SelectionPanels.Garrison = {
 				"playername": g_Players[entState.player].name
 			});
 
-		data.guiSelection.sprite = getPlayerHighlightColor(entState.player);
+		data.guiSelection.sprite = "color:" + g_DiplomacyColors.getPlayerColor(entState.player, 160);
 		data.button.sprite_disabled = data.button.sprite;
 
 		// Selection panel buttons only appear disabled if they
-		// also appear disabled to the owner of the building.
+		// also appear disabled to the owner of the structure.
 		data.icon.sprite =
 			(canUngarrison || g_IsObserver ? "" : "grayscale:") +
 			"stretched:session/portraits/" + template.icon;
@@ -400,8 +360,9 @@ g_SelectionPanels.Garrison = {
 g_SelectionPanels.Gate = {
 	"getMaxNumberOfItems": function()
 	{
-		return 24 - getNumberOfRightPanelButtons();
+		return 40 - getNumberOfRightPanelButtons();
 	},
+	"rowLength": 10,
 	"getItems": function(unitEntStates)
 	{
 		let hideLocked = unitEntStates.every(state => !state.gate || !state.gate.locked);
@@ -441,8 +402,9 @@ g_SelectionPanels.Gate = {
 g_SelectionPanels.Pack = {
 	"getMaxNumberOfItems": function()
 	{
-		return 24 - getNumberOfRightPanelButtons();
+		return 40 - getNumberOfRightPanelButtons();
 	},
+	"rowLength": 10,
 	"getItems": function(unitEntStates)
 	{
 		let checks = {};
@@ -531,22 +493,37 @@ g_SelectionPanels.Queue = {
 	 */
 	"getItems": function(unitEntStates)
 	{
-		let queue = [];
+		const queue = [];
 		let foundNew = true;
 		for (let i = 0; foundNew; ++i)
 		{
 			foundNew = false;
-			for (let state of unitEntStates)
+			for (const state of unitEntStates)
 			{
 				if (!state.production || !state.production.queue[i])
 					continue;
 				queue.push({
 					"producingEnt": state.id,
-					"queuedItem": state.production.queue[i]
+					"queuedItem": state.production.queue[i],
+					"autoqueue": state.production.autoqueue && state.production.queue[i].unitTemplate,
 				});
 				foundNew = true;
 			}
 		}
+		if (!queue.length)
+			return queue;
+		// Add 'ghost' items to show autoqueues.
+		const repeat = [];
+		for (const item of queue)
+			if (item.autoqueue)
+			{
+				const ghostItem = clone(item);
+				ghostItem.ghost = true;
+				repeat.push(ghostItem);
+			}
+		if (repeat.length)
+			for (let i = 0; queue.length < g_SelectionPanels.Queue.getMaxNumberOfItems(); ++i)
+				queue.push(repeat[i % repeat.length]);
 		return queue;
 	},
 	"resizePanel": function(numberOfItems, rowLength)
@@ -574,38 +551,54 @@ g_SelectionPanels.Queue = {
 			warning("Unknown production queue template " + uneval(queuedItem));
 			return false;
 		}
-
 		data.button.onPress = function() { removeFromProductionQueue(data.item.producingEnt, queuedItem.id); };
 
-		let tooltip = getEntityNames(template);
+		const tooltips = [getEntityNames(template)];
+		if (data.item.ghost)
+			tooltips.push(translate("The auto-queue will try to train this item later."));
 		if (queuedItem.neededSlots)
 		{
-			tooltip += "\n" + coloredText(translate("Insufficient population capacity:"), "red");
-			tooltip += "\n" + sprintf(translate("%(population)s %(neededSlots)s"), {
+			tooltips.push(coloredText(translate("Insufficient population capacity:"), "red"));
+			tooltips.push(sprintf(translate("%(population)s %(neededSlots)s"), {
 				"population": resourceIcon("population"),
 				"neededSlots": queuedItem.neededSlots
-			});
+			}));
 		}
-		data.button.tooltip = tooltip;
+		tooltips.push(showTemplateViewerOnRightClickTooltip(template));
+		data.button.tooltip = tooltips.join("\n");
 
 		data.countDisplay.caption = queuedItem.count > 1 ? queuedItem.count : "";
 
-		// Show the time remaining to finish the first item
-		if (data.i == 0)
-			Engine.GetGUIObjectByName("queueTimeRemaining").caption =
-				Engine.FormatMillisecondsIntoDateStringGMT(queuedItem.timeRemaining, translateWithContext("countdown format", "m:ss"));
+		if (data.item.ghost)
+		{
+			data.button.enabled = false;
+			Engine.GetGUIObjectByName("unitQueueProgressSlider[" + data.i + "]").sprite="color:0 150 250 50";
+		}
+		else
+		{
+			// Show the time remaining to finish the first item
+			if (data.i == 0)
+				Engine.GetGUIObjectByName("queueTimeRemaining").caption =
+					Engine.FormatMillisecondsIntoDateStringGMT(queuedItem.timeRemaining, translateWithContext("countdown format", "m:ss"));
 
-		let guiObject = Engine.GetGUIObjectByName("unitQueueProgressSlider[" + data.i + "]");
-		let size = guiObject.size;
+			const guiObject = Engine.GetGUIObjectByName("unitQueueProgressSlider[" + data.i + "]");
+			guiObject.sprite = "queueProgressSlider";
+			const size = guiObject.size;
 
-		// Buttons are assumed to be square, so left/right offsets can be used for top/bottom.
-		size.top = size.left + Math.round(queuedItem.progress * (size.right - size.left));
-		guiObject.size = size;
+			// Buttons are assumed to be square, so left/right offsets can be used for top/bottom.
+			size.top = size.left + Math.round(queuedItem.progress * (size.right - size.left));
+			guiObject.size = size;
+
+			data.button.enabled = controlsPlayer(data.player);
+		}
 
 		if (template.icon)
-			data.icon.sprite = "stretched:session/portraits/" + template.icon;
+			data.icon.sprite = (data.item.ghost ? "grayscale:" : "") + "stretched:session/portraits/" + template.icon;
 
-		data.button.enabled = controlsPlayer(data.player);
+
+		const showTemplateFunc = () => { showTemplateDetails(data.item.queuedItem.unitTemplate || data.item.queuedItem.technologyTemplate, data.playerState.civ); };
+		data.button.onPressRight = showTemplateFunc;
+		data.button.onPressRightDisabled = showTemplateFunc;
 
 		setPanelObjectPosition(data.button, data.i, data.rowLength);
 		return true;
@@ -615,8 +608,9 @@ g_SelectionPanels.Queue = {
 g_SelectionPanels.Research = {
 	"getMaxNumberOfItems": function()
 	{
-		return 8;
+		return 10;
 	},
+	"rowLength": 10,
 	"getItems": function(unitEntStates)
 	{
 		let ret = [];
@@ -625,13 +619,20 @@ g_SelectionPanels.Research = {
 				unitEntStates[0].production.technologies.map(tech => ({
 					"tech": tech,
 					"techCostMultiplier": unitEntStates[0].production.techCostMultiplier,
-					"researchFacilityId": unitEntStates[0].id
+					"researchFacilityId": unitEntStates[0].id,
+					"isUpgrading": !!unitEntStates[0].upgrade && unitEntStates[0].upgrade.isUpgrading
 				}));
 
-		for (let state of unitEntStates)
+		let sortedEntStates = unitEntStates.sort((a, b) =>
+			(!b.upgrade || !b.upgrade.isUpgrading) - (!a.upgrade || !a.upgrade.isUpgrading) ||
+		 	(!a.production ? 0 : a.production.queue.length) - (!b.production ? 0 : b.production.queue.length)
+		 );
+
+		for (let state of sortedEntStates)
 		{
 			if (!state.production || !state.production.technologies)
 				continue;
+
 			// Remove the techs we already have in ret (with the same name and techCostMultiplier)
 			let filteredTechs = state.production.technologies.filter(
 				tech => tech != null && !ret.some(
@@ -650,7 +651,8 @@ g_SelectionPanels.Research = {
 				ret = ret.concat(filteredTechs.map(tech => ({
 					"tech": tech,
 					"techCostMultiplier": state.production.techCostMultiplier,
-					"researchFacilityId": state.id
+					"researchFacilityId": state.id,
+					"isUpgrading": !!state.upgrade && state.upgrade.isUpgrading
 				})));
 		}
 		return ret;
@@ -692,6 +694,16 @@ g_SelectionPanels.Research = {
 			if (!template)
 				return false;
 
+			// Not allowed by civ.
+			if (!template.reqs)
+			{
+				// One of the pair may still be researchable by the current civ,
+				// hence don't hide everything.
+				Engine.GetGUIObjectByName("unitResearchButton[" + data.i + "]").hidden = true;
+				pair.hidden = true;
+				continue;
+			}
+
 			for (let res in template.cost)
 				template.cost[res] *= data.item.techCostMultiplier[res];
 
@@ -746,7 +758,7 @@ g_SelectionPanels.Research = {
 
 						entityCounts.push(sprintf(translatePlural("%(number)s entity of class %(class)s", "%(number)s entities of class %(class)s", remaining), {
 							"number": remaining,
-							"class": entity.class
+							"class": translate(entity.class)
 						}));
 					}
 
@@ -763,11 +775,14 @@ g_SelectionPanels.Research = {
 				addResearchToQueue(data.item.researchFacilityId, t);
 			})(tech);
 
-			button.onPressRight = (t => function () {
+			let showTemplateFunc =  (t => function() {
 				showTemplateDetails(
 					t,
 					GetTemplateData(data.unitEntStates.find(state => state.id == data.item.researchFacilityId).template).nativeCiv);
-			})(tech);
+			});
+
+			button.onPressRight = showTemplateFunc(tech);
+			button.onPressRightDisabled = showTemplateFunc(tech);
 
 			if (data.item.tech.pair)
 			{
@@ -796,6 +811,14 @@ g_SelectionPanels.Research = {
 			else
 				button.enabled = controlsPlayer(data.player);
 
+			if (data.item.isUpgrading)
+			{
+				button.enabled = false;
+				modifier += "color:0 0 0 127:grayscale:";
+				button.tooltip += "\n" + coloredText(translate("Cannot research while upgrading."), "red");
+
+			}
+
 			if (template.icon)
 				icon.sprite = modifier + "stretched:session/portraits/" + template.icon;
 
@@ -873,7 +896,7 @@ g_SelectionPanels.Selection = {
 			});
 		data.button.tooltip = tooltip;
 
-		data.guiSelection.sprite = getPlayerHighlightColor(unitOwner);
+		data.guiSelection.sprite = "color:" + g_DiplomacyColors.getPlayerColor(unitOwner, 160);
 		data.guiSelection.hidden = !g_IsObserver;
 
 		data.countDisplay.caption = data.item.ents.length || "";
@@ -924,15 +947,16 @@ g_SelectionPanels.Stance = {
 g_SelectionPanels.Training = {
 	"getMaxNumberOfItems": function()
 	{
-		return 24 - getNumberOfRightPanelButtons();
+		return 40 - getNumberOfRightPanelButtons();
 	},
+	"rowLength": 10,
 	"getItems": function()
 	{
 		return getAllTrainableEntitiesFromSelection();
 	},
 	"setupButton": function(data)
 	{
-		let template = GetTemplateData(data.item);
+		let template = GetTemplateData(data.item, data.player);
 		if (!template)
 			return false;
 
@@ -941,7 +965,7 @@ g_SelectionPanels.Training = {
 			"player": data.player
 		});
 
-		let unitIds = data.unitEntStates.map(status => status.id)
+		let unitIds = data.unitEntStates.map(status => status.id);
 		let [buildingsCountToTrainFullBatch, fullBatchSize, remainderBatch] =
 			getTrainingStatus(unitIds, data.item, data.playerState);
 
@@ -958,9 +982,10 @@ g_SelectionPanels.Training = {
 			if (!neededResources)
 				addTrainingToQueue(unitIds, data.item, data.playerState);
 		};
-		data.button.onPressRight = function() {
-			showTemplateDetails(data.item);
-		};
+
+		let showTemplateFunc = () => { showTemplateDetails(data.item, data.playerState.civ); };
+		data.button.onPressRight = showTemplateFunc;
+		data.button.onPressRightDisabled = showTemplateFunc;
 
 		data.countDisplay.caption = trainNum > 1 ? trainNum : "";
 
@@ -971,22 +996,23 @@ g_SelectionPanels.Training = {
 			getVisibleEntityClassesFormatted(template),
 			getAurasTooltip(template),
 			getEntityTooltip(template),
-			getEntityCostTooltip(template, unitIds[0], buildingsCountToTrainFullBatch, fullBatchSize, remainderBatch)
+			getEntityCostTooltip(template, data.player, unitIds[0], buildingsCountToTrainFullBatch, fullBatchSize, remainderBatch)
 		];
-
 		let limits = getEntityLimitAndCount(data.playerState, data.item);
-		tooltips.push(formatLimitString(limits.entLimit, limits.entCount, limits.entLimitChangers));
+		tooltips.push(formatLimitString(limits.entLimit, limits.entCount, limits.entLimitChangers),
+			formatMatchLimitString(limits.matchLimit, limits.matchCount, limits.type));
 
 		if (Engine.ConfigDB_GetValue("user", "showdetailedtooltips") === "true")
 			tooltips = tooltips.concat([
 				getHealthTooltip,
 				getAttackTooltip,
-				getSplashDamageTooltip,
 				getHealerTooltip,
-				getArmorTooltip,
+				getResistanceTooltip,
 				getGarrisonTooltip,
+				getTurretsTooltip,
 				getProjectilesTooltip,
-				getSpeedTooltip
+				getSpeedTooltip,
+				getResourceDropsiteTooltip
 			].map(func => func(template)));
 
 		tooltips.push(showTemplateViewerOnRightClickTooltip());
@@ -1010,6 +1036,13 @@ g_SelectionPanels.Training = {
 				modifier = resourcesToAlphaMask(neededResources) + ":";
 		}
 
+		if (data.unitEntStates.every(state => state.upgrade && state.upgrade.isUpgrading))
+		{
+			data.button.enabled = false;
+			modifier = "color:0 0 0 127:grayscale:";
+			data.button.tooltip += "\n" + coloredText(translate("Cannot train while upgrading."), "red");
+		}
+
 		if (template.icon)
 			data.icon.sprite = modifier + "stretched:session/portraits/" + template.icon;
 
@@ -1023,8 +1056,9 @@ g_SelectionPanels.Training = {
 g_SelectionPanels.Upgrade = {
 	"getMaxNumberOfItems": function()
 	{
-		return 24 - getNumberOfRightPanelButtons();
+		return 40 - getNumberOfRightPanelButtons();
 	},
+	"rowLength": 10,
 	"getItems": function(unitEntStates)
 	{
 		// Interface becomes complicated with multiple different units and this is meant per-entity, so prevent it if the selection has multiple different units.
@@ -1039,6 +1073,9 @@ g_SelectionPanels.Upgrade = {
 		if (!template)
 			return false;
 
+		let progressOverlay = Engine.GetGUIObjectByName("unitUpgradeProgressSlider[" + data.i + "]");
+		progressOverlay.hidden = true;
+
 		let technologyEnabled = true;
 
 		if (data.item.requiredTechnology)
@@ -1047,61 +1084,73 @@ g_SelectionPanels.Upgrade = {
 				"player": data.player
 			});
 
+		let limits = getEntityLimitAndCount(data.playerState, data.item.entity);
+		let upgradingEntStates = data.unitEntStates.filter(state => state.upgrade.template == data.item.entity);
+
+		let upgradableEntStates = data.unitEntStates.filter(state =>
+			!state.upgrade.progress &&
+			(!state.production || !state.production.queue || !state.production.queue.length));
+
 		let neededResources = data.item.cost && Engine.GuiInterfaceCall("GetNeededResources", {
-			"cost": multiplyEntityCosts(data.item, data.unitEntStates.length),
+			"cost": multiplyEntityCosts(data.item, upgradableEntStates.length),
 			"player": data.player
 		});
 
-		let limits = getEntityLimitAndCount(data.playerState, data.item.entity);
-		let progress = data.unitEntStates[0].upgrade.progress || 0;
-		let isUpgrading = data.unitEntStates[0].upgrade.template == data.item.entity;
-
 		let tooltip;
-		if (!progress)
+		let modifier = "";
+		if (!upgradingEntStates.length && upgradableEntStates.length)
 		{
+			let primaryName = g_SpecificNamesPrimary ? template.name.specific : template.name.generic;
+			let secondaryName;
+			if (g_ShowSecondaryNames)
+				secondaryName = g_SpecificNamesPrimary ? template.name.generic : template.name.specific;
+
 			let tooltips = [];
-			if (data.item.tooltip)
-				tooltips.push(sprintf(translate("Upgrade into a %(name)s. %(tooltip)s"), {
-					"name": template.name.generic,
-					"tooltip": translate(data.item.tooltip)
-				}));
+			if (g_ShowSecondaryNames)
+			{
+				if (data.item.tooltip)
+					tooltips.push(sprintf(translate("Upgrade to a %(primaryName)s (%(secondaryName)s). %(tooltip)s"), {
+						"primaryName": primaryName,
+						"secondaryName": secondaryName,
+						"tooltip": translate(data.item.tooltip)
+					}));
+				else
+					tooltips.push(sprintf(translate("Upgrade to a %(primaryName)s (%(secondaryName)s)."), {
+						"primaryName": primaryName,
+						"secondaryName": secondaryName
+					}));
+			}
 			else
-				tooltips.push(sprintf(translate("Upgrade into a %(name)s."), {
-					"name": template.name.generic
-				}));
+			{
+				if (data.item.tooltip)
+					tooltips.push(sprintf(translate("Upgrade to a %(primaryName)s. %(tooltip)s"), {
+						"primaryName": primaryName,
+						"tooltip": translate(data.item.tooltip)
+					}));
+				else
+					tooltips.push(sprintf(translate("Upgrade to a %(primaryName)s."), {
+						"primaryName": primaryName
+					}));
+			}
 
 			tooltips.push(
-				getEntityCostComponentsTooltipString(data.item, undefined, data.unitEntStates.length),
+				getEntityCostTooltip(data.item, undefined, undefined, data.unitEntStates.length),
 				formatLimitString(limits.entLimit, limits.entCount, limits.entLimitChangers),
+				formatMatchLimitString(limits.matchLimit, limits.matchCount, limits.type),
 				getRequiredTechnologyTooltip(technologyEnabled, data.item.requiredTechnology, GetSimState().players[data.player].civ),
 				getNeededResourcesTooltip(neededResources),
 				showTemplateViewerOnRightClickTooltip());
 
 			tooltip = tooltips.filter(tip => tip).join("\n");
 
-			data.button.onPress = function() { upgradeEntity(data.item.entity); };
-		}
-		else if (isUpgrading)
-		{
-			tooltip = translate("Cancel Upgrading");
-			data.button.onPress = function() { cancelUpgradeEntity(); };
-		}
-		else
-		{
-			tooltip = translate("Cannot upgrade when the entity is already upgrading.");
-			data.button.onPress = function() {};
-		}
-		data.button.enabled = controlsPlayer(data.player);
-		data.button.tooltip = tooltip;
+			data.button.onPress = function() {
+				upgradeEntity(
+				    data.item.entity,
+				    upgradableEntStates.map(state => state.id));
+			};
 
-		data.button.onPressRight = function() {
-			showTemplateDetails(data.item.entity);
-		};
-
-		let modifier = "";
-		if (!isUpgrading)
-			if (progress || !technologyEnabled || limits.canBeAddedCount == 0 &&
-				!hasSameRestrictionCategory(data.item.entity, data.unitEntStates[0].template))
+			if (!technologyEnabled || limits.canBeAddedCount == 0 &&
+				!upgradableEntStates.some(state => hasSameRestrictionCategory(data.item.entity, state.template)))
 			{
 				data.button.enabled = false;
 				modifier = "color:0 0 0 127:grayscale:";
@@ -1112,25 +1161,58 @@ g_SelectionPanels.Upgrade = {
 				modifier = resourcesToAlphaMask(neededResources) + ":";
 			}
 
+			data.countDisplay.caption = upgradableEntStates.length > 1 ? upgradableEntStates.length : "";
+		}
+		else if (upgradingEntStates.length)
+		{
+			tooltip = translate("Cancel Upgrading");
+			data.button.onPress = function() { cancelUpgradeEntity(); };
+			data.countDisplay.caption = upgradingEntStates.length > 1 ? upgradingEntStates.length : "";
+
+			let progress = 0;
+			for (let state of upgradingEntStates)
+				progress = Math.max(progress, state.upgrade.progress || 1);
+			let progressOverlaySize = progressOverlay.size;
+			// TODO This is bad: we assume the progressOverlay is square
+			progressOverlaySize.top = progressOverlaySize.bottom + Math.round((1 - progress) * (progressOverlaySize.left - progressOverlaySize.right));
+			progressOverlay.size = progressOverlaySize;
+			progressOverlay.hidden = false;
+		}
+		else
+		{
+			tooltip = coloredText(translatePlural(
+				"Cannot upgrade when the entity is training, researching or already upgrading.",
+				"Cannot upgrade when all entities are training, researching or already upgrading.",
+				data.unitEntStates.length), "red");
+
+			data.button.onPress = function() {};
+
+			data.button.enabled = false;
+			modifier = "color:0 0 0 127:grayscale:";
+		}
+		data.button.enabled = controlsPlayer(data.player);
+		data.button.tooltip = tooltip;
+
+		let showTemplateFunc = () => { showTemplateDetails(data.item.entity, data.playerState.civ); };
+		data.button.onPressRight = showTemplateFunc;
+		data.button.onPressRightDisabled = showTemplateFunc;
+
 		data.icon.sprite = modifier + "stretched:session/" +
 			(data.item.icon || "portraits/" + template.icon);
 
-		data.countDisplay.caption = data.unitEntStates.length > 1 ? data.unitEntStates.length : "";
-
-		let progressOverlay = Engine.GetGUIObjectByName("unitUpgradeProgressSlider[" + data.i + "]");
-		if (isUpgrading)
-		{
-			let size = progressOverlay.size;
-			size.top = size.left + Math.round(progress * (size.right - size.left));
-			progressOverlay.size = size;
-		}
-		progressOverlay.hidden = !isUpgrading;
-
 		setPanelObjectPosition(data.button, data.i + getNumberOfRightPanelButtons(), data.rowLength);
 		return true;
 	}
 };
 
+function initSelectionPanels()
+{
+
+	let unitBarterPanel = Engine.GetGUIObjectByName("unitBarterPanel");
+	if (BarterButtonManager.IsAvailable(unitBarterPanel))
+		g_SelectionPanelBarterButtonManager = new BarterButtonManager(unitBarterPanel);
+}
+
 /**
  * Pauses game and opens the template details viewer for a selected entity or technology.
  *
@@ -1141,13 +1223,17 @@ g_SelectionPanels.Upgrade = {
  */
 function showTemplateDetails(templateName, civCode)
 {
-	pauseGame();
+	if (inputState != INPUT_NORMAL)
+		return;
+	g_PauseControl.implicitPause();
 
-	Engine.PushGuiPage("page_viewer.xml", {
-		"templateName": templateName,
-		"callback": "resumeGame",
-		"civ": civCode
-	});
+	Engine.PushGuiPage(
+		"page_viewer.xml",
+		{
+			"templateName": templateName,
+			"civ": civCode
+		},
+		resumeGame);
 }
 
 /**
@@ -1174,7 +1260,6 @@ let g_PanelsOrder = [
 
 	// UNIQUE PANES (importance doesn't matter)
 	"Command",
-	"AllyCommand",
 	"Queue",
 	"Selection",
 ];
