@@ -8,6 +8,8 @@ const UPGRADING_CHOSEN_OTHER = -1;
 
 function canMoveSelectionIntoFormation(formationTemplate)
 {
+	if (formationTemplate == NULL_FORMATION)
+		return true;
 	if (!(formationTemplate in g_canMoveIntoFormation))
 		g_canMoveIntoFormation[formationTemplate] = Engine.GuiInterfaceCall("CanMoveEntsIntoFormation", {
 			"ents": g_Selection.toList(),
@@ -31,11 +33,6 @@ function hasSameRestrictionCategory(templateName1, templateName2)
 	return false;
 }
 
-function getPlayerHighlightColor(player)
-{
-	return "color:" + rgbToGuiColor(g_DisplayedPlayerColors[player], 160);
-}
-
 /**
  * Returns a "color:255 0 0 Alpha" string based on how many resources are needed.
  */
@@ -120,6 +117,66 @@ function formatLimitString(trainEntLimit, trainEntCount, trainEntLimitChangers)
 	return text;
 }
 
+/**
+ * Format template match count/limit message for the tooltip.
+ *
+ * @param {number} matchEntLimit - The limit of the entity.
+ * @param {number} matchEntCount - The count of the entity.
+ * @param {string} type - The type of the action (i.e. "build" or "training").
+ *
+ * @return {string} - The string to show the user with information regarding the limit of this template.
+ */
+function formatMatchLimitString(matchEntLimit, matchEntCount, type)
+{
+	if (matchEntLimit == undefined)
+		return "";
+
+	let passedLimit = matchEntCount >= matchEntLimit;
+	let count = matchEntLimit - matchEntCount;
+	let text;
+	if (type == "build")
+	{
+		if (passedLimit)
+			text = sprintf(translatePlural("Could only be constructed once.", "Could only be constructed %(limit)s times.", matchEntLimit), {
+				"limit": matchEntLimit
+			});
+		else if (matchEntLimit == 1)
+			text = translate("Can be constructed only once.");
+		else
+			text = sprintf(translatePlural("Can be constructed %(count)s more time.", "Can be constructed %(count)s more times.", count), {
+				"count": count
+			});
+	}
+	else if (type == "training")
+	{
+		if (passedLimit)
+			text = sprintf(translatePlural("Could only be trained once.", "Could only be trained %(limit)s times.", matchEntLimit), {
+				"limit": matchEntLimit
+			});
+		else if (matchEntLimit == 1)
+			text = translate("Can be trained only once.");
+		else
+			text = sprintf(translatePlural("Can be trained %(count)s more time.", "Can be trained %(count)s more times.", count), {
+				"count": count
+			});
+	}
+	else
+	{
+		if (passedLimit)
+			text = sprintf(translatePlural("Could only be created once.", "Could only be created %(limit)s times.", matchEntLimit), {
+				"limit": matchEntLimit
+			});
+		else if (matchEntLimit == 1)
+			text = translate("Can be created only once.");
+		else
+			text = sprintf(translatePlural("Can be created %(count)s more time.", "Can be created %(count)s more times.", count), {
+				"count": count
+			});
+	}
+
+	return passedLimit ? coloredText(text, "red") : text;
+}
+
 /**
  * Format batch training string for the tooltip
  * Examples:
@@ -149,7 +206,7 @@ function formatBatchTrainingString(buildingsCountToTrainFullBatch, fullBatchSize
 		fullBatchesString = fullBatchSize;
 
 	// We need to display the batch details part if there is either more than
-	// one building with full batch or one building with the full batch and
+	// one structure with full batch or one structure with the full batch and
 	// another with a partial batch
 	let batchString;
 	if (buildingsCountToTrainFullBatch > 1 ||
@@ -188,20 +245,23 @@ function jumpCamera(index)
 		return;
 
 	let threshold = Engine.ConfigDB_GetValue("user", "gui.session.camerajump.threshold");
+	let cameraPivot = Engine.GetCameraPivot();
 	if (g_JumpCameraLast &&
-	    Math.abs(Engine.CameraGetX() - position.x) < threshold &&
-	    Math.abs(Engine.CameraGetZ() - position.z) < threshold)
+	    Math.abs(cameraPivot.x - position.x) < threshold &&
+	    Math.abs(cameraPivot.z - position.z) < threshold)
+	{
 		Engine.CameraMoveTo(g_JumpCameraLast.x, g_JumpCameraLast.z);
+	}
 	else
 	{
-		g_JumpCameraLast = { "x": Engine.CameraGetX(), "z": Engine.CameraGetZ() };
+		g_JumpCameraLast = cameraPivot;
 		Engine.CameraMoveTo(position.x, position.z);
 	}
 }
 
 function setJumpCamera(index)
 {
-	g_JumpCameraPositions[index] = { "x": Engine.CameraGetX(), "z": Engine.CameraGetZ() };
+	g_JumpCameraPositions[index] = Engine.GetCameraPivot();
 }
 
 /**
@@ -242,30 +302,10 @@ function performCommand(entStates, commandName)
 	if (!entStates.length)
 		return;
 
-	// Don't check all entities, because we assume a player cannot
-	// select entities from more than one player
-	if (!controlsPlayer(entStates[0].player) &&
-	    !(g_IsObserver && commandName == "focus-rally"))
-		return;
-
-	if (g_EntityCommands[commandName])
+	if (getCommandInfo(commandName, entStates))
 		g_EntityCommands[commandName].execute(entStates);
 }
 
-function performAllyCommand(entity, commandName)
-{
-	if (!entity)
-		return;
-
-	let entState = GetEntityState(entity);
-	let playerState = GetSimState().players[Engine.GetPlayerID()];
-	if (!playerState.isMutualAlly[entState.player] || g_IsObserver)
-		return;
-
-	if (g_AllyEntityCommands[commandName])
-		g_AllyEntityCommands[commandName].execute(entState);
-}
-
 function performFormation(entities, formationTemplate)
 {
 	if (!entities)
@@ -274,7 +314,7 @@ function performFormation(entities, formationTemplate)
 	Engine.PostNetworkCommand({
 		"type": "formation",
 		"entities": entities,
-		"name": formationTemplate
+		"formation": formationTemplate
 	});
 }
 
@@ -319,11 +359,11 @@ function cancelPackUnit(pack)
 	});
 }
 
-function upgradeEntity(Template)
+function upgradeEntity(Template, selection)
 {
 	Engine.PostNetworkCommand({
 		"type": "upgrade",
-		"entities": g_Selection.toList(),
+		"entities": selection,
 		"template": Template,
 		"queued": false
 	});
@@ -368,41 +408,16 @@ function unloadTemplate(template, owner)
 		"template": template,
 		"owner": owner,
 		// Filter out all entities that aren't garrisonable.
-		"garrisonHolders": g_Selection.toList().filter(ent => {
+		"garrisonHolders": g_Selection.filter(ent => {
 			let state = GetEntityState(ent);
 			return state && !!state.garrisonHolder;
 		})
 	});
 }
 
-function unloadSelection()
-{
-	let parent = 0;
-	let ents = [];
-	for (let ent in g_Selection.selected)
-	{
-		let state = GetEntityState(+ent);
-		if (!state || !state.turretParent)
-			continue;
-		if (!parent)
-		{
-			parent = state.turretParent;
-			ents.push(+ent);
-		}
-		else if (state.turretParent == parent)
-			ents.push(+ent);
-	}
-	if (parent)
-		Engine.PostNetworkCommand({
-			"type": "unload",
-			"entities": ents,
-			"garrisonHolder": parent
-		});
-}
-
 function unloadAll()
 {
-	let garrisonHolders = g_Selection.toList().filter(e => {
+	const garrisonHolders = g_Selection.filter(e => {
 		let state = GetEntityState(e);
 		return state && !!state.garrisonHolder;
 	});
@@ -434,12 +449,64 @@ function unloadAll()
 		});
 }
 
+function unloadAllTurrets()
+{
+	const turretHolders = g_Selection.filter(e => {
+		let state = GetEntityState(e);
+		return state && !!state.turretHolder;
+	});
+
+	if (!turretHolders.length)
+		return;
+
+	let ownedHolders = [];
+	let ejectables = [];
+	for (let ent of turretHolders)
+	{
+		let turretHolderState = GetEntityState(ent);
+		if (controlsPlayer(turretHolderState.player))
+			ownedHolders.push(ent);
+		else
+		{
+			for (let turret of turretHolderState.turretHolder.turretPoints.map(tp => tp.entity))
+				if (turret && controlsPlayer(GetEntityState(turret).player))
+					ejectables.push(turret);
+		}
+	}
+
+	if (ejectables.length)
+		Engine.PostNetworkCommand({
+			"type": "leave-turret",
+			"entities": ejectables
+		});
+
+	if (ownedHolders.length)
+		Engine.PostNetworkCommand({
+			"type": "unload-turrets",
+			"entities": ownedHolders
+		});
+}
+
+function leaveTurretPoints()
+{
+	const entities = g_Selection.filter(entity => {
+		let entState = GetEntityState(entity);
+		return entState && entState.turretable &&
+			entState.turretable.holder != INVALID_ENTITY;
+	});
+
+	Engine.PostNetworkCommand({
+		"type": "leave-turret",
+		"entities": entities
+	});
+}
+
 function backToWork()
 {
 	Engine.PostNetworkCommand({
 		"type": "back-to-work",
 		// Filter out all entities that can't go back to work.
-		"entities": g_Selection.toList().filter(ent => {
+		"entities": g_Selection.filter(ent => {
 			let state = GetEntityState(ent);
 			return state && state.unitAI && state.unitAI.hasWorkOrders;
 		})
@@ -451,7 +518,7 @@ function removeGuard()
 	Engine.PostNetworkCommand({
 		"type": "remove-guard",
 		// Filter out all entities that are currently guarding/escorting.
-		"entities": g_Selection.toList().filter(ent => {
+		"entities": g_Selection.filter(ent => {
 			let state = GetEntityState(ent);
 			return state && state.unitAI && state.unitAI.isGuarding;
 		})
@@ -462,7 +529,7 @@ function raiseAlert()
 {
 	Engine.PostNetworkCommand({
 		"type": "alert-raise",
-		"entities": g_Selection.toList().filter(ent => {
+		"entities": g_Selection.filter(ent => {
 			let state = GetEntityState(ent);
 			return state && !!state.alertRaiser;
 		})
@@ -473,9 +540,33 @@ function endOfAlert()
 {
 	Engine.PostNetworkCommand({
 		"type": "alert-end",
-		"entities": g_Selection.toList().filter(ent => {
+		"entities": g_Selection.filter(ent => {
 			let state = GetEntityState(ent);
 			return state && !!state.alertRaiser;
 		})
 	});
 }
+
+function turnAutoQueueOn()
+{
+	Engine.PostNetworkCommand({
+		"type": "autoqueue-on",
+		"entities": g_Selection.filter(ent => {
+			let state = GetEntityState(ent);
+			return !!state?.production?.entities.length &&
+				!state.production.autoqueue;
+		})
+	});
+}
+
+function turnAutoQueueOff()
+{
+	Engine.PostNetworkCommand({
+		"type": "autoqueue-off",
+		"entities": g_Selection.filter(ent => {
+			let state = GetEntityState(ent);
+			return !!state?.production?.entities.length &&
+				state.production.autoqueue;
+		})
+	});
+}
